/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

import Map from './Map';

export default class GoogleMaps extends Map {
  constructor(pickupSelector, idMap, showPopupMarker) {
    super(pickupSelector, idMap, showPopupMarker);
    // eslint-disable-next-line no-undef
    this.infoWindow = new google.maps.InfoWindow();
    this.infoWindow.setOptions({minWidth: 290});
    this.reloadPickupListTimeout = null;
  }

  /**
   * Load and show map with markers
   */
  initMap(pickups, selectedPickupId) {
    if (!super.initMap(pickups)) {
      return false;
    }

    // eslint-disable-next-line no-undef
    this.map = new google.maps.Map(document.getElementById(this.idMap), {
      center: {lat: 0, lng: 0},
      zoom: 12,
    });

    this.markers = {};
    const $this = this;

    Object.keys(pickups).forEach((id) => {
      $this.addMapMarker(pickups[id]);
    });

    this.fitMapBounds();

    if (selectedPickupId) {
      if ($this.showPopupMarker) {
        this.openMapPopupMarker(selectedPickupId);
      } else {
        this.dancingWithMarker(selectedPickupId);
        this.pickupSelector.selectPickupInList(selectedPickupId, true);
      }
    }

    this.map.addListener('dragend', () => {
      if ($this.reloadPickupListTimeout) {
        clearTimeout($this.reloadPickupListTimeout);
      }
      $this.reloadPickupListTimeout = setTimeout(() => {
        // eslint-disable-next-line no-undef
        const geocoder = new google.maps.Geocoder();
        geocoder
          .geocode({location: {lat: $this.map.center.lat(), lng: $this.map.center.lng()}})
          .then((response) => {
            if (response.results[0]) {
              let streetNumber = '';
              let address = '';
              let postcode = '';
              let city = '';
              let country = '';
              response.results[0].address_components.forEach((component) => {
                const shortName = component.short_name;
                component.types.forEach((type) => {
                  if (type === 'street_number') {
                    streetNumber = shortName;
                  } else if (type === 'route') {
                    address = shortName;
                  } else if (type === 'postal_code') {
                    postcode = shortName;
                  } else if (type === 'locality') {
                    city = shortName;
                  } else if (type === 'country') {
                    country = shortName;
                  }
                });
              });
              if (postcode && country) {
                $this.pickupSelector.loadPickupList(`${streetNumber} ${address}`.trim(), city, postcode, country, false);
              }
            }
          })
          .catch((e) => () => { console.log(e); });
      }, 1500);
    });

    this.map.addListener('zoom_changed', () => {
      if ($this.reloadPickupListTimeout) {
        clearTimeout($this.reloadPickupListTimeout);
      }
    });

    return true;
  }

  /**
   * Fit map bounds
   */
  fitMapBounds() {
    if (!this.map) {
      return;
    }
    // eslint-disable-next-line no-undef
    const bounds = new google.maps.LatLngBounds();

    Object.keys(this.markers).forEach((id) => {
      bounds.extend(this.markers[id].getPosition());
    });
    this.map.fitBounds(bounds);
  }

  /**
   * Open popup marker for pickup and scroll to map
   * @param id
   */
  openMapPopupMarker(id) {
    if (!this.map || this.markers[id] === undefined) {
      return;
    }

    if (this.markers[id].content === undefined) {
      this.markers[id].content = this.getInfoWindowContent(this.pickupSelector.pickups[id]);
    }
    const marker = this.markers[id];

    this.infoWindow.close();
    this.infoWindow.setContent(marker.content);
    this.infoWindow.open(marker.map, marker);

    super.openMapPopupMarker(id);
  }

  /**
   * Highlight marker for pickup
   * @param id
   */
  dancingWithMarker(id) {
    Object.keys(this.markers).forEach((pickupId) => {
      if (parseInt(pickupId, 10) === parseInt(id, 10)) {
        // eslint-disable-next-line no-undef
        this.markers[pickupId].setAnimation(google.maps.Animation.BOUNCE);
      } else {
        this.markers[pickupId].setAnimation(null);
      }
    });
  }

  /**
   * Add pickup marker on the map
   * @param pickup
   */
  addMapMarker(pickup) {
    const $this = this;

    const type = parseInt(pickup.type, 10) === 4 ? 'locker' : 'point';

    // eslint-disable-next-line no-undef
    const marker = new google.maps.Marker({
      position: {lat: pickup.latitude, lng: pickup.longitude},
      map: this.map,
      title: pickup.name,
      icon: `${this.config.markerPath}poi-${type}.png`,
    });

    marker.addListener('click', () => {
      if ($this.showPopupMarker) {
        $this.openMapPopupMarker(pickup.id);
      } else {
        $this.dancingWithMarker(pickup.id);
      }
      $this.pickupSelector.selectPickupInList(pickup.id, true);
    });

    this.markers[pickup.id] = marker;
  }
}
