/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

import Map from './Map';

export default class OpenStreetMap extends Map {
  constructor(pickupSelector, idMap, showPopupMarker) {
    super(pickupSelector, idMap, showPopupMarker);
    this.featureGroup = null;
  }

  /**
   * Load and show map with markers
   */
  initMap(pickups, selectedPickupId) {
    if (!super.initMap(pickups)) {
      return;
    }

    if (this.featureGroup !== null) {
      this.featureGroup.clearLayers();
    }

    try {
      this.map = this.config.leaflet.map(this.idMap)
        .setView([0, 0], 12);
      this.config.leaflet.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
        attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
      }).addTo(this.map);
    } catch (e) {
      // Map container is already initialized
    }

    this.featureGroup = this.config.leaflet.featureGroup();
    this.featureGroup.addTo(this.map);

    const $this = this;

    this.markers = {};
    Object.keys(pickups).forEach((id) => {
      $this.addMapMarker(pickups[id]);
    });

    this.fitMapBounds();

    if (selectedPickupId) {
      if ($this.showPopupMarker) {
        this.openMapPopupMarker(selectedPickupId);
      } else {
        this.dancingWithMarker(selectedPickupId);
        this.pickupSelector.selectPickupInList(selectedPickupId, true);
      }
    }
  }

  /**
   * Fit map bounds
   */
  fitMapBounds() {
    if (!this.map || !this.featureGroup) {
      return;
    }
    this.map.fitBounds(this.featureGroup.getBounds());
  }

  /**
   * Open popup marker for pickup and scroll to map
   * @param string ID pickup
   */
  openMapPopupMarker(id) {
    if (!this.map || this.markers[id] === undefined) {
      return;
    }

    this.markers[id].openPopup();

    super.openMapPopupMarker(id);
  }

  /**
   * Highlight marker for pickup
   * @param string ID pickup
   */
  dancingWithMarker(id) {
    Object.keys(this.markers).forEach((pickupId) => {
      if (pickupId === id) {
        this.markers[pickupId].setOpacity(0.5);
      } else {
        this.markers[pickupId].setOpacity(1);
      }
    });
  }

  /**
   * Add pickup marker on the map
   * @param pickup
   */
  addMapMarker(pickup) {
    const $this = this;
    const type = parseInt(pickup.type, 10) === 4 ? 'locker' : 'point';
    const icon = this.config.leaflet.icon({
      iconUrl: `${this.config.markerPath}poi-${type}.png`,
      iconSize: [32, 36],
    });

    const marker = this.config.leaflet.marker([
      pickup.latitude,
      pickup.longitude,
    ], {
      icon,
    });

    marker.addTo(this.featureGroup);
    marker.on('click', () => {
      if (!$this.showPopupMarker) {
        $this.dancingWithMarker(pickup.id);
      }
      $this.pickupSelector.selectPickupInList(pickup.id, true);
    });

    if ($this.showPopupMarker) {
      marker.bindPopup(() => $this.getInfoWindowContent(pickup));
    }

    this.markers[pickup.id] = marker;
  }
}
