/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

import GoogleMaps from './GoogleMaps';
import OpenStreetMap from './OpenStreetMap';

export default class PickupSelector {
  constructor(
    carrierExtraContent,
    ajaxUrl,
    showPopupMarker,
    availableCountries,
    defaultCountry = 'BE',
  ) {
    // eslint-disable-next-line no-undef
    this.config = bpostshm;
    this.carrierExtraContent = $(carrierExtraContent);
    this.ajaxUrl = ajaxUrl;
    this.showPopupMarker = showPopupMarker;
    this.defaultCountry = defaultCountry;
    this.availableCountries = availableCountries;
    this.widget = this.carrierExtraContent.find(this.config.selectors.pickupWidget.parent);
    this.result = this.carrierExtraContent.find(this.config.selectors.pickupWidget.result);
    this.pickupList = this.carrierExtraContent.find(this.config.selectors.pickupWidget.pickupList);
    this.addressArea = this.carrierExtraContent.find(this.config.selectors.pickupAddress.parent);
    this.idCarrier = this.carrierExtraContent.data('carrier-id');
    this.type = this.carrierExtraContent.data('carrier-type');
    this.design = this.carrierExtraContent.data('design');
    this.pickupListAjaxQuery = null;
    this.selectPickupAjaxQuery = null;
    this.map = null;
    this.pickups = {};

    if (this.config.map === 'GoogleMaps') {
      this.map = new GoogleMaps(
        this,
        this.config.selectors.pickupWidget.map + this.idCarrier,
        showPopupMarker,
      );
    } else if (this.config.map === 'OpenStreetMap') {
      this.map = new OpenStreetMap(
        this,
        this.config.selectors.pickupWidget.map + this.idCarrier,
        showPopupMarker,
      );
    }
    if (!this.map) {
      this.showPopupMarker = false;
    }
  }

  /**
   * Show pickup selector
   */
  show() {
    this.carrierExtraContent.show();
  }

  /**
   * Hide pickup selector
   */
  hide() {
    this.carrierExtraContent.hide();
  }

  /**
   * Loading Start
   * @param string loading type
   */
  loadingStart(type) {
    let select = '';

    if (type) {
      select = `-${type}`;
    }
    this.carrierExtraContent.addClass(`bpostshm-loading${select}`);
    this.show();
  }

  /**
   * Loading Finish
   * @param string select loading type
   */
  loadingFinish(type) {
    let select = '';

    if (type) {
      select = `-${type}`;
    }
    this.carrierExtraContent.removeClass(`bpostshm-loading${select}`);
  }

  /**
   * Show an error message about pickup search
   * @param string msg
   */
  showError(msg) {
    this.widget.find(this.config.selectors.alert)
      .text(msg)
      .show();
  }

  /**
   * Hide error message
   */
  hideError() {
    this.widget.find(this.config.selectors.alert)
      .hide();
  }

  /**
   * Show pickup selector and load pickup list and map
   */
  open() {
    this.openWidget();
    this.loadPickupList();
  }

  /**
   * Open widget (show inline or open modal)
   */
  openWidget() {
    if (this.design === 'modal') {
      this.widget.show();
      this.widget.modal('show');
    } else {
      this.carrierExtraContent
        .find(`${this.config.selectors.pickupAddress.parent} ${this.config.selectors.changePickupBtn}`)
        .hide();
      this.widget.show();
    }
  }

  /**
   * Close Widget  (hide inline or close modal)
   */
  closeWidget() {
    if (this.design === 'modal') {
      this.widget.modal('hide');
    } else {
      this.carrierExtraContent
        .find(`${this.config.selectors.pickupAddress.parent} ${this.config.selectors.changePickupBtn}`)
        .show();
      this.widget.hide();
    }
  }

  /**
   * Hide pickup selector and hide map
   */
  close() {
    this.hide();
    if (this.map) {
      this.map.hide();
    }
  }

  /**
   * Get selected pickup
   * @returns string ID
   */
  getSelectedPickup() {
    return `${this.carrierExtraContent.data('selected-pickup')}`;
  }

  /**
   * Select pickup in list
   * @param int id
   * @param bool scrollTo
   */
  selectPickupInList(id, scrollTo) {
    const $pickup = this.carrierExtraContent.find(`.${this.config.selectors.pickupWidget.marker}${id}`);

    if ($pickup.length) {
      if (this.showPopupMarker) {
        $pickup.prop('checked', true);
      } else {
        this.carrierExtraContent
          .find('.pickup-open')
          .removeClass('pickup-open');
        if (this.pickups[id].openingHoursHtml === undefined) {
          this.applyPickupDetails(id);
        }
        if (this.pickups[id].openingHoursHtml !== undefined) {
          $pickup.find('.pickup-details').html(`
            ${this.pickups[id].openingHoursHtml}
            <div class="select-pickup">${this.pickups[id].selectBtnHtml}</div>
          `);
        }
        $pickup.addClass('pickup-open');
        if (scrollTo) {
          this.carrierExtraContent.find(this.config.selectors.pickupWidget.pickupList).scrollTop(0);
          this.carrierExtraContent.find(this.config.selectors.pickupWidget.pickupList).animate({
            scrollTop: $pickup.position().top,
          }, 'slow');
        }
      }
    }
  }

  /**
   * Load and show map with markers if exists
   */
  initMap() {
    if (this.map) {
      this.map.initMap(this.pickups, this.getSelectedPickup());
    }
  }

  /**
   * Load pickup list based on the search
   */
  loadPickupList(addressInfo, cityInfo, postcodeInfo, countryInfo, reloadAll = true) {
    const searchSelector = this.config.selectors.pickupWidget.search;

    let defaultCountry = this.carrierExtraContent.find(searchSelector.country).val();

    if (!defaultCountry || this.availableCountries[defaultCountry] === undefined) {
      defaultCountry = this.defaultCountry;
    }

    const country = countryInfo || defaultCountry;

    if (defaultCountry !== country) {
      return;
    }

    const deliveryDate = this.carrierExtraContent.find(searchSelector.deliveryDate).val();

    const address = addressInfo || this.carrierExtraContent.find(searchSelector.address).val();
    const city = cityInfo || this.carrierExtraContent.find(searchSelector.city).val();
    const postcode = postcodeInfo || this.carrierExtraContent.find(searchSelector.postcode).val();

    const $this = this;
    const $processingIcon = this.carrierExtraContent.find('.processing');
    this.pickups = {};

    if (reloadAll) {
      this.loadingStart();
    } else {
      $processingIcon.show();
    }

    if (this.pickupListAjaxQuery) {
      this.pickupListAjaxQuery.abort();
    }
    $.ajax({
      url: this.ajaxUrl,
      type: 'POST',
      dataType: 'json',
      data: {
        ajax: 1,
        action: 'getPickupPoints',
        address: encodeURIComponent(address),
        city: encodeURIComponent(city),
        postcode,
        country: defaultCountry,
        deliveryDate,
        type: this.type,
      },
      beforeSend: (jqXHR) => {
        this.pickupListAjaxQuery = jqXHR;
      },
    }).then((resp) => {
      if (reloadAll) {
        $this.result.hide();
      }

      if (reloadAll) {
        $this.hideError();
      }
      if (resp.errorMsg) {
        if (reloadAll) {
          $this.showError(resp.errorMsg);
          $this.loadingFinish();
        } else {
          $processingIcon.hide();
        }
        return;
      }

      if (!('pickupPoints' in resp)) {
        if (reloadAll) {
          $this.loadingFinish();
        } else {
          $processingIcon.hide();
        }
        return;
      }

      let htmlPickupList = '';

      for (let i = 0; i < resp.pickupPoints.length; i += 1) {
        const pickup = resp.pickupPoints[i];
        const id = `${pickup.id}`;

        if (id) {
          pickup.selectBtnHtml = `<a class="btn btn-select-pickup" data-id="${pickup.id}" href="#">${this.config.trans.select} »</a>`;

          if ($this.pickups[id] !== undefined) {
            if (reloadAll) {
              console.log(`Pick-up with ID ${id} already exists:`);
              console.log($this.pickups[id]);
              console.log('Other pick-up:');
              console.log(pickup);
            }
          } else {
            $this.pickups[id] = pickup;
          }
        } else {
          console.log(`Pick-up ID invalid "${id}":`);
          console.log(pickup);
        }

        const checked = resp.pickupPoints[i].id === $this.getSelectedPickup() ? 'checked' : '';

        const sep = ' - ';
        const pickupInfo = `
          <span class="bpostshm-name">${pickup.name}</span><span class="bpostshm-sep">${sep}</span>
          <span class="bpostshm-address">${pickup.address}</span><span class="bpostshm-sep">${sep}</span>
          <span class="bpostshm-postcode">${pickup.postcode}</span> <span class="bpostshm-city">${pickup.city}</span>
        `;

        const type = parseInt(pickup.type, 10) === 4 ? 'locker' : 'point';

        if ($this.showPopupMarker) {
          htmlPickupList += `
          <div class="checkbox">
            <label>
              <input
                type="radio"
                name="bpostshm_pickup"
                class="${$this.config.selectors.pickupWidget.marker}${pickup.id}"
                value="${pickup.id}"
                ${checked}
              > ${pickupInfo}
            </label>
          </div>`;
        } else {
          htmlPickupList += `
          <div class="pickup-point ${$this.config.selectors.pickupWidget.marker}${pickup.id}" data-pickup-id="${pickup.id}">
            <div class="pickup-icon">
              <img src="${this.config.markerPath}poi-${type}.png" />
            </div>
            <div class="pickup-info">${pickupInfo}</div>
            <div class="pickup-details"></div>
          </div>`;
        }
      }

      $this.pickupList.html(htmlPickupList);

      if (htmlPickupList && reloadAll) {
        $this.result.show();
      }

      if (reloadAll) {
        $this.loadingFinish();
      } else {
        $processingIcon.hide();
      }

      $this.initMap();

      prestashop.emit('bpostshmPickupListUpdated', {
        selector: $this,
        list: resp.pickupPoints,
      });
    })
      .fail((resp) => {
        if (reloadAll) {
          $this.loadingFinish();
          if (resp.statusText !== 'abort') {
            if (resp.statusText) {
              $this.showError(`${resp.statusText}, code:  ${resp.statusCode}`);
            }
            prestashop.emit('handleError', {
              eventType: 'bpostshmUpdatePickupList',
              resp,
            });
          }
        } else {
          $processingIcon.hide();
        }
      });
  }

  /**
   * Saves selected pickup in DB
   * @param id
   * @param bool openMapPopupMarker
   */
  selectPickup(id, openMapPopupMarker) {
    const pickup = this.pickups[id];

    if (!pickup) {
      return;
    }

    if (this.map.reloadPickupListTimeout !== undefined
      && this.map.reloadPickupListTimeout !== null) {
      clearTimeout(this.map.reloadPickupListTimeout);
    }

    const $this = this;
    const address = this.addressArea.find(this.config.selectors.pickupAddress.address);

    this.loadingStart('select');

    if (this.selectPickupAjaxQuery) {
      this.selectPickupAjaxQuery.abort();
    }
    $.ajax({
      type: 'POST',
      dataType: 'json',
      url: this.ajaxUrl,
      data: {
        ajax: 1,
        action: 'selectPickupPoint',
        infoPoint: JSON.stringify(pickup),
        idCarrier: this.idCarrier,
      },
      beforeSend: (jqXHR) => {
        this.selectPickupAjaxQuery = jqXHR;
      },
    }).then((resp) => {
      $this.carrierExtraContent.data('selected-pickup', pickup.id);
      $this.widget
        .find(`.${$this.config.selectors.pickupWidget.marker}${pickup.id}`)
        .prop('checked', true);

      let {country} = pickup;

      if (resp.pickupPoint.country !== undefined) {
        country = resp.pickupPoint.country;
      }

      this.addressArea.show();
      address.data('country', pickup.country);
      address.html(`
        ${pickup.name}<br>
        ${pickup.address}<br>
        ${pickup.postcode} ${pickup.city}<br>
        ${country}
      `);

      $this.closeWidget();

      $this.loadingFinish('select');

      if (openMapPopupMarker && this.map) {
        this.map.openMapPopupMarker(pickup.id);
      }

      prestashop.emit('bpostshmPickupUpdated', {
        selector: $this,
        data: resp,
        pickup,
        address,
      });
    })
      .fail((resp) => {
        $this.loadingFinish('select');
        if (resp.statusText !== 'abort') {
          if (resp.statusText) {
            $this.showError(`${resp.statusText}, code:  ${resp.statusCode}`);
          }
          prestashop.emit('handleError', {
            eventType: 'bpostshmUpdatePickup',
            resp,
          });
        }
      });
  }

  /**
   * Retrieves pick-up point details (opening hours and notes)
   * @param pickup
   * @returns {*}
   */
  applyPickupDetails(id) {
    if (this.pickups[id] === undefined) {
      return null;
    }
    $.ajax({
      url: this.ajaxUrl,
      type: 'POST',
      dataType: 'json',
      async: false,
      data: {
        ajax: 1,
        action: 'getPickupPointDetails',
        id: this.pickups[id].id,
        country: this.pickups[id].country,
        type: this.pickups[id].type,
      },
      success: (resp) => {
        let details = '';

        if (resp.pickupPoint.openingHours !== undefined && resp.pickupPoint.openingHours) {
          resp.pickupPoint.openingHours.forEach((day) => {
            if (day.schedule !== null) {
              details += `
          <tr>
            <td>${this.config.trans.days[day.day - 1]}</td>
            <td>${day.schedule}</td>
          </tr>`;
            }
          });
          if (details) {
            details = `<table><tbody>${details}</tbody></table>`;
          }
        }
        if (resp.pickupPoint.note !== undefined && resp.pickupPoint.note) {
          details += `<p class="note">${resp.pickupPoint.note}</p>`;
        }
        this.pickups[id].openingHoursHtml = details;
      },
    });
    return this.pickups[id];
  }
}
