<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

$autoloadPath = __DIR__ . '/vendor/autoload.php';
if (file_exists($autoloadPath)) {
    require_once $autoloadPath;
}

use PrestaShop\Module\BpostShm\Domain\Exception\CarrierException;
use PrestaShop\Module\BpostShm\Helper\SettingsFormHelper;
use PrestaShop\Module\BpostShm\Hook\Cron;
use PrestaShop\Module\BpostShm\HookDispatcher;
use PrestaShop\Module\BpostShm\Service\ApiService;
use PrestaShop\Module\BpostShm\Service\CarrierService;
use PrestaShop\Module\BpostShm\Service\DeliveryOptionsService;
use PrestaShop\Module\BpostShm\Service\LabelService;
use PrestaShop\Module\BpostShm\Utils\Installer;
use PrestaShop\Module\BpostShm\Utils\Uninstaller;
use PrestaShop\PrestaShop\Core\Module\WidgetInterface;

/**
 * Class BpostShm
 */
class BpostShm extends CarrierModule implements WidgetInterface
{
    /**
     * Carrier ID (property for getOrderShippingCost method)
     *
     * @var int
     */
    public $id_carrier;

    /**
     * @var Db
     */
    private $database;

    /**
     * @var HookDispatcher
     */
    private $hookDispatcher;

    /**
     * @var SettingsFormHelper
     */
    private $settingsFormHelper;

    public function __construct()
    {
        $this->name = 'bpostshm';
        $this->tab = 'shipping_logistics';
        $this->version = '2.0.1';
        $this->author = 'Stigmi.eu';
        $this->need_instance = 0;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->trans('bpost Shipping Manager', [], 'Modules.Bpostshm.Admin');
        $this->description = $this->trans('Allow your customers to choose their preferred delivery method: delivery at home or the office, at a pick-up location or in a parcel locker.', [], 'Modules.Bpostshm.Admin');
        $this->ps_versions_compliancy = ['min' => '8', 'max' => _PS_VERSION_];
        $this->module_key = '5573531ee6e49b17bf91e0ff664b2a07';

        if ($this->active && !$this->hasCredentials()) {
            $this->warning = $this->trans(
                'bpost account details must be configured before using this module.',
                [],
                'Modules.Bpostshm.Admin'
            );
        }

        $this->hookDispatcher = new HookDispatcher($this);
    }

    /**
     * Get bpost tracking url template.
     *
     * @return string the tracking url template
     */
    public function getCarrierUrl(): string
    {
        return 'https://track.bpost.cloud/btr/web/#/search?itemCode=@&postalCode=';
    }

    /**
     * Get bpost shopping methods.
     *
     * @return array returns an array containing the shipping methods
     */
    public function getShippingMethods(): array
    {
        return [
            1 => [
                'key' => 'HOME',
                'name' => 'Home deliveries / Livraison à domicile / Thuislevering',
                'delay' => [
                    'en' => 'Receive your parcel at home or at the office.',
                    'fr' => 'Recevez votre colis à la maison ou au bureau.',
                    'nl' => 'Ontvang je pakje thuis of op kantoor.',
                ],
            ],
            2 => [
                'key' => 'PICKUP',
                'name' => 'Pick-up point / Point d\'enlèvement / Afhaalpunt',
                'delay' => [
                    'en' => '2200 bpost Pick-up points, always closeby and on average 30% less CO2 emissions.',
                    'fr' => '2200 Points d\'enlèvement bpost, toujours à proximité et en moyenne 30% d\'émissions de CO2 en moins.',
                    'nl' => '2200 bpost Afhaalpunten, altijd dichtbij en gemiddeld 30% minder CO2 uitstoot.',
                ],
            ],
            4 => [
                'key' => 'PARCEL_LOCKER',
                'name' => 'Parcel locker / Distributeur de Colis / Pakjesautomaat',
                'delay' => [
                    'en' => 'Available 24/7 and reduce CO2 emissions on average with 30%.',
                    'fr' => 'Disponible 24/7 et contribue à réduire les émissions de CO2 de 30% en moyenne.',
                    'nl' => '24/7 bereikbaar en gemiddeld 30% minder CO2-uitstoot.',
                ],
            ],
        ];
    }

    /**
     * Get bpost shipping method names.
     *
     * @return array returns an array containing the bpost shipping method names
     */
    public function getShippingMethodNames(): array
    {
        return [
            'HOME' => $this->trans('Home Delivery', [], 'Modules.Bpostshm.Admin'),
            'PICKUP' => $this->trans('Pick-up point', [], 'Modules.Bpostshm.Admin'),
            'PARCEL_LOCKER' => $this->trans('Parcel locker', [], 'Modules.Bpostshm.Admin'),
        ];
    }

    /**
     * Get the tabs to install.
     *
     * @return array returns an array containing the tab name by language
     */
    public function getTabNames(): array
    {
        return [
            'AdminParentShipping' => [
                'en' => 'bpost Shipping Manager',
                'fr' => 'bpost Shipping Manager',
                'nl' => 'bpost Shipping Manager',
            ],
            'AdminBpostshmOpenedOrder' => [
                'en' => 'Opened orders',
                'fr' => 'Commandes ouvertes',
                'nl' => 'Geopende bestellingen',
            ],
            'AdminBpostshmTreatedOrder' => [
                'en' => 'Treated orders',
                'fr' => 'Commandes traitées',
                'nl' => 'Afgehandelde bestellingen',
            ],
        ];
    }

    /**
     * Get the available delivery methods.
     *
     * @return array returns an array containing the available shipping methods
     */
    public function getDeliveryMethods(): array
    {
        return [
            'Easy Retour' => $this->trans('Easy Retour', [], 'Modules.Bpostshm.Admin'),
            '24h Pro' => $this->trans('24h Pro', [], 'Modules.Bpostshm.Admin'),
            '24h business' => $this->trans('24h business', [], 'Modules.Bpostshm.Admin'),
            '@bpost' => $this->trans('@bpost', [], 'Modules.Bpostshm.Admin'),
            '@bpost international' => $this->trans('@bpost international', [], 'Modules.Bpostshm.Admin'),
            'Click & Collect' => $this->trans('Click & Collect', [], 'Modules.Bpostshm.Admin'),
            'Parcel locker' => $this->trans('Parcel locker', [], 'Modules.Bpostshm.Admin'),
            'Bus' => $this->trans('Bus', [], 'Modules.Bpostshm.Admin'),
            'Pallet' => $this->trans('Pallet', [], 'Modules.Bpostshm.Admin'),
            'World Easy Return' => $this->trans('World Easy Return', [], 'Modules.Bpostshm.Admin'),
            'World Express Pro' => $this->trans('World Express Pro', [], 'Modules.Bpostshm.Admin'),
            'World Business' => $this->trans('World Business', [], 'Modules.Bpostshm.Admin'),
            'Europe Business' => $this->trans('Europe Business', [], 'Modules.Bpostshm.Admin'),
        ];
    }

    /**
     * Get the available states for a bpost order.
     *
     * @return array returns an array containing the available shipping methods
     */
    public function getStates(): array
    {
        return [
            'OPEN' => $this->trans('OPEN', [], 'Modules.Bpostshm.Admin'),
            'PENDING' => $this->trans('PENDING', [], 'Modules.Bpostshm.Admin'),
            'CANCELLED' => $this->trans('CANCELLED', [], 'Modules.Bpostshm.Admin'),
            'ON-HOLD' => $this->trans('ON-HOLD', [], 'Modules.Bpostshm.Admin'),
            'PRINTED' => $this->trans('PRINTED', [], 'Modules.Bpostshm.Admin'),
            'ANNOUNCED' => $this->trans('ANNOUNCED', [], 'Modules.Bpostshm.Admin'),
            'IN_TRANSIT' => $this->trans('IN TRANSIT', [], 'Modules.Bpostshm.Admin'),
            'AWAITING_PICKUP' => $this->trans('AWAITING PICKUP', [], 'Modules.Bpostshm.Admin'),
            'DELIVERED' => $this->trans('DELIVERED', [], 'Modules.Bpostshm.Admin'),
            'BACK_TO_SENDER' => $this->trans('BACK TO SENDER', [], 'Modules.Bpostshm.Admin'),
            'MULTIPLE' => $this->trans('MULTIPLE', [], 'Modules.Bpostshm.Admin'),
            'SHM_ONLY_PROCESSING' => $this->trans('ONLY ON SHM', [], 'Modules.Bpostshm.Admin'),
        ];
    }

    /**
     * Get css selectors for the current theme
     *
     * @return array
     */
    public function getSelectors(): array
    {
        // don't use ID!
        return [
            'carrierName' => '.js-delivery-option .carrier-name',
            'bpostshmCarrier' => '.bpostshm-carrier',
            'deliveryDate' => '.bpostshm-delivery-date',
            'deliverySaturdayAlert' => '.bpostshm-saturday-alert',
            'deliveryOption' => '.js-delivery-option',
            'alert' => '.alert-danger',
            'choosePickupBtn' => '.bpostshm-choose-pickup',
            'changePickupBtn' => '.bpostshm-change-pickup',
            'pickupCarrier' => '.bpostshm-pickup-carrier',
            'pickupWidget' => [
                'parent' => '.bpostshm-widget',
                'map' => 'bpostshm-map',
                'marker' => 'bpostshm-marker',
                'markerBtn' => '.btn-select-pickup',
                'result' => '.bpostshm-result',
                'pickupList' => '.bpostshm-pickup-list',
                'search' => [
                    'address' => '.bpostshm-search .poi-address',
                    'postcode' => '.bpostshm-search .poi-postcode',
                    'city' => '.bpostshm-search .poi-city',
                    'country' => '.bpostshm-search .poi-country',
                    'deliveryDate' => '.bpostshm-delivery-date select',
                    'btn' => '.bpostshm-search button',
                ],
            ],
            'pickupAddress' => [
                'parent' => '.bpostshm-pickup-point-address',
                'address' => 'address',
            ],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function isUsingNewTranslationSystem(): bool
    {
        return true;
    }

    /**
     * Return the current database instance
     *
     * @return Db
     */
    public function getDatabase(): Db
    {
        if ($this->database === null) {
            $this->database = Db::getInstance();
        }

        return $this->database;
    }

    /**
     * Return current context
     *
     * @return Context
     */
    public function getContext(): Context
    {
        return $this->context;
    }

    /**
     * @return HookDispatcher
     */
    public function getHookDispatcher(): HookDispatcher
    {
        return $this->hookDispatcher;
    }

    /**
     * @return SettingsFormHelper
     */
    public function getSettingsFormHelper(): SettingsFormHelper
    {
        if ($this->settingsFormHelper === null) {
            $this->settingsFormHelper = new SettingsFormHelper($this);
        }

        return $this->settingsFormHelper;
    }

    /**
     * Dispatch hooks
     *
     * @param string $methodName
     * @param array $arguments
     *
     * @return mixed
     */
    public function __call(string $methodName, array $arguments)
    {
        return $this->getHookDispatcher()->dispatch(
            $methodName,
            !empty($arguments[0]) ? $arguments[0] : []
        );
    }

    /**
     * Install method
     *
     * @return bool
     */
    public function install(): bool
    {
        if (false === extension_loaded('curl')) {
            $this->_errors[] = $this->trans(
                'You have to enable the cURL extension on your server to install this module.',
                [],
                'Modules.Bpostshm.Admin'
            );

            return false;
        }

        $installed = parent::install()
            && $this->registerHook($this->getHookDispatcher()->getAvailableHooks());

        if ($installed) {
            (new Installer($this))->install();
        } else {
            $this->uninstall();

            return false;
        }

        return true;
    }

    /**
     * Uninstall method
     *
     * @return bool
     */
    public function uninstall(): bool
    {
        try {
            (new Uninstaller($this))->uninstall();
        } catch (PrestaShopException|PrestaShopDatabaseException $exception) {
            $this->log($exception->getMessage());
        }

        return parent::uninstall();
    }

    /**
     * Check if the API credentials are set.
     *
     * @return bool returns true if the API credentials are set, otherwise false
     */
    public function hasCredentials(): bool
    {
        return Configuration::get('BPOSTSMH_ACCOUNT_ID') && Configuration::get('BPOSTSHM_ACCOUNT_PASSPHRASE');
    }

    /**
     * Retrieve credentials for api
     *
     * @return string[]
     */
    public function getCredentials(): array
    {
        return [
            'username' => Configuration::get('BPOSTSMH_ACCOUNT_ID'),
            'password' => Configuration::get('BPOSTSHM_ACCOUNT_PASSPHRASE'),
        ];
    }

    /**
     * @return string
     *
     * @throws CarrierException
     */
    public function getContent(): string
    {
        $settingsBuilder = $this->getSettingsFormHelper();

        $result = [];
        if (true === Tools::isSubmit('submitBpostshmModule')) {
            $result = $settingsBuilder->postProcess();
        }

        $serviceMsg = '';
        /** @var ApiService $apiService */
        $apiService = $this->get('bpostshm.api_service');
        if (!$apiService->isAlive()) {
            $serviceMsg = $this->displayError($this->trans(
                'bpost-API is not available at the moment. Try again later.',
                [],
                'Modules.Bpostshm.Admin'
            ));
        } elseif ($this->hasCredentials()) {
            if (!$apiService->checkCredentials()) {
                $serviceMsg = $this->displayError($this->trans(
                    'bpost Account is not successfully configured. Check credentials.',
                    [],
                    'Modules.Bpostshm.Admin'
                ));
            } else {
                $serviceMsg = $this->displayConfirmation($this->trans(
                    'bpost Account is successfully configured.',
                    [],
                    'Modules.Bpostshm.Admin'
                ));
            }
        }

        try {
            /** @var CarrierService $carrierService */
            $carrierService = $this->get('bpostshm.carrier_service');
            $carrierService->checkCarriersConfig();
        } catch (CarrierException $e) {
            $serviceMsg .= $this->displayError($e->getMessage());
        }

        /** @var LabelService $labelService */
        $labelService = $this->get('bpostshm.label_service');
        if (!$labelService->checkPhoneFieldsRequirements()) {
            $serviceMsg .= $this->displayWarning(
                $this->trans(
                    'Entering a telephone number is currently not compulsory in your address parameters.',
                    [],
                    'Modules.Bpostshm.Admin'
                ) . ' ' .
                $this->trans(
                    'Failure to enter a telephone number will prevent you from printing international labels.',
                    [],
                    'Modules.Bpostshm.Admin'
                )
            );
        }

        $layout = $this->fetch('module:bpostshm/views/templates/admin/settings.tpl');

        return str_replace('[[settings_form]]', $serviceMsg . $settingsBuilder->renderForm($result), $layout);
    }

    public function getDefaultCountryCode(): string
    {
        return 'BE';
    }

    /**
     * Render template
     *
     * @param string $template
     * @param array $params
     *
     * @return string
     */
    public function render(string $template, array $params = []): string
    {
        $this->context->smarty->assign($params);

        return $this->display(__FILE__, $template);
    }

    /**
     * @param $hookName
     * @param array $configuration
     *
     * @return void
     */
    public function renderWidget($hookName, array $configuration)
    {
        // silence is golden
    }

    /**
     * @param $hookName
     * @param array $configuration
     *
     * @return void
     */
    public function getWidgetVariables($hookName, array $configuration)
    {
        // silence is golden
    }

    /**
     * {@inheritdoc}
     */
    public function getOrderShippingCost($cart, $shipping_cost)
    {
        /** @var DeliveryOptionsService $deliveryOptionsService */
        $deliveryOptionsService = $this->get('bpostshm.delivery_options_service');
        $additional = $deliveryOptionsService->getAdditionalShippingCost($cart, $this->id_carrier);
        if ($additional === false) {
            return false;
        }

        return $shipping_cost + $additional;
    }

    /**
     * {@inheritdoc}
     */
    public function getOrderShippingCostExternal($cart)
    {
        $deliveryOptionsService = $this->get('bpostshm.delivery_options_service');

        return $deliveryOptionsService->getAdditionalShippingCost($cart, $this->id_carrier);
    }

    /**
     * Get cron frequency
     *
     * @return int[] cron frequency
     */
    public function getCronFrequency(): array
    {
        return Cron::FREQUENCY;
    }

    /**
     * Adds a log item to the database and send a mail if configured for this $severity.
     *
     * @param string $msg the log message
     * @param int $severity
     * @param int|null $errorCode
     * @param string|null $objectType
     * @param int|null $objectId
     *
     * @return bool true if succeed
     */
    public function log(
        string $msg,
        int $severity = 2,
        ?int $errorCode = null,
        ?string $objectType = null,
        ?int $objectId = null
    ): bool {
        $msg = 'BPOSTSHM: [' . $_SERVER['REMOTE_ADDR'] . '] ' . $msg;

        return PrestaShopLogger::addLog($msg, $severity, $errorCode, $objectType, $objectId);
    }
}
