<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\Module\BpostShm\Service\ApiService;
use PrestaShop\Module\BpostShm\Service\PickupService;

/**
 * Class BpostshmPickupModuleFrontController
 */
class BpostshmPickupModuleFrontController extends ModuleFrontController
{
    /**
     * Displays the selected pickup point and stores the pickup details.
     *
     * @return void
     */
    public function displayAjaxSelectPickupPoint()
    {
        /** @var PickupService $pickupService */
        $pickupService = $this->module->get('bpostshm.pickup_service');
        $errorMsg = '';
        $pickup = '';
        try {
            $carrier = new Carrier(Tools::getValue('idCarrier'));
            if (!Validate::isLoadedObject($carrier)) {
                throw new PrestaShopException($this->context->getTranslator()->trans('Carrier not found.', [], 'Modules.Bpostshm.Shop'));
            }
            if (Validate::isLoadedObject($this->context->cart) && !$this->context->cart->id_carrier) {
                $this->context->cart->id_carrier = $carrier->id;
                $this->context->cart->update();
            }

            $infoPoint = json_decode(Tools::getValue('infoPoint'), true);
            if (is_array($infoPoint)) {
                $infoPoint['id_lang'] = $this->context->language->id;
            }

            $pickupService->storePickup($carrier->id, (int) $this->context->cart->id, $infoPoint);
            $pickup = $pickupService->getSelectedPickupAddress(
                $carrier->id,
                $this->context->language->id,
                $this->context->cart->id
            );
        } catch (Exception $e) {
            $this->module->log($e->getMessage(), 3);
            $errorMsg = $e->getMessage();
        }

        $this->ajaxRender(json_encode([
            'errorMsg' => $errorMsg,
            'pickupPoint' => $pickup,
        ]));
    }

    /**
     * Fetches and displays the pickup points based on the provided address and other parameters.
     *
     * @return void
     */
    public function displayAjaxGetPickupPoints()
    {
        $address = urldecode(Tools::getValue('address'));
        $postcode = Tools::getValue('postcode');
        $deliveryDate = Tools::getValue('deliveryDate');
        if ($deliveryDate && strtotime($deliveryDate) !== false) {
            $deliveryDate = date('d-m-Y', strtotime($deliveryDate));
        } else {
            $deliveryDate = null;
        }
        $type = Tools::getValue('type', PickupService::PICKUP_TYPE);
        $city = null;
        $country = null;
        $trans = $this->context->getTranslator();

        if (Tools::getIsset('city') && Tools::getValue('city') != 'unknown') {
            $city = Tools::getValue('city');
        }

        if (Tools::getIsset('country') && Tools::getValue('country') != 'unknown') {
            $country = Tools::getValue('country');
        }

        try {
            /** @var ApiService $apiService */
            $apiService = $this->module->get('bpostshm.api_service');
            /** @var PickupService $pickupService */
            $pickupService = $this->module->get('bpostshm.pickup_service');
            $pickupPoints = $pickupService->getPickupPoints($address, $postcode, $city, $country, $deliveryDate, $type);
            if (empty($pickupPoints)) {
                $pickupPoints = [
                    'errorCode' => 1,
                    'errorMsg' => $trans->trans(
                        'No pick-up point found. Please Check Address.',
                        [],
                        'Modules.Bpostshm.Shop'
                    ),
                ];
            } else {
                $pickupPoints = [
                    'pickupPoints' => $pickupPoints,
                    'errorCode' => 0,
                    'errorMsg' => '',
                ];
            }
        } catch (Exception $e) {
            if (!$apiService->isAlive()) {
                $e = new PrestaShopException(
                    $trans->trans('The bpost service is temporarily unavailable.', [], 'Modules.Bpostshm.Shop')
                );
            }
            $this->module->log($e->getMessage(), 3);
            $pickupPoints = ['errorCode' => 1, 'errorMsg' => $e->getMessage()];
        }

        $this->ajaxRender(json_encode($pickupPoints));
    }

    /**
     * Fetches the pickup point details.
     *
     * @return void
     */
    public function displayAjaxGetPickupPointDetails()
    {
        $id = (int) Tools::getValue('id');
        $country = Tools::getValue('country');
        $type = Tools::getValue('type', PickupService::PICKUP_TYPE);
        $trans = $this->context->getTranslator();

        /** @var PickupService $pickupService */
        $pickupService = $this->module->get('bpostshm.pickup_service');

        $availableCountries = array_keys(
            $pickupService->getAvailableCountries($type == PickupService::PARCEL_LOCKER_TYPE)
        );
        if (!in_array($country, $availableCountries)) {
            $country = $this->module->getDefaultCountryCode();
        }

        try {
            /** @var ApiService $apiService */
            $apiService = $this->module->get('bpostshm.api_service');
            $pickupPoint = $pickupService->getPickupDetails($id, $this->context->language->iso_code, $country, $type);
            if (empty($pickupPoint)) {
                $pickupPoint = [
                    'errorCode' => 1,
                    'errorMsg' => $trans->trans(
                        'No pick-up point found.',
                        [],
                        'Modules.Bpostshm.Shop'
                    ),
                ];
            } else {
                $pickupPoint = [
                    'pickupPoint' => $pickupPoint,
                    'errorCode' => 0,
                    'errorMsg' => '',
                ];
            }
        } catch (Exception $e) {
            if (!$apiService->isAlive()) {
                $e = new PrestaShopException(
                    $trans->trans('The bpost service is temporarily unavailable.', [], 'Modules.Bpostshm.Shop')
                );
            }
            $this->module->log($e->getMessage(), 3);
            $pickupPoint = ['errorCode' => 1, 'errorMsg' => $e->getMessage()];
        }

        $this->ajaxRender(json_encode($pickupPoint));
    }
}
