<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Adapter\BpostshmLabel\CommandHandler;

use PrestaShop\Module\BpostShm\Adapter\BpostshmLabel\Command\BulkDownloadLabelsCommand;
use PrestaShop\Module\BpostShm\Adapter\Order\Repository\OrderRepository;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\DownloadLabelsException;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\NoLabelFoundException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\NotABpostCarrierException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\ValueObject\Order;
use PrestaShop\Module\BpostShm\Service\LabelService;

/**
 * @internal
 */
final class BulkDownloadLabelsHandler implements BulkDownloadLabelsHandlerInterface
{
    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * @var LabelService
     */
    private $labelService;

    /**
     * @var OrderRepository
     */
    private $orderRepository;

    /**
     * @param \BpostShm $module
     * @param LabelService $labelService
     * @param OrderRepository $orderRepository
     */
    public function __construct(
        \BpostShm $module,
        LabelService $labelService,
        OrderRepository $orderRepository
    ) {
        $this->module = $module;
        $this->labelService = $labelService;
        $this->orderRepository = $orderRepository;
    }

    /**
     * @param BulkDownloadLabelsCommand $command
     *
     * @return array
     *
     * @throws DownloadLabelsException
     */
    public function handle(BulkDownloadLabelsCommand $command): array
    {
        $ordersWithoutLabels = [];
        $ordersWithGeneratedLabelsIds = [];
        $labels = [];
        foreach ($command->getOrderIds() as $orderId) {
            try {
                $order = $this->orderRepository->get($orderId);

                $orderCarrier = new \Carrier($order->id_carrier);
                if ($orderCarrier->external_module_name != $this->module->name) {
                    throw new NotABpostCarrierException($orderId);
                }

                $label = $this->labelService->getLabelPdfByOrder($order, $command->getRetour());
                if ($label) {
                    $labels[] = $label;
                } else {
                    throw new NoLabelFoundException();
                }

                $ordersWithGeneratedLabelsIds[] = $orderId->getValue();
            } catch (\Exception $e) {
                $ordersWithoutLabels[] = new Order($orderId, $e);
            }
        }

        if (!empty($ordersWithoutLabels)) {
            throw new DownloadLabelsException($ordersWithoutLabels, $ordersWithGeneratedLabelsIds);
        }

        return $labels;
    }
}
