<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Adapter\BpostshmLabel\CommandHandler;

use PrestaShop\Module\BpostShm\Adapter\BpostshmLabel\Command\BulkGenerateLabelsCommand;
use PrestaShop\Module\BpostShm\Adapter\Order\Repository\OrderRepository;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\GenerateLabelsException;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\NoLabelFoundException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\NotABpostCarrierException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\ValueObject\Order;
use PrestaShop\Module\BpostShm\Service\LabelService;
use PrestaShop\Module\BpostShm\Service\OrderService;

/**
 * @internal
 */
final class BulkGenerateLabelsHandler implements BulkGenerateLabelsHandlerInterface
{
    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * @var OrderService
     */
    private $orderService;

    /**
     * @var LabelService
     */
    private $labelService;

    /**
     * @var OrderRepository
     */
    private $orderRepository;

    /**
     * @param \BpostShm $module
     * @param LabelService $labelService
     * @param OrderService $orderService
     * @param OrderRepository $orderRepository
     */
    public function __construct(
        \BpostShm $module,
        LabelService $labelService,
        OrderService $orderService,
        OrderRepository $orderRepository
    ) {
        $this->module = $module;
        $this->labelService = $labelService;
        $this->orderService = $orderService;
        $this->orderRepository = $orderRepository;
    }

    /**
     * @param BulkGenerateLabelsCommand $command
     *
     * @return array
     *
     * @throws GenerateLabelsException
     * @throws NoLabelFoundException
     */
    public function handle(BulkGenerateLabelsCommand $command): array
    {
        $ordersWithFailedToGenerateLabels = [];
        $ordersWithGeneratedLabelsIds = [];

        $labels = [];
        foreach ($command->getOrderIds() as $orderId) {
            try {
                $order = $this->orderRepository->get($orderId);

                $orderCarrier = new \Carrier($order->id_carrier);
                if ($orderCarrier->external_module_name != $this->module->name) {
                    throw new NotABpostCarrierException($orderId);
                }

                $label = $this->labelService->getLabelPdfByOrder($order, $command->getRetour());
                if ($label) {
                    $labels[] = $label;
                } else {
                    $this->orderService->prepareBpostOrder($order, $command->getRetour());
                    $label = $this->labelService->generateLabel($order, $command->getRetour());
                    $labels[] = $label;
                    if ($label && \Configuration::get(
                        'BPOSTSHM_LABELS_MARK_PRINTED_ORDER_AS_TREATED',
                        null,
                        $order->id_shop_group,
                        $order->id_shop
                    )
                    ) {
                        $this->orderService->markOrderAsTreated($order);
                    }
                }

                $ordersWithGeneratedLabelsIds[] = $orderId->getValue();
            } catch (\Exception $e) {
                $ordersWithFailedToGenerateLabels[] = new Order($orderId, $e);
            }
        }

        if (!empty($ordersWithFailedToGenerateLabels)) {
            throw new GenerateLabelsException($ordersWithFailedToGenerateLabels, $ordersWithGeneratedLabelsIds);
        }

        if (empty($labels)) {
            throw new NoLabelFoundException();
        }

        return $labels;
    }
}
