<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Adapter\Presenter\Order;

use Address;
use Carrier;
use Doctrine\Common\Annotations\AnnotationException;
use Order;
use PrestaShop\Module\BpostShm\Service\CarrierService;
use PrestaShop\Module\BpostShm\Service\LabelService;
use PrestaShop\Module\BpostShm\Service\PickupService;
use PrestaShop\PrestaShop\Adapter\Presenter\Object\ObjectPresenter;

/**
 * Class OrderLazyArray
 */
class OrderLazyArray extends \PrestaShop\PrestaShop\Adapter\Presenter\Order\OrderLazyArray
{
    /** @var BpostShm */
    private $module;

    /** @var \Order */
    private $order;

    /** @var ObjectPresenter */
    private $objectPresenter;

    /**
     * Class Constructor
     *
     * @param mixed $module the module object
     * @param \Order $order the order object
     *
     * @throws AnnotationException
     * @throws \ReflectionException
     */
    public function __construct($module, \Order $order)
    {
        $this->module = $module;
        $this->order = $order;
        $this->objectPresenter = new ObjectPresenter();
        parent::__construct($order);
    }

    /**
     * Retrieves the carrier and translates the carrier name
     *
     * @arrayAccess
     *
     * @return array
     */
    public function getCarrier(): array
    {
        $orderCarrier = parent::getCarrier();

        if (isset($orderCarrier['name'])) {
            /** @var CarrierService $carrierService */
            $carrierService = $this->module->get('bpostshm.carrier_service');

            $orderCarrier['name'] = $carrierService->getTranslatedCarrierName(
                $orderCarrier['name'],
                $this->order->getAssociatedLanguage()
            );
        }

        return $orderCarrier;
    }

    /**
     * @inerhitDoc
     *
     * @arrayAccess
     */
    public function getDetails(): OrderDetailLazyArray
    {
        return new OrderDetailLazyArray($this->module, $this->order);
    }

    /**
     * Retrieves the delivery and invoice addresses for the order.
     *
     * If the order is sent to a pickup-up point,
     * the address of this pickup-up point replaces the customer's delivery address.
     *
     * @arrayAccess
     *
     * @return array the delivery and invoice addresses
     *
     * @throws \Exception
     */
    public function getAddresses(): array
    {
        $order = $this->order;

        $orderAddresses = [
            'delivery' => [],
        ];

        if (!$order->isVirtual()) {
            $carrier = new \Carrier($order->id_carrier);
            $addressDelivery = new \Address((int) $order->id_address_delivery);

            /** @var PickupService $pickupService */
            $pickupService = $this->module->get('bpostshm.pickup_service');
            if ($pickupService->isPickup($carrier)) {
                $addressDelivery = $pickupService->getSelectedPickupAddress(
                    $carrier->id,
                    \Context::getContext()->language->id,
                    $order->id_cart,
                    true,
                    $addressDelivery->firstname,
                    $addressDelivery->lastname
                );
            }

            $orderAddresses['delivery'] = $this->objectPresenter->present($addressDelivery);
            $orderAddresses['delivery']['formatted'] =
                \AddressFormat::generateAddress($addressDelivery, [], '<br />');
        }

        $addressInvoice = new \Address((int) $order->id_address_invoice);
        $orderAddresses['invoice'] = $this->objectPresenter->present($addressInvoice);
        $orderAddresses['invoice']['formatted'] = \AddressFormat::generateAddress($addressInvoice, [], '<br />');

        return $orderAddresses;
    }

    /**
     * Get the follow-up tracking URL for the order, especially adapted for bpost carrier.
     *
     * @arrayAccess
     *
     * @return string the follow-up tracking URL for the order
     */
    public function getFollowUp(): string
    {
        /** @var LabelService $labelService */
        $labelService = $this->module->get('bpostshm.label_service');

        $trackingNumbers = $labelService->getTrackingNumbers($this->order->id);
        foreach ($trackingNumbers as $tracking) {
            if ($tracking['url']) {
                return $tracking['url'];
            }
        }

        return '';
    }
}
