<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Api\Bpost\Order;

use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\AtHome;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\International;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\National;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Customer\Sender;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidValueException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostNotImplementedException;

class Box
{
    const BOX_STATUS_OPEN = 'OPEN';
    const BOX_STATUS_PENDING = 'PENDING';
    const BOX_STATUS_PRINTED = 'PRINTED';
    const BOX_STATUS_CANCELLED = 'CANCELLED';
    const BOX_STATUS_ON_HOLD = 'ON-HOLD';
    const BOX_STATUS_ANNOUNCED = 'ANNOUNCED';
    const BOX_STATUS_IN_TRANSIT = 'IN_TRANSIT';
    const BOX_STATUS_AWAITING_PICKUP = 'AWAITING_PICKUP';
    const BOX_STATUS_DELIVERED = 'DELIVERED';
    const BOX_STATUS_BACK_TO_SENDER = 'BACK_TO_SENDER';

    /**
     * @var Sender
     */
    private $sender;

    /**
     * @var AtHome
     */
    private $nationalBox;

    /**
     * @var International
     */
    private $internationalBox;

    /**
     * @var string
     */
    private $remark;

    /**
     * @var string
     */
    private $status;

    /** @var string */
    private $barcode;

    /** @var string */
    private $additionalCustomerReference;

    /**
     * @param International $internationalBox
     */
    public function setInternationalBox(International $internationalBox)
    {
        $this->internationalBox = $internationalBox;
    }

    /**
     * @return International
     */
    public function getInternationalBox()
    {
        return $this->internationalBox;
    }

    /**
     * @param National $nationalBox
     */
    public function setNationalBox(National $nationalBox)
    {
        $this->nationalBox = $nationalBox;
    }

    /**
     * @return National
     */
    public function getNationalBox()
    {
        return $this->nationalBox;
    }

    /**
     * @param string $remark
     */
    public function setRemark($remark)
    {
        $this->remark = $remark;
    }

    /**
     * @return string
     */
    public function getRemark()
    {
        return $this->remark;
    }

    /**
     * @param Sender $sender
     */
    public function setSender(Sender $sender)
    {
        $this->sender = $sender;
    }

    /**
     * @return Sender
     */
    public function getSender()
    {
        return $this->sender;
    }

    /**
     * @param string $status
     *
     * @throws BpostInvalidValueException
     */
    public function setStatus($status)
    {
        $status = strtoupper($status);
        if (!in_array($status, self::getPossibleStatusValues())) {
            throw new BpostInvalidValueException('status', $status, self::getPossibleStatusValues());
        }

        $this->status = $status;
    }

    /**
     * @param string $barcode
     */
    public function setBarcode($barcode)
    {
        $this->barcode = strtoupper((string) $barcode);
    }

    /**
     * @return string
     */
    public function getBarcode()
    {
        return $this->barcode;
    }

    /**
     * @return string
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @param string $additionalCustomerReference
     */
    public function setAdditionalCustomerReference($additionalCustomerReference)
    {
        $this->additionalCustomerReference = (string) $additionalCustomerReference;
    }

    /**
     * @return string
     */
    public function getAdditionalCustomerReference()
    {
        return $this->additionalCustomerReference;
    }

    /**
     * @return array
     */
    public static function getPossibleStatusValues()
    {
        return [
            self::BOX_STATUS_OPEN,
            self::BOX_STATUS_PENDING,
            self::BOX_STATUS_PRINTED,
            self::BOX_STATUS_CANCELLED,
            self::BOX_STATUS_ON_HOLD,
            self::BOX_STATUS_ANNOUNCED,
            self::BOX_STATUS_IN_TRANSIT,
            self::BOX_STATUS_AWAITING_PICKUP,
            self::BOX_STATUS_DELIVERED,
            self::BOX_STATUS_BACK_TO_SENDER,
        ];
    }

    /**
     * Return the object as an array for usage in the XML
     *
     * @param \DOMDocument $document
     * @param null $prefix
     *
     * @return \DOMElement
     *
     * @throws \DOMException
     */
    public function toXML(\DOMDocument $document, $prefix = null)
    {
        $tagName = 'box';
        if ($prefix !== null) {
            $tagName = $prefix . ':' . $tagName;
        }

        $box = $document->createElement($tagName);

        $this->senderToXML($document, $prefix, $box);
        $this->boxToXML($document, $prefix, $box);
        $this->remarkToXML($document, $prefix, $box);
        $this->additionalCustomerReferenceToXML($document, $prefix, $box);
        $this->barcodeToXML($document, $prefix, $box);

        return $box;
    }

    /**
     * @param \SimpleXMLElement $xml
     *
     * @return Box
     *
     * @throws BpostInvalidValueException
     * @throws BpostNotImplementedException
     */
    public static function createFromXML(\SimpleXMLElement $xml)
    {
        $box = new Box();
        if (isset($xml->sender)) {
            $box->setSender(
                Sender::createFromXML(
                    $xml->sender->children(
                        'http://schema.post.be/shm/deepintegration/v3/common'
                    )
                )
            );
        }
        if (isset($xml->nationalBox)) {
            /** @var \SimpleXMLElement $nationalBoxData */
            $nationalBoxData = $xml->nationalBox->children('http://schema.post.be/shm/deepintegration/v3/national');

            // build classname based on the tag name
            $className = '\\PrestaShop\\Module\\BpostShm\\Api\\Bpost\\Order\\Box\\' . ucfirst($nationalBoxData->getName());
            if ($nationalBoxData->getName() == 'at24-7') {
                $className = '\\PrestaShop\\Module\\BpostShm\\Api\\Bpost\\Order\\Box\\At247';
            }

            if (!method_exists($className, 'createFromXML')) {
                throw new BpostNotImplementedException('No createFromXML found into ' . $className);
            }

            $nationalBox = call_user_func(
                [$className, 'createFromXML'],
                $nationalBoxData
            );

            $box->setNationalBox($nationalBox);
        }
        if (isset($xml->internationalBox)) {
            /** @var \SimpleXMLElement $internationalBoxData */
            $internationalBoxData = $xml->internationalBox->children('http://schema.post.be/shm/deepintegration/v3/international');

            // build classname based on the tag name
            $className = '\\PrestaShop\\Module\\BpostShm\\Api\\Bpost\\Order\\Box\\' . ucfirst($internationalBoxData->getName());

            if (!method_exists($className, 'createFromXML')) {
                throw new BpostNotImplementedException('No createFromXML found into ' . $className);
            }

            $internationalBox = call_user_func(
                [$className, 'createFromXML'],
                $internationalBoxData
            );

            $box->setInternationalBox($internationalBox);
        }
        if (isset($xml->remark) && $xml->remark != '') {
            $box->setRemark((string) $xml->remark);
        }
        if (isset($xml->additionalCustomerReference) && $xml->additionalCustomerReference != '') {
            $box->setAdditionalCustomerReference((string) $xml->additionalCustomerReference);
        }
        if (!empty($xml->barcode)) {
            $box->setBarcode((string) $xml->barcode);
        }
        if (isset($xml->status) && $xml->status != '') {
            $box->setStatus((string) $xml->status);
        }

        return $box;
    }

    /**
     * @param \DOMDocument $document
     * @param $prefix
     * @param \DOMElement $box
     */
    private function barcodeToXML(\DOMDocument $document, $prefix, \DOMElement $box)
    {
        if ($this->getBarcode() !== null) {
            $tagName = 'barcode';
            if ($prefix !== null) {
                $tagName = $prefix . ':' . $tagName;
            }
            $box->appendChild(
                $document->createElement(
                    $tagName,
                    $this->getBarcode()
                )
            );
        }
    }

    /**
     * @param \DOMDocument $document
     * @param $prefix
     * @param \DOMElement $box
     */
    private function boxToXML(\DOMDocument $document, $prefix, \DOMElement $box)
    {
        if ($this->getNationalBox() !== null) {
            $box->appendChild(
                $this->getNationalBox()->toXML($document, $prefix)
            );
        }
        if ($this->getInternationalBox() !== null) {
            $box->appendChild(
                $this->getInternationalBox()->toXML($document, $prefix)
            );
        }
    }

    /**
     * @param \DOMDocument $document
     * @param $prefix
     * @param \DOMElement $box
     */
    private function senderToXML(\DOMDocument $document, $prefix, \DOMElement $box)
    {
        if ($this->getSender() !== null) {
            $box->appendChild(
                $this->getSender()->toXML($document, $prefix)
            );
        }
    }

    /**
     * @param \DOMDocument $document
     * @param $prefix
     * @param \DOMElement $box
     *
     * @throws \DOMException
     */
    private function remarkToXML(\DOMDocument $document, $prefix, \DOMElement $box)
    {
        if ($this->getRemark() !== null) {
            $tagName = 'remark';
            if ($prefix !== null) {
                $tagName = $prefix . ':' . $tagName;
            }
            $box->appendChild(
                $document->createElement(
                    $tagName,
                    $this->getRemark()
                )
            );
        }
    }

    /**
     * @param \DOMDocument $document
     * @param $prefix
     * @param \DOMElement $box
     *
     * @throws \DOMException
     */
    private function additionalCustomerReferenceToXML(\DOMDocument $document, $prefix, \DOMElement $box)
    {
        if ($this->getAdditionalCustomerReference() !== null) {
            $tagName = 'additionalCustomerReference';
            if ($prefix !== null) {
                $tagName = $prefix . ':' . $tagName;
            }
            $box->appendChild(
                $document->createElement(
                    $tagName,
                    $this->getAdditionalCustomerReference()
                )
            );
        }
    }
}
