<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Api\Bpost\Order\Box;

use PrestaShop\Module\BpostShm\Api\Bpost\Order\Customer\Receiver;
use PrestaShop\Module\BpostShm\Api\Bpost\ProductConfiguration\Product;
use PrestaShop\Module\BpostShm\Api\BpostException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidValueException;
use PrestaShop\Module\BpostShm\Api\Exception\XmlException\BpostXmlInvalidItemException;

class AtHome extends National
{
    /** @var Receiver */
    private $receiver;

    /** @var string */
    protected $requestedDeliveryDate;

    /**
     * @param string $product
     *
     * @see getPossibleProductValues
     *
     * @throws BpostInvalidValueException
     */
    public function setProduct($product)
    {
        if (!in_array($product, self::getPossibleProductValues())) {
            throw new BpostInvalidValueException('product', $product, self::getPossibleProductValues());
        }

        parent::setProduct($product);
    }

    /**
     * @return array
     */
    public static function getPossibleProductValues()
    {
        return [
            Product::PRODUCT_NAME_BPACK_24H_PRO,
            Product::PRODUCT_NAME_BPACK_24H_BUSINESS,
            Product::PRODUCT_NAME_BPACK_BUSINESS,
            Product::PRODUCT_NAME_BPACK_PALLET,
            Product::PRODUCT_NAME_BPACK_EASY_RETOUR,
        ];
    }

    /**
     * @param Receiver $receiver
     */
    public function setReceiver($receiver)
    {
        $this->receiver = $receiver;
    }

    /**
     * @return Receiver
     */
    public function getReceiver()
    {
        return $this->receiver;
    }

    /**
     * @return string
     */
    public function getRequestedDeliveryDate()
    {
        return $this->requestedDeliveryDate;
    }

    /**
     * @param string $requestedDeliveryDate
     */
    public function setRequestedDeliveryDate($requestedDeliveryDate)
    {
        $this->requestedDeliveryDate = (string) $requestedDeliveryDate;
    }

    /**
     * Return the object as an array for usage in the XML
     *
     * @param \DOMDocument $document
     * @param string $prefix
     * @param string $type
     *
     * @return \DOMElement
     *
     * @throws \DOMException
     */
    public function toXML(\DOMDocument $document, $prefix = null, $type = null)
    {
        $nationalElement = $document->createElement($this->getPrefixedTagName('nationalBox', $prefix));
        $boxElement = parent::toXML($document, null, 'atHome');
        $nationalElement->appendChild($boxElement);

        if ($this->getReceiver() !== null) {
            $boxElement->appendChild(
                $this->getReceiver()->toXML($document)
            );
        }

        $this->addToXmlRequestedDeliveryDate($document, $boxElement);

        return $nationalElement;
    }

    /**
     * @param \DOMDocument $document
     * @param \DOMElement $typeElement
     *
     * @throws \DOMException
     */
    protected function addToXmlRequestedDeliveryDate(\DOMDocument $document, \DOMElement $typeElement)
    {
        if ($this->getRequestedDeliveryDate() !== null) {
            $typeElement->appendChild(
                $document->createElement(
                    'requestedDeliveryDate',
                    $this->getRequestedDeliveryDate()
                )
            );
        }
    }

    /**
     * @param \SimpleXMLElement $xml
     * @param National|null $self
     *
     * @return AtHome
     *
     * @throws BpostXmlInvalidItemException
     * @throws BpostException
     */
    public static function createFromXML(\SimpleXMLElement $xml, ?National $self = null)
    {
        if ($self === null) {
            $self = new self();
        }

        if (!isset($xml->atHome)) {
            throw new BpostXmlInvalidItemException("$xml->atHome doesn't exist");
        }

        $atHomeXml = $xml->atHome[0];

        $self = parent::createFromXML($atHomeXml, $self);

        if (isset($atHomeXml->receiver)) {
            $self->setReceiver(
                Receiver::createFromXML(
                    $atHomeXml->receiver->children('http://schema.post.be/shm/deepintegration/v3/common')
                )
            );
        }

        if (isset($atHomeXml->requestedDeliveryDate) && $atHomeXml->requestedDeliveryDate != '') {
            $self->setRequestedDeliveryDate(
                $atHomeXml->requestedDeliveryDate
            );
        }

        return $self;
    }
}
