<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Api\Bpost\Order\Box;

use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\CustomsInfo\CustomsInfo;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\Option\Messaging;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\Option\Option;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Customer\Receiver;
use PrestaShop\Module\BpostShm\Api\Bpost\ProductConfiguration\Product;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidLengthException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidValueException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostNotImplementedException;

class International implements IBox
{
    const TAG_NAME = 'International';

    /**
     * @var string
     */
    protected $product;

    /**
     * @var array
     */
    protected $options;

    /**
     * @var Receiver
     */
    protected $receiver;

    /**
     * @var int
     */
    protected $parcelWeight;

    /**
     * @var CustomsInfo
     */
    protected $customsInfo;

    /**
     * @param CustomsInfo $customsInfo
     */
    public function setCustomsInfo($customsInfo)
    {
        $this->customsInfo = $customsInfo;
    }

    /**
     * @return CustomsInfo
     */
    public function getCustomsInfo()
    {
        return $this->customsInfo;
    }

    /**
     * @param Option[] $options
     */
    public function setOptions($options)
    {
        $this->options = $options;
    }

    /**
     * @return Option[]
     */
    public function getOptions()
    {
        return $this->options;
    }

    /**
     * @param Option $option
     */
    public function addOption(Option $option)
    {
        $this->options[] = $option;
    }

    /**
     * @param int $parcelWeight
     */
    public function setParcelWeight($parcelWeight)
    {
        $this->parcelWeight = $parcelWeight;
    }

    /**
     * @return int
     */
    public function getParcelWeight()
    {
        return $this->parcelWeight;
    }

    /**
     * @param string $product
     *
     * @throws BpostInvalidValueException
     */
    public function setProduct($product)
    {
        if (!in_array($product, self::getPossibleProductValues())) {
            throw new BpostInvalidValueException('product', $product, self::getPossibleProductValues());
        }

        $this->product = $product;
    }

    /**
     * @return string
     */
    public function getProduct()
    {
        return $this->product;
    }

    /**
     * @return array
     */
    public static function getPossibleProductValues()
    {
        return [
            Product::PRODUCT_NAME_BPACK_WORLD_BUSINESS,
            Product::PRODUCT_NAME_BPACK_WORLD_EASY_RETURN,
            Product::PRODUCT_NAME_BPACK_WORLD_EXPRESS_PRO,
            Product::PRODUCT_NAME_BPACK_EUROPE_BUSINESS,
            Product::PRODUCT_NAME_BPACK_AT_BPOST_WORLD,
        ];
    }

    /**
     * @param Receiver $receiver
     */
    public function setReceiver($receiver)
    {
        $this->receiver = $receiver;
    }

    /**
     * @return Receiver
     */
    public function getReceiver()
    {
        return $this->receiver;
    }

    /**
     * Return the object as an array for usage in the XML
     *
     * @param \DOMDocument $document
     * @param null $prefix
     *
     * @return \DOMElement
     *
     * @throws \DOMException
     */
    public function toXML(\DOMDocument $document, $prefix = null)
    {
        $tagName = 'internationalBox';
        if ($prefix !== null) {
            $tagName = $prefix . ':' . $tagName;
        }

        $internationalBox = $document->createElement($tagName);
        $international = $document->createElement('international:' . static::TAG_NAME);
        $internationalBox->appendChild($international);

        if ($this->getProduct() !== null) {
            $international->appendChild(
                $document->createElement(
                    'international:product',
                    $this->getProduct()
                )
            );
        }

        $options = $this->getOptions();
        if (!empty($options)) {
            $optionsElement = $document->createElement('international:options');
            foreach ($options as $option) {
                $optionsElement->appendChild(
                    $option->toXML($document)
                );
            }
            $international->appendChild($optionsElement);
        }

        if ($this->getReceiver() !== null) {
            $international->appendChild(
                $this->getReceiver()->toXML($document, 'international')
            );
        }

        if ($this->getParcelWeight() !== null) {
            $international->appendChild(
                $document->createElement(
                    'international:parcelWeight',
                    $this->getParcelWeight()
                )
            );
        }

        if ($this->getCustomsInfo() !== null) {
            $international->appendChild(
                $this->getCustomsInfo()->toXML($document, 'international')
            );
        }

        $this->alsoToXML($document, $international);

        return $internationalBox;
    }

    /**
     * @param \SimpleXMLElement $xml
     *
     * @return International
     *
     * @throws BpostNotImplementedException
     * @throws BpostInvalidLengthException
     */
    public static function createFromXML(\SimpleXMLElement $xml)
    {
        $international = new static();

        $type = static::TAG_NAME;
        $xml = $xml->$type;

        if (isset($xml->product) && $xml->product != '') {
            $international->setProduct(
                (string) $xml->product
            );
        }

        if (isset($xml->options)) {
            /** @var \SimpleXMLElement $optionData */
            foreach ($xml->options as $optionData) {
                $optionData = $optionData->children('http://schema.post.be/shm/deepintegration/v3/common');

                if (in_array($optionData->getName(), Messaging::getPossibleTypeValues())) {
                    $option = Messaging::createFromXML($optionData);
                } else {
                    $className = '\\PrestaShop\\Module\\BpostShm\\Api\\Bpost\\Order\\Box\\Option\\' . ucfirst($optionData->getName());
                    if (!method_exists($className, 'createFromXML')) {
                        throw new BpostNotImplementedException('No createFromXML found into ' . $className);
                    }
                    $option = call_user_func(
                        [$className, 'createFromXML'],
                        $optionData
                    );
                }

                $international->addOption($option);
            }
        }
        if (isset($xml->parcelWeight) && $xml->parcelWeight != '') {
            $international->setParcelWeight(
                (int) $xml->parcelWeight
            );
        }
        if (isset($xml->receiver)) {
            $receiverData = $xml->receiver->children(
                'http://schema.post.be/shm/deepintegration/v3/common'
            );
            $international->setReceiver(
                Receiver::createFromXML($receiverData)
            );
        }
        if (isset($xml->customsInfo)) {
            $international->setCustomsInfo(
                CustomsInfo::createFromXML($xml->customsInfo)
            );
        }

        $international->createAlsoFromXML($xml);

        return $international;
    }

    /**
     * Additional elements within descendants
     *
     * @param \DOMDocument $document
     * @param \DOMElement $element
     *
     * @return \DOMElement
     */
    protected function alsoToXML(\DOMDocument $document, \DOMElement $element)
    {
    }

    /**
     * @param \SimpleXMLElement $element
     *
     * @return Derivative instance
     */
    protected function createAlsoFromXML(\SimpleXMLElement $element)
    {
    }
}
