<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Api\Bpost\Order\Box;

use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\OpeningHour\Day;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\Option\Messaging;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\Option\Option;
use PrestaShop\Module\BpostShm\Api\BpostException;
use PrestaShop\Module\BpostShm\Api\Common\ComplexAttribute;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidLengthException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidValueException;
use PrestaShop\Module\BpostShm\Api\Exception\XmlException\BpostXmlInvalidItemException;

abstract class National extends ComplexAttribute implements IBox
{
    /** @var string */
    protected $product;

    /** @var Option[] */
    protected $options;

    /** @var int */
    protected $weight;

    /** @var Day[] */
    private $openingHours;

    /** @var string */
    private $desiredDeliveryPlace;

    /**
     * @param Option[] $options
     */
    public function setOptions($options)
    {
        $this->options = $options;
    }

    /**
     * @return Option[]
     */
    public function getOptions()
    {
        return $this->options;
    }

    /**
     * @param Option $option
     */
    public function addOption(Option $option)
    {
        $this->options[] = $option;
    }

    /**
     * @param string $product
     */
    public function setProduct($product)
    {
        $this->product = $product;
    }

    /**
     * @return string
     */
    public function getProduct()
    {
        return $this->product;
    }

    /**
     * @remark should be implemented by the child class
     *
     * @return array
     */
    public static function getPossibleProductValues()
    {
        return [];
    }

    /**
     * @param int $weight
     */
    public function setWeight($weight)
    {
        $this->weight = $weight;
    }

    /**
     * @return int
     */
    public function getWeight()
    {
        return $this->weight;
    }

    /**
     * @param Day[] $openingHours
     */
    public function setOpeningHours(array $openingHours)
    {
        $this->openingHours = $openingHours;
    }

    /**
     * @param Day $day
     */
    public function addOpeningHour(Day $day)
    {
        $this->openingHours[] = $day;
    }

    /**
     * @return Day[]
     */
    public function getOpeningHours()
    {
        return $this->openingHours;
    }

    /**
     * @param string $desiredDeliveryPlace
     */
    public function setDesiredDeliveryPlace($desiredDeliveryPlace)
    {
        $this->desiredDeliveryPlace = $desiredDeliveryPlace;
    }

    /**
     * @return string
     */
    public function getDesiredDeliveryPlace()
    {
        return $this->desiredDeliveryPlace;
    }

    /**
     * Return the object as an array for usage in the XML
     *
     * @param \DOMDocument $document
     * @param null $prefix
     * @param null $type
     *
     * @return \DOMElement
     *
     * @throws \DOMException
     */
    public function toXML(\DOMDocument $document, $prefix = null, $type = null)
    {
        $typeElement = $document->createElement($type);

        if ($this->getProduct() !== null) {
            $tagName = 'product';
            if ($prefix !== null) {
                $tagName = $prefix . ':' . $tagName;
            }
            $typeElement->appendChild(
                $document->createElement(
                    $tagName,
                    $this->getProduct()
                )
            );
        }

        $options = $this->getOptions();
        if (!empty($options)) {
            $optionsElement = $document->createElement('options');
            foreach ($options as $option) {
                $optionsElement->appendChild(
                    $option->toXML($document)
                );
            }
            $typeElement->appendChild($optionsElement);
        }

        if ($this->getWeight() !== null) {
            $typeElement->appendChild(
                $document->createElement($this->getPrefixedTagName('weight', $prefix), $this->getWeight())
            );
        }

        $openingHours = $this->getOpeningHours();
        if (!empty($openingHours)) {
            $openingHoursElement = $document->createElement('openingHours');
            /** @var Day $day */
            foreach ($openingHours as $day) {
                $openingHoursElement->appendChild(
                    $day->toXML($document)
                );
            }
            $typeElement->appendChild($openingHoursElement);
        }

        if ($this->getDesiredDeliveryPlace() !== null) {
            $typeElement->appendChild(
                $document->createElement(
                    $this->getPrefixedTagName('desiredDeliveryPlace', $prefix),
                    $this->getDesiredDeliveryPlace()
                )
            );
        }

        return $typeElement;
    }

    /**
     * @param \SimpleXMLElement $nationalXml
     * @param National|null $self
     *
     * @return AtHome
     *
     * @throws BpostException
     * @throws BpostXmlInvalidItemException
     * @throws BpostInvalidLengthException
     * @throws BpostInvalidValueException
     */
    public static function createFromXML(\SimpleXMLElement $nationalXml, ?National $self = null)
    {
        if (null === $self) {
            throw new BpostException('Set an instance of National');
        }

        if (isset($nationalXml->product) && $nationalXml->product != '') {
            $self->setProduct(
                (string) $nationalXml->product
            );
        }

        if (isset($nationalXml->options) && !empty($nationalXml->options)) {
            /** @var \SimpleXMLElement $optionData */
            foreach ($nationalXml->options as $optionData) {
                $optionData = $optionData->children('http://schema.post.be/shm/deepintegration/v3/common');

                if (in_array($optionData->getName(), [
                    Messaging::MESSAGING_TYPE_INFO_DISTRIBUTED,
                    Messaging::MESSAGING_TYPE_INFO_NEXT_DAY,
                    Messaging::MESSAGING_TYPE_INFO_REMINDER,
                    Messaging::MESSAGING_TYPE_KEEP_ME_INFORMED,
                ])
                ) {
                    $option = Messaging::createFromXML($optionData);
                } else {
                    $option = self::getOptionFromOptionData($optionData);
                }

                $self->addOption($option);
            }
        }

        if (isset($nationalXml->weight) && $nationalXml->weight != '') {
            $self->setWeight(
                (int) $nationalXml->weight
            );
        }

        if (isset($nationalXml->openingHours) && $nationalXml->openingHours != '') {
            foreach ($nationalXml->openingHours->children() as $day => $value) {
                $self->addOpeningHour(new Day($day, (string) $value));
            }
        }

        if (isset($nationalXml->desiredDeliveryPlace) && $nationalXml->desiredDeliveryPlace != '') {
            $self->setDesiredDeliveryPlace(
                (string) $nationalXml->desiredDeliveryPlace
            );
        }

        return $self;
    }

    /**
     * @param \SimpleXMLElement $optionData
     *
     * @return Option
     *
     * @throws BpostXmlInvalidItemException
     */
    protected static function getOptionFromOptionData(\SimpleXMLElement $optionData)
    {
        $class = ucfirst($optionData->getName());
        $className = '\\PrestaShop\\Module\\BpostShm\\Api\\Bpost\\Order\\Box\\Option\\' . $class;
        if (!method_exists($className, 'createFromXML')) {
            throw new BpostXmlInvalidItemException("$className doesn't have createFromXML method");
        }

        return call_user_func(
            [$className, 'createFromXML'],
            $optionData
        );
    }
}
