<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\National;

use PrestaShop\Module\BpostShm\Api\Common\BasicAttribute\EmailAddressCharacteristic;
use PrestaShop\Module\BpostShm\Api\Common\BasicAttribute\Language;
use PrestaShop\Module\BpostShm\Api\Common\BasicAttribute\PhoneNumber;
use PrestaShop\Module\BpostShm\Api\Common\ComplexAttribute;

class Unregistered extends ComplexAttribute
{
    /** @var Language */
    private $language;

    /** @var PhoneNumber */
    private $mobilePhone;

    /** @var EmailAddressCharacteristic */
    private $emailAddress;

    /** @var int */
    private $reducedMobilityZone;

    /**
     * @return bool
     */
    public function hasLanguage()
    {
        return $this->language !== null;
    }

    /**
     * @return string
     */
    public function getLanguage()
    {
        return $this->hasLanguage() ? $this->language->getValue() : null;
    }

    /**
     * @param string $language
     */
    public function setLanguage($language)
    {
        $this->language = new Language(strtoupper($language));
    }

    /**
     * @return bool
     */
    public function hasMobilePhone()
    {
        return $this->mobilePhone !== null;
    }

    /**
     * @return string
     */
    public function getMobilePhone()
    {
        return $this->hasMobilePhone() ? $this->mobilePhone->getValue() : null;
    }

    /**
     * @param string $mobilePhone
     */
    public function setMobilePhone($mobilePhone)
    {
        if ($mobilePhone) {
            $this->mobilePhone = new PhoneNumber($mobilePhone);
        }
    }

    /**
     * @return bool
     */
    public function hasEmailAddress()
    {
        return $this->emailAddress !== null;
    }

    /**
     * @return string
     */
    public function getEmailAddress()
    {
        return $this->hasEmailAddress() ? $this->emailAddress->getValue() : null;
    }

    /**
     * @param string $emailAddress
     */
    public function setEmailAddress($emailAddress)
    {
        $this->emailAddress = new EmailAddressCharacteristic($emailAddress);
    }

    /**
     * @return bool
     */
    public function getReducedMobilityZone()
    {
        return $this->reducedMobilityZone;
    }

    /**
     * @param bool $reducedMobilityZone
     */
    public function setReducedMobilityZone(bool $reducedMobilityZone)
    {
        $this->reducedMobilityZone = $reducedMobilityZone;
    }

    /**
     * @param \DOMDocument $document
     * @param string $prefix
     * @param string $type
     *
     * @return \DOMElement
     */
    public function toXml(\DOMDocument $document, $prefix = null, $type = null)
    {
        $tagName = $this->getPrefixedTagName('unregistered', $prefix);

        $xml = $document->createElement($tagName);

        if ($this->hasLanguage()) {
            $tagName = $this->getPrefixedTagName('language', $prefix);
            $xml->appendChild($document->createElement($tagName, $this->getLanguage()));
        }

        if ($this->getMobilePhone() !== null) {
            $tagName = $this->getPrefixedTagName('mobilePhone', $prefix);
            $xml->appendChild($document->createElement($tagName, $this->getMobilePhone()));
        }

        if ($this->getEmailAddress() !== null) {
            $tagName = $this->getPrefixedTagName('emailAddress', $prefix);
            $xml->appendChild($document->createElement($tagName, $this->getEmailAddress()));
        }

        if ($this->getReducedMobilityZone() !== null) {
            $tagName = $this->getPrefixedTagName('reducedMobilityZone', $prefix);
            if ($this->getReducedMobilityZone()) {
                $xml->appendChild($document->createElement($tagName));
            }
        }

        return $xml;
    }

    /**
     * @param \SimpleXMLElement $xml
     *
     * @return Unregistered
     */
    public static function createFromXml(\SimpleXMLElement $xml)
    {
        $self = new self();

        if (isset($xml->language) && $xml->language != '') {
            $self->setLanguage((string) $xml->language);
        }

        if (isset($xml->mobilePhone) && $xml->mobilePhone != '') {
            $self->setMobilePhone((string) $xml->mobilePhone);
        }

        if (isset($xml->emailAddress) && $xml->emailAddress != '') {
            $self->setEmailAddress((string) $xml->emailAddress);
        }

        if (isset($xml->reducedMobilityZone)) {
            $self->setReducedMobilityZone((bool) $xml->reducedMobilityZone);
        }

        return $self;
    }
}
