<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\Option;

use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidLengthException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidValueException;

class Messaging extends Option
{
    const MESSAGING_LANGUAGE_EN = 'EN';
    const MESSAGING_LANGUAGE_NL = 'NL';
    const MESSAGING_LANGUAGE_FR = 'FR';
    const MESSAGING_LANGUAGE_DE = 'DE';

    const MESSAGING_TYPE_INFO_DISTRIBUTED = 'infoDistributed';
    const MESSAGING_TYPE_INFO_NEXT_DAY = 'infoNextDay';
    const MESSAGING_TYPE_INFO_REMINDER = 'infoReminder';
    const MESSAGING_TYPE_KEEP_ME_INFORMED = 'keepMeInformed';

    /**
     * @var string
     */
    private $type;

    /**
     * @var string
     */
    private $language;

    /**
     * @var string
     */
    private $emailAddress;

    /**
     * @var string
     */
    private $mobilePhone;

    /**
     * @param string $emailAddress
     *
     * @throws BpostInvalidLengthException
     */
    public function setEmailAddress($emailAddress)
    {
        $length = 50;
        if (mb_strlen($emailAddress) > $length) {
            throw new BpostInvalidLengthException('emailAddress', mb_strlen($emailAddress), $length);
        }

        $this->emailAddress = $emailAddress;
    }

    /**
     * @return string
     */
    public function getEmailAddress()
    {
        return $this->emailAddress;
    }

    /**
     * @param string $language
     *
     * @throws BpostInvalidValueException
     */
    public function setLanguage($language)
    {
        $language = strtoupper($language);

        if (!in_array($language, self::getPossibleLanguageValues())) {
            throw new BpostInvalidValueException('language', $language, self::getPossibleLanguageValues());
        }

        $this->language = $language;
    }

    /**
     * @return string
     */
    public function getLanguage()
    {
        return $this->language;
    }

    /**
     * @return array
     */
    public static function getPossibleLanguageValues()
    {
        return [
            self::MESSAGING_LANGUAGE_EN,
            self::MESSAGING_LANGUAGE_NL,
            self::MESSAGING_LANGUAGE_FR,
            self::MESSAGING_LANGUAGE_DE,
        ];
    }

    /**
     * @param string $mobilePhone
     *
     * @throws BpostInvalidLengthException
     */
    public function setMobilePhone($mobilePhone)
    {
        $length = 20;
        if (mb_strlen($mobilePhone) > $length) {
            throw new BpostInvalidLengthException('mobilePhone', mb_strlen($mobilePhone), $length);
        }

        $this->mobilePhone = $mobilePhone;
    }

    /**
     * @return string
     */
    public function getMobilePhone()
    {
        return $this->mobilePhone;
    }

    /**
     * @return array
     */
    public static function getPossibleTypeValues()
    {
        return [
            self::MESSAGING_TYPE_INFO_DISTRIBUTED,
            self::MESSAGING_TYPE_INFO_NEXT_DAY,
            self::MESSAGING_TYPE_INFO_REMINDER,
            self::MESSAGING_TYPE_KEEP_ME_INFORMED,
        ];
    }

    /**
     * @param string $type
     *
     * @throws BpostInvalidValueException
     */
    public function setType($type)
    {
        if (!in_array($type, self::getPossibleTypeValues())) {
            throw new BpostInvalidValueException('type', $type, self::getPossibleTypeValues());
        }

        $this->type = $type;
    }

    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param string $type
     * @param string $language
     * @param string|null $emailAddress
     * @param string|null $mobilePhone
     *
     * @throws BpostInvalidLengthException
     * @throws BpostInvalidValueException
     */
    public function __construct($type, $language, $emailAddress = null, $mobilePhone = null)
    {
        $this->setType($type);
        $this->setLanguage($language);

        if ($emailAddress !== null) {
            $this->setEmailAddress($emailAddress);
        }
        if ($mobilePhone !== null) {
            $this->setMobilePhone($mobilePhone);
        }
    }

    /**
     * @inerhitDoc
     */
    public function toXML(\DOMDocument $document, $prefix = 'common')
    {
        $tagName = $this->getType();
        if ($prefix !== null) {
            $tagName = $prefix . ':' . $tagName;
        }

        $messaging = $document->createElement($tagName);
        $messaging->setAttribute('language', $this->getLanguage());

        if ($this->getEmailAddress() !== null) {
            $tagName = 'emailAddress';
            if ($prefix !== null) {
                $tagName = $prefix . ':' . $tagName;
            }
            $messaging->appendChild(
                $document->createElement(
                    $tagName,
                    $this->getEmailAddress()
                )
            );
        }
        if ($this->getMobilePhone() !== null) {
            $tagName = 'mobilePhone';
            if ($prefix !== null) {
                $tagName = $prefix . ':' . $tagName;
            }
            $messaging->appendChild(
                $document->createElement(
                    $tagName,
                    $this->getMobilePhone()
                )
            );
        }

        return $messaging;
    }

    /**
     * @param \SimpleXMLElement $xml
     *
     * @return Messaging
     *
     * @throws BpostInvalidLengthException
     */
    public static function createFromXML(\SimpleXMLElement $xml)
    {
        $messaging = new Messaging($xml->getName(), (string) $xml->attributes()->language);

        $data = $xml->{$xml->getName()};
        if (isset($data->emailAddress) && $data->emailAddress != '') {
            $messaging->setEmailAddress((string) $data->emailAddress);
        }
        if (isset($data->mobilePhone) && $data->mobilePhone != '') {
            $messaging->setMobilePhone((string) $data->mobilePhone);
        }

        return $messaging;
    }
}
