<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Api\Bpost\Order\Box;

use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidLengthException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidPatternException;

class ParcelContent
{
    /**
     * @var int
     */
    private $numberOfItemType;

    /**
     * @var int
     */
    private $valueOfItem;

    /**
     * @var string
     */
    private $itemDescription;

    /**
     * @var int
     */
    private $nettoWeight;

    /**
     * @var string
     */
    private $hsTariffCode;

    /**
     * @var string
     */
    private $originOfGoods;

    /**
     * @param int $quantity
     */
    public function setNumberOfItemType(int $quantity)
    {
        $this->numberOfItemType = $quantity;
    }

    /**
     * @return int
     */
    public function getNumberOfItemType()
    {
        return $this->numberOfItemType;
    }

    /**
     * @param int $value
     */
    public function setValueOfItem($value)
    {
        $this->valueOfItem = (int) $value;
    }

    /**
     * @return int
     */
    public function getValueOfItem()
    {
        return $this->valueOfItem;
    }

    /**
     * @param string $itemDescription
     *
     * @throws BpostInvalidLengthException
     */
    public function setItemDescription(string $itemDescription)
    {
        $length = 30;
        if (mb_strlen($itemDescription) > $length) {
            throw new BpostInvalidLengthException('itemDescription', mb_strlen($itemDescription), $length);
        }

        $this->itemDescription = $itemDescription;
    }

    /**
     * @return string
     */
    public function getItemDescription()
    {
        return $this->itemDescription;
    }

    /**
     * @param int $netWeight
     */
    public function setNettoWeight(int $netWeight)
    {
        $this->nettoWeight = (int) $netWeight;
    }

    /**
     * @return int
     */
    public function getNettoWeight()
    {
        return $this->nettoWeight;
    }

    /**
     * @param string $hsCode
     *
     * @throws BpostInvalidPatternException
     */
    public function setHsTariffCode(string $hsCode)
    {
        $pattern = '/^\d{4,9}$/';
        if (1 !== preg_match($pattern, $hsCode)) {
            throw new BpostInvalidPatternException('hsTariffDode', $hsCode, $pattern);
        }
        $this->hsTariffCode = $hsCode;
    }

    /**
     * @return string
     */
    public function getHsTariffCode()
    {
        return $this->hsTariffCode;
    }

    /**
     * @param string $countryIso
     *
     * @throws BpostInvalidPatternException
     */
    public function setOriginOfGoods(string $countryIso)
    {
        $pattern = '/^[a-zA-Z]{2}$/';
        if (1 !== preg_match($pattern, (string) $countryIso)) {
            throw new BpostInvalidPatternException('originOfGoods', $countryIso, $pattern);
        }
        $this->originOfGoods = $countryIso;
    }

    /**
     * @return string
     */
    public function getOriginOfGoods()
    {
        return $this->originOfGoods;
    }

    /**
     * Return the object as an array for usage in the XML
     *
     * @param \DOMDocument $document
     * @param null $prefix
     *
     * @return \DOMElement
     *
     * @throws \DOMException
     */
    public function toXML(\DOMDocument $document, $prefix = null)
    {
        $tagName = 'parcelContent';
        if ($prefix !== null) {
            $tagName = $prefix . ':' . $tagName;
        }
        $parcelContent = $document->createElement($tagName);

        if ($this->getNumberOfItemType() !== null) {
            $tagName = 'numberOfItemType';
            if ($prefix !== null) {
                $tagName = $prefix . ':' . $tagName;
            }
            $parcelContent->appendChild(
                $document->createElement(
                    $tagName,
                    $this->getNumberOfItemType()
                )
            );
        }
        if ($this->getValueOfItem() !== null) {
            $tagName = 'valueOfItem';
            if ($prefix !== null) {
                $tagName = $prefix . ':' . $tagName;
            }
            $parcelContent->appendChild(
                $document->createElement(
                    $tagName,
                    $this->getValueOfItem()
                )
            );
        }
        if ($this->getItemDescription() !== null) {
            $tagName = 'itemDescription';
            if ($prefix !== null) {
                $tagName = $prefix . ':' . $tagName;
            }
            $parcelContent->appendChild(
                $document->createElement(
                    $tagName,
                    $this->getItemDescription()
                )
            );
        }
        if ($this->getNettoWeight() !== null) {
            $tagName = 'nettoWeight';
            if ($prefix !== null) {
                $tagName = $prefix . ':' . $tagName;
            }
            $parcelContent->appendChild(
                $document->createElement(
                    $tagName,
                    $this->getNettoWeight()
                )
            );
        }
        if ($this->getHsTariffCode() !== null) {
            $tagName = 'hsTariffCode';
            if ($prefix !== null) {
                $tagName = $prefix . ':' . $tagName;
            }
            $parcelContent->appendChild(
                $document->createElement(
                    $tagName,
                    $this->getHsTariffCode()
                )
            );
        }
        if ($this->getOriginOfGoods() !== null) {
            $tagName = 'originOfGoods';
            if ($prefix !== null) {
                $tagName = $prefix . ':' . $tagName;
            }
            $parcelContent->appendChild(
                $document->createElement(
                    $tagName,
                    $this->getOriginOfGoods()
                )
            );
        }

        return $parcelContent;
    }

    /**
     * @param \SimpleXMLElement $xml
     *
     * @return ParcelContent
     *
     * @throws BpostInvalidLengthException
     * @throws BpostInvalidPatternException
     */
    public static function createFromXML(\SimpleXMLElement $xml)
    {
        $parcelContent = new ParcelContent();

        if (isset($xml->numberOfItemType) && $xml->numberOfItemType != '') {
            $parcelContent->setNumberOfItemType((int) $xml->numberOfItemType);
        }
        if (isset($xml->valueOfItem) && $xml->valueOfItem != '') {
            $parcelContent->setValueOfItem((int) $xml->valueOfItem);
        }
        if (isset($xml->itemDescription) && $xml->itemDescription != '') {
            $parcelContent->setItemDescription((string) $xml->itemDescription);
        }
        if (isset($xml->nettoWeight) && $xml->nettoWeight != '') {
            $parcelContent->setNettoWeight((int) $xml->nettoWeight);
        }
        if (isset($xml->hsTariffCode) && $xml->hsTariffCode != '') {
            $parcelContent->setHsTariffCode((string) $xml->hsTariffCode);
        }
        if (isset($xml->originOfGoods) && $xml->originOfGoods != '') {
            $parcelContent->setOriginOfGoods((string) $xml->originOfGoods);
        }

        return $parcelContent;
    }
}
