<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Api\Bpost\ProductConfiguration;

class Product
{
    const PRODUCT_NAME_BPACK_EASY_RETOUR = 'bpack Easy Retour';
    const PRODUCT_NAME_BPACK_24H_PRO = 'bpack 24h Pro';
    const PRODUCT_NAME_BPACK_24H_BUSINESS = 'bpack 24h business';
    const PRODUCT_NAME_BPACK_AT_BPOST = 'bpack@bpost';
    const PRODUCT_NAME_BPACK_AT_BPOST_WORLD = 'bpack@bpost international';
    const PRODUCT_NAME_BPACK_CLICK_AND_COLLECT = 'bpack Click & Collect';
    const PRODUCT_NAME_BPACK_24_7 = 'bpack 24/7';
    const PRODUCT_NAME_BPACK_BUSINESS = 'bpack Bus';
    const PRODUCT_NAME_BPACK_PALLET = 'bpack Pallet';
    const PRODUCT_NAME_BPACK_WORLD_EASY_RETURN = 'bpack World Easy Return';
    const PRODUCT_NAME_BPACK_WORLD_EXPRESS_PRO = 'bpack World Express Pro';
    const PRODUCT_NAME_BPACK_WORLD_BUSINESS = 'bpack World Business';
    const PRODUCT_NAME_BPACK_EUROPE_BUSINESS = 'bpack Europe Business';

    /** @var bool */
    private $default;
    /** @var string */
    private $name;

    /** @var Price[] */
    private $prices = [];
    /** @var Option[] */
    private $options = [];

    /**
     * @param \SimpleXMLElement $xml
     *
     * @return Product
     */
    public static function createFromXML(\SimpleXMLElement $xml)
    {
        /*
        <product default="true" name="bpack 24/7">
          <price price20To30="750" price10To20="650" price5To10="550" price2To5="450" priceLessThan2="350" countryIso2Code="BE"/>
          <option visiblity="NOT_VISIBLE_BY_CONSUMER_OPTIONAL" price="0" name="Saturday"/>
          <option visiblity="NOT_VISIBLE_BY_CONSUMER_OPTIONAL" price="0" name="Info &quot;Distributed&quot;"/>
          <option visiblity="NOT_VISIBLE_BY_CONSUMER_OPTIONAL" price="0" name="Insurance"/>
        </product>
        */
        $attributes = $xml->attributes();
        $children = $xml->children();

        $product = new self();
        $product->setDefault($attributes['default'] == 'true');
        $product->setName($attributes['name']);

        if (isset($children->price)) {
            foreach ($children->price as $priceXml) {
                $product->addPrice(Price::createFromXML($priceXml));
            }
        }
        if (isset($children->option)) {
            foreach ($children->option as $optionXml) {
                $product->addOption(Option::createFromXML($optionXml));
            }
        }

        return $product;
    }

    /**
     * @return bool
     */
    public function isDefault()
    {
        return $this->default;
    }

    /**
     * @param bool $default
     */
    public function setDefault($default)
    {
        $this->default = (bool) $default;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = (string) $name;
    }

    /**
     * @return Price[]
     */
    public function getPrices()
    {
        return $this->prices;
    }

    /**
     * @param Price $price
     */
    public function addPrice(Price $price)
    {
        $this->prices[] = $price;
    }

    /**
     * @return Option[]
     */
    public function getOptions()
    {
        return $this->options;
    }

    /**
     * @param Option $option
     */
    public function addOption(Option $option)
    {
        $this->options[] = $option;
    }

    /**
     * @return bool
     */
    public function isForNationalShipping()
    {
        switch ($this->getName()) {
            case self::PRODUCT_NAME_BPACK_EASY_RETOUR:
            case self::PRODUCT_NAME_BPACK_24H_PRO:
            case self::PRODUCT_NAME_BPACK_24H_BUSINESS:
            case self::PRODUCT_NAME_BPACK_AT_BPOST:
            case self::PRODUCT_NAME_BPACK_CLICK_AND_COLLECT:
            case self::PRODUCT_NAME_BPACK_24_7:
                return true;

            case self::PRODUCT_NAME_BPACK_WORLD_EASY_RETURN:
            case self::PRODUCT_NAME_BPACK_WORLD_EXPRESS_PRO:
            case self::PRODUCT_NAME_BPACK_WORLD_BUSINESS:
            case self::PRODUCT_NAME_BPACK_EUROPE_BUSINESS:
            default:
                return false;
        }
    }
}
