<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Api\Common;

use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidLengthException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidPatternException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidValueException;

abstract class BasicAttribute
{
    /** @var mixed */
    private $value;

    /** @var string */
    private $key;

    /**
     * BasicAttribute constructor.
     *
     * @param mixed $value
     * @param string $key
     *
     * @throws BpostLogicException
     */
    public function __construct($value, $key = '')
    {
        $this->value = $value;
        $this->setKey($key);
        $this->validate();
    }

    /**
     * @return mixed
     */
    public function getValue()
    {
        return $this->value;
    }

    /**
     * @param string $key
     */
    private function setKey($key)
    {
        $this->key = (string) ($key ?: $this->getDefaultKey());
    }

    /**
     * @return string
     */
    public function getKey()
    {
        return $this->key;
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return (string) $this->getValue();
    }

    /**
     * Prefix $tagName with the $prefix, if needed
     *
     * @param string $prefix
     * @param string $tagName
     *
     * @return string
     */
    public function getPrefixedTagName($tagName, $prefix = null)
    {
        if (empty($prefix)) {
            return $tagName;
        }

        return $prefix . ':' . $tagName;
    }

    /**
     * @param int $length
     *
     * @throws BpostInvalidLengthException
     */
    public function validateLength($length)
    {
        if (mb_strlen($this->getValue()) > $length) {
            throw new BpostInvalidLengthException($this->getKey(), mb_strlen($this->getValue()), $length);
        }
    }

    /**
     * @param array $allowedValues
     *
     * @throws BpostInvalidValueException
     */
    public function validateChoice(array $allowedValues)
    {
        if (!in_array($this->getValue(), $allowedValues)) {
            throw new BpostInvalidValueException($this->getKey(), $this->getValue(), $allowedValues);
        }
    }

    /**
     * @param string $regexPattern
     *
     * @throws BpostInvalidPatternException
     */
    public function validatePattern($regexPattern)
    {
        if (!preg_match("/^$regexPattern\$/", $this->getValue())) {
            throw new BpostInvalidPatternException($this->getKey(), $this->getValue(), $regexPattern);
        }
    }

    /**
     * @return string
     */
    abstract protected function getDefaultKey();

    /**
     * @throws BpostLogicException
     */
    abstract public function validate();
}
