<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Api;

use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidLengthException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidValueException;

class FormHandler
{
    /**
     * bPost instance
     *
     * @var Bpost
     */
    private $bpost;

    /**
     * The parameters
     *
     * @var array
     */
    private $parameters = [];

    /**
     * Create bPostFormHandler instance
     *
     * @param string $accountId
     * @param string $passPhrase
     * @param string $apiUrl
     */
    public function __construct($accountId, $passPhrase, $apiUrl = Bpost::API_URL)
    {
        $this->bpost = new Bpost($accountId, $passPhrase, $apiUrl);
    }

    /**
     * Calculate the hash
     *
     * @return string
     */
    private function getChecksum()
    {
        $keysToHash = [
            'accountId',
            'action',
            'costCenter',
            'customerCountry',
            'deliveryMethodOverrides',
            'extraSecure',
            'orderReference',
            'orderWeight',
        ];
        $base = 'accountId=' . $this->bpost->getAccountId() . '&';

        foreach ($keysToHash as $key) {
            if (isset($this->parameters[$key])) {
                if (!is_array($this->parameters[$key])) {
                    $base .= $key . '=' . $this->parameters[$key] . '&';
                } else {
                    foreach ($this->parameters[$key] as $entry) {
                        $base .= $key . '=' . $entry . '&';
                    }
                }
            }
        }

        // add passphrase
        $base .= $this->bpost->getPassPhrase();

        // return the hash
        return hash('sha256', $base);
    }

    /**
     * Get the parameters
     *
     * @param bool $form
     * @param bool $includeChecksum
     *
     * @return array
     */
    public function getParameters($form = false, $includeChecksum = true)
    {
        $return = $this->parameters;

        if ($form && isset($return['orderLine'])) {
            foreach ($return['orderLine'] as $key => $value) {
                $return['orderLine[' . $key . ']'] = $value;
            }

            unset($return['orderLine']);
        }

        if ($includeChecksum) {
            $return['accountId'] = $this->bpost->getAccountId();
            $return['checksum'] = $this->getChecksum();
        }

        return $return;
    }

    /**
     * Set a parameter
     *
     * @param string $key
     * @param mixed $value
     *
     * @throws BpostInvalidValueException
     * @throws BpostInvalidLengthException
     */
    public function setParameter($key, $value)
    {
        switch ((string) $key) {
            // limited values
            case 'action':
            case 'lang':
                $allowedValues = [];
                $allowedValues['action'] = ['START', 'CONFIRM'];
                $allowedValues['lang'] = ['NL', 'FR', 'EN', 'DE', 'Default'];

                if (!in_array($value, $allowedValues[$key])) {
                    throw new BpostInvalidValueException($key, $value, $allowedValues[$key]);
                }
                $this->parameters[$key] = $value;
                break;

                // maximum 2 chars
            case 'customerCountry':
                if (mb_strlen($value) > 2) {
                    throw new BpostInvalidLengthException($key, mb_strlen($value), 2);
                }
                $this->parameters[$key] = (string) $value;
                break;

                // maximum 8 chars
            case 'customerStreetNumber':
            case 'customerBox':
                if (mb_strlen($value) > 8) {
                    throw new BpostInvalidLengthException($key, mb_strlen($value), 8);
                }
                $this->parameters[$key] = (string) $value;
                break;

                // maximum 20 chars
            case 'customerPhoneNumber':
                if (mb_strlen($value) > 20) {
                    throw new BpostInvalidLengthException($key, mb_strlen($value), 20);
                }
                $this->parameters[$key] = (string) $value;
                break;

                // maximum 32 chars
            case 'customerPostalCode':
                if (mb_strlen($value) > 32) {
                    throw new BpostInvalidLengthException($key, mb_strlen($value), 32);
                }
                $this->parameters[$key] = (string) $value;
                break;

                // maximum 40 chars
            case 'customerFirstName':
            case 'customerLastName':
            case 'customerCompany':
            case 'customerStreet':
            case 'customerCity':
                if (mb_strlen($value) > 40) {
                    throw new BpostInvalidLengthException($key, mb_strlen($value), 40);
                }
                $this->parameters[$key] = (string) $value;
                break;

                // maximum 50 chars
            case 'orderReference':
            case 'costCenter':
            case 'customerEmail':
                if (mb_strlen($value) > 50) {
                    throw new BpostInvalidLengthException($key, mb_strlen($value), 50);
                }
                $this->parameters[$key] = (string) $value;
                break;

                // integers
            case 'orderTotalPrice':
            case 'orderWeight':
                $this->parameters[$key] = (int) $value;
                break;

                // array
            case 'orderLine':
                if (!isset($this->parameters[$key])) {
                    $this->parameters[$key] = [];
                }
                $this->parameters[$key][] = $value;
                break;

                // unknown
            case 'deliveryMethodOverrides':
            case 'extra':
            case 'extraSecure':
            case 'confirmUrl':
            case 'cancelUrl':
            case 'errorUrl':
            default:
                if (is_array($value)) {
                    sort($value);
                }
                $this->parameters[$key] = $value;
        }
    }
}
