<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Controller\Admin\Improve\Shipping;

use iio\libmergepdf\Merger;
use PrestaShop\Module\BpostShm\Adapter\BpostshmLabel\Command\BulkDeleteLabelsCommand;
use PrestaShop\Module\BpostShm\Adapter\BpostshmLabel\Command\BulkDownloadLabelsCommand;
use PrestaShop\Module\BpostShm\Adapter\BpostshmLabel\Command\BulkGenerateLabelsCommand;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\DeleteLabelsException;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\DownloadLabelsException;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\GenerateLabelsException;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\NoTrackingUrlFound;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\NotABpostCarrierException;
use PrestaShop\PrestaShop\Core\Domain\Order\ValueObject\OrderId;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;

/**
 * Class LabelController
 */
class LabelController extends OrderController
{
    /**
     * @param $orderId
     * @param $retour
     * @param Request $request
     *
     * @return RedirectResponse|Response
     */
    public function generateAction($orderId, $retour, Request $request)
    {
        try {
            $orderIds = $this->getOrderIds($orderId, $request);
            $labels = $this->getCommandBus()->handle(
                new BulkGenerateLabelsCommand($orderIds, (bool) $retour)
            );
        } catch (GenerateLabelsException $e) {
            $this->handleGenerateLabelsException($e);
        } catch (\Exception $e) {
            $this->addFlash('error', $this->getErrorMessageForException($e, $this->getErrorMessages($e)));
        }

        if (empty($labels)) {
            return $this->redirectToReferer($request);
        }

        return $this->getDownloadResponse($labels);
    }

    /**
     * @param $orderId
     * @param $retour
     * @param Request $request
     *
     * @return RedirectResponse|Response
     */
    public function downloadAction($orderId, $retour, Request $request)
    {
        try {
            $orderIds = $this->getOrderIds($orderId, $request);
            $labels = $this->getCommandBus()->handle(
                new BulkDownloadLabelsCommand($orderIds, (bool) $retour)
            );
        } catch (DownloadLabelsException $e) {
            $this->handleDownloadLabelsException($e);
        } catch (\Exception $e) {
            $this->addFlash('error', $this->getErrorMessageForException($e, $this->getErrorMessages($e)));
        }

        if (empty($labels)) {
            return $this->redirectToReferer($request);
        }

        return $this->getDownloadResponse($labels);
    }

    /**
     * @param $orderId
     * @param $retour
     * @param Request $request
     *
     * @return RedirectResponse
     */
    public function trackAction($orderId, $retour, Request $request): RedirectResponse
    {
        try {
            $order = $this->get('prestashop.adapter.order.repository.order_repository')
                        ->get(new OrderId((int) $orderId));

            $trackingNumbers = $this->get('bpostshm.label_service')->getTrackingNumbers($orderId, $retour);
            foreach ($trackingNumbers as $tracking) {
                if ($tracking['url']) {
                    return $this->redirect($tracking['url']);
                }
            }

            $msg = '';
            $orderId = new OrderId($order->id);
            $orderCarrier = new \Carrier($order->id_carrier);
            if ($orderCarrier->external_module_name != $this->get('bpostshm')->name) {
                $notABpostCarrierException = new NotABpostCarrierException($orderId);
                $msg = $this->getErrorMessageForException(
                    $notABpostCarrierException,
                    $this->getErrorMessages($notABpostCarrierException)
                );
            }

            throw new NoTrackingUrlFound($orderId, $msg);
        } catch (\Exception $e) {
            $this->addFlash('error', $this->getErrorMessageForException($e, $this->getErrorMessages($e)));
        }

        return $this->redirectToReferer($request);
    }

    /**
     * @unused
     *
     * @param $orderId
     * @param $retour
     * @param Request $request
     *
     * @return RedirectResponse
     */
    public function deleteAction($orderId, $retour, Request $request): RedirectResponse
    {
        try {
            $orderIds = $this->getOrderIds($orderId, $request);
            $this->getCommandBus()->handle(
                new BulkDeleteLabelsCommand($orderIds, $retour)
            );

            $this->addFlash(
                'success',
                $this->trans(
                    'Label successfully deleted for order #%s.',
                    'Modules.Bpostshm.Admin',
                    ['#%s' => $orderId]
                )
            );
        } catch (DeleteLabelsException $e) {
            $this->handleDeleteLabelsException($e);
        } catch (\Exception $e) {
            $this->addFlash('error', $this->getErrorMessageForException($e, $this->getErrorMessages($e)));
        }

        if ($orderId) {
            return $this->redirectToRoute('admin_orders_view', ['orderId' => (int) $orderId]);
        }

        return $this->redirectToReferer($request);
    }

    /**
     * Generate a download response for the provided labels
     *
     * If multiple labels are provided, the method will create a zip archive containing the labels' PDF files
     * and send it as a download.
     * If only one label is provided, the method will directly send the PDF file as a download.
     *
     * @param array $labels An array of labels. Each label should have a 'file' and 'pdf' entry.
     *
     * @return Response
     */
    private function getDownloadResponse(array $labels): Response
    {
        $contentType = 'pdf';

        if (count($labels) > 1) {
            if ('pdf' == 'zip') {
                $contentType = 'zip';
                $zip = new \ZipArchive();
                $tmpFile = tempnam(sys_get_temp_dir(), $contentType);
                $zip->open($tmpFile, \ZipArchive::CREATE);
                foreach ($labels as $label) {
                    $fileName = basename($label['file']);
                    $pdf = $label['pdf'];
                    $zip->addFromString($fileName, $pdf);
                }
                $zip->close();
                $content = \Tools::file_get_contents($tmpFile);
                unlink($tmpFile);
            } else {
                $merger = new Merger();
                foreach ($labels as $label) {
                    $merger->addFile($label['file']);
                }
                $content = $merger->merge();
            }

            $response = new Response($content);

            $disposition = $response->headers->makeDisposition(
                ResponseHeaderBag::DISPOSITION_ATTACHMENT,
                'bpost-label-' . date('Ymdhis') . '.' . $contentType
            );
        } else {
            $response = new Response($labels[0]['pdf']);
            $disposition = $response->headers->makeDisposition(
                ResponseHeaderBag::DISPOSITION_ATTACHMENT,
                basename($labels[0]['file'])
            );
        }

        $response->headers->set('Content-Disposition', $disposition);
        $response->headers->set('Content-Type', 'application/' . $contentType);

        return $response;
    }

    /**
     * @param GenerateLabelsException $e
     *
     * @return void
     */
    private function handleGenerateLabelsException(GenerateLabelsException $e)
    {
        $orderIds = $e->getOrdersWithGeneratedLabelsIds();
        if (count($orderIds)) {
            $this->addFlash(
                'success',
                $this->trans(
                    'The following order(s) has successfully generated labels: %s',
                    'Modules.Bpostshm.Admin',
                    ['%s' => implode(',', $orderIds)]
                )
            );
        }

        $orders = $e->getOrdersWithFailedToGenerateLabels();
        foreach ($orders as $order) {
            $this->addFlash(
                'error',
                $this->trans(
                    'An error occurred while generating labels for order #%d: %s',
                    'Modules.Bpostshm.Admin',
                    [
                        '#%d' => $order->getOrderId()->getValue(),
                        '%s' => $this->getErrorMessageForException(
                            $order->getException(),
                            $this->getErrorMessages($order->getException())
                        ),
                    ]
                )
            );
        }
    }

    /**
     * @param DownloadLabelsException $e
     *
     * @return void
     */
    private function handleDownloadLabelsException(DownloadLabelsException $e)
    {
        $orderIds = $e->getOrdersWithGeneratedLabelsIds();
        if (count($orderIds)) {
            $this->addFlash(
                'success',
                $this->trans(
                    'The following order(s) have a label: %s',
                    'Modules.Bpostshm.Admin',
                    ['%s' => implode(',', $orderIds)]
                )
            );
        }

        $orders = $e->getOrdersWithoutLabels();
        foreach ($orders as $order) {
            $this->addFlash(
                'error',
                $this->trans(
                    'No label is available for order #%d: %s',
                    'Modules.Bpostshm.Admin',
                    [
                        '#%d' => $order->getOrderId()->getValue(),
                        '%s' => $this->getErrorMessageForException(
                            $order->getException(),
                            $this->getErrorMessages($order->getException())
                        ),
                    ]
                )
            );
        }
    }

    /**
     * @param DeleteLabelsException $e
     *
     * @return void
     */
    private function handleDeleteLabelsException(DeleteLabelsException $e)
    {
        $orderIds = $e->getOrdersWithDeletedLabelsIds();
        if (count($orderIds)) {
            $this->addFlash(
                'success',
                $this->trans(
                    'The labels for the following order(s) was successfully deleted: %s',
                    'Modules.Bpostshm.Admin',
                    ['%s' => implode(',', $orderIds)]
                )
            );
        }

        $orders = $e->getOrdersWithErrors();
        foreach ($orders as $order) {
            $this->addFlash(
                'error',
                $this->trans(
                    'An error occurred while deleting labels for order #%d: #%s',
                    'Modules.Bpostshm.Admin',
                    [
                        '#%d' => $order->getOrderId()->getValue(),
                        '%s' => $this->getErrorMessageForException(
                            $order->getException(),
                            $this->getErrorMessages($order->getException())
                        ),
                    ]
                )
            );
        }
    }
}
