<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Controller\Admin\Improve\Shipping;

use PrestaShop\Module\BpostShm\Adapter\BpostshmOrder\Command\BulkMarkTreatedCommand;
use PrestaShop\Module\BpostShm\Adapter\BpostshmOrder\Command\BulkRefreshStatusCommand;
use PrestaShop\Module\BpostShm\Adapter\BpostshmOrder\Command\CancelOrderCommand;
use PrestaShop\Module\BpostShm\Adapter\BpostshmOrder\Command\PrepareMissingOrdersCommand;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\MarkTreatedException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\PrepareMissingOrdersException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\RefreshStatusException;
use PrestaShop\Module\BpostShm\Grid\Definition\Factory\AbstractOrderGridDefinitionFactory;
use PrestaShop\Module\BpostShm\Search\Filters\OpenedOrderFilters;
use PrestaShop\Module\BpostShm\Service\OrderService;
use PrestaShop\PrestaShop\Core\Domain\Order\Exception\OrderException;
use PrestaShop\PrestaShop\Core\Domain\Order\ValueObject\OrderId;
use PrestaShopBundle\Security\Annotation\AdminSecurity;
use PrestaShopBundle\Service\Grid\ResponseBuilder;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Manages "Improve > Shipping > bpost Shipping Manager"
 */
class OpenedOrderController extends OrderController
{
    /**
     * Show opened order listing page.
     *
     * @AdminSecurity("is_granted('read', request.get('_legacy_controller'))")
     *
     * @param Request $request
     * @param OpenedOrderFilters $openedOrderFilters
     *
     * @return Response
     */
    public function indexAction(
        Request $request,
        OpenedOrderFilters $openedOrderFilters
    ): Response {
        $openedOrderGridFactory = $this->get('bpostshm.grid.factory.opened_order');
        $openedOrderGrid = $openedOrderGridFactory->getGrid($openedOrderFilters);
        /** @var OrderService $orderService */
        $orderService = $this->get('bpostshm.order_service');
        $orders = $orderService->getMissingBpostOrders();

        return $this->render('@Modules/bpostshm/views/PrestaShop/Admin/Improve/Shipping/OpenedOrder/index.html.twig', [
            'enableSidebar' => true,
            'help_link' => $this->generateSidebarLink($request->attributes->get('_legacy_controller')),
            'openedOrderGrid' => $this->presentGrid($openedOrderGrid),
            'showAddMissingOrdersBtn' => count($orders) > 0,
        ]);
    }

    /**
     * Provides filters functionality
     *
     * @AdminSecurity("is_granted('read', request.get('_legacy_controller'))", redirectRoute="admin_bpostshm_index")
     *
     * @param Request $request
     *
     * @return RedirectResponse
     */
    public function searchAction(Request $request): RedirectResponse
    {
        $gridDefinitionFactory = 'bpostshm.grid.definition.factory.opened_order';
        $filterId = AbstractOrderGridDefinitionFactory::GRID_ID;
        if ($request->get('order') && !$request->get('bpost_order')) {
            $_POST[$filterId] = $_POST['order'];
            $request = Request::createFromGlobals();
        }

        /** @var ResponseBuilder $responseBuilder */
        $responseBuilder = $this->get('prestashop.bundle.grid.response_builder');

        return $responseBuilder->buildSearchResponse(
            $this->get($gridDefinitionFactory),
            $request,
            $filterId,
            'admin_bpostshm_index'
        );
    }

    /**
     * @param Request $request
     *
     * @return RedirectResponse
     */
    public function prepareMissingOrdersAction(Request $request): RedirectResponse
    {
        /** @var BpostShm $module */
        $module = $this->get('bpostshm');

        if (!$module->hasCredentials()) {
            $this->addFlash('error', $this->trans('You must first provide your bpost account information.', 'Modules.Bpostshm.Admin'));
        } else {
            /** @var OrderService $orderService */
            $orderService = $this->get('bpostshm.order_service');

            $orders = $orderService->getMissingBpostOrders();
            if (count($orders)) {
                try {
                    $this->getCommandBus()->handle(
                        new PrepareMissingOrdersCommand($orders)
                    );
                    $this->addFlash('success', $this->trans('Successful update', 'Admin.Notifications.Success'));
                } catch (PrepareMissingOrdersException $e) {
                    $this->handlePrepareMissingOrdersException($e);
                }
            } else {
                $this->addFlash('success', $this->trans('No missing orders', 'Modules.Bpostshm.Admin'));
            }
        }

        return $this->redirectToReferer($request);
    }

    /**
     * @param int $orderId
     * @param Request $request
     *
     * @return RedirectResponse
     */
    public function prepareMissingOrderAction(int $orderId, Request $request): RedirectResponse
    {
        /** @var BpostShm $module */
        $module = $this->get('bpostshm');

        if (!$module->hasCredentials()) {
            $this->addFlash('error', $this->trans('You must first provide your bpost account information.', 'Modules.Bpostshm.Admin'));
        } else {
            /** @var OrderService $orderService */
            $orderService = $this->get('bpostshm.order_service');
            if ($orderService->hasMissingBpostOrder($orderId)) {
                try {
                    $this->getCommandBus()->handle(
                        new PrepareMissingOrdersCommand([new \Order($orderId)])
                    );
                    $this->addFlash('success', $this->trans('Successful update', 'Admin.Notifications.Success'));
                } catch (PrepareMissingOrdersException $e) {
                    $this->handlePrepareMissingOrdersException($e);
                }
            } else {
                $this->addFlash('success', $this->trans('No missing orders', 'Modules.Bpostshm.Admin'));
            }
        }

        return $this->redirectToReferer($request);
    }

    /**
     * @param int $orderId
     *
     * @return RedirectResponse
     *
     * @throws OrderException
     */
    public function cancelOrderAction(int $orderId): RedirectResponse
    {
        try {
            $this->getCommandBus()->handle(
                new CancelOrderCommand(new OrderId($orderId))
            );
            $this->addFlash('success', $this->trans('Successful update', 'Admin.Notifications.Success'));
        } catch (Exception $e) {
            $this->addFlash('error', $this->getErrorMessageForException($e, $this->getErrorMessages($e)));
        }

        return $this->redirectToRoute('admin_bpostshm_index');
    }

    /**
     * @param $orderId
     * @param Request $request
     *
     * @return RedirectResponse
     *
     * @throws OrderException
     */
    public function markTreatedAction($orderId, Request $request): RedirectResponse
    {
        try {
            $orderIds = $this->getOrderIds($orderId, $request);
            $this->getCommandBus()->handle(
                new BulkMarkTreatedCommand($orderIds)
            );
            $this->addFlash('success', $this->trans('Successful update', 'Admin.Notifications.Success'));
        } catch (MarkTreatedException $e) {
            $this->handleMarkTreatedException($e);
        } catch (Exception $e) {
            $this->addFlash('error', $this->getErrorMessageForException($e, $this->getErrorMessages($e)));
        }

        return $this->redirectToReferer($request);
    }

    /**
     * @param $orderId
     * @param Request $request
     *
     * @return RedirectResponse
     *
     * @throws OrderException
     */
    public function refreshStatusAction($orderId, Request $request): RedirectResponse
    {
        try {
            $orderIds = $this->getOrderIds($orderId, $request);
            $this->getCommandBus()->handle(
                new BulkRefreshStatusCommand($orderIds)
            );
            $this->addFlash('success', $this->trans('Successful update', 'Admin.Notifications.Success'));
        } catch (RefreshStatusException $e) {
            $this->handleRefreshStatusException($e);
        } catch (Exception $e) {
            $this->addFlash('error', $this->getErrorMessageForException($e, $this->getErrorMessages($e)));
        }

        return $this->redirectToReferer($request);
    }

    /**
     * @param PrepareMissingOrdersException $e
     */
    private function handlePrepareMissingOrdersException(PrepareMissingOrdersException $e)
    {
        $orderIds = $e->getOrdersWithPreparedIds();
        if (count($orderIds)) {
            $this->addFlash(
                'success',
                $this->trans(
                    'The following order(s) have been successfully prepared: %s',
                    'Modules.Bpostshm.Admin',
                    ['%s' => implode(',', $orderIds)]
                )
            );
        }

        $orders = $e->getFailedPreparedOrders();
        foreach ($orders as $order) {
            $this->addFlash(
                'error',
                $this->trans(
                    'An error occurred while preparing bpost order #%d: %s',
                    'Modules.Bpostshm.Admin',
                    [
                        '#%d' => $order->getOrderId()->getValue(),
                        '%s' => $this->getErrorMessageForException(
                            $order->getException(),
                            $this->getErrorMessages($order->getException())
                        ),
                    ]
                )
            );
        }
    }

    /**
     * @param MarkTreatedException $e
     */
    private function handleMarkTreatedException(MarkTreatedException $e)
    {
        $orderIds = $e->getTreatedOrderIds();
        if (count($orderIds)) {
            $this->addFlash(
                'success',
                $this->trans(
                    'The following order(s) have been successfully marked as treated: %s',
                    'Modules.Bpostshm.Admin',
                    ['%s' => implode(',', $orderIds)]
                )
            );
        }

        $orders = $e->getOrdersWithFailedMarkTreated();
        foreach ($orders as $order) {
            $this->addFlash(
                'error',
                $this->trans(
                    'An error occurred while marking the bpost order #%d as treated: %s',
                    'Modules.Bpostshm.Admin',
                    [
                        '#%d' => $order->getOrderId()->getValue(),
                        '%s' => $this->getErrorMessageForException(
                            $order->getException(),
                            $this->getErrorMessages($order->getException())
                        ),
                    ]
                )
            );
        }
    }

    /**
     * @param RefreshStatusException $e
     */
    private function handleRefreshStatusException(RefreshStatusException $e)
    {
        $orderIds = $e->getRefreshedStatusOrderIds();
        if (count($orderIds)) {
            $this->addFlash(
                'success',
                $this->trans(
                    'The following order(s) have had their statuses successfully refreshed: %s',
                    'Modules.Bpostshm.Admin',
                    ['%s' => implode(',', $orderIds)]
                )
            );
        }

        $orders = $e->getFailedStatusRefreshOrders();
        foreach ($orders as $order) {
            $this->addFlash(
                'error',
                $this->trans(
                    'An error occurred while refreshing the status for bpost order #%d: %s',
                    'Modules.Bpostshm.Admin',
                    [
                        '#%d' => $order->getOrderId()->getValue(),
                        '%s' => $this->getErrorMessageForException(
                            $order->getException(),
                            $this->getErrorMessages($order->getException())
                        ),
                    ]
                )
            );
        }
    }
}
