<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Controller\Admin\Improve\Shipping;

use PrestaShop\Module\BpostShm\Api\BpostException;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\FileException;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\NoLabelFoundException;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\NonExpectedValueException;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\NoTrackingUrlFound;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\BpostStatusNotFoundException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\CancelStatusNotFoundException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\HSCodeException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\MissingReferenceException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\NotABpostCarrierException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\OriginCountryException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\ProductMaxWeightException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\ProductMinWeightException;
use PrestaShop\Module\BpostShm\Service\ParcelService;
use PrestaShop\PrestaShop\Core\Domain\Order\Exception\OrderEmailSendException;
use PrestaShop\PrestaShop\Core\Domain\Order\Exception\OrderNotFoundException;
use PrestaShopBundle\Controller\Admin\FrameworkBundleAdminController;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Manages "Improve > Shipping > bpost Shipping Manager"
 */
class OrderController extends FrameworkBundleAdminController
{
    /**
     * @param Request $request
     * @param string $defaultRoute
     *
     * @return RedirectResponse
     */
    protected function redirectToReferer(
        Request $request,
        string $defaultRoute = 'admin_bpostshm_index'
    ): RedirectResponse {
        $referer = $request->server->has('HTTP_REFERER') ? $request->server->get('HTTP_REFERER') : false;

        return $referer ? $this->redirect($referer) : $this->redirectToRoute($defaultRoute);
    }

    /**
     * @param $orderId
     * @param Request $request
     *
     * @return int[]|mixed
     */
    protected function getOrderIds($orderId, Request $request)
    {
        $orderIds = $request->request->get('order_orders_bulk');
        if (!is_array($orderIds)) {
            $orderIds = [(int) $orderId];
        }

        return $orderIds;
    }

    /**
     * Get error by exception from given messages
     *
     * @param Exception $e
     * @param array $messages
     *
     * @return string
     */
    protected function getErrorMessageForException(\Exception $e, array $messages)
    {
        if ($e instanceof BpostException && !empty($e->getMessage())) {
            return $e->getMessage();
        }

        return parent::getErrorMessageForException($e, $messages);
    }

    /**
     * @param \Exception $e
     *
     * @return array
     */
    protected function getErrorMessages(\Exception $e): array
    {
        return [
            FileException::class => $e->getMessage(),
            HSCodeException::class => $e->getMessage(),
            OriginCountryException::class => $e->getMessage(),
            NonExpectedValueException::class => $e->getMessage(),
            MissingReferenceException::class => $e->getMessage(),
            ProductMinWeightException::class => $e instanceof ProductMinWeightException ? $this->trans(
                'Product #%d has no specified weight.',
                'Modules.Bpostshm.Admin',
                [
                    '%d' => $e->getProductId()->getValue(),
                ]
            ) : '',
            ProductMaxWeightException::class => $e instanceof ProductMaxWeightException ? $this->trans(
                'Product #%d exceeds maximum allowed weight (%max% Kg).',
                'Modules.Bpostshm.Admin',
                [
                    '%d' => $e->getProductId()->getValue(),
                    '%max%' => ParcelService::MAX_WEIGHT / 1000,
                ]
            ) : '',
            NoLabelFoundException::class => $this->trans(
                'No label found.',
                'Modules.Bpostshm.Admin'
            ),
            CancelStatusNotFoundException::class => $this->trans(
                'Cancel Status not found. Please create a "Cancelled" order state using "order_canceled" template.',
                'Modules.Bpostshm.Admin'
            ),
            NoTrackingUrlFound::class => $e instanceof NoTrackingUrlFound ? $this->trans(
                'No bpost tracking url found for order #%d. %s',
                'Modules.Bpostshm.Admin',
                ['#%d' => $e->getOrderId()->getValue(), '%s' => $e->getMessage()]
            ) : '',
            BpostStatusNotFoundException::class => $e instanceof BpostStatusNotFoundException ?
                trim($this->trans(
                    'Unable to retrieve bpost status for order #%d. %s',
                    'Modules.Bpostshm.Admin',
                    ['#%d' => $e->getOrderId()->getValue(), '%s' => $e->getMessage()]
                )) : '',
            NotABpostCarrierException::class => $e instanceof NotABpostCarrierException ?
                $this->trans(
                    'It appears the order #%d hasn\'t a bpost carrier.',
                    'Modules.Bpostshm.Admin',
                    ['#%d' => $e->getOrderId()->getValue()]
                ) : '',
            OrderNotFoundException::class => $e instanceof OrderNotFoundException ?
                $this->trans(
                    'Order #%d cannot be loaded.',
                    'Admin.Orderscustomers.Notification',
                    ['#%d' => $e->getOrderId()->getValue()]
                ) : '',
            OrderEmailSendException::class => $this->trans(
                'An error occurred while sending the e-mail to the customer.',
                'Admin.Orderscustomers.Notification'
            ),
        ];
    }
}
