<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Grid\Definition\Factory;

use PrestaShop\Module\BpostShm\Grid\Column\Type\BpostDeliveryMethodColumn;
use PrestaShop\Module\BpostShm\Grid\Column\Type\BpostLabelColumn;
use PrestaShop\Module\BpostShm\Grid\Column\Type\BpostStatusColumn;
use PrestaShop\Module\BpostShm\Grid\Column\Type\DateTimeColumn;
use PrestaShop\Module\BpostShm\Grid\Column\Type\ListActionColumn;
use PrestaShop\PrestaShop\Core\ConfigurationInterface;
use PrestaShop\PrestaShop\Core\Form\FormChoiceProviderInterface;
use PrestaShop\PrestaShop\Core\Grid\Action\Bulk\BulkActionCollection;
use PrestaShop\PrestaShop\Core\Grid\Action\Bulk\Type\ButtonBulkAction;
use PrestaShop\PrestaShop\Core\Grid\Action\Bulk\Type\SubmitBulkAction;
use PrestaShop\PrestaShop\Core\Grid\Action\GridActionCollection;
use PrestaShop\PrestaShop\Core\Grid\Action\Row\AccessibilityChecker\AccessibilityCheckerInterface;
use PrestaShop\PrestaShop\Core\Grid\Action\Row\RowActionCollection;
use PrestaShop\PrestaShop\Core\Grid\Action\Row\Type\LinkRowAction;
use PrestaShop\PrestaShop\Core\Grid\Action\Type\SimpleGridAction;
use PrestaShop\PrestaShop\Core\Grid\Column\ColumnCollection;
use PrestaShop\PrestaShop\Core\Grid\Column\Type\Common\ActionColumn;
use PrestaShop\PrestaShop\Core\Grid\Column\Type\Common\BulkActionColumn;
use PrestaShop\PrestaShop\Core\Grid\Column\Type\Common\IdentifierColumn;
use PrestaShop\PrestaShop\Core\Grid\Column\Type\DataColumn;
use PrestaShop\PrestaShop\Core\Grid\Column\Type\PreviewColumn;
use PrestaShop\PrestaShop\Core\Grid\Definition\Factory\AbstractGridDefinitionFactory;
use PrestaShop\PrestaShop\Core\Grid\Definition\Factory\BulkDeleteActionTrait;
use PrestaShop\PrestaShop\Core\Grid\Definition\Factory\DeleteActionTrait;
use PrestaShop\PrestaShop\Core\Grid\Filter\Filter;
use PrestaShop\PrestaShop\Core\Grid\Filter\FilterCollection;
use PrestaShop\PrestaShop\Core\Hook\HookDispatcherInterface;
use PrestaShopBundle\Form\Admin\Type\DateRangeType;
use PrestaShopBundle\Form\Admin\Type\SearchAndResetType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextType;

/**
 * Class OrderGridDefinitionFactory creates grid definition Bpost Orders data.
 */
abstract class AbstractOrderGridDefinitionFactory extends AbstractGridDefinitionFactory
{
    use BulkDeleteActionTrait;
    use DeleteActionTrait;

    public const GRID_ID = 'bpost_order';

    /**
     * @var ConfigurationInterface
     */
    private $configuration;

    /**
     * @var \Context
     */
    private $context;

    /**
     * @var AccessibilityCheckerInterface
     */
    private $cancelOrderAccessibilityChecker;

    /**
     * @var AccessibilityCheckerInterface
     */
    private $generateLabelsAccessibilityChecker;

    /**
     * @var AccessibilityCheckerInterface
     */
    private $generateRetourLabelsAccessibilityChecker;

    /**
     * @var AccessibilityCheckerInterface
     */
    private $markAsTreatedAccessibilityChecker;

    /**
     * @var AccessibilityCheckerInterface
     */
    private $printLabelsAccessibilityChecker;

    /**
     * @var AccessibilityCheckerInterface
     */
    private $refreshStatusAccessibilityChecker;

    /**
     * @var AccessibilityCheckerInterface
     */
    private $viewTrackAccessibilityChecker;

    /**
     * @var FormChoiceProviderInterface
     */
    private $statusesChoiceProvider;

    /**
     * @param HookDispatcherInterface $dispatcher
     * @param ConfigurationInterface $configuration
     * @param \Context $context
     * @param AccessibilityCheckerInterface $cancelOrderAccessibilityChecker
     * @param AccessibilityCheckerInterface $generateLabelsAccessibilityChecker
     * @param AccessibilityCheckerInterface $generateRetourLabelsAccessibilityChecker
     * @param AccessibilityCheckerInterface $markAsTreatedAccessibilityChecker
     * @param AccessibilityCheckerInterface $printLabelsAccessibilityChecker
     * @param AccessibilityCheckerInterface $refreshStatusAccessibilityChecker
     * @param AccessibilityCheckerInterface $viewTrackAccessibilityChecker
     * @param FormChoiceProviderInterface $statusesChoiceProvider
     */
    public function __construct(
        HookDispatcherInterface $dispatcher,
        ConfigurationInterface $configuration,
        \Context $context,
        AccessibilityCheckerInterface $cancelOrderAccessibilityChecker,
        AccessibilityCheckerInterface $generateLabelsAccessibilityChecker,
        AccessibilityCheckerInterface $generateRetourLabelsAccessibilityChecker,
        AccessibilityCheckerInterface $markAsTreatedAccessibilityChecker,
        AccessibilityCheckerInterface $printLabelsAccessibilityChecker,
        AccessibilityCheckerInterface $refreshStatusAccessibilityChecker,
        AccessibilityCheckerInterface $viewTrackAccessibilityChecker,
        FormChoiceProviderInterface $statusesChoiceProvider
    ) {
        parent::__construct($dispatcher);

        $this->configuration = $configuration;
        $this->context = $context;
        $this->cancelOrderAccessibilityChecker = $cancelOrderAccessibilityChecker;
        $this->generateLabelsAccessibilityChecker = $generateLabelsAccessibilityChecker;
        $this->generateRetourLabelsAccessibilityChecker = $generateRetourLabelsAccessibilityChecker;
        $this->markAsTreatedAccessibilityChecker = $markAsTreatedAccessibilityChecker;
        $this->printLabelsAccessibilityChecker = $printLabelsAccessibilityChecker;
        $this->refreshStatusAccessibilityChecker = $refreshStatusAccessibilityChecker;
        $this->viewTrackAccessibilityChecker = $viewTrackAccessibilityChecker;
        $this->statusesChoiceProvider = $statusesChoiceProvider;
    }

    /**
     * {@inheritdoc}
     */
    protected function getId(): string
    {
        return static::GRID_ID;
    }

    /**
     * {@inheritdoc}
     */
    protected function getName(): string
    {
        return $this->trans('Opened Orders', [], 'Modules.Bpostshm.Admin');
    }

    protected function getRedirectRoute(): string
    {
        return 'admin_bpostshm_index';
    }

    /**
     * {@inheritdoc}
     */
    protected function getColumns()
    {
        $previewColumn = (new PreviewColumn('preview'))
            ->setOptions([
                'icon_expand' => 'keyboard_arrow_down',
                'icon_collapse' => 'keyboard_arrow_up',
                'preview_data_route' => 'admin_orders_preview',
                'preview_route_params' => [
                    'orderId' => 'id_order',
                ],
            ]);

        return (new ColumnCollection())
            ->add(
                (new BulkActionColumn('orders_bulk'))
                    ->setOptions([
                        'bulk_field' => 'id_order',
                    ])
            )
            ->add(
                (new ActionColumn('bpost_actions'))
                ->setName($this->trans('', [], 'Admin.Global'))
                ->setOptions([
                    'actions' => $this->getBpostRowActions(),
                ])
            )
            ->add(
                (new IdentifierColumn('bpost_reference'))
                ->setName($this->trans('Reference', [], 'Admin.Global'))
                ->setOptions([
                    'identifier_field' => 'bpost_reference',
                    'preview' => $previewColumn,
                    'clickable' => false,
                ])
            )
            ->add(
                (new BpostDeliveryMethodColumn('delivery_method'))
                ->setName($this->trans('Delivery Method', [], 'Modules.Bpostshm.Admin'))
                ->setOptions([
                    'field' => 'delivery_method',
                    'field_opt_saturday' => 'opt_saturday',
                    'field_opt_insurance' => 'opt_insurance',
                    'field_opt_signature' => 'opt_signature',
                    'field_opt_2nd_presentation' => 'opt_2nd_presentation',
                ])
            )
            ->add(
                (new DataColumn('recipient'))
                ->setName($this->trans('Recipient', [], 'Modules.Bpostshm.Admin'))
                ->setOptions([
                    'field' => 'recipient',
                ])
            )
            ->add(
                (new DateTimeColumn('drop_date'))
                ->setName($this->trans('Drop date', [], 'Modules.Bpostshm.Admin'))
                ->setOptions([
                    'field' => 'drop_date',
                    'format' => $this->context->language->date_format_lite,
                    'empty_data' => '--',
                    'clickable' => true,
                ])
            )
            ->add(
                (new BpostStatusColumn('status'))
                ->setName($this->trans('Status', [], 'Admin.Global'))
                ->setOptions([
                    'field' => 'status',
                ])
            )
            ->add(
                (new DateTimeColumn('date_add'))
                ->setName($this->trans('Creation Date', [], 'Modules.Bpostshm.Admin'))
                ->setOptions([
                    'field' => 'date_add',
                    'format' => $this->context->language->date_format_full,
                    'clickable' => true,
                ])
            )
            ->add(new BpostLabelColumn('label'))
            ->add(
                (new ListActionColumn('actions'))
                ->setName($this->trans('Actions', [], 'Admin.Global'))
                ->setOptions([
                    'actions' => $this->getRowActions(),
                ])
            );
    }

    /**
     * {@inheritdoc}
     */
    protected function getFilters()
    {
        $filters = new FilterCollection();

        $filters
            ->add(
                (new Filter('bpost_reference', TextType::class))
                ->setTypeOptions([
                    'required' => false,
                    'attr' => [
                        'placeholder' => $this->trans('Search reference', [], 'Admin.Actions'),
                    ],
                ])
                ->setAssociatedColumn('bpost_reference')
            )
            ->add(
                (new Filter('delivery_method', TextType::class))
                ->setTypeOptions([
                    'required' => false,
                    'attr' => [
                        'placeholder' => $this->trans('Search method', [], 'Modules.Bpostshm.Admin'),
                    ],
                ])
                ->setAssociatedColumn('delivery_method')
            )
            ->add(
                (new Filter('recipient', TextType::class))
                ->setTypeOptions([
                    'required' => false,
                    'attr' => [
                        'placeholder' => $this->trans('Search recipient', [], 'Modules.Bpostshm.Admin'),
                    ],
                ])
                ->setAssociatedColumn('recipient')
            )
            ->add(
                (new Filter('drop_date', DateRangeType::class))
                ->setTypeOptions([
                    'required' => false,
                ])
                ->setAssociatedColumn('drop_date')
            )
            ->add(
                (new Filter('status', ChoiceType::class))
                ->setTypeOptions([
                    'required' => false,
                    'attr' => [
                        'placeholder' => $this->trans('Search Status', [], 'Admin.Actions'),
                    ],
                    'choices' => $this->statusesChoiceProvider->getChoices(),
                    'translation_domain' => false,
                ])
                ->setAssociatedColumn('status')
            )
            ->add(
                (new Filter('date_add', DateRangeType::class))
                ->setTypeOptions([
                    'required' => false,
                ])
                ->setAssociatedColumn('date_add')
            )
            ->add(
                (new Filter('actions', SearchAndResetType::class))
                ->setTypeOptions([
                    'reset_route' => 'admin_common_reset_search_by_filter_id',
                    'reset_route_params' => [
                        'filterId' => $this->getId(),
                    ],
                    'redirect_route' => $this->getRedirectRoute(),
                ])
                ->setAssociatedColumn('actions')
            );

        return $filters;
    }

    /**
     * {@inheritdoc}
     */
    protected function getGridActions()
    {
        return (new GridActionCollection())
            ->add(
                (new SimpleGridAction('common_refresh_list'))
                    ->setName($this->trans('Refresh list', [], 'Admin.Advparameters.Feature'))
                    ->setIcon('refresh')
            )
            ->add(
                (new SimpleGridAction('common_show_query'))
                    ->setName($this->trans('Show SQL query', [], 'Admin.Actions'))
                    ->setIcon('code')
            )
            ->add(
                (new SimpleGridAction('common_export_sql_manager'))
                    ->setName($this->trans('Export to SQL Manager', [], 'Admin.Actions'))
                    ->setIcon('storage')
            );
    }

    /**
     * {@inheritdoc}
     */
    protected function getBulkActions()
    {
        return (new BulkActionCollection())
            ->add(
                (new SubmitBulkAction('mark_treated'))
                ->setName($this->trans('Mark as treated', [], 'Modules.Bpostshm.Admin'))
                ->setOptions([
                    'submit_route' => 'admin_bpostshm_bulk_mark_treated',
                ])
            )
            ->add(
                (new SubmitBulkAction('print_labels'))
                ->setName($this->trans('Print labels', [], 'Modules.Bpostshm.Admin'))
                ->setOptions([
                    'submit_route' => 'admin_bpostshm_label_bulk_generate',
                ])
            )
            ->add(
                (new SubmitBulkAction('refresh_status'))
                ->setName($this->trans('Refresh Status', [], 'Modules.Bpostshm.Admin'))
                ->setOptions([
                    'submit_route' => 'admin_bpostshm_bulk_refresh_status',
                ])
            )
            ->add(
                (new ButtonBulkAction('open_tabs'))
                ->setName($this->trans('Open in new tabs', [], 'Admin.Orderscustomers.Feature'))
                ->setOptions([
                    'class' => 'open_tabs',
                    'attributes' => [
                        'data-route' => 'admin_orders_view',
                        'data-route-param-name' => 'orderId',
                        'data-tabs-blocked-message' => $this->trans(
                            'It looks like you have exceeded the number of tabs allowed. Check your browser settings to open multiple tabs.',
                            [],
                            'Admin.Orderscustomers.Feature'
                        ),
                    ],
                ])
            );
    }

    /**
     * @return RowActionCollection
     */
    private function getBpostRowActions(): RowActionCollection
    {
        $rowActionCollection = new RowActionCollection();
        $rowActionCollection->add(
            (new LinkRowAction('print_labels'))
            ->setName($this->trans('Print labels', [], 'Modules.Bpostshm.Admin'))
            ->setIcon('printer')
            ->setOptions([
                'accessibility_checker' => $this->printLabelsAccessibilityChecker,
                'route' => 'admin_bpostshm_label_download',
                'route_param_name' => 'orderId',
                'route_param_field' => 'id_order',
                'use_inline_display' => true,
            ])
        )
            ->add(
                (new LinkRowAction('generate_labels'))
                ->setName($this->trans('Print labels', [], 'Modules.Bpostshm.Admin'))
                ->setIcon('add_circle')
                ->setOptions([
                    'accessibility_checker' => $this->generateLabelsAccessibilityChecker,
                    'route' => 'admin_bpostshm_label_generate',
                    'route_param_name' => 'orderId',
                    'route_param_field' => 'id_order',
                    'use_inline_display' => true,
                ])
            )
            ->add(
                (new LinkRowAction('view_track'))
                ->setName($this->trans('View Track & Trace status', [], 'Modules.Bpostshm.Admin'))
                ->setIcon('schedule')
                ->setOptions([
                    'accessibility_checker' => $this->viewTrackAccessibilityChecker,
                    'route' => 'admin_bpostshm_label_track',
                    'route_param_name' => 'orderId',
                    'route_param_field' => 'id_order',
                    'use_inline_display' => true,
                    'target' => '_blank',
                ])
            )
            ->add(
                (new LinkRowAction('view'))
                ->setName($this->trans('View', [], 'Admin.Actions'))
                ->setIcon('zoom_in')
                ->setOptions([
                    'route' => 'admin_orders_view',
                    'route_param_name' => 'orderId',
                    'route_param_field' => 'id_order',
                    'clickable_row' => true,
                    'use_inline_display' => true,
                    'target' => '_blank',
                ])
            );

        return $rowActionCollection;
    }

    /**
     * @return RowActionCollection
     */
    private function getRowActions(): RowActionCollection
    {
        $rowActionCollection = new RowActionCollection();
        $rowActionCollection->add(
            (new LinkRowAction('mark_as_treated'))
                ->setName($this->trans('Mark as treated', [], 'Modules.Bpostshm.Admin'))
                ->setIcon('assignment_turned_in')
                ->setOptions([
                    'accessibility_checker' => $this->markAsTreatedAccessibilityChecker,
                    'route' => 'admin_bpostshm_mark_treated',
                    'route_param_name' => 'orderId',
                    'route_param_field' => 'id_order',
                    'use_inline_display' => true,
                ])
        )
            ->add(
                (new LinkRowAction('print_labels'))
                ->setName($this->trans('Print labels', [], 'Modules.Bpostshm.Admin'))
                ->setIcon('printer')
                ->setOptions([
                    'accessibility_checker' => $this->printLabelsAccessibilityChecker,
                    'route' => 'admin_bpostshm_label_download',
                    'route_param_name' => 'orderId',
                    'route_param_field' => 'id_order',
                    'use_inline_display' => true,
                ])
            )
            ->add(
                (new LinkRowAction('generate_labels'))
                ->setName($this->trans('Print labels', [], 'Modules.Bpostshm.Admin'))
                ->setIcon('add_circle')
                ->setOptions([
                    'accessibility_checker' => $this->generateLabelsAccessibilityChecker,
                    'route' => 'admin_bpostshm_label_generate',
                    'route_param_name' => 'orderId',
                    'route_param_field' => 'id_order',
                    'use_inline_display' => true,
                ])
            )
            ->add(
                (new LinkRowAction('generate_retour_labels'))
                ->setName($this->trans('Create Retour', [], 'Modules.Bpostshm.Admin'))
                ->setIcon('add_circle')
                ->setOptions([
                    'accessibility_checker' => $this->generateRetourLabelsAccessibilityChecker,
                    'route' => 'admin_bpostshm_label_download',
                    'route_param_name' => 'orderId',
                    'route_param_field' => 'id_order',
                    'use_inline_display' => true,
                ])
            )
            ->add(
                (new LinkRowAction('view_track'))
                ->setName($this->trans('View Track & Trace status', [], 'Modules.Bpostshm.Admin'))
                ->setIcon('schedule')
                ->setOptions([
                    'accessibility_checker' => $this->viewTrackAccessibilityChecker,
                    'route' => 'admin_bpostshm_label_track',
                    'route_param_name' => 'orderId',
                    'route_param_field' => 'id_order',
                    'use_inline_display' => true,
                    'target' => '_blank',
                ])
            )
            ->add(
                (new LinkRowAction('refresh_status'))
                ->setName($this->trans('Refresh status', [], 'Modules.Bpostshm.Admin'))
                ->setIcon('refresh')
                ->setOptions([
                    'accessibility_checker' => $this->refreshStatusAccessibilityChecker,
                    'route' => 'admin_bpostshm_refresh_status',
                    'route_param_name' => 'orderId',
                    'route_param_field' => 'id_order',
                    'use_inline_display' => true,
                ])
            )
            ->add(
                (new LinkRowAction('cancel'))
                ->setName($this->trans('Cancel Order', [], 'Admin.Actions'))
                ->setIcon('cancel')
                ->setOptions([
                    'accessibility_checker' => $this->cancelOrderAccessibilityChecker,
                    'confirm_message' => $this->trans(
                        'Are you sure you want to cancel the order?',
                        [],
                        'Modules.Bpostshm.Admin'
                    ),
                    'route' => 'admin_bpostshm_cancel_order',
                    'route_param_name' => 'orderId',
                    'route_param_field' => 'id_order',
                    'use_inline_display' => true,
                ])
            )
            ->add(
                (new LinkRowAction('view'))
                ->setName($this->trans('View', [], 'Admin.Actions'))
                ->setIcon('zoom_in')
                ->setOptions([
                    'route' => 'admin_orders_view',
                    'route_param_name' => 'orderId',
                    'route_param_field' => 'id_order',
                    'clickable_row' => true,
                    'use_inline_display' => true,
                    'target' => '_blank',
                ])
            );

        return $rowActionCollection;
    }
}
