<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace PrestaShop\Module\BpostShm\Grid\Query;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Query\QueryBuilder;
use PrestaShop\Module\BpostShm\Helper\SettingsFormHelper;
use PrestaShop\PrestaShop\Core\ConfigurationInterface;
use PrestaShop\PrestaShop\Core\Grid\Query\AbstractDoctrineQueryBuilder;
use PrestaShop\PrestaShop\Core\Grid\Query\DoctrineSearchCriteriaApplicatorInterface;
use PrestaShop\PrestaShop\Core\Grid\Search\SearchCriteriaInterface;

/**
 * Class AbstractOrderQueryBuilder builds queries for Bpost Orders grid.
 */
abstract class AbstractOrderQueryBuilder extends AbstractDoctrineQueryBuilder
{
    /**
     * @var ConfigurationInterface
     */
    protected $configuration;

    /**
     * @var int
     */
    protected $contextLangId;

    /**
     * @var DoctrineSearchCriteriaApplicatorInterface
     */
    protected $criteriaApplicator;

    /**
     * @param Connection $connection
     * @param string $dbPrefix
     * @param ConfigurationInterface $configuration
     * @param DoctrineSearchCriteriaApplicatorInterface $criteriaApplicator
     * @param int $contextLangId
     * @param int[] $contextShopIds
     */
    public function __construct(
        Connection $connection,
        $dbPrefix,
        DoctrineSearchCriteriaApplicatorInterface $criteriaApplicator,
        int $contextLangId,
        array $contextShopIds,
        ConfigurationInterface $configuration
    ) {
        parent::__construct($connection, $dbPrefix);
        $this->contextLangId = $contextLangId;
        $this->criteriaApplicator = $criteriaApplicator;
        $this->contextShopIds = $contextShopIds;
        $this->configuration = $configuration;
    }

    /**
     * @var array
     */
    protected $contextShopIds;

    /**
     * {@inheritdoc}
     */
    public function getSearchQueryBuilder(SearchCriteriaInterface $searchCriteria)
    {
        $qb = $this
            ->getBaseQueryBuilder($searchCriteria->getFilters())
            ->addSelect('o.id_order, bo.`date_add`')
            ->addSelect('bo.`bpost_reference`, bo.`delivery_method`, bo.`recipient`')
            ->addSelect('bo.`opt_saturday`, bo.`opt_signature`, bo.`opt_insurance`, bo.`opt_2nd_presentation`')
            ->addSelect('bo.`drop_date` , bo.`status`')
        ;

        $this->applySorting($qb, $searchCriteria);

        $this->criteriaApplicator
            ->applyPagination($searchCriteria, $qb)
        ;

        return $qb;
    }

    /**
     * {@inheritdoc}
     */
    public function getCountQueryBuilder(SearchCriteriaInterface $searchCriteria): QueryBuilder
    {
        $qb = $this->getBaseQueryBuilder($searchCriteria->getFilters());
        $qb->select('count(o.id_order)');

        return $qb;
    }

    /**
     * @param array $filters
     *
     * @return QueryBuilder
     */
    protected function getBaseQueryBuilder(array $filters): QueryBuilder
    {
        $qb = $this->connection
            ->createQueryBuilder()
            ->from($this->dbPrefix . 'orders', 'o')
            ->innerJoin('o', $this->dbPrefix . 'bpostshm_order', 'bo', 'o.id_order = bo.id_order')
            ->leftJoin('o', $this->dbPrefix . 'shop', 's', 'o.id_shop = s.id_shop')
            ->andWhere('o.`id_shop` IN (:context_shop_ids)')
            ->andWhere('bo.`status` <> "SHM_ONLY_PROCESSING"')
            ->setParameter('context_lang_id', $this->contextLangId, \PDO::PARAM_INT)
            ->setParameter('context_shop_ids', $this->contextShopIds, Connection::PARAM_INT_ARRAY)
        ;

        $displayStates = $this->configuration->get('BPOSTSHM_LABELS_ORDER_DISPLAY_STATES');
        if (SettingsFormHelper::isValidJSON($displayStates)) {
            $displayStates = json_decode($displayStates, true);
            $qb->andWhere('o.`current_state` IN (:display_state_ids)')
                ->setParameter('display_state_ids', $displayStates, Connection::PARAM_INT_ARRAY);
        }

        $this->addCountLabelsSubFields($qb);
        $qb->addSelect('bo.treated');

        $strictComparisonFilters = [
            'id_order' => 'o.id_order',
            'status' => 'bo.status',
            'carrier_method' => 'bo.carrier_method',
            'retour' => 'bo.retour',
            'opt_saturday' => 'bo.opt_saturday',
            'opt_signature' => 'bo.opt_signature',
            'opt_insurance' => 'bo.opt_insurance',
            'opt_2nd_presentation' => 'bo.opt_2nd_presentation',
            'treated' => 'bo.treated',
        ];

        $likeComparisonFilters = [
            'bpost_reference' => 'bo.`bpost_reference`',
            'delivery_method' => 'bo.`delivery_method`',
            'recipient' => 'bo.`recipient`',
        ];

        $dateComparisonFilters = [
            'drop_date' => 'bo.`drop_date`',
            'date_add' => 'bo.`date_add`',
        ];

        if (!isset($filters['date_add'])) {
            $displayDays = (int) $this->configuration->get('BPOSTSHM_LABELS_ORDER_DISPLAY_DAYS');
            if ($displayDays > 0) {
                $filters['date_add']['from'] = date('Y-m-d', strtotime("- $displayDays days"));
            }
        }

        foreach ($filters as $filterName => $filterValue) {
            if (isset($strictComparisonFilters[$filterName])) {
                $alias = $strictComparisonFilters[$filterName];

                $qb->andWhere("$alias = :$filterName");
                $qb->setParameter($filterName, $filterValue);

                continue;
            }

            if (isset($likeComparisonFilters[$filterName])) {
                $alias = $likeComparisonFilters[$filterName];

                $qb->andWhere("$alias LIKE :$filterName");
                $qb->setParameter($filterName, '%' . $filterValue . '%');

                continue;
            }

            if (isset($dateComparisonFilters[$filterName])) {
                $alias = $dateComparisonFilters[$filterName];

                if (isset($filterValue['from'])) {
                    $name = sprintf('%s_from', $filterName);

                    $qb->andWhere("$alias >= :$name");
                    $qb->setParameter($name, sprintf('%s %s', $filterValue['from'], '0:0:0'));
                }

                if (isset($filterValue['to'])) {
                    $name = sprintf('%s_to', $filterName);

                    $qb->andWhere("$alias <= :$name");
                    $qb->setParameter($name, sprintf('%s %s', $filterValue['to'], '23:59:59'));
                }
            }
        }

        return $qb;
    }

    /**
     * Returns the subquery to count labels
     */
    private function getCountLabelsSubSelect(): string
    {
        return $this->connection
            ->createQueryBuilder()
            ->select('count(lb.id_label)')
            ->from($this->dbPrefix . 'bpostshm_label', 'lb')
            ->andWhere('lb.id_order = o.id_order')
            ->andWhere('lb.tracking IS NOT NULL AND lb.tracking <> ""')
            ->orderBy('id_label', 'desc')
            ->setMaxResults(1)
            ->getSQL()
        ;
    }

    /**
     * Returns the subquery to count retour labels
     */
    private function getCountRetourLabelsSubSelect(): string
    {
        return $this->connection
            ->createQueryBuilder()
            ->select('count(lb.id_label)')
            ->from($this->dbPrefix . 'bpostshm_label', 'lb')
            ->andWhere('lb.id_order = o.id_order')
            ->andWhere('lb.tracking_retour IS NOT NULL AND lb.tracking_retour <> ""')
            ->orderBy('id_label', 'desc')
            ->setMaxResults(1)
            ->getSQL()
        ;
    }

    /**
     * @param QueryBuilder $qb
     */
    private function addCountLabelsSubFields(QueryBuilder $qb): void
    {
        $qb->addSelect('(' . $this->getCountLabelsSubSelect() . ') AS count_labels');
        $qb->addSelect('(' . $this->getCountRetourLabelsSubSelect() . ') AS count_retour_labels');
    }

    /**
     * @param QueryBuilder $qb
     * @param SearchCriteriaInterface $criteria
     */
    private function applySorting(QueryBuilder $qb, SearchCriteriaInterface $criteria)
    {
        $sortableFields = [
            'id_order' => 'o.id_order',
            'bpost_reference' => 'bo.bpost_reference',
            'delivery_method' => 'bo.delivery_method',
            'recipient' => 'bo.recipient',
            'carrier_method' => 'bo.carrier_method',
            'opt_saturday' => 'bo.opt_saturday',
            'opt_signature' => 'bo.opt_signature',
            'opt_insurance' => 'bo.opt_insurance',
            'opt_2nd_presentation' => 'bo.opt_2nd_presentation',
            'status' => 'bo.status',
            'drop_date' => 'bo.`drop_date`',
            'date_add' => 'bo.`date_add`',
        ];

        if (isset($sortableFields[$criteria->getOrderBy()])) {
            $qb->orderBy(
                $sortableFields[$criteria->getOrderBy()],
                $criteria->getOrderWay()
            );
        }
    }
}
