<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Helper;

use PrestaShop\Module\BpostShm\Domain\Exception\CarrierException;
use PrestaShop\Module\BpostShm\Service\CarrierService;
use PrestaShop\Module\BpostShm\Service\DeliveryDateService;
use PrestaShop\Module\BpostShm\Service\PickupService;
use PrestaShop\Module\BpostShm\Utils\BpostTools;
use PrestaShopBundle\Entity\Repository\TabRepository;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * Class SettingsBuilder
 */
class SettingsFormHelper
{
    /**
     * The GitHub directory for the BPOSTSHM project.
     */
    public const BPOSTSHM_GITHUB_DIRECTORY = 'Stigmi/Bpostshm';

    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * @var \Context
     */
    private $context;

    /**
     * @var Db
     */
    private $database;

    /**
     * @var TranslatorInterface
     */
    private $translator;

    /**
     * SettingsBuilder constructor.
     *
     * @param \BpostShm $module
     */
    public function __construct(\BpostShm $module)
    {
        $this->module = $module;
        $this->context = $module->getContext();
        $this->database = $module->getDatabase();
        $this->translator = $module->getTranslator();
    }

    /**
     * Get the configuration fields and their default values.
     *
     * @param int|null $idShopGroup
     * @param int|null $idShop
     *
     * @return array the configuration fields and their default values
     */
    public function getConfigFieldsAndDefaultValues(?int $idShopGroup = null, ?int $idShop = null): array
    {
        $defaultSender = BpostTools::cleanStr(\Configuration::get('PS_SHOP_NAME', null, $idShopGroup, $idShop));
        $defaultAddr = \Configuration::get('PS_SHOP_ADDR1', null, $idShopGroup, $idShop);
        $defaultPostcode = \Configuration::get('PS_SHOP_CODE', null, $idShopGroup, $idShop);
        $defaultCity = \Configuration::get('PS_SHOP_CITY', null, $idShopGroup, $idShop);
        $country = new \Country(\Configuration::get('PS_SHOP_COUNTRY_ID', null, $idShopGroup, $idShop));
        $defaultCountry = \Validate::isLoadedObject($country) ? $country->iso_code : 'BE';
        $defaultEmail = \Configuration::get('PS_SHOP_EMAIL', null, $idShopGroup, $idShop);
        if (!\Validate::isEmail($defaultEmail)) {
            $defaultEmail = '';
        }
        $defaultPhone = BpostTools::cleanPhone(\Configuration::get('PS_SHOP_PHONE', null, $idShopGroup, $idShop));

        $defaultOrderStatesArray = [];
        $osExclude = [
            1, // Awaiting check payment
            6, // Canceled
            7, // Refunded
            10, // Awaiting bank wire payment
        ];
        foreach (\OrderState::getOrderStates((int) \Context::getContext()->language->id) as $state) {
            if (!in_array($state['id_order_state'], $osExclude)) {
                $defaultOrderStatesArray[] = $state['id_order_state'];
            }
        }

        return [
            'BPOSTSMH_ACCOUNT_ID' => ['default' => '', 'old' => 'BPOST_ACCOUNT_ID'],
            'BPOSTSHM_ACCOUNT_PASSPHRASE' => ['default' => '', 'old' => 'BPOST_ACCOUNT_PASSPHRASE'],
            'BPOSTSHM_STORE_DETAILS_SENDER' => ['default' => $defaultSender],
            'BPOSTSHM_STORE_DETAILS_ADDRESS' => ['default' => $defaultAddr],
            'BPOSTSHM_STORE_DETAILS_POSTCODE' => ['default' => $defaultPostcode],
            'BPOSTSHM_STORE_DETAILS_CITY' => ['default' => $defaultCity],
            'BPOSTSHM_STORE_DETAILS_COUNTRY' => ['default' => $defaultCountry],
            'BPOSTSHM_STORE_DETAILS_EMAIL' => ['default' => $defaultEmail],
            'BPOSTSHM_STORE_DETAILS_PHONE' => ['default' => $defaultPhone],

            'BPOSTSHM_ID_CARRIER_HOME' => ['default' => '', 'global' => true],
            'BPOSTSHM_ID_CARRIER_PICKUP' => ['default' => '', 'global' => true],
            'BPOSTSHM_ID_CARRIER_PARCEL_LOCKER' => ['default' => '', 'global' => true],

            'BPOSTSHM_PICKUP_SELECTOR_MAP_ENABLED' => ['default' => '1'],
            'BPOSTSHM_PICKUP_SELECTOR_GMAPS_API_KEY' => ['default' => '', 'old' => 'BPOST_GMAPS_API_KEY'],
            'BPOSTSHM_PICKUP_SELECTOR_DESIGN' => ['default' => 'modal'],

            'BPOSTSHM_DELIVERY_OPTIONS_HOME_BUSINESS' => ['default' => 'pro'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_ENABLED' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_FROM' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_COST' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE_FROM' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_SIGNATURE_FROM' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE_COST' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_SIGNATURE_COST' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_2ND_PRESENTATION_ENABLED' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_2ND_PRESENTATION_FROM' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_2ND_PRESENTATION_COST' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_SATURDAY_ENABLED' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_SATURDAY_FROM' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_SATURDAY_COST' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INSURANCE_ENABLED' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INSURANCE_FROM' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INSURANCE_COST' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_SATURDAY_ENABLED' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_SATURDAY_FROM' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_SATURDAY_COST' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_INSURANCE_ENABLED' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_INSURANCE_FROM' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_INSURANCE_COST' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_INTL_BUSINESS' => ['default' => 'business'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_INTL_INSURANCE_ENABLED' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_INTL_INSURANCE_FROM' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_INTL_INSURANCE_COST' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INTL_INSURANCE_ENABLED' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INTL_INSURANCE_FROM' => ['default' => '0'],
            'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INTL_INSURANCE_COST' => ['default' => '0'],

            'BPOSTSHM_DELIVERY_DATE_ENABLED' => ['default' => '0', 'old' => 'BPOST_DISPLAY_DELIVERY_DATE'],
            'BPOSTSHM_DELIVERY_DATE_CHOOSE' => ['default' => '0', 'old' => 'BPOST_CHOOSE_DELIVERY_DATE'],
            'BPOSTSHM_DELIVERY_DATE_NUM_DATES_SHOWN' => ['default' => '5', 'old' => 'BPOST_NUM_DATES_SHOWN'],
            'BPOSTSHM_DELIVERY_DATE_SHIP_DELAY_DAYS' => ['default' => '0', 'old' => 'BPOST_SHIP_DELAY_DAYS'],
            'BPOSTSHM_DELIVERY_DATE_CUTOFF_TIME' => ['default' => '1500', 'old' => 'BPOST_CUTOFF_TIME'],
            'BPOSTSHM_DELIVERY_DATE_HIDE_OOS' => ['default' => '0', 'old' => 'BPOST_HIDE_DATE_OOS'],

            'BPOSTSHM_INTL_OUTSIDE_EU_ENABLED' => ['default' => '0'],
            'BPOSTSHM_INTL_HS_CODE_TYPE' => ['default' => 'fixed'],
            'BPOSTSHM_INTL_HS_CODE_FIXED_VALUE' => ['default' => '1234'],
            'BPOSTSHM_INTL_HS_CODE_FEATURE_VALUE' => ['default' => ''],
            'BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_TYPE' => ['default' => 'fixed'],
            'BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_FIXED_VALUE' => ['default' => 'BE'],
            'BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_FEATURE_VALUE' => ['default' => ''],

            'BPOSTSHM_LABELS_USE_PS' => ['default' => '0', 'old' => 'BPOST_USE_PS_LABELS'],
            'BPOSTSHM_LABELS_ORDER_DISPLAY_DAYS' => ['default' => '15', 'old' => 'BPOST_ORDER_DISPLAY_DAYS'],
            'BPOSTSHM_LABELS_CLEAN_AFTER_DAYS' => ['default' => '0', 'global' => true],
            'BPOSTSHM_LABELS_ORDER_DISPLAY_STATES' => [
                'default' => json_encode($defaultOrderStatesArray),
                'multiple' => true,
            ],
            'BPOSTSHM_LABELS_TREATED_ORDER_STATE' => [
                'default' => '3' /* Processing in progress */,
                'old' => 'BPOST_TREATED_ORDER_STATE',
            ],
            'BPOSTSHM_LABELS_MARK_PRINTED_ORDER_AS_TREATED' => ['default' => '0', 'old' => 'BPOST_TREAT_PRINTED_ORDER'],
            'BPOSTSHM_LABELS_PDF_FORMAT' => ['default' => 'A4', 'old' => 'BPOST_LABEL_PDF_FORMAT'],
            'BPOSTSHM_LABELS_DEFAULT_PRODUCT_WEIGHT' => ['default' => '100' /* g */],
            'BPOSTSHM_LABELS_AUTO_RETOUR' => ['default' => '0', 'old' => 'BPOST_AUTO_RETOUR_LABEL'],
        ];
    }

    /**
     * Render the configuration form for the module.
     *
     * @param array $result the result of the form submission
     *
     * @return string the rendered form HTML
     */
    public function renderForm(array $result = []): string
    {
        $helper = new \HelperForm();

        $helper->module = $this->module;
        $context = \Context::getContext();
        $helper->default_form_language = $context->language->id;

        $helper->submit_action = 'submitBpostshmModule';
        $helper->currentIndex = $context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->module->name . '&tab_module=' .
            $this->module->tab . '&module_name=' . $this->module->name;
        $helper->token = \Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFormValues(),
            'languages' => $context->controller->getLanguages(),
            'id_language' => $context->language->id,
        ];

        $configForm = $this->getConfigForm();
        $msg = '';

        if (isset($result['warning'])) {
            $msg .= $this->module->displayWarning($result['warning']);
        }

        if (isset($result['error'])) {
            $msg .= $this->module->displayError($result['error']);
        }

        if (isset($result['success'])) {
            $msg .= $this->module->displayConfirmation(
                $this->translator->trans(
                    'Settings updated.',
                    [],
                    'Modules.Bpostshm.Admin'
                )
            );
        }

        if ($this->hasAnUpdateAvailable()) {
            $dir = SettingsFormHelper::BPOSTSHM_GITHUB_DIRECTORY;
            $configForm[0]['form']['description'] = $this->translator->trans(
                'There is a new version for bpost module.',
                [],
                'Modules.Bpostshm.Admin'
            ) .
            ' <a href="https://github.com/' . $dir . '/releases" target="_blank">' .
                $this->translator->trans('Click here, to read more information', [], 'Modules.Bpostshm.Admin') .
            '</a>';
        }

        return $msg . $helper->generateForm($configForm);
    }

    /**
     * Get the configuration form.
     *
     * @return array the configuration form
     */
    protected function getConfigForm(): array
    {
        $linkBusiness = $this->translator->trans(
            'https://bpost.freshdesk.com/solution/articles/4000102198-bpack-24h-pro-vs-bpack-24h-business',
            [],
            'Modules.Bpostshm.Admin'
        );
        $linkRetour = $this->translator->trans(
            'https://bpost.freshdesk.com/support/solutions/articles/4000033756',
            [],
            'Modules.Bpostshm.Admin'
        );

        /** @var PickupService $pickupService */
        $pickupService = $this->module->get('bpostshm.pickup_service');
        /** @var CarrierService $carrierService */
        $carrierService = $this->module->get('bpostshm.carrier_service');

        $currency = $this->context->currency->symbol;
        $currency .= ' ' . $this->translator->trans('(VAT excl.)', [], 'Modules.Bpostshm.Admin');

        $descGlobalConfig = \Shop::isFeatureActive() ?
            (' <b>' . $this->translator->trans('[ALL SHOPS]', [], 'Modules.Bpostshm.Admin') . '</b> ') : '';

        $enableLinkOption = 'https://bpost.freshdesk.com/support/solutions/articles/4000033755';

        $tabAccountSettings = $this->translator->trans('Account settings', [], 'Modules.Bpostshm.Admin');
        $tabCarrierSettings = $this->translator->trans('Carrier settings', [], 'Modules.Bpostshm.Admin');
        $tabPickupSettings = $this->translator->trans('Pick-up point settings', [], 'Modules.Bpostshm.Admin');
        $tabDeliveryOptions = $this->translator->trans('Delivery options', [], 'Modules.Bpostshm.Admin');
        $tabDeliverySettings = $this->translator->trans('Delivery settings', [], 'Modules.Bpostshm.Admin');
        $tabIntlSettings = $this->translator->trans('International settings', [], 'Modules.Bpostshm.Admin');
        $tabLabelSettings = $this->translator->trans('Label settings', [], 'Modules.Bpostshm.Admin');
        $tabDoc = $this->translator->trans('Doc', [], 'Modules.Bpostshm.Admin');

        $form = [
            'form' => [
                'legend' => [
                    'title' => '<img src="' . $this->module->getPathUri() . 'logo.png" alt="bpost" />' .
                        $this->translator->trans('bpost Shipping Manager', [], 'Modules.Bpostshm.Admin'),
                ],
                'tabs' => [
                    'account_settings' => $tabAccountSettings,
                    'carrier_settings' => $tabCarrierSettings,
                    'pickup_settings' => $tabPickupSettings,
                    'delivery_options' => $tabDeliveryOptions,
                    'delivery_settings' => $tabDeliverySettings,
                    'international_settings' => $tabIntlSettings,
                    'label_settings' => $tabLabelSettings,
                    'doc' => $tabDoc,
                ],
                //
                // Account Settings
                //
                'input' => [
                    [
                        'type' => 'free',
                        'tab' => 'account_settings',
                        'name' => 'important_notice',
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'account_settings',
                        'label' => $this->translator->trans('Account ID', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSMH_ACCOUNT_ID',
                        'desc' => $this->translator->trans(
                            'Your 6 digits bpost account ID used for the Shipping Manager.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'account_settings',
                        'label' => $this->translator->trans('Passphrase', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_ACCOUNT_PASSPHRASE',
                        'desc' => $this->translator->trans(
                            'The passphrase you entered in bpost Shipping Manager back-office application. This is not the password used to access bpost portal.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'account_settings',
                        'label' => $this->translator->trans('Sender name', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_STORE_DETAILS_SENDER',
                        'desc' => $this->translator->trans(
                            'Displayed on the labels, as the sender.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'account_settings',
                        'label' => $this->translator->trans('Address', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_STORE_DETAILS_ADDRESS',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'account_settings',
                        'label' => $this->translator->trans('City', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_STORE_DETAILS_CITY',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'account_settings',
                        'label' => $this->translator->trans('Postcode', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_STORE_DETAILS_POSTCODE',
                        'required' => true,
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'account_settings',
                        'label' => $this->translator->trans('Country', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_STORE_DETAILS_COUNTRY',
                        'options' => $this->getPrestaShopCountries(),
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'account_settings',
                        'label' => $this->translator->trans('Email', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_STORE_DETAILS_EMAIL',
                        'required' => false,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'account_settings',
                        'label' => $this->translator->trans('Phone', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_STORE_DETAILS_PHONE',
                        'desc' => $this->translator->trans('No dots or spaces.', [], 'Modules.Bpostshm.Admin'),
                        'required' => false,
                    ],
                    //
                    // Carrier Settings
                    //
                    [
                        'type' => 'select',
                        'tab' => 'carrier_settings',
                        'label' => $this->translator->trans('Carrier for Home Delivery', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_ID_CARRIER_HOME',
                        'options' => [
                            'query' => $carrierService->getCarrierOptions(),
                            'id' => 'id_carrier',
                            'name' => 'name',
                        ],
                        'required' => true,
                        'desc' => $descGlobalConfig,
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'carrier_settings',
                        'label' => $this->translator->trans('Carrier for Pick-up point', [], 'Modules.Bpostshm.Admin'),
                        'desc' => $descGlobalConfig . $this->translator->trans(
                            'Only for %countries%.',
                            [
                                '%countries%' => implode(
                                    ', ',
                                    $pickupService->getAvailableCountries()
                                ),
                            ],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_ID_CARRIER_PICKUP',
                        'options' => [
                            'query' => $carrierService->getCarrierOptions(),
                            'id' => 'id_carrier',
                            'name' => 'name',
                        ],
                        'required' => true,
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'carrier_settings',
                        'label' => $this->translator->trans('Carrier for parcel locker', [], 'Modules.Bpostshm.Admin'),
                        'desc' => $descGlobalConfig . $this->translator->trans(
                            'Only for %countries%.',
                            [
                                '%countries%' => implode(
                                    ', ',
                                    $pickupService->getAvailableCountries(true)
                                ),
                            ],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_ID_CARRIER_PARCEL_LOCKER',
                        'options' => [
                            'query' => $carrierService->getCarrierOptions(),
                            'id' => 'id_carrier',
                            'name' => 'name',
                        ],
                        'required' => true,
                    ],
                    [
                        'type' => 'hidden',
                        'tab' => 'carrier_settings',
                        'name' => 'BPOSTSHM_ID_CARRIER_IDS',
                    ],
                    //
                    // Pickup Settings
                    //
                    [
                        'type' => 'switch',
                        'tab' => 'pickup_settings',
                        'label' => $this->translator->trans('Enable map', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_PICKUP_SELECTOR_MAP_ENABLED',
                        'is_bool' => true,
                        'desc' => $this->translator->trans(
                            'Show map? Otherwise a list of PU is shown.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'pickup_settings',
                        'label' => $this->translator->trans('Google maps API key', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_PICKUP_SELECTOR_GMAPS_API_KEY',
                        'desc' => $this->translator->trans(
                            'If not set, OpenStreetMap is used.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'pickup_settings',
                        'label' => $this->translator->trans('Design', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_PICKUP_SELECTOR_DESIGN',
                        'options' => $this->getPickupSelectorDesigns(),
                        'required' => true,
                    ],
                    //
                    // Delivery Options
                    //
                    [
                        'type' => 'html',
                        'tab' => 'delivery_options',
                        'name' => 'home-delivery-belgium-title',
                        'html_content' => $this->getDeliveryOptionsTitleHtml(
                            $this->translator->trans(
                                'Home Delivery Belgium',
                                [],
                                'Modules.Bpostshm.Admin'
                            )
                        ),
                    ],
                    [
                        'type' => 'radio',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans(
                            'Product',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_BUSINESS',
                        'values' => $this->getProducts(false),
                        'hint' => $this->translator->trans(
                            'Which product should I use [1]MORE here[/1].',
                            [
                                '[1]' => '<a href="' . $linkBusiness . '" target="_blank">',
                                '[/1]' => '</a>',
                            ],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => true,
                    ],
                    [
                        'type' => 'switch',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('Saturday Delivery', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_ENABLED',
                        'is_bool' => true,
                        'hint' => $this->translator->trans(
                            'Allow delivery of your goods on Saturdays.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('as from', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_FROM',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Minimum purchase total required in order to trigger the option excluding taxes & shipping costs.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('with additional cost', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_COST',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Saturday Delivery with additional cost.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('Delivery option', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE',
                        'options' => $this->getInsuranceTypes(),
                        'hint' => $this->translator->trans(
                            'Signature: The delivery happens against signature by the receiver.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ) .
                            $this->translator->trans(
                                'Each package is equipped with a unique barcode, and thanks to our online tracking system, e-Tracker, you can know at any time where it is.',
                                [],
                                'Modules.Bpostshm.Admin'
                            ) . '<br/>' .
                            $this->translator->trans(
                                'Insurance: Insurance to insure your goods to a maximum of 500,00 euro.',
                                [],
                                'Modules.Bpostshm.Admin'
                            ),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('as from', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE_FROM',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Minimum purchase total required in order to trigger the option excluding taxes & shipping costs.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('as from', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_SIGNATURE_FROM',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Minimum purchase total required in order to trigger the option excluding taxes & shipping costs.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('with additional cost', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE_COST',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Additionnal cost for insurance.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('with additional cost', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_SIGNATURE_COST',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Additionnal cost for signature.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'switch',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('2nd Presentation', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_2ND_PRESENTATION_ENABLED',
                        'is_bool' => true,
                        'hint' => $this->translator->trans(
                            'This option automatically provides a second presentation at the receiver\'s address should he be absent upon first presentation.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('as from', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_2ND_PRESENTATION_FROM',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Minimum purchase total required in order to trigger the option excluding taxes & shipping costs.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('with additional cost', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_2ND_PRESENTATION_COST',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Additionnal cost for 2nd presentation.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'html',
                        'tab' => 'delivery_options',
                        'name' => 'pickup-belgium-title',
                        'html_content' => $this->getDeliveryOptionsTitleHtml(
                            $this->translator->trans(
                                'Pick-up point Belgium',
                                [],
                                'Modules.Bpostshm.Admin'
                            )
                        ),
                    ],
                    [
                        'type' => 'switch',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans(
                            'Saturday Delivery',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_SATURDAY_ENABLED',
                        'is_bool' => true,
                        'hint' => $this->translator->trans(
                            'Allow delivery of your goods on Saturdays.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('as from', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_SATURDAY_FROM',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Minimum purchase total required in order to trigger the option excluding taxes & shipping costs.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('with additional cost', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_SATURDAY_COST',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Saturday Delivery with additional cost.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'switch',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('Insurance', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INSURANCE_ENABLED',
                        'is_bool' => true,
                        'hint' => $this->translator->trans(
                            'Insurance: Insurance to insure your goods to a maximum of 500,00 euro.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('as from', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INSURANCE_FROM',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Minimum purchase total required in order to trigger the option excluding taxes & shipping costs.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('with additional cost', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INSURANCE_COST',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Additionnal cost for insurance.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'html',
                        'tab' => 'delivery_options',
                        'name' => 'parcel-locker-belgium-title',
                        'html_content' => $this->getDeliveryOptionsTitleHtml(
                            $this->translator->trans(
                                'Parcel locker Belgium',
                                [],
                                'Modules.Bpostshm.Admin'
                            )
                        ),
                    ],
                    [
                        'type' => 'switch',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans(
                            'Saturday Delivery',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_SATURDAY_ENABLED',
                        'is_bool' => true,
                        'hint' => $this->translator->trans(
                            'Allow delivery of your goods on Saturdays.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('as from', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_SATURDAY_FROM',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Minimum purchase total required in order to trigger the option excluding taxes & shipping costs.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('with additional cost', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_SATURDAY_COST',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Saturday Delivery with additional cost.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'switch',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('Insurance', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_INSURANCE_ENABLED',
                        'is_bool' => true,
                        'hint' => $this->translator->trans(
                            'Insurance: Insurance to insure your goods to a maximum of 500,00 euro.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('as from', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_INSURANCE_FROM',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Minimum purchase total required in order to trigger the option excluding taxes & shipping costs.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('with additional cost', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_INSURANCE_COST',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Additionnal cost for insurance.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'html',
                        'tab' => 'delivery_options',
                        'name' => 'home-delivery-intl-title',
                        'html_content' => $this->getDeliveryOptionsTitleHtml(
                            $this->translator->trans(
                                'Home Delivery International',
                                [],
                                'Modules.Bpostshm.Admin'
                            )
                        ),
                    ],
                    [
                        'type' => 'radio',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans(
                            'Product',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_INTL_BUSINESS',
                        'values' => $this->getProducts(true),
                        'hint' => $this->translator->trans(
                            'Choose international delivery option.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => true,
                    ],
                    [
                        'type' => 'switch',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('Insurance', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_INTL_INSURANCE_ENABLED',
                        'is_bool' => true,
                        'hint' => $this->translator->trans(
                            'Insurance: Insurance to insure your goods to a maximum of 500,00 euro.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('as from', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_INTL_INSURANCE_FROM',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Minimum purchase total required in order to trigger the option excluding taxes & shipping costs.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('with additional cost', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_INTL_INSURANCE_COST',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Additionnal cost for insurance.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'html',
                        'tab' => 'delivery_options',
                        'name' => 'post-point-intl-title',
                        'html_content' => $this->getDeliveryOptionsTitleHtml(
                            $this->translator->trans(
                                'Post Point International',
                                [],
                                'Modules.Bpostshm.Admin'
                            )
                        ),
                    ],
                    [
                        'type' => 'switch',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans(
                            'Insurance',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INTL_INSURANCE_ENABLED',
                        'is_bool' => true,
                        'hint' => $this->translator->trans(
                            'Insurance: Insurance to insure your goods to a maximum of 500,00 euro.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('as from', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INTL_INSURANCE_FROM',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Minimum purchase total required in order to trigger the option excluding taxes & shipping costs.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_options',
                        'label' => $this->translator->trans('with additional cost', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INTL_INSURANCE_COST',
                        'suffix' => $currency,
                        'hint' => $this->translator->trans(
                            'Additionnal cost for insurance.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'required' => false,
                    ],
                    //
                    // Delivery Settings
                    //
                    [
                        'type' => 'switch',
                        'tab' => 'delivery_settings',
                        'label' => $this->translator->trans('Display delivery date', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_DATE_ENABLED',
                        'is_bool' => true,
                        'desc' => $this->translator->trans(
                            'Option to display the expected delivery date to the client (Belgium only).',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'switch',
                        'tab' => 'delivery_settings',
                        'label' => $this->translator->trans('Choose delivery date', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_DELIVERY_DATE_CHOOSE',
                        'is_bool' => true,
                        'desc' => $this->translator->trans(
                            'Allow customers to choose their delivery date.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_settings',
                        'label' => $this->translator->trans(
                            'Number of delivery dates shown',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_DELIVERY_DATE_NUM_DATES_SHOWN',
                        'desc' => $this->translator->trans('Minimum 2, maximum 7.', [], 'Modules.Bpostshm.Admin'),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_settings',
                        'label' => $this->translator->trans(
                            'Days between order and shipment',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_DELIVERY_DATE_SHIP_DELAY_DAYS',
                        'desc' => $this->translator->trans(
                            'Default is 0 (next day delivery), maximum 8.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'delivery_settings',
                        'label' => $this->translator->trans(
                            'Next day delivery allowed until',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_DELIVERY_DATE_CUTOFF_TIME',
                        'desc' => $this->translator->trans('Format: hh:mm', [], 'Modules.Bpostshm.Admin'),
                    ],
                    [
                        'type' => 'switch',
                        'tab' => 'delivery_settings',
                        'label' => $this->translator->trans(
                            'Hide delivery date when out of stock',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_DELIVERY_DATE_HIDE_OOS',
                        'is_bool' => true,
                        'desc' => $this->translator->trans(
                            'Do not display delivery date when at least one product in the cart, is out of stock.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'values' => $this->getSwitchValues(),
                    ],
                    //
                    // International Settings
                    //
                    [
                        'col' => '12',
                        'label' => '',
                        'type' => 'html',
                        'tab' => 'international_settings',
                        'name' => 'intl',
                        'html_content' => $this->getIntlHtmlContent(),
                    ],
                    [
                        'type' => 'switch',
                        'tab' => 'international_settings',
                        'label' => $this->translator->trans('Shipments outside EU', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_INTL_OUTSIDE_EU_ENABLED',
                        'is_bool' => true,
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'international_settings',
                        'label' => $this->translator->trans('HS Code', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_INTL_HS_CODE_TYPE',
                        'options' => $this->getHsCodeTypes(),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'international_settings',
                        'label' => $this->translator->trans('Apply to all products', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_INTL_HS_CODE_FIXED_VALUE',
                        'desc' => $this->translator->trans(
                            'HS Code value (min 4 digits, max 9 digits)',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'international_settings',
                        'label' => $this->translator->trans(
                            'Use the following product feature',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_INTL_HS_CODE_FEATURE_VALUE',
                        'options' => $this->getPrestaShopFeatures(),
                        'desc' => $this->translator->trans('HS Code value', [], 'Modules.Bpostshm.Admin'),
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'international_settings',
                        'label' => $this->translator->trans('Country of Origin', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_TYPE',
                        'options' => $this->getCountryOfOriginTypes(),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'international_settings',
                        'label' => $this->translator->trans('Apply to all products', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_FIXED_VALUE',
                        'desc' => $this->translator->trans(
                            'Country of origin value (iso code of country)',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'international_settings',
                        'label' => $this->translator->trans(
                            'Use the following product feature',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_FEATURE_VALUE',
                        'options' => $this->getPrestaShopFeatures(),
                        'desc' => $this->translator->trans('Country of origin value', [], 'Modules.Bpostshm.Admin'),
                    ],
                    //
                    // Label Settings
                    //
                    [
                        'type' => 'switch',
                        'tab' => 'label_settings',
                        'label' => $this->translator->trans(
                            'Use PrestaShop to manage labels',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_LABELS_USE_PS',
                        'is_bool' => true,
                        'desc' => $this->translator->trans(
                            'If you enable this option, labels are generated directly within PrestaShop. It is not needed to use the bpost Shipping Manager for these tasks.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ) . '<br/>' .
                            $this->translator->trans(
                                'Pop-ups must be enabled in your browser, in order to view the printed labels.',
                                [],
                                'Modules.Bpostshm.Admin'
                            ) . '<br/>' .
                            $this->translator->trans(
                                '[1]Click here[/1] to learn more about this option.',
                                [
                                    '[1]' => '<a href="' . $enableLinkOption . '">',
                                    '[/1]' => '</a>',
                                ],
                                'Modules.Bpostshm.Admin'
                            ),
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'free',
                        'tab' => 'label_settings',
                        'name' => 'cron',
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'label_settings',
                        'label' => $this->translator->trans('Days to display orders', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_LABELS_ORDER_DISPLAY_DAYS',
                        'desc' => $this->translator->trans(
                            'Set to 0 to display all orders.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'suffix' => $this->translator->trans('days', [], 'Modules.Bpostshm.Admin'),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'label_settings',
                        'label' => $this->translator->trans(
                            'Delete labels dating back more than',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'suffix' => $this->translator->trans('days', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_LABELS_CLEAN_AFTER_DAYS',
                        'desc' => $descGlobalConfig . $this->translator->trans(
                            'Set to 0 to never delete the labels.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'label_settings',
                        'multiple' => true,
                        'class' => 'chosen',
                        'label' => $this->translator->trans(
                            'Filter display by order states',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_LABELS_ORDER_DISPLAY_STATES',
                        'options' => $this->getPrestaShopOrderStatuses(),
                        'desc' => $this->translator->trans(
                            'Leave empty to display all bpost orders.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                    ],
                    [
                        'type' => 'select',
                        'tab' => 'label_settings',
                        'label' => $this->translator->trans('Treated order state', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_LABELS_TREATED_ORDER_STATE',
                        'options' => $this->getPrestaShopOrderStatuses(),
                        'desc' => $this->translator->trans(
                            'When you "Process" a bpost order, you can choose which PrestaShop status it will take.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                    ],
                    [
                        'type' => 'switch',
                        'tab' => 'label_settings',
                        'label' => $this->translator->trans(
                            'Mark printed order as treated',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'name' => 'BPOSTSHM_LABELS_MARK_PRINTED_ORDER_AS_TREATED',
                        'is_bool' => true,
                        'desc' => $this->translator->trans(
                            'Printed orders are automatically marked as processed.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                        'values' => $this->getSwitchValues(),
                    ],
                    [
                        'type' => 'radio',
                        'tab' => 'label_settings',
                        'label' => $this->translator->trans('Label format', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_LABELS_PDF_FORMAT',
                        'values' => $this->getPdfFormats(),
                    ],
                    [
                        'type' => 'text',
                        'tab' => 'label_settings',
                        'label' => $this->translator->trans('Default product weight', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_LABELS_DEFAULT_PRODUCT_WEIGHT',
                        'suffix' => $this->translator->trans('g', [], 'Modules.Bpostshm.Admin'),
                        'desc' => $this->translator->trans(
                            'When a product weight is less or equal 0.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ),
                    ],
                    [
                        'type' => 'switch',
                        'tab' => 'label_settings',
                        'label' => $this->translator->trans('Retour label', [], 'Modules.Bpostshm.Admin'),
                        'name' => 'BPOSTSHM_LABELS_AUTO_RETOUR',
                        'is_bool' => true,
                        'desc' => $this->translator->trans(
                            'If you enable this option, a retour label is automatically added and printed when generating labels.',
                            [],
                            'Modules.Bpostshm.Admin'
                        ) . '<br/>' .
                            $this->translator->trans(
                                'If disabled, you are able to manually create retour labels.',
                                [],
                                'Modules.Bpostshm.Admin'
                            ) . '<br/>' .
                            $this->translator->trans(
                                '[1]Click here[/1] to learn more about this option.',
                                [
                                    '[1]' => '<a href="' . $linkRetour . '" target="_blank">',
                                    '[/1]' => '</a>',
                                ],
                                'Modules.Bpostshm.Admin'
                            ),
                        'values' => $this->getSwitchValues(),
                    ],
                    //
                    // Doc
                    //
                    [
                        'col' => '12',
                        'label' => '',
                        'name' => 'doc',
                        'tab' => 'doc',
                        'type' => 'html',
                        'html_content' => $this->getDocHtmlContent(),
                    ],
                ],
                'submit' => [
                    'title' => $this->translator->trans('Save', [], 'Modules.Bpostshm.Admin'),
                ],
            ],
        ];

        return [$form];
    }

    /**
     * Get the HTML content of the documentation template file.
     *
     * @return string the HTML content of the documentation template file
     */
    public function getDocHtmlContent(): string
    {
        return \Context::getContext()->smarty->fetch(
            'module:bpostshm/views/templates/admin/settings/doc.tpl'
        );
    }

    /**
     * Get the HTML content for the international settings.
     *
     * @return string the HTML content
     */
    public function getIntlHtmlContent(): string
    {
        $intlList = [];
        $ppiList = [];

        $hasCredentials = false;
        if (\Configuration::get('BPOSTSMH_ACCOUNT_ID') && \Configuration::get('BPOSTSHM_ACCOUNT_PASSPHRASE')) {
            $hasCredentials = true;
            $config = \Configuration::getGlobalValue('BPOSTSHM_INTL_COUNTRIES');
            if (SettingsFormHelper::isValidJSON($config)) {
                $data = json_decode($config, true);
                $countries = [];
                $idLang = (int) \Context::getContext()->language->id;
                foreach (\Country::getCountries($idLang, false, false, false) as $country) {
                    $countries[$country['iso_code']] = $country;
                }
                if (isset($data['intl']) && is_array($data['intl'])) {
                    foreach ($data['intl'] as $country) {
                        if (isset($countries[$country])) {
                            $intlList[$country] = $countries[$country]['name'];
                        }
                    }
                }
                if (isset($data['ppi']) && is_array($data['ppi'])) {
                    foreach ($data['ppi'] as $country) {
                        if (isset($countries[$country])) {
                            $ppiList[$country] = $countries[$country]['name'];
                        }
                    }
                }
            }
        }

        sort($intlList);
        sort($ppiList);
        $linkIntl = 'https://bpost.freshdesk.com/support/solutions/articles/4000068592'
                    . '--how-to-configure-country-for-international-delivery-for-the-frontend-';
        $linkManageFeatures = 'https://docs.prestashop-project.org/v.8-documentation/user-guide/'
                                . 'selling/managing-catalog/managing-product-features';
        \Context::getContext()->smarty->assign([
            'hasCredentials' => $hasCredentials,
            'intlList' => $intlList,
            'ppiList' => $ppiList,
            'linkIntlDelivery' => $this->translator->trans($linkIntl, [], 'Modules.Bpostshm.Admin'),
            'linkFeatures' => $this->translator->trans($linkManageFeatures, [], 'Modules.Bpostshm.Admin'),
        ]);

        return \Context::getContext()->smarty->fetch(
            'module:bpostshm/views/templates/admin/settings/intl.tpl'
        );
    }

    /**
     * Get the HTML content for the title in delivery options section.
     *
     * @return string the HTML content
     */
    public function getDeliveryOptionsTitleHtml($title): string
    {
        return '<h4>' . $title . '</h4>';
    }

    /**
     * Process the form data and update configuration.
     *
     * @return array the result of the process, with 'success' set to true if it was successful,
     *               and 'error' set to an error message if there was an error
     *
     * @throws CarrierException
     */
    public function postProcess(): array
    {
        $result = ['success' => true];
        $errors = [];
        $mandatoryFields = [
            'BPOSTSMH_ACCOUNT_ID' => $this->translator->trans(
                'The "Account ID" field is required.',
                [],
                'Modules.Bpostshm.Admin'
            ),
            'BPOSTSHM_ACCOUNT_PASSPHRASE' => $this->translator->trans(
                'The "Passphrase" field is required.',
                [],
                'Modules.Bpostshm.Admin'
            ),
            'BPOSTSHM_STORE_DETAILS_SENDER' => $this->translator->trans(
                'The "Sender" field is required.',
                [],
                'Modules.Bpostshm.Admin'
            ),
            'BPOSTSHM_STORE_DETAILS_ADDRESS' => $this->translator->trans(
                'The "Address" field is required.',
                [],
                'Modules.Bpostshm.Admin'
            ),
            'BPOSTSHM_STORE_DETAILS_CITY' => $this->translator->trans(
                'The "City" field is required.',
                [],
                'Modules.Bpostshm.Admin'
            ),
            'BPOSTSHM_STORE_DETAILS_POSTCODE' => $this->translator->trans(
                'The "Postcode" field is required.',
                [],
                'Modules.Bpostshm.Admin'
            ),
            'BPOSTSHM_STORE_DETAILS_COUNTRY' => $this->translator->trans(
                'The "Country" field is required.',
                [],
                'Modules.Bpostshm.Admin'
            ),
        ];

        $formValues = $this->getConfigFormValues();
        foreach ($formValues as $key => $value) {
            $value = \Tools::getValue($key);
            if (isset($mandatoryFields[$key]) && strlen(trim($value)) <= 0) {
                $errors[] = $mandatoryFields[$key];
            }
            if (\Tools::getValue('BPOSTSHM_DELIVERY_DATE_ENABLED')) {
                if (\Tools::getValue('BPOSTSHM_DELIVERY_DATE_CHOOSE')) {
                    if ($key == 'BPOSTSHM_DELIVERY_DATE_NUM_DATES_SHOWN' && ((int) $value < 2 || (int) $value > 7)) {
                        $value = (int) $value;
                        $errors[] = $this->translator->trans(
                            'Number of delivery dates shown: Please choose a value between %min% and %max%.',
                            ['%min%' => 2, '%max%' => 7],
                            'Modules.Bpostshm.Admin'
                        );
                    }
                }
                if ($key == 'BPOSTSHM_DELIVERY_DATE_SHIP_DELAY_DAYS' && ((int) $value < 0 || (int) $value > 8)) {
                    $value = (int) $value;
                    $errors[] = $this->translator->trans(
                        'Days between order and shipment: Please choose a value between %min% and %max%.',
                        ['%min%' => 0, '%max%' => 8],
                        'Modules.Bpostshm.Admin'
                    );
                }
                if ($key == 'BPOSTSHM_DELIVERY_DATE_CUTOFF_TIME' && !BpostTools::isTime($value)) {
                    $errors[] = $this->translator->trans(
                        'Next day delivery allowed until: Bad time format.',
                        [],
                        'Modules.Bpostshm.Admin'
                    );
                }
            }
            if (\Tools::getValue('BPOSTSHM_INTL_OUTSIDE_EU_ENABLED')) {
                if (\Tools::getValue('BPOSTSHM_INTL_HS_CODE_TYPE') == 'fixed') {
                    if ($key == 'BPOSTSHM_INTL_HS_CODE_FIXED_VALUE' && !BpostTools::isHsCode($value)) {
                        $errors[] = $this->translator->trans(
                            'Bad HS code format.',
                            [],
                            'Modules.Bpostshm.Admin'
                        );
                    }
                }
                if (\Tools::getValue('BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_TYPE') == 'fixed') {
                    if ($key == 'BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_FIXED_VALUE' && !BpostTools::isOriginCountry($value)) {
                        $errors[] = $this->translator->trans(
                            'Bad Origin Of Country format.',
                            [],
                            'Modules.Bpostshm.Admin'
                        );
                    }
                }
            }
        }

        $carriers = [];

        if ($formValues['BPOSTSHM_ID_CARRIER_IDS'] != \Tools::getValue('BPOSTSHM_ID_CARRIER_IDS')) {
            $errors[] = $this->translator->trans(
                'Carrier config was no more up to date. Please re-try.',
                [],
                'Modules.Bpostshm.Admin'
            );
        } else {
            $carriers = [
                'BPOSTSHM_ID_CARRIER_HOME' => (int) \Tools::getValue('BPOSTSHM_ID_CARRIER_HOME'),
                'BPOSTSHM_ID_CARRIER_PICKUP' => (int) \Tools::getValue('BPOSTSHM_ID_CARRIER_PICKUP'),
                'BPOSTSHM_ID_CARRIER_PARCEL_LOCKER' => (int) \Tools::getValue('BPOSTSHM_ID_CARRIER_PARCEL_LOCKER'),
            ];
            if ((
                $carriers['BPOSTSHM_ID_CARRIER_HOME'] && $carriers['BPOSTSHM_ID_CARRIER_PICKUP']
                    && $carriers['BPOSTSHM_ID_CARRIER_HOME'] == $carriers['BPOSTSHM_ID_CARRIER_PICKUP']
            ) || (
                $carriers['BPOSTSHM_ID_CARRIER_PICKUP'] && $carriers['BPOSTSHM_ID_CARRIER_PARCEL_LOCKER']
                && $carriers['BPOSTSHM_ID_CARRIER_PICKUP'] == $carriers['BPOSTSHM_ID_CARRIER_PARCEL_LOCKER']
            ) || (
                $carriers['BPOSTSHM_ID_CARRIER_PARCEL_LOCKER'] && $carriers['BPOSTSHM_ID_CARRIER_HOME']
                && $carriers['BPOSTSHM_ID_CARRIER_PARCEL_LOCKER'] == $carriers['BPOSTSHM_ID_CARRIER_HOME']
            )) {
                $errors[] = $this->translator->trans(
                    'You must choose a different carrier for each delivery method.',
                    [],
                    'Modules.Bpostshm.Admin'
                );
            } else {
                try {
                    /** @var CarrierService $carrierService */
                    $carrierService = $this->module->get('bpostshm.carrier_service');
                    $carrierService->updateCarrierConfig($carriers);
                } catch (\PrestaShopException $e) {
                    $errors[] = $e->getMessage();
                }
            }
        }

        if (count($errors)) {
            unset($result['success']);
            $result['error'] = implode('<br>', $errors);
        } else {
            $applyBusinessConfig = false;
            $applyLabelUsePSConfig = false;
            $exceptions = array_keys($carriers);
            $formDefaultValues = SettingsFormHelper::getConfigFieldsAndDefaultValues();
            foreach ($formValues as $key => $value) {
                // Because the PrestaShops form helper adds these brackets to a select field, with multiple set as true.
                // We have to manually remove them to get the correct value
                $key = trim($key, '[]');

                if (in_array($key, $exceptions)) {
                    continue;
                }

                $newValue = \Tools::getValue($key);
                if ($key == 'BPOSTSHM_DELIVERY_DATE_CUTOFF_TIME') {
                    $newValue = BpostTools::cleanTime($newValue);
                }
                if ($key == 'BPOSTSHM_LABELS_CLEAN_AFTER_DAYS' || $key == 'BPOSTSHM_LABELS_USE_PS') {
                    $newValue = (int) $newValue;
                    if ($key == 'BPOSTSHM_LABELS_USE_PS') {
                        $applyLabelUsePSConfig = (int) $value != $newValue;
                    }
                }
                if ($key == 'BPOSTSHM_DELIVERY_OPTIONS_HOME_BUSINESS' && $newValue == 'business') {
                    $applyBusinessConfig = true;
                }

                $isGlobal = isset($formDefaultValues[$key]['global']) && $formDefaultValues[$key]['global'];
                if (is_array($value)) {
                    if (\Tools::getValue($key) === false) {
                        if ($isGlobal) {
                            \Configuration::updateGlobalValue($key, '');
                        } else {
                            \Configuration::updateValue($key, '');
                        }
                    } else {
                        if ($isGlobal) {
                            \Configuration::updateGlobalValue($key, json_encode($newValue));
                        } else {
                            \Configuration::updateValue($key, json_encode($newValue));
                        }
                    }
                } else {
                    if ($isGlobal) {
                        \Configuration::updateGlobalValue($key, $newValue);
                    } else {
                        \Configuration::updateValue($key, $newValue);
                    }
                }
            }
            if ($applyBusinessConfig) {
                $this->applyBusinessOptions();
            }
            if ($applyLabelUsePSConfig) {
                $this->applyLabelUsePSOptions();
            }
        }

        \Configuration::deleteByName(DeliveryDateService::CACHED_CONFIG_KEY);

        return $result;
    }

    /**
     * When business option is enabled, several options are automatically enabled
     *
     * @param null $idShopGroup
     * @param null $idShop
     *
     * @return void
     */
    public function applyBusinessOptions($idShopGroup = null, $idShop = null)
    {
        \Configuration::updateValue(
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE',
            'insurance',
            false,
            $idShopGroup,
            $idShop
        );
        \Configuration::updateValue(
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE_FROM',
            '0',
            false,
            $idShopGroup,
            $idShop
        );
        \Configuration::updateValue(
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_2ND_PRESENTATION_ENABLED',
            '1',
            false,
            $idShopGroup,
            $idShop
        );
        \Configuration::updateValue(
            'BPOSTSHM_DELIVERY_OPTIONS_HOME_2ND_PRESENTATION_FROM',
            '0',
            false,
            $idShopGroup,
            $idShop
        );
    }

    /**
     * When "Use PS for manage labels" option is enabled, we enable bpost Shipping Manager tab.
     *
     * @return void
     *
     * @throws \PrestaShopException
     * @throws \PrestaShopDatabaseException
     */
    public function applyLabelUsePSOptions()
    {
        $enabled = (bool) \Configuration::getGlobalValue('BPOSTSHM_LABELS_USE_PS');
        if (\Shop::isFeatureActive()) {
            foreach (\Shop::getShops(true) as $shop) {
                $enabled |= (bool) \Configuration::get(
                    'BPOSTSHM_LABELS_USE_PS',
                    null,
                    $shop['id_shop_group'],
                    $shop['id_shop']
                );
            }
        }
        /** @var TabRepository $tabRepository */
        $tabRepository = $this->module->get('prestashop.core.admin.tab.repository');
        if ($idTab = $tabRepository->findOneIdByClassName('AdminParentBpostshm')) {
            $tab = new \Tab($idTab);
            $tab->enabled = $enabled;
            $tab->active = $enabled;
            $tab->update();
        }
    }

    /**
     * Get the switch values options.
     *
     * @return array the switch values options
     */
    protected function getSwitchValues(): array
    {
        return [
            [
                'id' => 'bpostshm_value_active_on',
                'value' => true,
                'label' => $this->translator->trans('Enabled', [], 'Modules.Bpostshm.Admin'),
            ],
            [
                'id' => 'bpostshm_value_active_off',
                'value' => false,
                'label' => $this->translator->trans('Disabled', [], 'Modules.Bpostshm.Admin'),
            ],
        ];
    }

    /**
     * Get the PrestaShop countries options.
     *
     * @return array the PrestaShop countries options
     */
    protected function getPrestaShopCountries(): array
    {
        $countries = \Country::getCountries((int) \Context::getContext()->language->id, true);
        $options = [];
        foreach ($countries as $country) {
            $options['query'][] = [
                'id' => $country['iso_code'],
                'name' => $country['name'],
                'label' => $country['name'],
            ];
        }
        $options['id'] = 'id';
        $options['name'] = 'name';

        return $options;
    }

    /**
     * Get the PrestaShop order statuses options.
     *
     * @return array the PrestaShop order statuses options
     */
    protected function getPrestaShopOrderStatuses(): array
    {
        $prestaShopOrderStatuses = \OrderState::getOrderStates(\Context::getContext()->language->id);
        $prestaShopOrderStatusesOptions = [];
        $prestaShopOrderStatusesOptions['query'][] = [
            'id' => '',
            'name' => '',
            'label' => '',
        ];
        foreach ($prestaShopOrderStatuses as $prestaShopOrderStatus) {
            $prestaShopOrderStatusesOptions['query'][] = [
                'id' => $prestaShopOrderStatus['id_order_state'],
                'name' => $prestaShopOrderStatus['name'],
                'label' => $prestaShopOrderStatus['name'],
            ];
        }
        $prestaShopOrderStatusesOptions['id'] = 'id';
        $prestaShopOrderStatusesOptions['name'] = 'name';

        return $prestaShopOrderStatusesOptions;
    }

    /**
     * Get the PrestaShop features options.
     *
     * @return array the PrestaShop features options
     */
    protected function getPrestaShopFeatures(): array
    {
        $prestaShopFeatures = \Feature::getFeatures(\Context::getContext()->language->id);
        $prestaShopFeaturesOptions = [];
        $prestaShopFeaturesOptions['query'][] = [
            'id' => '',
            'name' => '',
            'label' => '',
        ];
        foreach ($prestaShopFeatures as $prestaShopFeature) {
            $prestaShopFeaturesOptions['query'][] = [
                'id' => $prestaShopFeature['id_feature'],
                'name' => $prestaShopFeature['name'],
                'label' => $prestaShopFeature['name'],
            ];
        }
        $prestaShopFeaturesOptions['id'] = 'id';
        $prestaShopFeaturesOptions['name'] = 'name';

        return $prestaShopFeaturesOptions;
    }

    /**
     * Get the pickup selector designs options.
     *
     * @return array the pickup selector designs options
     */
    protected function getPickupSelectorDesigns(): array
    {
        $options = [
            ['id' => 'inline', 'name' => $this->translator->trans('Inline', [], 'Modules.Bpostshm.Admin')],
            ['id' => 'modal', 'name' => $this->translator->trans('Open modal', [], 'Modules.Bpostshm.Admin')],
        ];
        foreach ($options as $option) {
            $options['query'][] = [
                'id' => $option['id'],
                'name' => $option['name'],
                'label' => $option['name'],
            ];
        }
        $options['id'] = 'id';
        $options['name'] = 'name';

        return $options;
    }

    /**
     * Get the products options.
     *
     * @param bool $intl a boolean indicating whether the international option is selected
     *
     * @return array the products options
     */
    protected function getProducts(bool $intl): array
    {
        $pro = $intl ? $this->translator->trans(
            'World Express Pro',
            [],
            'Modules.Bpostshm.Admin'
        ) : $this->translator->trans(
            '24H Pro',
            [],
            'Modules.Bpostshm.Admin'
        );
        $business = $intl ? $this->translator->trans(
            'World Business',
            [],
            'Modules.Bpostshm.Admin'
        ) : $this->translator->trans(
            '24H Business',
            [],
            'Modules.Bpostshm.Admin'
        );

        return [
            [
                'id' => 'bpostshm_value_' . ($intl ? 'intl_' : '') . 'pro',
                'value' => 'pro',
                'label' => $pro,
            ],
            [
                'id' => 'bpostshm_value_' . ($intl ? 'intl_' : '') . 'business',
                'value' => 'business',
                'label' => $business,
            ],
        ];
    }

    /**
     * Get the insurance types options.
     *
     * @return array the insurance types options
     */
    protected function getInsuranceTypes(): array
    {
        $options = [
            ['id' => '', 'name' => ''],
            ['id' => 'signature', 'name' => $this->translator->trans('Signature', [], 'Modules.Bpostshm.Admin')],
            ['id' => 'insurance', 'name' => $this->translator->trans('Insurance', [], 'Modules.Bpostshm.Admin')],
        ];
        foreach ($options as $option) {
            $options['query'][] = [
                'id' => $option['id'],
                'name' => $option['name'],
                'label' => $option['name'],
            ];
        }
        $options['id'] = 'id';
        $options['name'] = 'name';

        return $options;
    }

    /**
     * Get the HS Code types options.
     *
     * @return array the HS Code types options
     */
    protected function getHsCodeTypes(): array
    {
        $options = [
            [
                'id' => 'fixed',
                'name' => $this->translator->trans('Apply to all products', [], 'Modules.Bpostshm.Admin'),
            ],
            [
                'id' => 'feature',
                'name' => $this->translator->trans('Use the following product feature', [], 'Modules.Bpostshm.Admin'),
            ],
        ];
        foreach ($options as $option) {
            $options['query'][] = [
                'id' => $option['id'],
                'name' => $option['name'],
                'label' => $option['name'],
            ];
        }
        $options['id'] = 'id';
        $options['name'] = 'name';

        return $options;
    }

    /**
     * Retrieves the types of country of origin available.
     *
     * This method internally calls the getHsCodeTypes method to retrieve the country of origin types.
     *
     * @return array an array containing the types of country of origin
     */
    protected function getCountryOfOriginTypes(): array
    {
        return $this->getHsCodeTypes();
    }

    /**
     * Returns an array of PDF formats.
     *
     * @return array an array of PDF formats, each format represented as an associative array
     */
    protected function getPdfFormats(): array
    {
        return [
            [
                'id' => 'bpostshm_value_A4',
                'value' => 'A4',
                'label' => $this->translator->trans('Default format A4 (PDF)', [], 'Modules.Bpostshm.Admin'),
            ],
            [
                'id' => 'bpostshm_value_A6',
                'value' => 'A6',
                'label' => $this->translator->trans('Default format A6 (PDF)', [], 'Modules.Bpostshm.Admin'),
            ],
        ];
    }

    /**
     * Set values for the inputs.
     *
     * @return array
     */
    public function getConfigFormValues(): array
    {
        $configFormValues = [];
        foreach (SettingsFormHelper::getConfigFieldsAndDefaultValues() as $configKey => $configSettings) {
            $configValue = isset($configSettings['global']) && $configSettings['global'] ?
                \Configuration::getGlobalValue($configKey) : \Configuration::get($configKey);
            if (!isset($configSettings['multiple']) || !$configSettings['multiple']) {
                if ($configKey == 'BPOSTSHM_DELIVERY_DATE_CUTOFF_TIME' && strlen($configValue) === 4) {
                    $configValue = substr($configValue, 0, 2) . ':'
                                    . substr($configValue, 2, 2);
                }
                $configFormValues[$configKey] = $configValue;
            } else {
                $configFormValues[$configKey . '[]'] = $this->settingToArray($configValue);
            }
        }
        $configFormValues['BPOSTSHM_ID_CARRIER_IDS'] = implode('-', [
            (int) \Configuration::getGlobalValue('BPOSTSHM_ID_CARRIER_HOME'),
            (int) \Configuration::getGlobalValue('BPOSTSHM_ID_CARRIER_PICKUP'),
            (int) \Configuration::getGlobalValue('BPOSTSHM_ID_CARRIER_PARCEL_LOCKER'),
        ]);

        $tpl = 'views/templates/admin/settings/important_notice.tpl';
        $configFormValues['important_notice'] = $this->module->render($tpl);

        $cron = '';
        $cronjobs = \Module::getInstanceByName('cronjobs');
        if (!$cronjobs || !$cronjobs->active) {
            $cron = $this->module->render('views/templates/admin/settings/cron.tpl', [
                'cron_url' => \Context::getContext()->link->getModuleLink(
                    $this->module->name,
                    'cron',
                    ['secure_key' => \Configuration::getGlobalValue('BPOSTSHM_CRON_SECURE_KEY')]
                ),
            ]);
        }
        $configFormValues['cron'] = $cron;

        return $configFormValues;
    }

    /**
     * Check if an update is available for the module.
     *
     * @return bool returns true if an update is available, false otherwise
     */
    protected function hasAnUpdateAvailable(): bool
    {
        $options = [
            'http' => [
                'method' => 'GET',
                'header' => "Accept-language: en\r\n" .
                    "User-Agent: PHP\r\n",
            ],
        ];
        $context = stream_context_create($options);
        $content = \Tools::file_get_contents(
            '
            https://api.github.com/repos/' . SettingsFormHelper::BPOSTSHM_GITHUB_DIRECTORY . '/releases/latest',
            false,
            $context
        );
        if ($content) {
            $information = json_decode($content);
            if (version_compare($information->tag_name, $this->module->version, '>')) {
                return true;
            }
        }

        return false;
    }

    /**
     * Convert a setting value to an array.
     *
     * @param mixed $setting the setting value to be converted
     *
     * @return array the converted setting value as an array
     */
    protected function settingToArray($setting): array
    {
        if (is_string($setting) && !empty($setting)) {
            return json_decode($setting);
        }

        return [];
    }

    /**
     * Validates if a given string is a valid JSON.
     *
     * @param string $json the JSON string to validate
     *
     * @return bool true if the given string is a valid JSON, false otherwise
     */
    public static function isValidJSON(string $json): bool
    {
        $json = trim($json);
        $valid = !empty($json) && !is_numeric($json);
        if ($valid = $valid && in_array(\Tools::substr($json, 0, 1), ['{', '['])) {
            json_decode($json);
            $valid = JSON_ERROR_NONE === json_last_error();
        }

        return $valid;
    }
}
