<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Hook;

use Order;
use PrestaShop\Module\BpostShm\Service\LabelService;
use PrestaShop\Module\BpostShm\Service\OrderService;
use PrestaShop\Module\BpostShm\Service\PickupService;
use PrestaShop\Module\BpostShm\Utils\BpostTools;

/**
 * Class AdminOrderTab
 */
class AdminOrderTab extends AbstractHook
{
    /**
     * @inerhitDoc
     */
    const AVAILABLE_HOOKS = [
        'displayAdminOrderTabLink',
        'displayAdminOrderTabContent',
    ];

    /**
     * Display the admin order tab link.
     *
     * @param array $params The parameters for displaying the order tab link.
     *                      - id_order (int): The ID of the order.
     *
     * @return string The HTML representation of the admin order tab link.
     *                Returns an empty string if the order is not a Bpost order.
     */
    public function displayAdminOrderTabLink(array $params): string
    {
        $order = new \Order($params['id_order']);
        if (BpostTools::isBpostOrder($order)) {
            $smartyParams = [
                'logo_url' => $this->module->getPathUri() . 'logo.png',
            ];

            /** @var OrderService $orderService */
            $orderService = $this->module->get('bpostshm.order_service');
            if ($orderService->hasMissingBpostOrder($params['id_order'])) {
                return $this->displayAdminOrderTabLinkLabels($smartyParams);
            }

            if (!\Configuration::get('BPOSTSHM_LABELS_USE_PS', null, $order->id_shop_group, $order->id_shop)) {
                return '';
            }

            /** @var PickupService $pickupService */
            $pickupService = $this->module->get('bpostshm.pickup_service');
            $idPickup = $pickupService->getPickupIdByOrderId($order->id);
            $smartyParams['idPickup'] = $idPickup;

            $html = $idPickup ? $this->displayAdminOrderTabLinkPickup($smartyParams) : '';
            $html .= $this->displayAdminOrderTabLinkLabels($smartyParams);

            return $html;
        }

        return '';
    }

    /**
     * Display the admin order tab link labels.
     *
     * @param array $smartyParams The Smarty parameters for rendering the admin order tab labels.
     *                            - logo_url (string): The URL of the logo.
     *                            - idPickup (int|null): The ID of the pickup.
     *
     * @return string the HTML representation of the admin order tab link labels
     */
    protected function displayAdminOrderTabLinkLabels(array $smartyParams): string
    {
        return $this->module->render('views/templates/hook/admin-order-tab-label-link.tpl', $smartyParams);
    }

    /**
     * Display the admin order tab link for pickup.
     *
     * @param array $smartyParams The Smarty parameters for rendering the admin order tab link.
     *                            - logo_url (string): The URL of the logo image.
     *                            - idPickup (int): The ID of the pickup.
     *
     * @return string the HTML representation of the admin order tab link for pickup
     */
    protected function displayAdminOrderTabLinkPickup(array $smartyParams): string
    {
        return $this->module->render('views/templates/hook/admin-order-tab-pickup-link.tpl', $smartyParams);
    }

    /**
     * Display the admin order tab content.
     *
     * @param array $params The parameters for displaying the order tab content.
     *                      - id_order (int): The ID of the order.
     *
     * @return string The HTML representation of the admin order tab content.
     *                Returns an empty string if the order is not a Bpost order.
     */
    public function displayAdminOrderTabContent(array $params): string
    {
        $order = new \Order($params['id_order']);
        if (BpostTools::isBpostOrder($order)) {
            /** @var OrderService $orderService */
            $orderService = $this->module->get('bpostshm.order_service');
            if ($orderService->hasMissingBpostOrder($order->id)) {
                return $this->displayAdminOrderTabContentMissingOrder($order->id);
            }

            if (!\Configuration::get('BPOSTSHM_LABELS_USE_PS', null, $order->id_shop_group, $order->id_shop)) {
                return '';
            }

            /** @var PickupService $pickupService */
            $pickupService = $this->module->get('bpostshm.pickup_service');
            $idPickup = $pickupService->getPickupIdByOrderId($order->id);

            $html = $idPickup ? $this->displayAdminOrderTabContentPickup($order, $idPickup) : '';
            $html .= $this->displayAdminOrderTabContentLabels($order->id, $idPickup);

            return $html;
        }

        return '';
    }

    /**
     * Display the admin order tab content when missing bpost order.
     *
     * @param int $idOrder the ID of the order=
     *
     * @return string the rendered HTML content
     */
    protected function displayAdminOrderTabContentMissingOrder(int $idOrder): string
    {
        $urlPrepareMissingOrder = $this->context->link->getAdminLink(
            'AdminBpostshmOpenedOrder',
            true,
            [
                'route' => 'admin_bpostshm_prepare_missing_order',
                'orderId' => $idOrder,
            ]
        );

        $smartyParams = [
            'urlPrepareMissingOrder' => $urlPrepareMissingOrder,
        ];

        return $this->module->render('views/templates/hook/admin-order-tab-missing-order-content.tpl', $smartyParams);
    }

    /**
     * Display the admin order tab content for labels.
     *
     * @param int $idOrder the ID of the order
     * @param int $idPickup the ID of the pickup
     *
     * @return string the rendered HTML content
     */
    protected function displayAdminOrderTabContentLabels(int $idOrder, int $idPickup): string
    {
        $order = new \Order($idOrder);
        $urlDownloadLabel = $this->context->link->getAdminLink(
            'AdminBpostshmLabelOrder',
            true,
            [
                'route' => 'admin_bpostshm_label_download',
                'orderId' => $idOrder,
            ]
        );
        $urlDownloadLabelRetour = $this->context->link->getAdminLink(
            'AdminBpostshmLabelOrder',
            true,
            [
                'route' => 'admin_bpostshm_label_generate',
                'orderId' => $idOrder,
                'retour' => true,
            ]
        );
        $urlGenerateLabel = $this->context->link->getAdminLink(
            'AdminBpostshmLabelOrder',
            true,
            [
                'route' => 'admin_bpostshm_label_generate',
                'orderId' => $idOrder,
            ]
        );

        $urlGenerateRetourLabel = $this->context->link->getAdminLink(
            'AdminBpostshmLabelOrder',
            true,
            [
                'route' => 'admin_bpostshm_label_generate',
                'orderId' => $idOrder,
                'retour' => true,
            ]
        );

        /** @var LabelService $labelService */
        $labelService = $this->module->get('bpostshm.label_service');

        $label = $labelService->getFirstLabelByOrder($idOrder);
        $labelRetour = $labelService->getFirstLabelByOrder($idOrder, true);
        $labelsFollowUp = $labelService->getTrackingNumbers($idOrder);
        $labelsRetourFollowUp = $labelService->getTrackingNumbers($idOrder, true);

        $smartyParams = [
            'idPickup' => $idPickup,
            'autoRetour' => \Configuration::get(
                'BPOSTSHM_LABELS_AUTO_RETOUR',
                null,
                $order->id_shop_group,
                $order->id_shop
            ),
            'label' => $label,
            'labelsFollowUp' => $labelsFollowUp,
            'labelRetour' => $labelRetour,
            'labelsRetourFollowUp' => $labelsRetourFollowUp,
            'urlDownloadLabel' => $urlDownloadLabel,
            'urlDownloadLabelRetour' => $urlDownloadLabelRetour,
            'urlGenerateLabel' => $urlGenerateLabel,
            'urlGenerateRetourLabel' => $urlGenerateRetourLabel,
        ];

        return $this->module->render('views/templates/hook/admin-order-tab-label-content.tpl', $smartyParams);
    }

    /**
     * Display the admin order tab content for pickup.
     *
     * @param \Order $order the order object
     * @param int $idPickup the ID of the pickup
     *
     * @return string the rendered HTML content
     */
    protected function displayAdminOrderTabContentPickup(\Order $order, int $idPickup): string
    {
        $idCart = $order->id_cart;

        /** @var PickupService $pickupService */
        $pickupService = $this->module->get('bpostshm.pickup_service');
        $pickup = $pickupService->getSelectedPickupAddress(
            $order->id_carrier,
            $this->context->language->id,
            $idCart,
            true
        );

        $smartyParams = [
            'idPickup' => $idPickup,
            'canChangePickup' => $order->current_state != \Configuration::get('PS_OS_SHIPPING'),
            'pickupAddress' => $pickup,
        ];

        return $this->module->render('views/templates/hook/admin-order-tab-pickup-content.tpl', $smartyParams);
    }
}
