<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Hook;

use Address;
use Configuration;
use PrestaShop\Module\BpostShm\Service\CarrierService;
use PrestaShop\Module\BpostShm\Service\CustomerContactService;
use PrestaShop\Module\BpostShm\Service\DeliveryDateService;
use PrestaShop\Module\BpostShm\Service\PickupService;
use PrestaShop\Module\BpostShm\Utils\BpostTools;

/**
 * Class Carrier
 */
class Carrier extends AbstractHook
{
    /**
     * @inerhitDoc
     */
    const AVAILABLE_HOOKS = [
        'actionObjectCarrierDeleteAfter',
        'actionCarrierUpdate',
        'actionCheckoutRender',
        'displayCarrierExtraContent',
    ];

    /**
     * Delete carrier history after carrier object is deleted.
     *
     * @param array $params the parameters of the method
     *
     * @return void
     */
    public function actionObjectCarrierDeleteAfter(array $params)
    {
        \Db::getInstance()->delete('bpostshm_carrier_history', 'id_carrier = ' . (int) $params['object']->id);
    }

    /**
     * Update carrier configuration values after carrier is updated.
     *
     * @param array $params The parameters of the method.
     *                      - id_carrier: The ID of the carrier being updated.
     *                      - carrier: The updated carrier object.
     *
     * @return void
     */
    public function actionCarrierUpdate(array $params)
    {
        $carriers = [
            'BPOSTSHM_ID_CARRIER_HOME',
            'BPOSTSHM_ID_CARRIER_PICKUP',
            'BPOSTSHM_ID_CARRIER_PARCEL_LOCKER',
        ];
        foreach ($carriers as $key) {
            $newCarrierId = (int) $params['carrier']->id;
            $shippingMethod = null;
            if ((int) $params['id_carrier'] == (int) \Configuration::getGlobalValue($key)) {
                \Configuration::updateGlobalValue($key, $newCarrierId);
                $shippingMethod = str_replace('BPOSTSHM_ID_CARRIER_', '', $key);
            }
            if ($shippingMethod) {
                \Db::getInstance()->insert(
                    'bpostshm_carrier_history',
                    [
                        'id_carrier' => $newCarrierId,
                        'shipping_method' => $shippingMethod,
                    ],
                    false,
                    true,
                    \Db::REPLACE
                );
            }
        }
    }

    /**
     * Displays the extra content for a carrier.
     *
     * @param array $params An array of parameters
     *                      - 'carrier' (int) The ID of the carrier
     *
     * @return string|void The rendered content for the carrier
     *                     or void if the carrier is not available in the selected address
     */
    public function displayCarrierExtraContent(array $params)
    {
        $carrier = new \Carrier($params['carrier']['id']);

        $idCart = \Validate::isLoadedObject($this->context->cart) ? $this->context->cart->id : null;
        $idLang = $this->context->language->id;

        $deliveryAddress = null;
        $deliveryAddressFormatted = '';
        if ($idCart) {
            $deliveryAddress = new \Address($this->context->cart->id_address_delivery);
            $deliveryAddressFormatted = \AddressFormat::generateAddress($deliveryAddress, [], '<br />');
            $country = new \Country($deliveryAddress->id_country, $idLang);
        } else {
            $country = new \Country(\Country::getByIso($this->module->getDefaultCountryCode()), $idLang);
        }

        /** @var DeliveryDateService $deliveryDateService */
        $deliveryDateService = $this->module->get('bpostshm.delivery_date_service');
        $deliveryDate = false;
        $deliveryDates = false;
        $isSaturday = false;
        $needSaturdayAlert = \Configuration::get('BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_COST') > 0
                                && \Configuration::get('BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_ENABLED');
        if ($deliveryDateService->showDeliveryDate($idCart)) {
            $deliveryDate = $deliveryDateService->getDeliveryDate($idCart, $carrier->id);
            $deliveryDates = $deliveryDateService->getDeliveryDates($carrier, $this->context->cart);
            if (!$deliveryDate && $idCart) {
                foreach ($deliveryDates as $date) {
                    $deliveryDate = $date;
                    break;
                }
                $deliveryDateService->setDeliveryDate($idCart, $carrier->id, $deliveryDate);
            }
            if ($deliveryDate) {
                $isSaturday = $deliveryDateService->isSaturday($deliveryDate);
            }
            if (!\Configuration::get('BPOSTSHM_DELIVERY_DATE_CHOOSE')) {
                $deliveryDates = false;
                $needSaturdayAlert = false;
            }
        }
        $smartyParams = [
            'carrier' => $carrier,
            'deliveryAddress' => $deliveryAddress,
            'deliveryAddressFormatted' => $deliveryAddressFormatted,
            'deliveryDates' => $deliveryDates,
            'deliveryDate' => $deliveryDate,
            'needSaturdayAlert' => $needSaturdayAlert,
            'isSaturday' => $isSaturday,
            'country' => $country,
        ];

        /** @var PickupService $pickupService */
        $pickupService = $this->module->get('bpostshm.pickup_service');
        $pickupPath = '';
        if ($pickupService->isPickup($carrier)) {
            $carrierType = $pickupService->getPickupType($carrier);
            $carrierMethod = 'PICKUP';
            if ($carrierType == PickupService::PARCEL_LOCKER_TYPE) {
                $carrierMethod = 'PARCEL_LOCKER';
            }

            $countries = $pickupService->getAvailableCountries($carrierMethod == 'PARCEL_LOCKER');
            if (!isset($countries[$country->iso_code])) {
                return;
            }

            $pickup = $pickupService->getSelectedPickupAddress($carrier->id, $idLang, $idCart);

            if (!$idCart || !\Validate::isLoadedObject($deliveryAddress)) {
                // default address in guess mode
                $deliveryAddress = new \Address();
                $deliveryAddress->address1 = '';
                $deliveryAddress->postcode = '';
                $deliveryAddress->city = '';
                $deliveryAddress->id_country = $country->id;
            }

            /** @var CustomerContactService $customerContactService */
            $customerContactService = $this->module->get('bpostshm.customer_contact_service');

            $pickupPath = 'pickup/';
            $smartyParams = array_merge($smartyParams, [
                'deliveryAddress' => $deliveryAddress,
                'deliveryAddressFormatted' => $deliveryAddressFormatted,
                'needSaturdayAlert' => \Configuration::get("BPOSTSHM_DELIVERY_OPTIONS_{$carrierMethod}_SATURDAY_COST") > 0
                    && \Configuration::get("BPOSTSHM_DELIVERY_OPTIONS_{$carrierMethod}_SATURDAY_ENABLED"),
                'countries' => $pickupService->getAvailableCountries($pickupService->isParcelLocker($carrier)),
                'selectedPickupPoint' => $pickup,
                'carrierType' => $carrierType,
                'carrierMethod' => $carrierMethod,
                'showCustomerContactForm' => $carrierMethod == 'PARCEL_LOCKER',
                'customerContact' => $customerContactService->getCustomerContact(
                    $idCart,
                    $carrier->id,
                    $this->context->customer,
                    $deliveryAddress
                ),
                'showMap' => (bool) \Configuration::get('BPOSTSHM_PICKUP_SELECTOR_MAP_ENABLED'),
                'design' => \Configuration::get('BPOSTSHM_PICKUP_SELECTOR_DESIGN'),
            ]);
        }

        return $this->module->render("views/templates/hook/{$pickupPath}carrier-extra-content.tpl", $smartyParams);
    }

    /**
     * Saves customer contact info for parcel locker carrier during the checkout render action
     *
     * @param array $params an array of parameters
     *
     * @return void
     */
    public function actionCheckoutRender(array $params)
    {
        $email = BpostTools::cleanStr(\Tools::getValue('bpostshm-email'));
        $mobile = BpostTools::cleanStr(\Tools::getValue('bpostshm-mobile'));
        if ($email || $mobile) {
            /** @var CarrierService $carrierService */
            $carrierService = $this->module->get('bpostshm.carrier_service');
            if ($carrierService->getCarrierMethod($params['cart']->id_carrier) == 'PARCEL_LOCKER') {
                /** @var CustomerContactService $customerContactService */
                $customerContactService = $this->module->get('bpostshm.customer_contact_service');
                $res = $customerContactService->checkCustomerContact([
                    'email' => $email,
                    'mobile' => $mobile,
                ]);
                if ($res !== true) {
                    $this->context->controller->errors = array_merge($this->context->controller->errors, $res);
                }
                $customerContactService->setCustomerContact(
                    $this->context->cart->id,
                    $this->context->cart->id_carrier,
                    $email,
                    $mobile,
                    (int) \Tools::getValue('bpostshm-rmz')
                );
            }
        }
    }
}
