<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Hook;

use Configuration;
use PrestaShop\Module\BpostShm\Service\LabelService;
use PrestaShop\Module\BpostShm\Service\OrderService;

/**
 * Class Cron
 */
class Cron extends AbstractHook
{
    /**
     * @inerhitDoc
     */
    const AVAILABLE_HOOKS = [
        'actionCronJob',
    ];

    /**
     * each day at 01:00
     */
    const FREQUENCY = [
        'hour' => 1,
        'day' => -1,
        'month' => -1,
        'day_of_week' => -1,
    ];

    /**
     * Executes the cron job action.
     *
     * This method is responsible for executing the cron job action. It calls the
     * `prepareMissingBpostOrders()` and `cleanOldLabels()` methods to perform
     * specific tasks required for the cron job.
     *
     * @return void
     *
     * @throws \Exception if an error occurs during the cron job action
     */
    public function actionCronJob()
    {
        $this->refreshBpostOrdersStatuses();
        $this->prepareMissingBpostOrders();
        $this->cleanOldLabels();
    }

    /**
     * Refreshes statuses of bpost orders.
     *
     * @return void
     *
     * @throws \Exception if an error occurs while refreshing statuses of bpost orders
     */
    public function refreshBpostOrdersStatuses()
    {
        try {
            /** @var OrderService $orderService */
            $orderService = $this->module->get('bpostshm.order_service');
            if (\Shop::isFeatureActive()) {
                foreach (\Shop::getShops(true) as $shop) {
                    $orderService->refreshStatuses((int) $shop['id_shop']);
                }
            } else {
                $orderService->refreshStatuses();
            }
        } catch (\Exception $e) {
            $this->module->log('[CRON] ' . $this->translator->trans(
                'An error occurred while refreshing statuses of bpost orders: %msg%.',
                ['%msg%' => $e->getMessage()],
                'Modules.Bpostshm.Admin'
            ));
        }
    }

    /**
     * Prepares missing bpost orders.
     *
     * This method invokes the prepareMissingBpostOrders() method of the Order Service class
     * to perform necessary actions for preparing missing bpost orders.
     *
     * @return void
     *
     * @throws \Exception if an error occurs while preparing missing bpost orders
     */
    public function prepareMissingBpostOrders()
    {
        try {
            /** @var OrderService $orderService */
            $orderService = $this->module->get('bpostshm.order_service');
            $orderService->prepareMissingBpostOrders();
        } catch (\Exception $e) {
            $this->module->log('[CRON] ' . $this->translator->trans(
                'An error occurred while preparing missing bpost orders: %msg%.',
                ['%msg%' => $e->getMessage()],
                'Modules.Bpostshm.Admin'
            ));
        }
    }

    /**
     * Cleans old labels.
     *
     * This method retrieves the number of days to keep old labels from the configuration.
     * If the number of days is greater than 0, it invokes the cleanOldLabels() method
     * of the Label Service class to delete labels that are older than the configured number of days.
     *
     * @return void
     *
     * @throws \Exception if an error occurs while cleaning old labels
     */
    public function cleanOldLabels()
    {
        try {
            $days = (int) \Configuration::getGlobalValue('BPOSTSHM_LABELS_CLEAN_AFTER_DAYS');
            if ($days > 0) {
                /** @var LabelService $labelService */
                $labelService = $this->module->get('bpostshm.label_service');
                $labelService->cleanOldLabels('-' . $days . ' days');
            }
        } catch (\Exception $e) {
            $this->module->log('[CRON] ' . $this->translator->trans(
                'An error occurred while cleaning old labels: %msg%.',
                ['%msg%' => $e->getMessage()],
                'Modules.Bpostshm.Admin'
            ));
        }
    }
}
