<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Hook;

use AdminModulesController;
use Configuration;
use Media as PSMedia;
use OrderController;
use PrestaShop\Module\BpostShm\Service\PickupService;

/**
 * Class Media
 */
class Media extends AbstractHook
{
    /**
     * @inerhitDoc
     */
    const AVAILABLE_HOOKS = [
        'actionAdminControllerSetMedia',
        'actionFrontControllerSetMedia',
    ];

    /**
     * Sets the media (CSS and JS files) for the admin controller.
     *
     * If the current controller is an instance of AdminModulesController
     * and the 'configure' value in the request matches the module name,
     * then the 'back-settings.css' and 'back-settings.js' files will be added.
     *
     * If the current controller is 'AdminOrders',
     * then the following files will be added:
     * - 'widget.css' located in 'views/css/admin/' of the module
     * - 'leaflet.css' located in 'views/css/leaflet/' of the module
     * - 'fancybox' jQuery plugin
     * - 'leaflet.js' located in 'views/js/' of the module
     * - 'widget.js' located in 'views/js/admin/' of the module
     *
     * @return void
     */
    public function actionAdminControllerSetMedia()
    {
        if ($this->context->controller instanceof \AdminModulesController
            && \Tools::getValue('configure') == $this->module->name) {
            $this->context->controller->addCSS($this->module->getPathUri() . 'views/dist/back-settings.css');
            $this->context->controller->addJS($this->module->getPathUri() . 'views/dist/back-settings.js');
        }
    }

    /**
     * Sets the media (CSS and JS files) for the front controller.
     *
     * If the current controller is an instance of OrderController, the following media files will be added:
     * - If the 'BPOSTSHM_PICKUP_SELECTOR_MAP_ENABLED' configuration is enabled:
     *      - If the 'BPOSTSHM_PICKUP_SELECTOR_GMAPS_API_KEY' configuration is set,
     *          the Google Maps API will be added remotely.
     *      - If the 'BPOSTSHM_PICKUP_SELECTOR_GMAPS_API_KEY' configuration is not set,
     *          the following files will be added:
     *          - 'leaflet.js' located in 'modules/{module_name}/views/dist/'
     *          - 'leaflet.css' located in 'modules/{module_name}/views/dist/'
     * - 'front.js' located in 'modules/{module_name}/view/dist/'
     * - 'front.css' located in 'modules/{module_name}/view/dist/'
     *
     * Additionally, the method sets JavaScript definitions for the bpostshm namespace.
     * These definitions include:
     * - 'map': either set to 'GoogleMaps' or 'OpenStreetMap' based
     *          on the 'BPOSTSHM_PICKUP_SELECTOR_GMAPS_API_KEY' configuration
     * - 'leaflet': null, to be loaded in JavaScript if necessary
     * - 'design': the value of the 'BPOSTSHM_PICKUP_SELECTOR_DESIGN' configuration
     * - 'pickupAjaxUrl': the URL of the 'pickup' module link
     * - 'deliveryDateAjaxUrl': the URL of the 'deliveryDate' module link
     * - 'availableCountries': an array of available countries for parcel locker and pickup types
     * - 'defaultCountry': the default country code from the module
     * - 'markerPath': the base URI followed by 'modules/{module_name}/img/'
     * - 'selectors': the result of calling the 'getSelectors' method on the module
     * - 'trans': an array of translations for schedulesTitle, select, and days
     *
     * @return void
     */
    public function actionFrontControllerSetMedia()
    {
        if ($this->context->controller instanceof \OrderController) {
            if (\Configuration::get('BPOSTSHM_PICKUP_SELECTOR_MAP_ENABLED')) {
                if (\Configuration::get('BPOSTSHM_PICKUP_SELECTOR_GMAPS_API_KEY')) {
                    $this->context->controller->registerJavascript(
                        'modules-' . $this->module->name . '_google-maps-api',
                        'https://maps.googleapis.com/maps/api/js?key=' .
                        trim(\Configuration::get('BPOSTSHM_PICKUP_SELECTOR_GMAPS_API_KEY')),
                        ['server' => 'remote', 'attributes' => 'defer']
                    );
                } else {
                    $this->context->controller->registerJavascript(
                        'modules-' . $this->module->name . '_leaflet',
                        'modules/' . $this->module->name . '/views/dist/leaflet.js'
                    );
                    $this->context->controller->registerStylesheet(
                        'modules-' . $this->module->name . '_leaflet',
                        'modules/' . $this->module->name . '/views/dist/leaflet.css'
                    );
                }
            }

            $this->context->controller->registerJavascript(
                'modules-' . $this->module->name . '_front',
                'modules/' . $this->module->name . '/views/dist/front.js'
            );
            $this->context->controller->registerStylesheet(
                'modules-' . $this->module->name . '_front',
                'modules/' . $this->module->name . '/views/dist/front.css'
            );
            $this->context->controller->registerStylesheet(
                'modules-' . $this->module->name . '_custom',
                'modules/' . $this->module->name . '/views/css/custom.css'
            );

            /** @var PickupService $pickupService */
            $pickupService = $this->module->get('bpostshm.pickup_service');
            PSMedia::addJsDef([
                'bpostshm' => [
                    'map' => !\Configuration::get('BPOSTSHM_PICKUP_SELECTOR_MAP_ENABLED') ? '' : (
                        \Configuration::get('BPOSTSHM_PICKUP_SELECTOR_GMAPS_API_KEY') ? 'GoogleMaps' : 'OpenStreetMap'
                    ),
                    'leaflet' => null, // loaded in js if necessary
                    'design' => \Configuration::get('BPOSTSHM_PICKUP_SELECTOR_DESIGN'),
                    'pickupAjaxUrl' => $this->context->link->getModuleLink('bpostshm', 'pickup'),
                    'deliveryDateAjaxUrl' => $this->context->link->getModuleLink('bpostshm', 'deliveryDate'),
                    'availableCountries' => [
                        PickupService::PARCEL_LOCKER_TYPE => $pickupService->getAvailableCountries(true),
                        PickupService::PICKUP_TYPE => $pickupService->getAvailableCountries(),
                    ],
                    'defaultCountry' => $this->module->getDefaultCountryCode(),
                    'markerPath' => __PS_BASE_URI__ . 'modules/' . $this->module->name . '/img/',
                    'selectors' => $this->module->getSelectors(),
                    'trans' => [
                        'schedulesTitle' => $this->translator->trans('Opening hours', [], 'Modules.Bpostshm.Shop'),
                        'select' => $this->translator->trans('Select', [], 'Modules.Bpostshm.Shop'),
                        'days' => [
                            $this->translator->trans('Monday', [], 'Modules.Bpostshm.Shop'),
                            $this->translator->trans('Tuesday', [], 'Modules.Bpostshm.Shop'),
                            $this->translator->trans('Wednesday', [], 'Modules.Bpostshm.Shop'),
                            $this->translator->trans('Thursday', [], 'Modules.Bpostshm.Shop'),
                            $this->translator->trans('Friday', [], 'Modules.Bpostshm.Shop'),
                            $this->translator->trans('Saturday', [], 'Modules.Bpostshm.Shop'),
                            $this->translator->trans('Sunday', [], 'Modules.Bpostshm.Shop'),
                        ],
                    ],
                ],
            ]);
        }
    }
}
