<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Hook;

use Cart;
use Doctrine\Common\Annotations\AnnotationException;
use PrestaShop\Module\BpostShm\Adapter\Presenter\Order\OrderLazyArray;
use PrestaShop\Module\BpostShm\Service\CarrierService;
use PrestaShop\Module\BpostShm\Service\CustomerContactService;
use PrestaShop\Module\BpostShm\Service\OrderService;
use PrestaShop\Module\BpostShm\Service\PickupService;
use PrestaShop\Module\BpostShm\Utils\BpostTools;
use Validate;

/**
 * Class order
 */
class Order extends AbstractHook
{
    /**
     * @inerhitDoc
     */
    const AVAILABLE_HOOKS = [
        'actionFrontControllerInitAfter',
        'actionPresentOrder',
        'actionStigmiOpcPaymentProcess',
        'actionValidateOrder',
        'actionValidateStepComplete',
        'displayOrderConfirmation',
    ];

    /**
     * Updates the presented order in the specified params array.
     *
     * @param array $params The array containing the parameters.
     *                      - 'presentedOrder' (object): The presented order to update.
     *
     * @return void
     *
     * @throws AnnotationException if there is an error in the annotation
     * @throws \ReflectionException if there is an error in the reflection
     */
    public function actionPresentOrder(array $params)
    {
        $order = new \Order($params['presentedOrder']->details->id);
        if (BpostTools::isBpostOrder($order)) {
            $params['presentedOrder'] = new OrderLazyArray($this->module, $order);
        }
    }

    /**
     * Checks the customer contact for the given cart when the parcel locker carrier is selected.
     *
     * @param \Cart $cart the cart object to check the customer contact
     *
     * @return array|bool returns true if the customer contact is valid, an array with errors otherwise
     *
     * @throws AnnotationException if there is an error in the annotation
     * @throws \ReflectionException if there is an error in reflecting the class
     */
    protected function checkCustomerContact(\Cart $cart)
    {
        if (BpostTools::isBpostCart($cart)) {
            /** @var CarrierService $carrierService */
            $carrierService = $this->module->get('bpostshm.carrier_service');
            if ($carrierService->getCarrierMethod($cart->id_carrier) == 'PARCEL_LOCKER') {
                /** @var CustomerContactService $customerContactService */
                $customerContactService = $this->module->get('bpostshm.customer_contact_service');
                $customerContact = $customerContactService->getCustomerContact($cart->id, $cart->id_carrier);

                return $customerContactService->checkCustomerContact($customerContact);
            }
        }

        return true;
    }

    /**
     * Checks if a pick-up point is selected for the given cart
     * and returns an error message if a pick-up point is required but not selected.
     *
     * @param \Cart $cart the cart object
     *
     * @return array|true returns an error message if a pick-up point is required but not selected,
     *                    otherwise returns true
     */
    protected function checkPickupPoint(\Cart $cart)
    {
        if (BpostTools::isBpostCart($cart)) {
            /** @var PickupService $pickupService */
            $pickupService = $this->module->get('bpostshm.pickup_service');
            if ($pickupService->isPickup($cart->id_carrier)) {
                $pickupId = $pickupService->getSelectedPickupId($cart->id_carrier, $cart->id);
                if (!$pickupId) {
                    if ($pickupService->isParcelLocker($cart->id_carrier)) {
                        return [
                            'noParcelLockerSelected' => $this->translator->trans(
                                'Please choose a parcel locker.',
                                [],
                                'Modules.Bpostshm.Shop'
                            ),
                        ];
                    } else {
                        return [
                            'noPickupPointSelected' => $this->translator->trans(
                                'Please choose a pick-up point.',
                                [],
                                'Modules.Bpostshm.Shop'
                            ),
                        ];
                    }
                }

                $addressDelivery = new \Address($cart->id_address_delivery);
                $country = \Country::getIsoById($addressDelivery->id_country);

                $selectedPickup = $pickupService->getPickup($pickupId, $this->context->language, $country);
                if (isset($selectedPickup['iso_country'])) {
                    $diffCountryMsg = $this->translator->trans(
                        'Please create/select a delivery address that is in the same country as the pick-up point.',
                        [],
                        'Modules.Bpostshm.Shop'
                    );
                    if (\Validate::isLoadedObject($addressDelivery) && $selectedPickup['iso_country'] != $country) {
                        return ['differentCountry' => $diffCountryMsg];
                    }
                }
            }
        }

        return true;
    }

    /**
     * This method is called when the request is an AJAX request
     * with the action parameter set to 'checkCartStillOrderable'.
     * It checks if the customer contact form is valid and if a pick-up point is correctly selected.
     *
     * @param array $params the parameters passed to the method
     *
     * @return void
     *
     * @throws AnnotationException if there is an error in the annotation
     * @throws \ReflectionException if there is an error in the reflection
     */
    public function actionFrontControllerInitAfter(array $params)
    {
        if ($params['controller'] instanceof \OrderController) {
            if (\Tools::getValue('ajax') && \Tools::getValue('action') == 'checkCartStillOrderable') {
                $errors = [];
                $res = $this->checkCustomerContact($this->context->cart);
                if ($res !== true) {
                    $errors = $res;
                }

                $res = $this->checkPickupPoint($this->context->cart);
                if ($res !== true) {
                    $errors = array_merge($errors, $res);
                }

                if (count($errors)) {
                    $responseData = [
                        'errors' => true,
                        'cartUrl' => $this->context->link->getPageLink('order'),
                    ];

                    $params['controller']->errors = array_merge($this->context->controller->errors, $errors);

                    // redirect with notifications
                    $notifications = json_encode([
                        'error' => $params['controller']->errors,
                        'warning' => $params['controller']->warning,
                        'success' => $params['controller']->success,
                        'info' => $params['controller']->info,
                    ]);

                    if (session_status() == PHP_SESSION_ACTIVE) {
                        $_SESSION['notifications'] = $notifications;
                    } elseif (session_status() == PHP_SESSION_NONE) {
                        session_start();
                        $_SESSION['notifications'] = $notifications;
                    } else {
                        setcookie('notifications', $notifications);
                    }

                    header('Content-Type: application/json');
                    exit(json_encode($responseData));
                }
            }
        }
    }

    /**
     * This method is called to validate if the step is complete.
     * It checks if the customer contact form is valid and if a pick-up point is correctly selected.
     *
     * @param array $params The parameters passed to the method.
     *                      It should contain the 'cart' key, which represents the cart object.
     *
     * @return void
     *
     * @throws AnnotationException if there is an error in the annotation
     * @throws \ReflectionException if there is an error in the reflection
     */
    public function actionValidateStepComplete(array $params)
    {
        $res = $this->checkCustomerContact($params['cart']);
        if ($res !== true) {
            $params['completed'] = false;
            $this->context->controller->errors = array_merge($this->context->controller->errors, $res);
        }
        $res = $this->checkPickupPoint($params['cart']);
        if ($res !== true) {
            $params['completed'] = false;
            $this->context->controller->errors = array_merge($this->context->controller->errors, $res);
        }
    }

    /**
     * This method is called by the stigmiopc module to check if we can place order.
     * It checks if the customer contact form is valid and if a pick-up point is correctly selected.
     *
     * @param array $params The parameters passed to the method.
     *                      It should contain the following keys:
     *                      - cart: The cart object.
     *                      - canOrder: A boolean indicating if the order can be made.
     *                      - cannotOrderMessages: An array of error messages preventing the order.
     *
     * @return void
     *
     * @throws AnnotationException if there is an error in the annotation
     * @throws \ReflectionException if there is an error in the reflection
     */
    public function actionStigmiOpcPaymentProcess(array $params)
    {
        $res = $this->checkCustomerContact($params['cart']);
        if ($res !== true) {
            $params['canOrder'] = false;
            $params['cannotOrderMessages'] = $res;
        }
        $res = $this->checkPickupPoint($params['cart']);
        if ($res !== true) {
            $params['canOrder'] = false;
            if (isset($params['cannotOrderMessages'])) {
                $params['cannotOrderMessages'] = array_merge($params['cannotOrderMessages'], $res);
            } else {
                $params['cannotOrderMessages'] = $res;
            }
        }
    }

    /**
     * This method is called to validate an order.
     * It prepares a bpost order if the order is a bpost order.
     * It sets the pickup point for the order if the carrier is a pickup carrier.
     *
     * @param array $params The parameters passed to the method.
     *                      The 'order' parameter is required and should be an instance of the 'Order' class.
     *
     * @return void
     *
     * @throws \Exception if there is an error while preparing the bpost order
     */
    public function actionValidateOrder(array $params)
    {
        $order = $params['order'];

        if (BpostTools::isBpostOrder($order)) {
            /** @var OrderService $orderService */
            $orderService = $this->module->get('bpostshm.order_service');
            try {
                $orderService->prepareBpostOrder($order, false);
            } catch (\Exception $e) {
                $log = $this->translator->trans(
                    'Validate Order: Unable to prepare bpost order (ID %order%): %message%.',
                    [
                        '%order%' => $order->id,
                        '%message%' => $e->getMessage() ?: '[' . get_class($e) . ']',
                    ],
                    'Modules.Bpostshm.Admin'
                );
                $this->module->log($log, 3, null, 'Order', $order->id);
            }

            /** @var PickupService $pickupService */
            $pickupService = $this->module->get('bpostshm.pickup_service');

            if ($pickupService->isPickup($order->id_carrier)) {
                $deliveryAddress = new \Address($order->id_address_delivery);

                $pickupPoint = $pickupService->getPickup(
                    (int) $pickupService->getSelectedPickupId($order->id_cart),
                    new \Language($order->id_lang),
                    \Country::getIsoById($deliveryAddress->id_country)
                );
                if ($pickupPoint) {
                    $errors = $pickupService->setPickupForOrder($order->id, $pickupPoint);
                } else {
                    $errors = [
                        $this->translator->trans(
                            'Pick-up point not found for cart %cart% and carrier %carrier%.',
                            [
                                '%cart%' => $order->id_cart,
                                '%carrier%' => $order->id_carrier,
                            ],
                            'Modules.Bpostshm.Shop'
                        ),
                    ];
                }
                if (count($errors)) {
                    foreach ($errors as $error) {
                        $this->module->log($error, 3, null, 'Order', $order->id);
                    }
                }
            }
        }
    }

    /**
     * This method is responsible for displaying the order confirmation page.
     *
     * @param array $params The parameters passed to the method. It should contain either 'objOrder' or 'order'.
     *
     * @return void
     *
     * @throws \ReflectionException if there is an error in the reflection
     * @throws AnnotationException if there is an error in the annotation
     */
    public function displayOrderConfirmation(array $params)
    {
        $order = (isset($params['objOrder'])) ? $params['objOrder'] : $params['order'];

        /** @var PickupService $pickupService */
        $pickupService = $this->module->get('bpostshm.pickup_service');

        if ($pickupService->isPickup($order->id_carrier)) {
            $pickup = $pickupService->getSelectedPickupAddress($order->id_lang, $order->id_cart);
            if (!$pickup || !$order->id_cart) {
                $log = $this->translator->trans(
                    'Order Confirmation: Unable to find Pick-up point "%pickup%" for order %order% (cart: %cart%).',
                    [
                        '%pickup%' => $pickupService->getSelectedPickupId($order->id_cart),
                        '%order%' => $order->id,
                        '%cart%' => $order->id_cart,
                    ],
                    'Modules.Bpostshm.Shop'
                );
                $this->module->log($log, 3, null, 'Order', $order->id);
            }
            $this->context->smarty->assign([
                'pickup' => $pickup,
            ]);
        }
    }
}
