<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Service;

use BpostShm;
use Configuration;
use Exception;
use Order;
use PrestaShop\Module\BpostShm\Api\Bpost;
use PrestaShop\Module\BpostShm\Api\Bpost\Label;
use PrestaShop\Module\BpostShm\Api\Bpost\ProductConfiguration;
use PrestaShop\Module\BpostShm\Api\Exception\BpostApiResponseException\BpostCurlException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostApiResponseException\BpostInvalidResponseException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostApiResponseException\BpostInvalidSelectionException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostApiResponseException\BpostInvalidXmlResponseException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidValueException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostNotImplementedException;
use PrestaShop\Module\BpostShm\Api\Exception\XmlException\BpostXmlNoReferenceFoundException;
use PrestaShop\Module\BpostShm\Api\Geo6;

/**
 * Class ApiService
 */
class ApiService
{
    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * @var \Context
     */
    private $context;

    /**
     * @var Bpost
     */
    private $bpost;

    /**
     * @var Geo6
     */
    private $geo6;

    /**
     * Constructor
     *
     * @param \BpostShm $module the BpostShm module object
     */
    public function __construct(\BpostShm $module)
    {
        $this->module = $module;
        $this->context = $module->getContext();
    }

    /**
     * Get the Bpost API instance.
     *
     * @return Bpost
     */
    private function getBpostApi(): Bpost
    {
        if (!$this->bpost) {
            $credentials = $this->module->getCredentials();
            $this->bpost = new Bpost(
                $credentials['username'],
                $credentials['password']
            );
        }

        return $this->bpost;
    }

    /**
     * Get the instance of the Geo6Api.
     * If the instance does not exist, a new instance will be created with the given credentials.
     *
     * @return Geo6 the instance of the Geo6Api
     */
    private function getGeo6Api(): Geo6
    {
        if (!$this->geo6) {
            $this->geo6 = new Geo6('999999', '');
        }

        return $this->geo6;
    }

    /**
     * Check if the API credentials are valid and update the global configuration.
     *
     * @return bool True if the credentials are valid, false otherwise
     */
    public function checkCredentials(): bool
    {
        try {
            $config = $this->getBpostApi()->fetchProductConfig();

            // refresh countries config cache
            $intlCountries = [];
            $ppiCountries = [];

            foreach ($config->getDeliveryMethods() as $deliveryMethod) {
                foreach ($deliveryMethod->getProducts() as $product) {
                    $productName = $product->getName();
                    $productPrices = $product->getPrices();
                    if (count($productPrices)) {
                        if (preg_match('#bpack world#i', $productName)) {
                            foreach ($productPrices as $price) {
                                $intlCountries[] = $price->getCountryIso2();
                            }
                            break;
                        } elseif (preg_match('#international#i', $productName)) {
                            foreach ($productPrices as $price) {
                                $ppiCountries[] = $price->getCountryIso2();
                            }
                            break;
                        }
                    }
                }
            }
            \Configuration::updateGlobalValue('BPOSTSHM_INTL_COUNTRIES', json_encode([
                'intl' => $intlCountries,
                'ppi' => $ppiCountries,
            ]));
        } catch (\Exception $e) {
            return false;
        }

        return true;
    }

    /**
     * Get the product configuration.
     *
     * This method fetches the product configuration from the Bpost API.
     * It returns an array with the product configuration details.
     * If an exception occurs during the fetch, it will return false.
     *
     * @return ProductConfiguration|false The product configuration details
     *
     * @throws \Exception if an error occurs during the fetch
     */
    public function getProductConfig()
    {
        try {
            $config = $this->getBpostApi()->fetchProductConfig();
        } catch (\Exception $e) {
            return false;
        }

        return $config;
    }

    /**
     * Check if the bpost API is alive.
     * This method makes a request to the bpost API to check if it is alive and working properly.
     *
     * @return bool true if the API is alive and working properly, false otherwise
     */
    public function isAlive(): bool
    {
        try {
            $response = \Tools::file_get_contents('https://api.bpost.be/services/shm/');

            if (preg_match('/This request requires HTTP authentication/i', $response)) {
                return true;
            }
        } catch (\Exception $e) {
            // silence is golden.
        }

        return false;
    }

    /**
     * Get the details of a pickup point.
     *
     * This method retrieves the details of a pickup point specified by its ID.
     *
     * @param string $id The ID of the pickup point
     * @param string $country The country code of the pickup point (default: 'BE')
     * @param int $type The type of the pickup point (default: 3)
     *
     * @return Geo6\Poi The details of the pickup point
     *
     * @throws \Exception If an error occurs while retrieving the details
     */
    public function getPickupPointDetails(string $id, string $isoCode, string $country = 'BE', int $type = 3): Geo6\Poi
    {
        if (!$isoCode) {
            $isoCode = $this->context->language->iso_code;
        }

        return $this->getGeo6Api()->getServicePointDetails($id, \Tools::strtolower($isoCode), $type, $country);
    }

    /**
     * Get pick-up points
     *
     * @param string $street The street name
     * @param string $nr The street number
     * @param string $zone The geographic zone
     * @param int $type Requested point type, possible values are:
     *                  - 1: Post Office
     *                  - 2: Post Point
     *                  - 3: (1+2, Post Office + Post Point)
     *                  - 4: bpack 24/7
     *                  - 7: (1+2+4, Post Office + Post Point + bpack 24/7)
     *                  - 8: Click & Collect Shop
     *                  - 9: (1+8, Post Office + Click & Collect Shop)
     *                  - 10: (2+8, Post Point + Click & Collect Shop)
     *                  - 11: (1+2+8, Post Office + Post Point + Click & Collect Shop)
     *                  - 15: (1+2+4+8, Post Office + Post Point + bpack 24/7 + Click & Collect Shop)
     *                  - 16: Kariboo
     * @param string $country The country code
     * @param string|null $deliveryDate The delivery date in 'DD-MM-YYYY' format (optional, default: null)
     * @param int $limit The maximum number of pickup points to retrieve (optional, default: 20)
     *
     * @return array Array of pickup points
     */
    public function getPickupPoints(
        string $street,
        string $nr,
        string $zone,
        int $type,
        string $country,
        ?string $deliveryDate = null,
        int $limit = 20
    ): array {
        $points = [];

        try {
            $response = $this->getGeo6Api()->getAdvancedNearestServicePoint(
                $street,
                $nr,
                $zone,
                $this->context->language->iso_code,
                $type,
                $limit,
                $country,
                $deliveryDate
            );
            foreach ($response as $row) {
                $poi = [
                    'id' => $row['poi']->getId(),
                    'latitude' => $row['poi']->getLatitude(),
                    'longitude' => $row['poi']->getLongitude(),
                    'name' => $row['poi']->getOffice(),
                    'type' => $row['poi']->getType(),
                    'street' => $row['poi']->getstreet(),
                    'nr' => $row['poi']->getNr(),
                    'address' => $row['poi']->getstreet() . ', ' . $row['poi']->getNr(),
                    'city' => $row['poi']->getCity(),
                    'postcode' => $row['poi']->getZip(),
                    'country' => $country,
                    'distance' => $row['distance'],
                ];
                $points[] = $poi;
            }
        } catch (\Exception $e) {
            $this->module->log('Unable to retrieve pick-up points: ' . $e->getMessage(), 1);
        }

        return $points;
    }

    /**
     * Get the Bpost order reference for a given order.
     *
     * @param \Order $order The order object
     * @param bool $retour Determines whether the order is a return order or not
     *
     * @return string The Bpost order reference
     *
     * @throws \PrestaShopException If the order object is invalid
     */
    public function getBpostOrderReference(\Order $order, bool $retour): string
    {
        if (!\Validate::isLoadedObject($order)) {
            throw new \PrestaShopException('Invalid Order');
        }

        return $this->getBpostApi()->getAccountId() . '_' . $order->id . '_' . $order->reference
            . ($retour ? '_RETOUR' : '');
    }

    /**
     * Fetches the Bpost order from the API.
     *
     * Retrieves the Bpost\Order object for the given Order.
     *
     * @param \Order $order The order object to fetch Bpost order for
     * @param bool $retour Indicates whether to fetch return order or not
     *
     * @return Bpost\Order|null The fetched Bpost\Order object or null if not found
     *
     * @throws BpostInvalidValueException
     * @throws BpostCurlException
     * @throws BpostInvalidSelectionException
     * @throws BpostInvalidXmlResponseException
     * @throws BpostNotImplementedException
     * @throws BpostXmlNoReferenceFoundException
     */
    public function getBpostOrder(\Order $order, bool $retour): ?Bpost\Order
    {
        try {
            return $this->getBpostApi()->fetchOrder(
                $this->getBpostOrderReference($order, $retour)
            );
        } catch (BpostInvalidResponseException $e) {
            // silence is golden.
        }

        return null;
    }

    /**
     * Create or replace an order.
     *
     * This method creates or replaces an order using the provided Bpost\Order object.
     * - If the order does not exist, a new order will be created.
     * - If the order already exists, it will be replaced with the provided order object.
     *
     * @param Bpost\Order $order The Bpost\Order object representing the order
     *
     * @return bool Returns true if the order was successfully created or replaced, otherwise false
     *
     * @throws BpostCurlException
     * @throws BpostInvalidResponseException
     * @throws BpostInvalidSelectionException
     * @throws BpostInvalidXmlResponseException
     * @throws \DOMException
     */
    public function createOrReplaceOrder(Bpost\Order $order): bool
    {
        return $this->getBpostApi()->createOrReplaceOrder($order);
    }

    /**
     * Create the labels for all unprinted boxes in an order.
     * The service will return labels for all unprinted boxes for that order.
     * Boxes that were unprinted will get the status PRINTED, the boxes that
     * had already been printed will remain the same.
     *
     * @param string $reference The reference for an order
     * @param string $format The desired format, allowed values are: A4, A6
     * @param bool $retour Whether the order has a return or not
     * @param bool $asPdf Should we retrieve the PDF-version instead of PNG
     *
     * @return Label[]
     *
     * @throws BpostCurlException
     * @throws BpostInvalidResponseException
     * @throws BpostInvalidSelectionException
     * @throws BpostInvalidValueException
     * @throws BpostInvalidXmlResponseException
     */
    public function createLabelForOrder(string $reference, string $format, bool $retour, bool $asPdf = true): array
    {
        return $this->getBpostApi()->createLabelForOrder($reference, $format, $retour, $asPdf);
    }

    /**
     * Create the labels file from a barcode.
     *
     * @param string $barcode The barcode of the label
     * @param string $format The desired format, allowed values are: A4, A6
     * @param bool $retour Whether the order has a return or not
     * @param bool $asPdf Should we retrieve the PDF-version instead of PNG
     *
     * @return Label[]
     *
     * @throws BpostCurlException
     * @throws BpostInvalidResponseException
     * @throws BpostInvalidSelectionException
     * @throws BpostInvalidValueException
     * @throws BpostInvalidXmlResponseException
     */
    public function createLabelForBox(string $barcode, string $format, bool $retour, bool $asPdf = true): array
    {
        return $this->getBpostApi()->createLabelForBox($barcode, $format, $retour, $asPdf);
    }
}
