<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Service;

use Address;
use BpostShm;
use Carrier;
use Configuration;
use Context;
use Language;
use Order;
use PrestaShop\Module\BpostShm\Domain\Exception\CarrierException;
use PrestaShop\Module\BpostShm\Utils\BpostTools;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * Class CarrierService
 */
class CarrierService
{
    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * @var \Context
     */
    private $context;

    /**
     * @var \Db
     */
    private $database;

    /**
     * @var TranslatorInterface
     */
    private $translator;

    /**
     * Constructor
     *
     * @param \BpostShm $module
     */
    public function __construct(\BpostShm $module)
    {
        $this->module = $module;
        $this->context = $module->getContext();
        $this->database = $module->getDatabase();
        $this->translator = $module->getTranslator();
    }

    /**
     * Get the available shipping methods.
     *
     * @return array returns an array containing the available shipping methods
     */
    public function getMethods(): array
    {
        return $this->module->getShippingMethodNames();
    }

    /**
     * Retrieves the carrier method for the given carrier.
     *
     * @param int|string|\Carrier $carrier the carrier ID or carrier object
     *
     * @return string|null the carrier method if found, null otherwise
     */
    public function getCarrierMethod($carrier): ?string
    {
        if (is_int($carrier) || is_string($carrier)) {
            $carrier = new \Carrier($carrier);
        }

        $carrierMethod = null;

        if (BpostTools::isBpostCarrier($carrier)) {
            $sql = new \DbQuery();
            $sql->select('shipping_method');
            $sql->from('bpostshm_carrier_history');
            $sql->where('id_carrier = ' . (int) $carrier->id);
            $shippingMethod = \Db::getInstance()->getValue($sql);
            if ($shippingMethod) {
                $methods = $this->getMethods();
                if (isset($methods[$shippingMethod])) {
                    $carrierMethod = $shippingMethod;
                }
            }
        }

        return $carrierMethod;
    }

    /**
     * Retrieves the delivery method for an order.
     *
     * @param \Order $order The order object
     * @param bool $retour Whether it is a return delivery or not
     *
     * @return string|null The delivery method or null if no method found
     *
     * @throws CarrierException If there is no delivery address for the order
     */
    public function getDeliveryMethod(\Order $order, bool $retour): ?string
    {
        $carrier = new \Carrier($order->id_carrier);

        $deliveryMethod = null;
        $method = $this->getCarrierMethod($carrier);
        if ($method) {
            $deliveryAddress = new \Address($order->id_address_delivery);
            if (\Validate::isLoadedObject($deliveryAddress)) {
                $isoCountry = \Country::getIsoById($deliveryAddress->id_country);
                $intl = BpostTools::isIntl($isoCountry) ? '_INTL' : '';
            } else {
                throw new CarrierException($this->translator->trans('No delivery address for order #%order%!', ['%order%' => $order->id], 'Modules.Bpostshm.Admin'));
            }

            $isBusiness = \Configuration::get(
                "BPOSTSHM_DELIVERY_OPTIONS_HOME{$intl}_BUSINESS",
                null,
                $order->id_shop_group,
                $order->id_shop
            ) == 'business';

            switch ($method) {
                case $retour:
                case 'HOME':
                    if ($retour) {
                        $deliveryMethod = $intl ? 'World Easy Return' : 'Easy Retour';
                    } else {
                        if ($isBusiness) {
                            $deliveryMethod = $intl ? 'World Business' : '24h business';
                        } else {
                            $deliveryMethod = $intl ? 'World Express Pro' : '24h Pro';
                        }
                    }
                    break;
                case 'PICKUP':
                    $deliveryMethod = $intl ? '@bpost international' : '@bpost';
                    break;
                case 'PARCEL_LOCKER':
                    if ($intl) {
                        throw new CarrierException($this->translator->trans('Parcel locker is not available outside Belgium! (about order #%order%)', ['%order%' => $order->id], 'Modules.Bpostshm.Admin'));
                    }
                    $deliveryMethod = 'Parcel locker';
                    break;
            }

            $deliveryMethods = $this->module->getDeliveryMethods();
            if (!isset($deliveryMethods[$deliveryMethod])) {
                throw new CarrierException($this->translator->trans('The delivery method "%method%" is not referenced! (about order #%order%)', ['%method%' => $deliveryMethod, '%order%' => $order->id], 'Modules.Bpostshm.Admin'));
            }
        }

        return $deliveryMethod;
    }

    /**
     * Retrieves the carriers associated with the BpostShm module.
     *
     * @return array An array containing information about the carriers.
     *               The array structure is as follows:
     *               [
     *               'BPOSTSHM_ID_CARRIER_$key' => [
     *               'name' => (string) The name of the carrier,
     *               'id_carrier' => (int) The ID of the carrier,
     *               'carrier' => (Carrier|null) An instance of the Carrier class associated with the given ID,
     *               ],
     *               ...
     *               ]
     */
    public function getCarriers(): array
    {
        $carriers = [];
        foreach ($this->getMethods() as $key => $name) {
            $idCarrier = (int) \Configuration::getGlobalValue("BPOSTSHM_ID_CARRIER_$key");
            $carriers["BPOSTSHM_ID_CARRIER_$key"] = [
                'name' => $name,
                'id_carrier' => $idCarrier,
                'carrier' => $idCarrier > 0 ? new \Carrier($idCarrier) : null,
            ];
        }

        return $carriers;
    }

    /**
     * Retrieves the list of carrier options.
     *
     * The method merges an empty carrier option with the list of carriers retrieved
     * using the current context language. It loops through the list of carriers and
     * appends the carrier ID to the carrier name in the format "name (ID id_carrier)".
     *
     * @return array the list of carrier options as an associative array
     */
    public function getCarrierOptions(): array
    {
        $allCarriers = array_merge(
            [['id_carrier' => '0', 'name' => '']],
            \Carrier::getCarriers($this->context->language->id, false, false, false, null, \Carrier::ALL_CARRIERS)
        );

        foreach ($allCarriers as &$c) {
            if ($c['id_carrier']) {
                $c['name'] = sprintf('%s (ID %d)', $c['name'], $c['id_carrier']);
            }
        }

        return $allCarriers;
    }

    /**
     * Check if all carriers in the configuration are valid and linked to existing carriers.
     *
     * @throws CarrierException if a carrier is linked to a deleted carrier or an unknown carrier
     */
    public function checkCarriersConfig()
    {
        $hasCarrier = false;
        foreach ($this->getCarriers() as $key => $carrierConfig) {
            if (null !== $carrierConfig['carrier']) {
                if (\Validate::isLoadedObject($carrierConfig['carrier'])) {
                    if ($carrierConfig['carrier']->deleted) {
                        throw new CarrierException($this->translator->trans('The "%carrier_name%" bpost delivery method is linked with a deleted carrier! (ID %id%)', ['%id%' => $carrierConfig['id_carrier'], '%carrier_name%' => $carrierConfig['name']], 'Modules.Bpostshm.Admin'));
                    } else {
                        $sql = new \DbQuery();
                        $sql->select('shipping_method');
                        $sql->from('bpostshm_carrier_history');
                        $sql->where('id_carrier = ' . (int) $carrierConfig['id_carrier']);
                        $shippingMethod1 = \Db::getInstance()->getValue($sql);
                        if (!$shippingMethod1) {
                            throw new CarrierException($this->translator->trans('The "%carrier_name%" bpost delivery method is not found in `bpostshm_carrier_history` table! (ID %id%)', ['%id%' => $carrierConfig['id_carrier'], '%carrier_name%' => $carrierConfig['name']], 'Modules.Bpostshm.Admin'));
                        }
                        $shippingMethod2 = \Tools::strtoupper(str_replace('BPOSTSHM_ID_CARRIER_', '', $key));
                        if ($shippingMethod1 != $shippingMethod2) {
                            throw new CarrierException($this->translator->trans('The "%carrier_name%" bpost delivery method is linked with "%shipping_method1%" method instead of "%shipping_method2%" method in `bpostshm_carrier_history` table! (ID %id%)', ['%id%' => $carrierConfig['id_carrier'], '%carrier_name%' => $carrierConfig['name'], '%shipping_method1%' => $shippingMethod1, '%shipping_method2%' => $shippingMethod2], 'Modules.Bpostshm.Admin'));
                        }
                    }
                    $hasCarrier = true;
                } else {
                    throw new CarrierException($this->translator->trans('The "%carrier_name%" bpost delivery method is linked with an unknown carrier! (ID %id%)', ['%id%' => $carrierConfig['id_carrier'], '%carrier_name%' => $carrierConfig['name']], 'Modules.Bpostshm.Admin'));
                }
            }
        }
        if (!$hasCarrier) {
            throw new CarrierException($this->translator->trans('No carrier is linked for one of the bpost delivery method.', [], 'Modules.Bpostshm.Admin'));
        }
    }

    /**
     * Updates carrier configuration.
     *
     * This method updates the carrier configuration based on the given carrier array.
     *
     * @param string[] $carriers The array containing the carrier configuration.
     *                           The keys of the array should be the carrier configuration keys
     *                           (e.g., 'BPOSTSHM_ID_CARRIER_HOME') and the values should be the new carrier IDs.
     *
     * @return void
     *
     * @throws CarrierException
     */
    public function updateCarrierConfig(array $carriers)
    {
        $duplicatedCarriers = [];
        foreach ($carriers as $key => $newIdCarrier) {
            $currentIdCarrier = (int) \Configuration::getGlobalValue($key);
            $shippingMethod = str_replace('BPOSTSHM_ID_CARRIER_', '', $key);

            if ($newIdCarrier <= 0 && $currentIdCarrier > 0) {
                // remove current carrier link
                $currentCarrier = new \Carrier($currentIdCarrier);
                if (\Validate::isLoadedObject($currentCarrier) && !$currentCarrier->deleted) {
                    $duplicatedCarriers[$currentIdCarrier] = $this->duplicateCarrier(
                        $currentCarrier,
                        false,
                        '',
                        $shippingMethod
                    );
                }
                \Configuration::updateGlobalValue($key, 0);
            } elseif ($newIdCarrier <= 0) {
                \Configuration::updateGlobalValue($key, 0);
            } elseif ($newIdCarrier != $currentIdCarrier) {
                $currentCarrier = new \Carrier($currentIdCarrier);
                if (\Validate::isLoadedObject($currentCarrier) && !$currentCarrier->deleted) {
                    // remove current carrier link
                    $duplicatedCarriers[$currentIdCarrier] = $this->duplicateCarrier(
                        $currentCarrier,
                        false,
                        '',
                        $shippingMethod
                    );
                }
                \Configuration::updateGlobalValue($key, 0); // config updated temporarily

                if (isset($duplicatedCarriers[$newIdCarrier])) {
                    $newCarrier = $duplicatedCarriers[$newIdCarrier];
                    $newCarrier->deleted = false;
                    $newCarrier->is_module = true;
                    $newCarrier->external_module_name = $this->module->name;
                    $newCarrier->update();
                    \Configuration::updateGlobalValue($key, $newCarrier->id);
                } else {
                    $newCarrier = new \Carrier($newIdCarrier);
                    if (\Validate::isLoadedObject($newCarrier)) {
                        // add new carrier link
                        $carrier = $this->duplicateCarrier($newCarrier, true, $this->module->name, $shippingMethod);
                        \Configuration::updateGlobalValue($key, $carrier->id);
                    }
                }
            }
        }
    }

    /**
     * Duplicates a carrier and performs necessary updates.
     *
     * @param \Carrier $currentCarrier the carrier object to be duplicated
     * @param bool $is_module flag indicating if the carrier is a module carrier
     * @param string $external_module_name the external module name for the carrier
     * @param string $shippingMethod the shipping method for the carrier
     *
     * @return false|\Carrier the duplicated carrier object
     *
     * @throws CarrierException if an error occurred while saving the carrier
     */
    private function duplicateCarrier(
        \Carrier $currentCarrier,
        bool $is_module,
        string $external_module_name,
        string $shippingMethod
    ) {
        $newCarrier = $currentCarrier->duplicateObject();

        if (\Validate::isLoadedObject($newCarrier)) {
            // Set flag deteled to true for historization
            $currentCarrier->deleted = true;
            $currentCarrier->update();

            $newCarrier->deleted = false;
            $newCarrier->is_module = $is_module;
            $newCarrier->external_module_name = $external_module_name;
            $newCarrier->update();

            $groups = [];
            foreach ($currentCarrier->getGroups() as $group) {
                $groups[$group['id_group']] = $group['id_group'];
            }
            $newCarrier->setGroups($groups);

            if (\Configuration::get('PS_CARRIER_DEFAULT') == $currentCarrier->id) {
                \Configuration::updateGlobalValue('PS_CARRIER_DEFAULT', (int) $newCarrier->id);
            }
            if (\Shop::isFeatureActive()) {
                foreach (\Shop::getShops(true) as $shop) {
                    if (\Configuration::get(
                        'PS_CARRIER_DEFAULT',
                        null,
                        $shop['id_shop_group'],
                        $shop['id_shop']
                    ) == $currentCarrier->id) {
                        \Configuration::updateValue(
                            'PS_CARRIER_DEFAULT',
                            (int) $newCarrier->id,
                            false,
                            $shop['id_shop_group'],
                            $shop['id_shop']
                        );
                    }
                }
            }

            \Db::getInstance()->insert(
                'bpostshm_carrier_history',
                [
                    'id_carrier' => (int) $newCarrier->id,
                    'shipping_method' => $shippingMethod,
                ],
                false,
                true,
                \Db::REPLACE
            );

            $this->duplicateLogo($newCarrier->id, $currentCarrier->id);
        } else {
            throw new CarrierException($this->translator->trans('An error occurred while saving carrier.', [], 'Modules.Bpostshm.Admin'));
        }

        return $newCarrier;
    }

    /**
     * Method to duplicate a logo for a carrier
     *
     * @param int $new_id The ID of the new carrier
     * @param int $old_id The ID of the old carrier
     *
     * @return void
     */
    private function duplicateLogo(int $new_id, int $old_id)
    {
        $old_logo = _PS_SHIP_IMG_DIR_ . '/' . $old_id . '.jpg';
        if (file_exists($old_logo)) {
            copy($old_logo, _PS_SHIP_IMG_DIR_ . '/' . $new_id . '.jpg');
        }

        $old_tmp_logo = _PS_TMP_IMG_DIR_ . '/carrier_mini_' . $old_id . '.jpg';
        if (file_exists($old_tmp_logo)) {
            if (!isset($_FILES['logo'])) {
                copy($old_tmp_logo, _PS_TMP_IMG_DIR_ . '/carrier_mini_' . $new_id . '.jpg');
            }
            unlink($old_tmp_logo);
        }
    }

    /**
     * Translates carrier's name
     *
     * @param string $name the original carrier name
     * @param \Language $language the language object
     *
     * @return string the translated carrier name
     */
    public function getTranslatedCarrierName(string $name, \Language $language): string
    {
        if ($name) {
            $languages = ['en' => 0, 'fr' => 1, 'nl' => 2];
            $code = substr($language->iso_code, 0, 2);
            if (!isset($languages[$code])) {
                $code = 'en';
            }

            $names = explode('/', $name);
            if (count($names) === 3) {
                $name = trim($names[$languages[$code]]);
            }
        }

        return $name;
    }
}
