<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Service;

use Cart;
use Order;
use PrestaShop\Module\BpostShm\Utils\BpostTools;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * Class CustomerContactService
 */
class CustomerContactService
{
    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * @var \Context
     */
    private $context;

    /**
     * @var \Db
     */
    private $database;

    /**
     * @var TranslatorInterface
     */
    private $translator;

    /**
     * Constructor
     *
     * @param \BpostShm $module
     */
    public function __construct(\BpostShm $module)
    {
        $this->module = $module;
        $this->context = $module->getContext();
        $this->database = $module->getDatabase();
        $this->translator = $module->getTranslator();
    }

    /**
     * Retrieves the customer contact information for a given order.
     *
     * @param int|string|\Order $order the order ID or an Order object
     *
     * @return array An associative array containing the customer contact information.
     *               The array has the following keys:
     *               - email (string): The email address of the customer.
     *               - mobile (string): The mobile phone number of the customer.
     *               - reduced_mobility_zone (bool): Whether the customer requires reduced mobility assistance.
     */
    public function getCustomerContactForOrder($order): array
    {
        if (!$order) {
            return $this->getCustomerContact(0, 0);
        }

        if (is_int($order) || is_string($order)) {
            $order = new \Order($order);
        }

        return $this->getCustomerContact(\Cart::getCartIdByOrderId($order->id), $order->id_carrier);
    }

    /**
     * Retrieves customer contact information for a given cart and carrier.
     *
     * @param int $idCart the ID of the cart
     * @param int $idCarrier the ID of the carrier
     * @param object|null $customer An instance of the customer object. (optional)
     * @param object|null $address An instance of the address object. (optional)
     *
     * @return array An associative array containing the customer contact information.
     *               The array has the following keys:
     *               - email (string): The email address of the customer.
     *               - mobile (string): The mobile phone number of the customer.
     *               - reduced_mobility_zone (bool): Whether the customer requires reduced mobility assistance.
     */
    public function getCustomerContact(int $idCart, int $idCarrier, ?object $customer = null, ?object $address = null): array
    {
        $sql = new \DbQuery();
        $sql->select('email, mobile, reduced_mobility_zone');
        $sql->from('bpostshm_cart');
        $sql->where('id_cart = ' . $idCart . ' AND id_carrier = ' . $idCarrier);

        $infos = \Db::getInstance()->getRow($sql);
        if (!$infos) {
            $infos = [
                'email' => '',
                'mobile' => '',
                'reduced_mobility_zone' => false,
            ];
        }

        if (!$infos['email'] && $customer && !empty($customer->email)) {
            $infos['email'] = $customer->email;
        }
        if (!$infos['mobile'] && $address && !empty($address->phone_mobile)) {
            $infos['mobile'] = $address->phone_mobile;
        }

        return $infos;
    }

    /**
     * Set customer contact information for a cart.
     *
     * @param int $idCart the ID of the cart
     * @param int $idCarrier the ID of the carrier
     * @param string $email the email address of the customer
     * @param string $mobile the mobile number of the customer
     * @param int $reducedMobilityZone the ID of the reduced mobility zone
     *
     * @return void
     *
     * @throws \PrestaShopException if the cart is not found
     */
    public function setCustomerContact(
        int $idCart,
        int $idCarrier,
        string $email,
        string $mobile,
        int $reducedMobilityZone
    ) {
        $cart = new \Cart($idCart);
        if (!\Validate::isLoadedObject($cart)) {
            throw new \PrestaShopException($this->translator->trans('Set Customer Contact: Cart not found (%id%).', ['%id%' => $idCart], 'Modules.Bpostshm.Admin'));
        }
        \Db::getInstance()->insert(
            'bpostshm_cart',
            [
                'id_cart' => (int) $cart->id,
                'id_carrier' => $idCarrier,
                'email' => pSQL($email),
                'mobile' => pSQL($mobile),
                'reduced_mobility_zone' => $reducedMobilityZone,
            ],
            false,
            false,
            \Db::ON_DUPLICATE_KEY
        );
    }

    /**
     * Checks customer contact information for a cart.
     *
     * @param array $customerContact the customer contact information
     *
     * @return array|bool errors in an array or true if we have required information
     */
    public function checkCustomerContact(array $customerContact)
    {
        $errors = [];

        if (!\Validate::isEmail($customerContact['email'])) {
            $errors['noCustomerEmail'] = $this->translator->trans(
                'Please provide a valid email address for the parcel locker.',
                [],
                'Modules.Bpostshm.Shop'
            );
        }

        if ($customerContact['mobile'] && !BpostTools::isBeMobileNumber($customerContact['mobile'])) {
            $errors['noCustomerMobile'] = $this->translator->trans(
                'Please provide a valid mobile phone for the parcel locker.',
                [],
                'Modules.Bpostshm.Shop'
            );
        }

        return count($errors) ? $errors : true;
    }
}
