<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Service;

use Carrier;
use Cart;
use PrestaShop\Module\BpostShm\Utils\BpostTools;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * Class DeliveryDateService
 */
class DeliveryDateService
{
    const FORMAT = 'Y-m-d';

    const CACHED_CONFIG_KEY = 'BPOSTSHM_CACHED_DATES';

    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * @var CarrierService
     */
    private $carrierService;

    /**
     * @var \Context
     */
    private $context;

    /**
     * @var \Db
     */
    private $database;

    /**
     * @var TranslatorInterface
     */
    private $translator;

    /**
     * @var array
     */
    private $cachedDates;

    /**
     * Constructor
     *
     * @param \BpostShm $module
     * @param CarrierService $carrierService
     *
     * @throws \Exception
     */
    public function __construct(\BpostShm $module, CarrierService $carrierService)
    {
        $this->module = $module;
        $this->context = $module->getContext();
        $this->database = $module->getDatabase();
        $this->translator = $module->getTranslator();
        $this->carrierService = $carrierService;

        $storedDates = (string) \Configuration::get(self::CACHED_CONFIG_KEY);
        if (!empty($storedDates)) {
            $storedDates = json_decode($storedDates, true);
            if ((int) date('Ymd') < (int) $storedDates['expires']) {
                if (!empty($storedDates['dates']) && is_array($storedDates['dates'])) {
                    $this->setCachedDates($storedDates['dates']);
                }
            }
        }
        $this->generateNextDates();
    }

    /**
     * Shows delivery date for a given cart.
     *
     * @param int $idCart The ID of the cart
     * @param string|null $country The country code to check delivery for. Defaults to null.
     *
     * @return bool Return false if delivery date is not available, true otherwise
     */
    public function showDeliveryDate(int $idCart, ?string $country = null): bool
    {
        if ($country && BpostTools::isIntl($country) === false) {
            return false;
        }

        $cart = new \Cart($idCart);

        if (!\Configuration::get('BPOSTSHM_DELIVERY_DATE_ENABLED', null, $cart->id_shop_group, $cart->id_shop)) {
            return false;
        }

        if (\Configuration::get('BPOSTSHM_DELIVERY_DATE_HIDE_OOS', null, $cart->id_shop_group, $cart->id_shop)) {
            $cart = new \Cart($idCart);
            if (\Validate::isLoadedObject($cart)) {
                foreach ($cart->getProducts() as $product) {
                    if ((isset($product['quantity_available']) && $product['quantity_available'] > 0)
                        || (isset($product['product_quantity']) && $product['product_quantity'] > 0)) {
                        return true;
                    }
                }

                return false;
            }
        }

        return true;
    }

    /**
     * Get the delivery date for a given cart and carrier.
     *
     * @param int $idCart The ID of the cart
     * @param int $idCarrier The ID of the carrier
     *
     * @return string|null The delivery date, or null if not found
     */
    public function getDeliveryDate(int $idCart, int $idCarrier): ?string
    {
        $sql = new \DbQuery();
        $sql->select('delivery_date');
        $sql->from('bpostshm_cart');
        $sql->where('id_cart = ' . $idCart . ' AND id_carrier = ' . $idCarrier);

        return \Db::getInstance()->getValue($sql);
    }

    /**
     * Retrieves the delivery date for a given order.
     *
     * @param Order|int|string $order the order object, or the order's ID as int or string
     *
     * @return string|null the delivery date in format 'YYYY-MM-DD', or null if the order is invalid
     */
    public function getDeliveryDateForOrder($order): ?string
    {
        if (!$order) {
            return null;
        }

        if (is_int($order) || is_string($order)) {
            $order = new Order($order);
        }

        return $this->getDeliveryDate(\Cart::getCartIdByOrderId($order->id), $order->id_carrier);
    }

    /**
     * Set the delivery date for a cart and carrier.
     *
     * @param int $idCart The ID of the cart
     * @param int $idCarrier The ID of the carrier
     * @param string|null $date The delivery date in YYYY-MM-DD format (optional)
     *
     * @throws \PrestaShopException If the cart is not found or the date is incorrect
     */
    public function setDeliveryDate(int $idCart, int $idCarrier, ?string $date)
    {
        $cart = new \Cart($idCart);
        if (!\Validate::isLoadedObject($cart)) {
            throw new \PrestaShopException($this->translator->trans('Set Delivery Date: Cart not found (%id%).', ['%id%' => $idCart], 'Modules.Bpostshm.Admin'));
        }
        if ($date && !\Validate::isDate($date) || !strtotime($date)) {
            throw new \PrestaShopException($this->translator->trans('Set Delivery Date: Date is incorrect %date%.', ['%date%' => $date], 'Modules.Bpostshm.Admin'));
        }
        \Db::getInstance()->insert(
            'bpostshm_cart',
            [
                'id_cart' => $cart->id,
                'id_carrier' => $idCarrier,
                'delivery_date' => $date ?: null,
            ],
            false,
            false,
            \Db::ON_DUPLICATE_KEY
        );
    }

    /**
     * Retrieves the delivery dates for the specified carrier.
     *
     * @param mixed $carrier the carrier to retrieve delivery dates for
     * @param \Cart|null $cart the shopping cart to know the total order
     *
     * @return array the array of delivery dates
     */
    public function getDeliveryDates($carrier, ?\Cart $cart): array
    {
        $method = $this->carrierService->getCarrierMethod($carrier);
        if (!$method) {
            throw new \PrestaShopException(sprintf('Invalid bpost carrier (ID %d)', $carrier instanceof \Carrier ? $carrier->id : (int) $carrier));
        }
        $idShopGroup = \Validate::isLoadedObject($cart) ? $cart->id_shop_group : null;
        $idShop = \Validate::isLoadedObject($cart) ? $cart->id_shop : null;
        $withSaturday = false;
        if (\Configuration::get("BPOSTSHM_DELIVERY_OPTIONS_{$method}_SATURDAY_ENABLED", null, $idShopGroup, $idShop)) {
            $withSaturday = true;
            $from = (float) \Configuration::get(
                "BPOSTSHM_DELIVERY_OPTIONS_{$method}_SATURDAY_FROM",
                null,
                $idShopGroup,
                $idShop
            );
            if ($from > 0) {
                $withSaturday = false;
                if (\Validate::isLoadedObject($cart)) {
                    $orderTotal = (float) $cart->getOrderTotal(false, \Cart::ONLY_PRODUCTS);
                    if ($orderTotal >= $from) {
                        $withSaturday = true;
                    }
                }
            }
        }
        $nDates = (int) \Configuration::get('BPOSTSHM_DELIVERY_DATE_NUM_DATES_SHOWN', null, $idShopGroup, $idShop) ?: 1;
        $dropDate = $this->getDropDateToday($withSaturday, $idShopGroup, $idShop);

        return $this->getNDeliveryDatesFrom($nDates, $dropDate, $withSaturday);
    }

    /**
     * Retrieves the drop date based on the specified date.
     *
     * @param int $dateFrom The date from which to calculate the drop date. Default value is 0.
     * @param null $idShopGroup
     * @param null $idShop
     *
     * @return int|false the drop date if found, otherwise false
     */
    public function getDropDate(int $dateFrom = 0, $idShopGroup = null, $idShop = null)
    {
        $dropDate = false;
        if ($dateFrom > 0) {
            $workdays = $this->cachedDates['workdays'];
            $workDates = array_keys($workdays);
            foreach ($workDates as $date) {
                if ($date < $dateFrom) {
                    $dropDate = $date;
                } else {
                    break;
                }
            }
        } else {
            $dropDate = $this->getDropDateToday(false, $idShopGroup, $idShop);
        }

        return $dropDate;
    }

    /**
     * Retrieves the day for the given date from the cachedDates array.
     *
     * @param string $date the date for which to retrieve the day
     *
     * @return int|false the day of the given date if found in the cachedDates array, false otherwise
     */
    public function getDay(string $date)
    {
        return isset($this->cachedDates['all'][$date]) ? (int) $this->cachedDates['all'][$date] : false;
    }

    /**
     * Checks if a given date is a Saturday.
     *
     * @param string $date the date to check in format 'YYYY-MM-DD'
     *
     * @return bool returns true if the given date is a Saturday, false otherwise
     */
    public function isSaturday(string $date): bool
    {
        return 6 === $this->getDay($date);
    }

    /**
     * Sets the cached delivery dates.
     *
     * @param array $dates the array of delivery dates to be cached
     *
     * @return void
     */
    protected function setCachedDates(array $dates = [])
    {
        $cachedDates = [
            'all' => $dates,
            'workdays' => array_intersect($dates, range(1, 5)),
            'workdays_with_saturday' => array_intersect($dates, range(1, 6)),
        ];
        $this->cachedDates = $cachedDates;
    }

    /**
     * Generates and stores the next 30 dates.
     *
     * @return bool true on success, false on failure
     *
     * @throws \Exception
     */
    protected function generateNextDates(): bool
    {
        $dates = $this->generateDates(30, time());
        $this->setCachedDates($dates);
        $tomorrow = new \DateTime();
        $dateStored = [
            'expires' => (int) $tomorrow->modify('+1 day')->format('Ymd'),
            'dates' => $dates,
        ];

        return \Configuration::updateValue(self::CACHED_CONFIG_KEY, json_encode($dateStored));
    }

    /**
     * Generates an array of dates based on the specified number of days and timestamp.
     *
     * @param int $nDays the number of days
     * @param int $timestamp the timestamp to start generating dates from
     *
     * @return array the array of generated dates
     *
     * @throws \Exception
     */
    protected function generateDates(int $nDays, int $timestamp): array
    {
        // $iso = 'R30/'.date('Y-m-d').'T00:00:00Z/P1D';
        $iso = 'R' . $nDays . '/' . date('Y-m-d', $timestamp) . 'T00:00:00Z/P1D';
        $period = new \DatePeriod($iso);
        $dates = [];

        $easter = new \DateTime(date('d-m-Y', \easter_date()));
        $paak = clone $easter;
        $paak->modify('first monday');
        $ascension = clone $easter;
        $ascension->modify('+39 days');
        $pentecost = clone $easter;
        $pentecost->modify('+50 days');
        $holsFixed = [
            '01-01-*',
            '01-05-*',
            '21-07-*',
            '15-08-*',
            '01-11-*',
            '11-01-*',
            '25-12-*',
        ];
        $holsVar = [
            $paak->format(self::FORMAT),
            $ascension->format(self::FORMAT),
            $pentecost->format(self::FORMAT),
        ];
        foreach ($period as $date) {
            $dateFormat = $date->format(self::FORMAT);
            $isHoliday = in_array($date->format('d-m-*'), $holsFixed) || in_array($dateFormat, $holsVar);

            $dates[$dateFormat] = $isHoliday ? 0 : (int) $date->format('N');
        }

        return $dates;
    }

    /**
     * Retrieves a specified number of dates after a given date.
     *
     * @param int $nDays the number of days to retrieve after the given date
     * @param string $from the date from which to retrieve the following dates
     * @param array $dates the array of available dates
     *
     * @return string the date specified number of days after the given date
     */
    protected function getNDaysAfter(int $nDays, string $from, array $dates): string
    {
        $nDaysAfter = $from;
        if ($nDays) {
            foreach ($dates as $date) {
                if ($date < $from) {
                    continue;
                } elseif (0 == $nDays--) {
                    $nDaysAfter = $date;
                    break;
                }
            }
        }

        return $nDaysAfter;
    }

    /**
     * Retrieves a specified number of delivery dates starting from a given date,
     * considering the inclusion of Saturdays.
     *
     * @param int $nDates the number of delivery dates to retrieve
     * @param string $from the starting date, in format 'YYYY-MM-DD'
     * @param bool $withSaturday specifies whether to include Saturdays as valid delivery dates
     *
     * @return string[] an array of delivery dates in format 'YYYY-MM-DD'
     *
     * @throws \PrestaShopException if an invalid number of delivery dates is provided
     */
    protected function getNDeliveryDatesFrom(int $nDates, string $from, bool $withSaturday): array
    {
        if ($nDates < 1) {
            throw new \PrestaShopException('Invalid number of delivery dates');
        }

        $delDates = array_keys($this->cachedDates['workdays' . ($withSaturday ? '_with_saturday' : '')]);
        $firstDate = $this->getNDaysAfter(1, $from, $delDates);
        if ($nDates > 1 || 6 !== $this->cachedDates['all'][$firstDate]) {
            --$nDates;
        }

        $dates = [$firstDate];
        if ($nDates) {
            foreach ($delDates as $date) {
                if ($date <= $firstDate) {
                    continue;
                } elseif ($nDates-- > 0) {
                    $dates[] = $date;
                }
            }
        }

        return $dates;
    }

    /**
     * Retrieves the drop date for today, considering the inclusion of Saturdays.
     *
     * @param bool $withSaturday specifies whether to include Saturdays as valid drop dates
     * @param $idShopGroup
     * @param $idShop
     *
     * @return string the drop date in format 'YYYY-MM-DD'
     */
    protected function getDropDateToday(bool $withSaturday, $idShopGroup = null, $idShop = null): string
    {
        $delayDays = (int) \Configuration::get('BPOSTSHM_DELIVERY_DATE_SHIP_DELAY_DAYS', null, $idShopGroup, $idShop);
        $cutoff = (string) \Configuration::get('BPOSTSHM_DELIVERY_DATE_CUTOFF_TIME', null, $idShopGroup, $idShop);

        $today = date(self::FORMAT);
        $workdays = $this->cachedDates['workdays' . ($withSaturday ? '_with_saturday' : '')];
        $workDates = array_keys($workdays);
        $isWorkday = in_array($today, $workDates);

        $now = date('Hi');
        if ($isWorkday && $now >= $cutoff) {
            ++$delayDays;
        }

        $startDay = $today;
        foreach ($workDates as $date) {
            if ($date >= $startDay) {
                $startDay = $date;
                break;
            }
        }

        return $this->getNDaysAfter($delayDays, $startDay, $workDates);
    }
}
