<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Service;

use Carrier;
use Cart;
use Country;
use PrestaShop\Module\BpostShm\Service\Box\Option\Option;
use PrestaShop\Module\BpostShm\Utils\BpostTools;
use Product;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * Class DeliveryOptionsService
 */
class DeliveryOptionsService
{
    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * @var \Context
     */
    private $context;

    /**
     * @var \Db
     */
    private $database;

    /**
     * @var TranslatorInterface
     */
    private $translator;

    /**
     * @var PickupService
     */
    private $pickupService;

    /**
     * @var CarrierService
     */
    private $carrierService;

    /**
     * @var DeliveryDateService
     */
    private $deliveryDateService;

    /**
     * @var ParcelService
     */
    private $parcelService;

    /**
     * @var array
     */
    private $cachedShippingCosts;

    /**
     * Constructor
     *
     * @param \BpostShm $module
     * @param PickupService $pickupService
     * @param CarrierService $carrierService
     * @param DeliveryDateService $deliveryDateService
     * @param ParcelService $parcelService
     */
    public function __construct(
        \BpostShm $module,
        PickupService $pickupService,
        CarrierService $carrierService,
        DeliveryDateService $deliveryDateService,
        ParcelService $parcelService
    ) {
        $this->module = $module;
        $this->context = $module->getContext();
        $this->database = $module->getDatabase();
        $this->translator = $module->getTranslator();
        $this->pickupService = $pickupService;
        $this->carrierService = $carrierService;
        $this->deliveryDateService = $deliveryDateService;
        $this->parcelService = $parcelService;
        $this->cachedShippingCosts = [];
    }

    /**
     * Calculate additional shipping cost for a given cart and carrier.
     *
     * @param mixed $cart the cart object
     * @param int $idCarrier the ID of the carrier
     * @param int|null $idLang the ID of the language (optional)
     *
     * @return float the additional shipping cost
     */
    public function getAdditionalShippingCost($cart, int $idCarrier, ?int $idLang = null)
    {
        $cost = 0;

        if (!\Validate::isLoadedObject($cart) || !$idCarrier) {
            return $cost;
        }

        if (!$idLang) {
            $idLang = (int) $this->context->language->id;
        }
        $deliveryDate = $this->deliveryDateService->getDeliveryDate($cart->id, $idCarrier);
        $cacheKey = implode('|', [$cart->id, $idCarrier, $idLang, $deliveryDate]);
        if (isset($this->cachedShippingCosts[$cacheKey])) {
            return $this->cachedShippingCosts[$cacheKey];
        }

        $carrier = new \Carrier($idCarrier);
        if (!BpostTools::isBpostCarrier($carrier)) {
            $this->cachedShippingCosts[$cacheKey] = $cost;

            return $cost;
        }
        $method = $this->carrierService->getCarrierMethod($carrier);
        if (!$method) {
            $this->cachedShippingCosts[$cacheKey] = $cost;

            return $cost;
        }

        if (!$this->parcelService->canShip($cart)) {
            $this->cachedShippingCosts[$cacheKey] = false;

            return false;
        }

        $deliveryAddress = new \Address($cart->id_address_delivery);
        if (\Validate::isLoadedObject($deliveryAddress)) {
            $isoCountry = \Country::getIsoById($deliveryAddress->id_country);
        } else {
            $isoCountry = $this->module->getDefaultCountryCode();
        }
        $isIntl = (bool) BpostTools::isIntl($isoCountry);

        if ($method != 'HOME') {
            // PICKUP OR PARCEL_LOCKER
            $pickupAddress = $this->pickupService->getSelectedPickupAddress($carrier->id, $idLang, $cart->id);
            if ($pickupAddress) {
                $isIntl = (bool) BpostTools::isIntl(\Country::getIsoById($pickupAddress->id_country));
            }
        }

        $orderTotal = (float) $cart->getOrderTotal(false, \Cart::ONLY_PRODUCTS);
        $options = $this->getDeliveryOptions(
            $method,
            $isIntl,
            $deliveryDate && $this->deliveryDateService->isSaturday($deliveryDate),
            $orderTotal,
            $cart->id_shop_group,
            $cart->id_shop
        );
        foreach ($options as $option => $hasOption) {
            if ($hasOption) {
                $configKey = $this->getDeliveryOptionConfigKey($method, $isIntl, $option . '_COST');
                $configCost = (float) \Configuration::get($configKey, null, $cart->id_shop_group, $cart->id_shop);
                if ($configCost > 0) {
                    $cost += $configCost;
                }
            }
        }

        $this->cachedShippingCosts[$cacheKey] = $cost;

        return $this->cachedShippingCosts[$cacheKey];
    }

    /**
     * Get the available delivery options based on the given parameters.
     *
     * @param string $carrierMethod the carrier method
     * @param bool $isIntl whether the delivery is international or not
     * @param bool $isSaturday whether the delivery is on a Saturday or not
     * @param float $orderTotal The total amount of the order (tax excl.)
     * @param int|null $idShopGroup
     * @param int|null $idShop
     *
     * @return array The available delivery options as an associative array with the following keys:
     *               - 'saturday' (int): Whether Saturday delivery is enabled (0 or 1).
     *               - 'signature' (int): Whether signature is required (0 or 1).
     *               - 'insurance' (int): Whether insurance is enabled (0 or 1).
     *               - '2nd_presentation' (int): Whether second presentation is enabled (0 or 1).
     */
    public function getDeliveryOptions(
        string $carrierMethod,
        bool $isIntl,
        bool $isSaturday,
        float $orderTotal,
        ?int $idShopGroup = null,
        ?int $idShop = null
    ): array {
        $options = [
            'saturday' => 0,
            'signature' => 0,
            'insurance' => 0,
            '2nd_presentation' => 0,
        ];

        $configPrefix = $this->getDeliveryOptionConfigKey($carrierMethod, $isIntl);

        if (!$isIntl) {
            if ($isSaturday && \Configuration::get($configPrefix . 'SATURDAY_ENABLED', null, $idShopGroup, $idShop)) {
                if ($orderTotal >= (float) \Configuration::get(
                    $configPrefix . 'SATURDAY_FROM',
                    null,
                    $idShopGroup,
                    $idShop
                )) {
                    $options['saturday'] = 1;
                }
            }

            if ($carrierMethod == 'HOME') {
                $insuranceType = \Configuration::get($configPrefix . 'INSURANCE', null, $idShopGroup, $idShop);
                if ($insuranceType) {
                    $from = (float) \Configuration::get(
                        $configPrefix . \Tools::strtoupper($insuranceType) . '_FROM',
                        null,
                        $idShopGroup,
                        $idShop
                    );
                    if ($orderTotal >= $from) {
                        $options[$insuranceType] = 1;
                    }
                }

                if (\Configuration::get($configPrefix . '2ND_PRESENTATION_ENABLED', null, $idShopGroup, $idShop)) {
                    if ($orderTotal >= (float) \Configuration::get(
                        $configPrefix . '2ND_PRESENTATION_FROM',
                        null,
                        $idShopGroup,
                        $idShop
                    )) {
                        $options['2nd_presentation'] = 1;
                    }
                }
            }
        }

        if (\Configuration::get($configPrefix . 'INSURANCE_ENABLED', null, $idShopGroup, $idShop)) {
            if ($orderTotal >= (float) \Configuration::get(
                $configPrefix . 'INSURANCE_FROM',
                null,
                $idShopGroup,
                $idShop
            )) {
                $options['insurance'] = 1;
            }
        }

        return $options;
    }

    /**
     * Retrieves HS code and country of origin
     *
     * @param int $idProduct the ID of product
     * @param int $idLang the ID of language
     * @param int|null $idShopGroup the ID of shop group
     * @param int|null $idShop the ID of shop
     *
     * @return array
     */
    public function getHsCodeAndOriginOfCountry(
        int $idProduct,
        int $idLang,
        ?int $idShopGroup = null,
        ?int $idShop = null
    ): array {
        if (\Configuration::get('BPOSTSHM_INTL_HS_CODE_TYPE', null, $idShopGroup, $idShop) == 'fixed') {
            $hsCode = \Configuration::get('BPOSTSHM_INTL_HS_CODE_FIXED_VALUE', null, $idShopGroup, $idShop);
        } else {
            $hsCode = $this->getFeaturesValue(
                (int) \Configuration::get('BPOSTSHM_INTL_HS_CODE_FEATURE_VALUE', null, $idShopGroup, $idShop),
                $idProduct,
                $idLang
            );
        }

        if (\Configuration::get('BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_TYPE', null, $idShopGroup, $idShop) == 'fixed') {
            $originCountry = trim(\Configuration::get(
                'BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_FIXED_VALUE',
                null,
                $idShopGroup,
                $idShop
            ));
        } else {
            $originCountry = $this->getFeaturesValue(
                (int) \Configuration::get('BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_FEATURE_VALUE', null, $idShopGroup, $idShop),
                $idProduct,
                $idLang
            );
        }
        if (is_numeric($originCountry)) {
            $originCountry = \Country::getIsoById($originCountry);
        }
        if (strlen($originCountry) > 2) {
            $idCountry = \Country::getIdByName(
                \Configuration::get('PS_LANG_DEFAULT', null, $idShopGroup, $idShop),
                ucfirst(\Tools::strtolower($originCountry))
            );
            if ($idCountry) {
                $originCountry = \Country::getIsoById($idCountry);
            }
        }

        return [
            'hsCode' => trim($hsCode),
            'originCountry' => \Tools::strtoupper($originCountry),
        ];
    }

    /**
     * Retrieves feature value
     *
     * @param int $idFeature
     * @param int $idProduct
     * @param int $idLang
     *
     * @return string|null
     */
    private function getFeaturesValue(int $idFeature, int $idProduct, int $idLang): ?string
    {
        $value = null;
        $features = \Product::getFrontFeaturesStatic($idLang, $idProduct);
        foreach ($features as $feature) {
            if ($feature['id_feature'] == $idFeature) {
                $value = trim($feature['value']);
                break;
            }
        }

        return $value;
    }

    /**
     * Retrieves the delivery config key.
     *
     * @param string $carrierMethod the carrier method name
     * @param bool $isIntl true if is international
     * @param string $option the option name
     *
     * @return string the config key
     */
    private function getDeliveryOptionConfigKey(string $carrierMethod, bool $isIntl, string $option = ''): string
    {
        return 'BPOSTSHM_DELIVERY_OPTIONS_' . $carrierMethod . ($isIntl ? '_INTL_' : '_') . \Tools::strtoupper($option);
    }
}
