<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Service;

use Carrier;
use Db;
use Order;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Customer\Sender;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\FileException;
use PrestaShop\Module\BpostShm\Domain\BpostshmLabel\Exception\NonExpectedValueException;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * Class LabelService
 */
class LabelService
{
    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * @var Context
     */
    private $context;

    /**
     * @var \Db
     */
    private $database;

    /**
     * @var TranslatorInterface
     */
    private $translator;

    /**
     * @var ApiService
     */
    private $apiService;

    /**
     * @var OrderService
     */
    private $orderService;

    /**
     * Constructor.
     *
     * @param \BpostShm $module
     * @param ApiService $apiService
     * @param OrderService $orderService
     */
    public function __construct(
        \BpostShm $module,
        ApiService $apiService,
        OrderService $orderService
    ) {
        $this->module = $module;
        $this->context = $module->getContext();
        $this->database = $module->getDatabase();
        $this->translator = $module->getTranslator();
        $this->apiService = $apiService;
        $this->orderService = $orderService;
    }

    /**
     * Retrieve label PDF by order
     *
     * Retrieves the label PDF file path, contents, and tracking number for a given order.
     * If the tracking number is not provided, it will be fetched from the order using the getTracking method.
     *
     * @param \Order $order The order object for which to retrieve the label PDF
     * @param bool $retour (optional) Whether to retrieve the return label PDF. Defaults to false.
     * @param string $reference (optional) The bpost reference
     *
     * @return array|null returns an array with the following keys:
     *                    - 'file' - The file path of the label PDF,
     *                    - 'pdf' - The content of the label PDF file
     *                    Returns null if the label PDF file does not exist
     */
    public function getLabelPdfByOrder(\Order $order, bool $retour = false, ?string $reference = null)
    {
        if (!$reference) {
            $bpostshmOrder = $this->orderService->getBpostshmOrder($order, $retour);
            if ($bpostshmOrder) {
                $reference = $bpostshmOrder['bpost_reference'];
            }
        }

        if (!$reference) {
            return null;
        }

        $file = $this->getLabelFilePath($order, $reference);
        if (!file_exists($file)) {
            $this->restoreLabelFileForOrder($order, $reference, $retour);
        }
        if (!file_exists($file)) {
            return null;
        }

        return [
            'file' => $file,
            'pdf' => \Tools::file_get_contents($file),
        ];
    }

    /**
     * Retrieve first label data by order ID
     *
     * @param int $orderId
     * @param $retour
     *
     * @return array|bool|object|null
     */
    public function getFirstLabelByOrder($orderId, bool $retour = false)
    {
        $op = $retour ? '=' : '<>';
        $sql = new \DbQuery();
        $sql->select('*');
        $sql->from('bpostshm_label');
        $sql->where('id_order=' . (int) $orderId);
        $sql->where("`retour` $op 'ONLY'");
        $sql->orderBy('date_add DESC');

        return $this->database->getRow($sql);
    }

    /**
     * Delete label by order
     *
     * @param \Order $order
     * @param $retour
     */
    public function deleteLabel($order, $retour = false): void
    {
        $label = $this->getLabelPdfByOrder($order, $retour);
        if ($label && file_exists($label['file'])) {
            @unlink($label['file']);
        }

        $this->deleteLabelFromDb($order->id, $retour);

        if (!$retour) {
            $orderCarrier = new \OrderCarrier($order->getIdOrderCarrier());
            $orderCarrier->tracking_number = '';
            $orderCarrier->save();
        }
    }

    /**
     * Delete label in DB by order ID
     *
     * @param $orderId
     * @param $retour
     *
     * @return bool
     */
    private function deleteLabelFromDb($orderId, $retour = false): bool
    {
        $op = $retour ? '=' : '<>';

        return $this->database->delete(
            'bpostshm_label',
            'id_order=' . (int) $orderId . " AND `retour` $op 'ONLY'"
        );
    }

    /**
     * Generate label for an order
     *
     * @param \Order $order
     * @param bool $isRetour
     *
     * @throws MissingReferenceException
     * @throws NonExpectedValueException
     * @throws FileException
     */
    public function generateLabel(\Order $order, bool $isRetour = false): array
    {
        $bpostshmOrder = $this->orderService->getBpostshmOrder($order, $isRetour);
        if (!$bpostshmOrder) {
            throw new MissingReferenceException($this->translator->trans('Unable to generate label, reference is missing.', [], 'Modules.Bpostshm.Admin'));
        }
        $reference = $bpostshmOrder['bpost_reference'];
        $label = null;
        $retour = $isRetour ? 'ONLY' : (\Configuration::get('BPOSTSHM_LABELS_AUTO_RETOUR', null, $order->id_shop_group, $order->id_shop) ? 'WITH' : 'WITHOUT');
        $labels = $this->createLabelForOrder($reference, $retour != 'WITHOUT', $order->id_shop_group, $order->id_shop);

        if (empty($labels)) {
            // already printed and recently restored?
            $label = $this->getLabelPdfByOrder($order, $isRetour, $reference);
            if ($label) {
                return $label;
            }
        }

        foreach ($labels as $label) {
            $bpostshmLabels = [];
            $barcodes = $label->getBarcodes();
            if ($retour == 'ONLY') {
                foreach ($barcodes as $barcode) {
                    $bpostshmLabels[] = [
                        'tracking' => null,
                        'tracking_retour' => pSQL($barcode->getBarcode()),
                        'id_order' => (int) $order->id,
                        'date_add' => date('Y-m-d H:i:s'),
                        'retour' => pSQL($retour),
                    ];
                }
            } else {
                for ($i = 0; $i < count($barcodes); ++$i) {
                    $tracking = $barcodes[$i]->getBarcode();
                    $trackingRetour = null;
                    if ($retour == 'WITH') {
                        ++$i;
                        if (isset($barcodes[$i])) {
                            $trackingRetour = $barcodes[$i]->getBarcode();
                        } else {
                            $log = $this->translator->trans(
                                'No retour barcode found for %reference% (%delivery_method%).',
                                [
                                    '%reference%' => $reference,
                                    '%delivery_method%' => $bpostshmOrder['delivery_method'],
                                ],
                                'Modules.Bpostshm.Admin'
                            );
                            $this->module->log($log);
                        }
                    }
                    $bpostshmLabels[] = [
                        'tracking' => pSQL($tracking),
                        'tracking_retour' => pSQL($trackingRetour),
                        'id_order' => (int) $order->id,
                        'date_add' => date('Y-m-d H:i:s'),
                        'retour' => pSQL($retour),
                    ];
                }
            }
            $label = $label->getBytes();
        }

        if (count($bpostshmLabels) <= 0 || !$label) {
            throw new NonExpectedValueException($this->translator->trans('Unable to generate label, api returns a non-expected value.', [], 'Modules.Bpostshm.Admin'));
        }

        $this->deleteLabelFromDb($order->id, $isRetour);

        foreach ($bpostshmLabels as $bpostshmLabel) {
            $this->database->insert('bpostshm_label', $bpostshmLabel);
        }

        $file = $this->getLabelFilePath($order, $reference);
        if (!file_put_contents($file, $label)) {
            throw new FileException($this->translator->trans('Unable to save bpost label to %file%.', ['%file%' => $file], 'Modules.Bpostshm.Admin'));
        }

        if (!$isRetour) {
            $orderCarrier = new \OrderCarrier($order->getIdOrderCarrier());
            $orderCarrier->tracking_number = implode(', ', array_column($bpostshmLabels, 'tracking'));
            $orderCarrier->save();
        }

        return [
            'file' => $file,
            'pdf' => $label,
        ];
    }

    /**
     * Retrieves an associative array  with tracking number and tracking url by order ID
     *
     * @param int $orderId the order ID
     * @param bool $retour (optional) Whether to retrieve the return tracking. Defaults to false.
     *
     * @return array Returns an array with the following keys:
     *               - 'url' - The URL of the tracking if available otherwise an empty string.
     *               - 'number' - The tracking number.
     */
    public function getTrackingNumbers($orderId, bool $retour = false)
    {
        $field = 'tracking' . ($retour ? '_retour' : '');

        $sql = new \DbQuery();
        $sql->select("$field, c.url, o.id_carrier, o.id_lang, o.id_cart");
        $sql->from('bpostshm_label', 'l');
        $sql->leftJoin('orders', 'o', 'l.id_order = o.id_order');
        $sql->leftJoin('carrier', 'c', 'o.id_carrier = c.id_carrier');
        if (!$retour) {
            $sql->select('a.postcode');
            $sql->leftJoin('address', 'a', 'o.id_address_delivery = a.id_address');
        }
        $sql->where('l.id_order=' . (int) $orderId);
        $sql->where("$field IS NOT NULL AND $field <> ''");
        $sql->orderBy('l.date_add');
        $result = $this->database->executeS($sql);

        $trackings = [];
        if ($result) {
            $postcode = null;
            if ($retour) {
                $order = new \Order($orderId);
                $postcode = \Configuration::get('BPOSTSHM_STORE_DETAILS_POSTCODE', null, $order->id_shop_group, $order->id_shop);
            }
            $idLang = $this->context->language->id;

            foreach ($result as $row) {
                if ($row[$field]) {
                    if (null === $postcode) {
                        /** @var PickupService $pickupService */
                        $pickupService = $this->module->get('bpostshm.pickup_service');
                        if ($row['id_carrier']) {
                            $address = $pickupService->getSelectedPickupAddress(
                                $row['id_carrier'],
                                $row['id_lang'] ?: $idLang,
                                $row['id_cart']
                            );
                            if ($address) {
                                $postcode = $address->postcode;
                            }
                        }
                    }
                    if (null === $postcode) {
                        $postcode = $row['postcode'];
                    }
                    $trackings[] = [
                        'url' => $this->getTrackingUrl(
                            $row[$field],
                            $postcode,
                            $row['url']
                        ),
                        'number' => $row[$field],
                    ];
                }
            }
        }

        return $trackings;
    }

    /**
     * Retrieves the tracking URL.
     *
     * @param string $tracking the tracking number
     * @param string $postcode the postcode
     * @param string $carrier the carrier URL
     *
     * @return string the tracking URL, or an empty string if no tracking URL is available
     */
    public function getTrackingUrl($tracking, $postcode, $carrierUrl)
    {
        if (empty($tracking) || empty($carrierUrl) || empty($postcode)) {
            return '';
        }

        return str_replace('@', $tracking, $carrierUrl) . $postcode;
    }

    /**
     * Restore labels in DB from \PrestaShop\Module\BpostShm\Api\Bpost\Order object
     *
     * @param \PrestaShop\Module\BpostShm\Api\Bpost\Order $bpostOrder
     * @param int $idOrder
     * @param bool $isRetour
     *
     * @return void
     */
    public function restorelabels(\PrestaShop\Module\BpostShm\Api\Bpost\Order $bpostOrder, int $idOrder, $isRetour = null)
    {
        $labels = [];
        $labelsRetour = [];
        $boxes = $bpostOrder->getBoxes();

        if ($isRetour || ($isRetour === null && preg_match('#_RETOUR$#', $bpostOrder->getReference()))) {
            $retour = 'ONLY';
            foreach ($boxes as $box) {
                $this->database->insert('bpostshm_label', [
                    'tracking' => null,
                    'tracking_retour' => pSQL($box->getBarcode()),
                    'id_order' => $idOrder,
                    'date_add' => date('Y-m-d H:i:s'),
                    'retour' => pSQL($retour),
                ], null, false, \Db::INSERT_IGNORE);
            }
        } else {
            $firstSender = null;
            foreach ($boxes as $box) {
                if (!$firstSender) {
                    $firstSender = $box->getSender();
                    $labels[] = $box;
                } else {
                    if ($this->isSameSenders($firstSender, $box->getSender())) {
                        $labels[] = $box;
                    } else {
                        $labelsRetour[] = $box;
                    }
                }
            }
            $retour = 'WITHOUT';
            if (count($labels) == count($labelsRetour)) {
                $retour = 'WITH';
            } else {
                $labels = $boxes;
            }
        }

        foreach ($labels as $i => $label) {
            $trackingRetour = null;
            if ($retour == 'WITH') {
                $trackingRetour = $labelsRetour[$i]->getBarcode();
            }

            $this->database->insert('bpostshm_label', [
                'tracking' => pSQL($label->getBarcode()),
                'tracking_retour' => pSQL($trackingRetour),
                'id_order' => $idOrder,
                'date_add' => date('Y-m-d H:i:s'),
                'retour' => pSQL($retour),
            ], null, false, \Db::INSERT_IGNORE);
        }
    }

    /**
     * Clean old labels
     *
     * @param string $expirationDate param for strtotime()
     *
     * @return bool
     */
    public function cleanOldLabels(string $expirationDate): bool
    {
        $files = $this->scanAllDir(_PS_DOWNLOAD_DIR_ . 'bpostshm', strtotime($expirationDate));
        foreach ($files as $file) {
            if (preg_match('#\.pdf$#i', $file)) {
                @unlink(_PS_DOWNLOAD_DIR_ . 'bpostshm/' . $file);
            }
        }
        $this->delEmptyTreeDir(_PS_DOWNLOAD_DIR_);

        return $this->database->delete(
            'bpostshm_label',
            'date_add < "' . date('Y-m-d', strtotime($expirationDate)) . '"'
        );
    }

    /**
     * Check if one of phone fields is mandatory for intl labels
     *
     * @return bool true if one of phone fields is mandatory or false otherwise
     */
    public function checkPhoneFieldsRequirements(): bool
    {
        $requiredFields = \AddressFormat::getFieldsRequired();

        return in_array('phone', $requiredFields) || in_array('phone_mobile', $requiredFields);
    }

    /**
     * Create label for order
     *
     * @param string $reference
     * @param bool $retour
     *
     * @return array
     */
    private function createLabelForOrder(string $reference, $retour = false, $idShopGroup = null, $idShop = null): array
    {
        $format = \Configuration::get('BPOSTSHM_LABELS_PDF_FORMAT', null, $idShopGroup, $idShop);
        $response = $this->apiService->createLabelForOrder($reference, $format, $retour);

        return $response;
    }

    /**
     * Restores PDF file label for an order.
     *
     * @param \Order $order
     * @param string $reference
     * @param bool $retour
     *
     * @return array
     */
    private function restoreLabelFileForOrder(\Order $order, string $reference, $retour = false): bool
    {
        $op = $retour ? '=' : '<>';
        $sql = new \DbQuery();
        $sql->select('tracking');
        $sql->from('bpostshm_label');
        $sql->where('id_order=' . (int) $order->id);
        $sql->where("`retour` $op 'ONLY'");
        $sql->orderBy('date_add DESC');

        $tracking = $this->database->getValue($sql);
        if ($tracking) {
            $format = \Configuration::get('BPOSTSHM_LABELS_PDF_FORMAT', null, $order->id_shop_group, $order->id_shop);
            try {
                $labels = $this->apiService->createLabelForBox($tracking, $format, $retour);
                foreach ($labels as $label) {
                    $bytes = $label->getBytes();
                    if ($bytes) {
                        $file = $this->getLabelFilePath($order, $reference);
                        if (!file_put_contents($file, $bytes)) {
                            throw new FileException($this->translator->trans('Unable to save bpost label to %file%.', ['%file%' => $file], 'Modules.Bpostshm.Admin'));
                        }
                    }
                }
            } catch (\Exception $e) {
                // Silence is golden
            }
        }

        return false;
    }

    /**
     * Retrieves label bpost path.
     *
     * @param \Order $order
     * @param string $reference
     * @param bool $retour
     *
     * @return string
     */
    private function getLabelFilePath(\Order $order, string $reference): string
    {
        $dir = 'bpostshm/' . date('Ymd', strtotime($order->date_add));
        if (!is_dir(_PS_DOWNLOAD_DIR_ . $dir)) {
            mkdir(_PS_DOWNLOAD_DIR_ . $dir, 0775, true);
        }
        $file = _PS_DOWNLOAD_DIR_ . $dir . '/' . $reference . '.pdf';
        if (!file_exists($file)) {
            // without date directory parent
            $simplifiedVersion = _PS_DOWNLOAD_DIR_ . 'bpostshm/' . $reference . '.pdf';
            if (file_exists($simplifiedVersion)) {
                $file = $simplifiedVersion;
            }
            // before upgrade
            $oldVersion = _PS_DOWNLOAD_DIR_ . 'bpostshm/' . $reference . '/1.pdf';
            if (file_exists($oldVersion)) {
                $file = $oldVersion;
            }
        }

        return $file;
    }

    /**
     * Scan all directories
     *
     * @param string $dir directory
     * @param timestamp $expirationDate
     *
     * @return array
     */
    private function scanAllDir(string $dir, int $expirationDate): array
    {
        $result = [];
        foreach (scandir($dir) as $filename) {
            if ($filename[0] === '.') {
                continue;
            }
            $filePath = $dir . '/' . $filename;
            if (is_dir($filePath)) {
                foreach ($this->scanAllDir($filePath, $expirationDate) as $childFilename) {
                    $result[] = $filename . '/' . $childFilename;
                }
            } else {
                $stat = stat($filePath);
                if ($stat['mtime'] < $expirationDate) {
                    $result[] = $filename;
                }
            }
        }

        return $result;
    }

    /**
     * Remove empty directories
     *
     * @param $dir directory
     *
     * @return void
     */
    private function delEmptyTreeDir($dir): void
    {
        $files = glob($dir . '*', GLOB_MARK);
        foreach ($files as $file) {
            if (is_dir($file)) {
                $this->delEmptyTreeDir($file);
            }
        }
        if (is_dir($dir)) {
            @rmdir($dir);
        }
    }

    /**
     * Compares two sender by name, company and email
     *
     * @param Sender $sender1
     * @param Sender $sender2
     *
     * @return bool true if the senders are similars
     */
    private function isSameSenders(Sender $sender1, Sender $sender2)
    {
        return trim($sender1->getName()) == trim($sender2->getName())
            && trim($sender1->getCompany()) == trim($sender2->getCompany())
            && trim($sender1->getEmailAddress()) == trim($sender2->getEmailAddress());
    }
}
