<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Service;

use BpostShm;
use Db;
use Exception;
use Order;
use PrestaShop\Module\BpostShm\Api\Bpost;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Line;
use PrestaShop\Module\BpostShm\Api\Exception\BpostApiResponseException\BpostCurlException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostApiResponseException\BpostInvalidResponseException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostApiResponseException\BpostInvalidSelectionException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostApiResponseException\BpostInvalidXmlResponseException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidLengthException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidPatternException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidValueException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostNotImplementedException;
use PrestaShop\Module\BpostShm\Api\Exception\XmlException\BpostXmlNoReferenceFoundException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\BpostStatusNotFoundException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\HSCodeException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\NotABpostCarrierException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\OriginCountryException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\ProductMaxWeightException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\ProductMinWeightException;
use PrestaShop\Module\BpostShm\Domain\Exception\CarrierException;
use PrestaShop\Module\BpostShm\Domain\Exception\MissingSettingsException;
use PrestaShop\Module\BpostShm\Utils\BpostTools;
use PrestaShop\PrestaShop\Core\Domain\Order\Command\BulkChangeOrderStatusCommand;
use PrestaShop\PrestaShop\Core\Domain\Order\Exception\OrderException;
use PrestaShop\PrestaShop\Core\Domain\Order\ValueObject\OrderId;
use PrestaShop\PrestaShop\Core\Domain\Product\Exception\ProductConstraintException;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * Class OrderService
 */
class OrderService
{
    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * @var \Context
     */
    private $context;

    /**
     * @var \Db
     */
    private $database;

    /**
     * @var TranslatorInterface
     */
    private $translator;

    /**
     * @var ApiService
     */
    private $apiService;

    /**
     * @var CarrierService
     */
    private $carrierService;

    /**
     * @var ParcelService
     */
    private $parcelService;

    /**
     * @var DeliveryOptionsService
     */
    private $deliveryOptionsService;

    /**
     * @var DeliveryDateService
     */
    private $deliveryDateService;

    /**
     * Constructor
     *
     * @param \BpostShm $module
     * @param ApiService $apiService
     * @param CarrierService $carrierService
     * @param ParcelService $parcelService
     * @param DeliveryOptionsService $deliveryOptionsService
     * @param DeliveryDateService $deliveryDateService
     */
    public function __construct(
        \BpostShm $module,
        ApiService $apiService,
        CarrierService $carrierService,
        ParcelService $parcelService,
        DeliveryOptionsService $deliveryOptionsService,
        DeliveryDateService $deliveryDateService
    ) {
        $this->module = $module;
        $this->context = $module->getContext();
        $this->database = $module->getDatabase();
        $this->translator = $module->getTranslator();
        $this->apiService = $apiService;
        $this->carrierService = $carrierService;
        $this->parcelService = $parcelService;
        $this->deliveryOptionsService = $deliveryOptionsService;
        $this->deliveryDateService = $deliveryDateService;
    }

    /**
     * Retrieves the cancel order status ID.
     *
     * @return int the cancel order status ID
     */
    public function getCancelOrderStatusId(): int
    {
        $qb = new \DbQuery();
        $qb->select('id_order_state');
        $qb->from('order_state_lang');
        $qb->where('`template` = "order_canceled"');

        return (int) \Db::getInstance()->getValue($qb);
    }

    /**
     * Retrieves the bpost order from the SHM.
     *
     * @param \Order $order the order object
     * @param bool $retour whether the order has a retour or not
     *
     * @return Bpost\Order|null the bpost order retrieved from the SHM, or null if an exception is thrown
     *
     * @throws BpostInvalidValueException
     * @throws BpostCurlException
     * @throws BpostInvalidSelectionException
     * @throws BpostInvalidXmlResponseException
     * @throws BpostNotImplementedException
     * @throws BpostXmlNoReferenceFoundException
     */
    public function getBpostOrderInSHM(\Order $order, bool $retour): ?Bpost\Order
    {
        return $this->apiService->getBpostOrder($order, $retour);
    }

    /**
     * Retrieves the status of bpost order from the SHM.
     *
     * @param \Order|Bpost\Order $order the Order object
     * @param bool $retour whether the order has a retour or not
     *
     * @return string|null the bpost status for order in upper case
     *
     * @throws BpostCurlException
     * @throws BpostInvalidSelectionException
     * @throws BpostInvalidValueException
     * @throws BpostInvalidXmlResponseException
     * @throws BpostNotImplementedException
     * @throws BpostStatusNotFoundException
     * @throws BpostXmlNoReferenceFoundException
     * @throws OrderException
     */
    public function getBpostOrderStatus($order, bool $retour = false): ?string
    {
        $status = 'PENDING';

        if ($order instanceof \Order) {
            $bpostOrder = $this->getBpostOrderInSHM($order, $retour);
        } elseif ($order instanceof Bpost\Order) {
            $bpostOrder = $order;
        } else {
            throw new BpostStatusNotFoundException(new OrderId(0));
        }

        if ($bpostOrder) {
            $boxes = $bpostOrder->getBoxes();
            foreach ($boxes as $box) {
                $boxStatus = $box->getStatus();
                $status = $boxStatus;
                if (!$retour) {
                    break;
                }
            }
        }

        return \Tools::strtoupper($status);
    }

    /**
     * Refreshes the status of bpost order.
     *
     * @param \Order $order
     *
     * @return string the bpost status for order
     *
     * @throws BpostCurlException
     * @throws BpostInvalidSelectionException
     * @throws BpostInvalidValueException
     * @throws BpostInvalidXmlResponseException
     * @throws BpostNotImplementedException
     * @throws BpostStatusNotFoundException
     * @throws BpostXmlNoReferenceFoundException
     * @throws OrderException
     */
    public function refreshStatus(\Order $order): string
    {
        $status = $this->getBpostOrderStatus($order);
        if (!$status) {
            throw new BpostStatusNotFoundException(new OrderId((int) $order->id));
        }

        $states = $this->module->getStates();
        if (!isset($states[$status])) {
            throw new BpostStatusNotFoundException(new OrderId((int) $order->id), $this->translator->trans('The status "#%s" is not referenced.', ['#%s' => $status], 'Modules.Bpostshm.Admin'));
        }

        \Db::getInstance()->update('bpostshm_order', [
            'status' => pSQL($status),
        ], 'id_order = ' . (int) $order->id);

        return $status;
    }

    /**
     * Refreshes statuses of bpost orders
     *
     * @param int|null $idShop the shop ID of concerned orders
     *
     * @return void
     */
    public function refreshStatuses(?int $idShop = null)
    {
        $statusCond = 'bo.`status` IS NOT NULL AND bo.`status` NOT IN ("DELIVERED", "CANCELLED")';

        $sql = new \DbQuery();
        $sql->select('o.*');
        $sql->from('orders', 'o');
        $sql->innerJoin('bpostshm_order', 'bo', "bo.id_order = o.id_order AND $statusCond");

        if (null !== $idShop) {
            $sql->where('o.id_shop = ' . $idShop);
        }

        $days = (int) \Configuration::get('BPOSTSHM_LABELS_ORDER_DISPLAY_DAYS', null, null, $idShop);
        if ($days > 0) {
            $sql->where("DATEDIFF(NOW(), bo.`date_add`) <= $days");
        }

        $rows = \Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        foreach ($rows as $row) {
            $order = new \Order();
            try {
                $order->hydrate($row);
                $this->refreshStatus($order);
            } catch (\Exception $e) {
                $msg = '[refreshStatus]';
                if ($order->id) {
                    $msg .= '[order # ' . $order->id . ']';
                }
                $msg .= ' ' . $e->getMessage();
                $this->module->log($msg);
            }
        }
    }

    /**
     * Retrieves the bpost order from the database.
     *
     * @param \Order $order the order object
     * @param bool $retour whether the order has a retour or not
     *
     * @return array|null the bpost order data as an associative array, or null if the order is not found
     */
    public function getBpostshmOrder(\Order $order, bool $retour): ?array
    {
        $sql = new \DbQuery();
        $sql->select('*');
        $sql->from('bpostshm_order');
        $sql->where('id_order = ' . (int) $order->id);
        $sql->where('retour = ' . (int) $retour);

        $result = \Db::getInstance()->getRow($sql);
        if (isset($result['bpost_reference'])) {
            return $result;
        }

        return null;
    }

    /**
     * Prepares the bpost order for processing.
     *
     * @param \Order $order the order object
     * @param bool $isRetour whether the order has a retour or not
     *
     * @return void
     *
     * @throws BpostCurlException
     * @throws BpostInvalidLengthException
     * @throws BpostInvalidPatternException
     * @throws BpostInvalidResponseException
     * @throws BpostInvalidSelectionException
     * @throws BpostInvalidValueException
     * @throws BpostInvalidXmlResponseException
     * @throws BpostNotImplementedException
     * @throws BpostXmlNoReferenceFoundException
     * @throws CarrierException
     * @throws \DOMException
     * @throws HSCodeException
     * @throws MissingSettingsException
     * @throws NotABpostCarrierException
     * @throws OrderException
     * @throws OriginCountryException
     * @throws ProductConstraintException
     * @throws ProductMaxWeightException
     * @throws ProductMinWeightException
     */
    public function prepareBpostOrder(\Order $order, bool $isRetour)
    {
        if (!\Validate::isLoadedObject($order)) {
            throw new \PrestaShopException($this->translator->trans('Missing Order.', [], 'Modules.Bpostshm.Admin'));
        }
        $status = \Configuration::get(
            'BPOSTSHM_LABELS_USE_PS',
            null,
            $order->id_shop_group,
            $order->id_shop
        ) ? 'PENDING' : 'SHM_ONLY_PROCESSING';
        $bpostOrder = $this->getBpostOrderInSHM($order, $isRetour);
        if (!$bpostOrder) {
            $this->saveBpostshmOrder($order, $isRetour, $status);
            $this->createOrReplaceOrder($order, $isRetour);
        } else {
            //
            // Restoring DB data from SHM...
            //
            /** @var LabelService $labelService */
            $labelService = $this->module->get('bpostshm.label_service');
            for ($i = 0; $i < 2; ++$i) {
                try {
                    $newStatus = $this->getBpostOrderStatus($bpostOrder);
                    if ($newStatus) {
                        $status = $newStatus;
                    }
                } catch (\Exception $e) {
                    // silence is golden.
                }
                $this->saveBpostshmOrder($order, $isRetour, $status);
                $labelService->restoreLabels($bpostOrder, (int) $order->id, $isRetour);

                $isRetour = !$isRetour;
                $bpostOrder = $this->getBpostshmOrder($order, $isRetour);
                if (!$bpostOrder) {
                    break;
                }
            }
        }
    }

    /**
     * Prepares the missing bpost orders.
     *
     * This method retrieves the missing bpost orders from the database and prepares them for processing.
     * It iterates over each order, calling the prepareBpostOrder() method. If an exception occurs during
     * the preparation process, an error log is created using the translator and the module's log() method.
     *
     * @return void
     */
    public function prepareMissingBpostOrders()
    {
        $orders = $this->getMissingBpostOrders();
        foreach ($orders as $order) {
            try {
                $this->prepareBpostOrder($order, false);
            } catch (\Exception $e) {
                $log = $this->translator->trans(
                    'Missing Order: Unable to prepare bpost order (ID %order%): %message%.',
                    [
                        '%order%' => $order->id,
                        '%message%' => $e->getMessage(),
                    ],
                    'Modules.Bpostshm.Admin'
                );
                $this->module->log($log, 3, null, 'Order', $order->id);
            }
        }
    }

    /**
     * Retrieves orders that have not been prepared and are not referenced in the bpostshm_order table.
     *
     * @param int|null $maxDays The maximum number of days to consider when selecting orders. Default to 30
     *                          or "BPOSTSHM_LABELS_ORDER_DISPLAY_DAYS" config value if < 30
     *
     * @return array the missing bpost orders
     */
    public function getMissingBpostOrders(?int $maxDays = 30): array
    {
        if ($maxDays) {
            $displayDays = (int) \Configuration::get('BPOSTSHM_LABELS_ORDER_DISPLAY_DAYS');
            if ($displayDays > 0) {
                $maxDays = min($displayDays, $maxDays);
            }
        }

        $sql = new \DbQuery();
        $sql->select('o.id_order as id, o.*');
        $sql->from('orders', 'o');
        $sql->innerJoin('bpostshm_carrier_history', 'h', 'h.id_carrier = o.id_carrier');
        $sql->leftJoin('bpostshm_order', 'bo', 'bo.id_order = o.id_order AND bo.retour = 0');
        $sql->where('bo.id_order IS NULL');
        if ((int) $maxDays > 0) {
            $sql->where('o.date_add >= "' . date('Y-m-d 00:00:00', strtotime("- $maxDays DAYS")) . '"');
        }

        $orders = [];
        $rows = \Db::getInstance()->executeS($sql);
        if (is_array($rows)) {
            foreach ($rows as $row) {
                $order = new \Order();
                $order->hydrate($row);
                $orders[$order->id] = $order;
            }
        }

        return $orders;
    }

    /**
     * Checks if order has missing bpost order.
     *
     * @param int $idOrder The Order ID
     *
     * @return bool true if missing bpost order or false otherwise
     */
    public function hasMissingBpostOrder(int $idOrder): bool
    {
        $sql = new \DbQuery();
        $sql->select('o.id_order as id');
        $sql->from('orders', 'o');
        $sql->innerJoin('bpostshm_carrier_history', 'h', 'h.id_carrier = o.id_carrier');
        $sql->leftJoin('bpostshm_order', 'bo', 'bo.id_order = o.id_order AND bo.retour = 0');
        $sql->where('o.id_order = ' . $idOrder);
        $sql->where('bo.id_order IS NULL');

        return (int) \Db::getInstance()->getValue($sql) == $idOrder;
    }

    /**
     * Marks the order as treated in the database.
     *
     * @param \Order $order the order object
     *
     * @throws OrderException If we have an error during the status change
     */
    public function markOrderAsTreated(\Order $order)
    {
        \Db::getInstance()->update('bpostshm_order', [
            'treated' => true,
        ], 'id_order = ' . (int) $order->id);

        $idOrderState = (int) \Configuration::get(
            'BPOSTSHM_LABELS_TREATED_ORDER_STATE',
            null,
            $order->id_shop_group,
            $order->id_shop
        );
        $currentOrderState = $order->getCurrentOrderState();
        if ($currentOrderState && $currentOrderState->id != $idOrderState) {
            $this->module->get('prestashop.core.command_bus')->handle(new BulkChangeOrderStatusCommand(
                [$order->id],
                $idOrderState
            ));
        }
    }

    /**
     * Saves the bpost order in the database.
     *
     * @param \Order $order the order object
     * @param bool $retour whether the order has a retour or not
     * @param string $status the status of the bpost order
     *
     * @return void
     *
     * @throws BpostInvalidLengthException
     * @throws CarrierException
     * @throws NotABpostCarrierException
     * @throws OrderException
     */
    private function saveBpostshmOrder(\Order $order, bool $retour, string $status)
    {
        \Db::getInstance()->insert(
            'bpostshm_order',
            $this->getBpostshmOrderData($order, $retour, $status),
            false,
            true,
            \Db::REPLACE
        );
    }

    /**
     * Retrieves the bpost order details.
     *
     * @param \Order $order the order object
     * @param bool $retour whether the order has a retour or not
     * @param string|null $status the Bpost order status (optional)
     *
     * @return array the Bpost order details
     *
     * @throws BpostInvalidLengthException
     * @throws NotABpostCarrierException
     * @throws OrderException
     * @throws CarrierException
     */
    private function getBpostshmOrderData(\Order $order, bool $retour, ?string $status = null): array
    {
        $deliveryDate = $this->deliveryDateService->getDeliveryDateForOrder($order);
        $carrierMethod = $this->carrierService->getCarrierMethod($order->id_carrier);
        if (!$carrierMethod) {
            throw new NotABpostCarrierException(new OrderId((int) $order->id));
        }
        $receiverAndSender = $this->parcelService->getReceiverAndSender($order, $retour, $carrierMethod);
        $options = $this->deliveryOptionsService->getDeliveryOptions(
            $carrierMethod,
            BpostTools::isIntl($receiverAndSender['country']),
            $deliveryDate && $this->deliveryDateService->isSaturday($deliveryDate),
            (float) $order->total_products,
            $order->id_shop_group,
            $order->id_shop
        );
        $date = date('Y-m-d H:i:s');

        return [
            'id_order' => (int) $order->id,
            'bpost_reference' => pSQL($this->apiService->getBpostOrderReference($order, $retour)),
            'delivery_method' => pSQL($this->carrierService->getDeliveryMethod($order, $retour)),
            'recipient' => pSQL($receiverAndSender['recipient']),
            'drop_date' => $deliveryDate ? pSQL($deliveryDate) : null,
            'opt_saturday' => $options['saturday'],
            'opt_signature' => $options['signature'],
            'opt_insurance' => $options['insurance'],
            'opt_2nd_presentation' => $options['2nd_presentation'],
            'retour' => $retour,
            'treated' => 0,
            'status' => pSQL($status),
            'date_add' => $date,
            'date_upd' => $date,
        ];
    }

    /**
     * Creates or replaces a bpost order.
     *
     * @param \Order $order the order object
     * @param bool $retour Whether the order has a retour or not. Default is false.
     *
     * @return void
     *
     * @throws BpostCurlException
     * @throws BpostInvalidLengthException
     * @throws BpostInvalidPatternException
     * @throws BpostInvalidResponseException
     * @throws BpostInvalidSelectionException
     * @throws BpostInvalidValueException
     * @throws BpostInvalidXmlResponseException
     * @throws \DOMException
     * @throws HSCodeException
     * @throws MissingSettingsException
     * @throws OriginCountryException
     * @throws ProductMaxWeightException
     * @throws ProductMinWeightException
     * @throws ProductConstraintException
     */
    private function createOrReplaceOrder(\Order $order, bool $retour = false): void
    {
        $bpostshmOrder = $this->getBpostshmOrder($order, $retour);
        if (!$bpostshmOrder) {
            throw new MissingReferenceException($this->translator->trans('Reference is missing in bpostshm_order table.', [], 'Modules.Bpostshm.Admin'));
        }

        $reference = $bpostshmOrder['bpost_reference'];
        $bpostOrder = new Bpost\Order($reference);

        $orderProducts = [];
        if ($retour) {
            $retours = \OrderReturn::getOrdersReturn($order->id_customer, $order->id);
            foreach ($retours as $ret) {
                $orderProducts = array_merge(
                    $orderProducts,
                    \OrderReturn::getOrdersReturnProducts($ret['id_order_return'], $order)
                );
            }
        }
        if (count($orderProducts) <= 0) {
            $orderProducts = $order->getProducts();
        }

        foreach ($orderProducts as $product) {
            $qty = (int) $product['product_quantity'];
            $line = new Line(BpostTools::cleanStr($product['product_name']), $qty);
            $bpostOrder->addLine($line);
        }

        $bpostOrder = $this->parcelService->createBoxes($bpostshmOrder, $order, $bpostOrder, $orderProducts, $retour);

        $this->apiService->createOrReplaceOrder($bpostOrder);
    }
}
