<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Service;

use BpostShm;
use Cart;
use Order;
use PrestaShop\Module\BpostShm\Adapter\Parcel;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Address\IntlPugoAddress;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Address\ParcelsDepotAddress;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Address\PugoAddress;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\AtIntlHome;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\AtIntlPugo;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\CustomsInfo\CustomsInfo;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Box\ParcelContent;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Customer\Receiver;
use PrestaShop\Module\BpostShm\Api\Bpost\Order\Customer\Sender;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidLengthException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidPatternException;
use PrestaShop\Module\BpostShm\Api\Exception\BpostLogicException\BpostInvalidValueException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\HSCodeException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\OriginCountryException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\ProductMaxWeightException;
use PrestaShop\Module\BpostShm\Domain\BpostshmOrder\Exception\ProductMinWeightException;
use PrestaShop\Module\BpostShm\Domain\Exception\MissingSettingsException;
use PrestaShop\Module\BpostShm\Utils\BpostTools;
use PrestaShop\PrestaShop\Core\Domain\Product\Exception\ProductConstraintException;
use PrestaShop\PrestaShop\Core\Domain\Product\ValueObject\ProductId;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * Class ParcelService
 */
class ParcelService
{
    const MAX_WEIGHT = 30000;

    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * @var \Context
     */
    private $context;

    /**
     * @var \Db
     */
    private $database;

    /**
     * @var TranslatorInterface
     */
    private $translator;

    /**
     * @var PickupService
     */
    private $pickupService;

    /**
     * @var CustomerContactService
     */
    private $customerContactService;

    /**
     * Constructor
     *
     * @param \BpostShm $module
     * @param PickupService $pickupService
     * @param CustomerContactService $customerContactService
     */
    public function __construct(
        \BpostShm $module,
        PickupService $pickupService,
        CustomerContactService $customerContactService
    ) {
        $this->module = $module;
        $this->context = $module->getContext();
        $this->database = $module->getDatabase();
        $this->translator = $module->getTranslator();
        $this->pickupService = $pickupService;
        $this->customerContactService = $customerContactService;
    }

    /**
     * Creates boxes for the order, ensuring that each box does not exceed the 30kg limit.
     *
     * @param array $bpostshmOrder the bpostshmOrder data
     * @param \Order $order the order
     * @param \PrestaShop\Module\BpostShm\Api\Bpost\Order $bpostOrder the bpost Order object for API
     * @param array $products the concerned products
     * @param bool $retour whether the order has a retour or not
     *
     * @return \PrestaShop\Module\BpostShm\Api\Bpost\Order the bpost Order object for API
     *
     * @throws BpostInvalidLengthException
     * @throws BpostInvalidPatternException
     * @throws BpostInvalidValueException
     * @throws HSCodeException
     * @throws MissingSettingsException
     * @throws OriginCountryException
     * @throws ProductConstraintException
     * @throws ProductMaxWeightException
     * @throws ProductMinWeightException
     */
    public function createBoxes(
        array $bpostshmOrder,
        \Order $order,
        \PrestaShop\Module\BpostShm\Api\Bpost\Order $bpostOrder,
        array $products,
        bool $retour
    ): \PrestaShop\Module\BpostShm\Api\Bpost\Order {
        /** @var CarrierService $carrierService */
        $carrierService = $this->module->get('bpostshm.carrier_service');
        $carrierMethod = $carrierService->getCarrierMethod($order->id_carrier);

        $shippers = $this->getReceiverAndSender($order, $retour, $carrierMethod);
        $receiver = $shippers['receiver'];
        $sender = $shippers['sender'];
        $isIntl = BpostTools::isIntl($shippers['country']);

        $pickup = null;
        $language = new \Language($order->id_lang);
        if ($this->pickupService->isPickup($order->id_carrier)) {
            $pickup = $this->pickupService->getPickup(
                (int) $this->pickupService->getPickupIdByOrderId($order->id),
                $language,
                $shippers['country']
            );
        }

        $defaultMobile = $shippers['phone_mobile'];

        $deliveryMethod = $bpostshmOrder['delivery_method'];
        $options = $this->getDeliveryBoxOptions($bpostshmOrder, $retour);

        $parcels = $this->getParcels(
            $order,
            $products,
            $deliveryMethod,
            $options,
            $retour,
            $receiver,
            $sender,
            $isIntl,
            $pickup,
            $defaultMobile,
            $language
        );
        foreach ($parcels as $parcel) {
            $box = $this->createBox($parcel);
            $bpostOrder->addBox($box);
        }

        return $bpostOrder;
    }

    /**
     * Creates box for a parcel.
     *
     * @param Parcel $parcel the parcel
     *
     * @return Box the box of the parcel
     *
     * @throws BpostInvalidLengthException
     * @throws MissingSettingsException
     * @throws BpostInvalidPatternException
     * @throws BpostInvalidValueException
     */
    private function createBox(Parcel $parcel): Box
    {
        $sender = $parcel->getSender();
        $receiver = $parcel->getReceiver();
        $order = $parcel->getOrder();
        $products = [];
        $customsInfo = new CustomsInfo();

        $box = new Box();
        $box->setStatus('OPEN');
        $box->setSender($sender);

        $needCN23 = false;
        if ($parcel->isIntl()) {
            $customsInfo->setParcelValue($parcel->getValue());
            $customsInfo->setContentDescription(BpostTools::cleanStr('ORDER #' . $order->reference, 50));
            $customsInfo->setShipmentType(CustomsInfo::CUSTOM_INFO_SHIPMENT_TYPE_GOODS);
            $customsInfo->setParcelReturnInstructions(
                CustomsInfo::CUSTOM_INFO_PARCEL_RETURN_INSTRUCTION_RTS
            );
            $customsInfo->setPrivateAddress(false);

            $as = $sender->getAddress();
            $isoFrom = $as->getCountryCode();
            $ar = $receiver->getAddress();
            $isoTo = $ar->getCountryCode();
            $postcode = $ar->getPostalCode();
            $needCN23 = BpostTools::needCN23($isoFrom, $isoTo, $postcode);
            if ($needCN23) {
                if (!\Configuration::get(
                    'BPOSTSHM_INTL_OUTSIDE_EU_ENABLED',
                    null,
                    $order->id_shop_group,
                    $order->id_shop
                )) {
                    throw new MissingSettingsException($this->translator->trans('Missing international non-EU settings.', [], 'Modules.Bpostshm.Admin'));
                }
                $products = $parcel->getProducts();
                $productsIds = array_column($products, 'id_product');
                $eadCats = $this->getEadCategories($productsIds);
                if (!empty($eadCats)) {
                    $customsInfo->setContentDescription(BpostTools::cleanStr($eadCats, 50));
                }
                $currency = new \Currency((int) $order->id_currency);
                $customsInfo->setCurrency($currency->iso_code);
                $customsInfo->setAmtPostagePaidByAddresse((float) $order->total_shipping_tax_incl);
            }
        }

        $pickup = $parcel->getPickup();
        if (!$pickup || $parcel->isRetour()) {
            if (!$parcel->isIntl()) {
                $atHome = new Box\AtHome();
                // bpack 24h Pro, bpack 24h business, bpack Bus, bpack Pallet, bpack Easy Retour
                $atHome->setProduct('bpack ' . $parcel->getDeliveryMethod());
                $atHome->setReceiver($receiver);
                $atHome->setWeight($parcel->getWeight());
                $options = $parcel->getDeliveryBoxOptions();
                foreach ($options as $option) {
                    $atHome->addOption($option);
                }
                $box->setNationalBox($atHome);
            } else {
                $atHome = new AtIntlHome();
                // bpack World Business, bpack World Express Pro, bpack Europe Business, bpack World Easy Return
                $atHome->setProduct('bpack ' . $parcel->getDeliveryMethod());
                if ($needCN23) {
                    foreach ($products as $product) {
                        $parcelContent = new ParcelContent();
                        $parcelContent->setNumberOfItemType((int) $product['quantity']);
                        $parcelContent->setValueOfItem($product['value']);
                        $parcelContent->setItemDescription($product['description']);
                        $parcelContent->setNettoWeight($product['weight']);
                        $parcelContent->setHsTariffCode($product['hsCode']);
                        $parcelContent->setOriginOfGoods($product['originCountry']);
                        $atHome->addParcelContent($parcelContent);
                    }
                }
                $atHome->setReceiver($receiver);
                $atHome->setParcelWeight($parcel->getWeight());
                $atHome->setCustomsInfo($customsInfo);
                $options = $parcel->getDeliveryBoxOptions();
                foreach ($options as $option) {
                    $atHome->addOption($option);
                }
                $box->setInternationalBox($atHome);
            }
        } else {
            if (PickupService::PARCEL_LOCKER_TYPE != $pickup['type']) {
                if (!$parcel->isIntl()) {
                    $atBpost = new Box\AtBpost();
                    $pugoAddress = new PugoAddress(
                        $pickup['street'],
                        null,
                        $pickup['nr'],
                        null,
                        $pickup['zipcode'],
                        $pickup['city'],
                        $pickup['iso_country']
                    );
                    $atBpost->setReceiverName(\Tools::substr($receiver->getName(), 0, 40));
                    $atBpost->setReceiverCompany(\Tools::substr($receiver->getCompany(), 0, 40));
                    $atBpost->setWeight($parcel->getWeight());
                } else {
                    $atBpost = new AtIntlPugo();
                    $pugoAddress = new IntlPugoAddress(
                        $pickup['street'],
                        null,
                        $pickup['nr'],
                        null,
                        $pickup['zipcode'],
                        $pickup['city'],
                        $pickup['iso_country']
                    );
                    $atBpost->setReceiver($receiver);
                    $atBpost->setParcelWeight($parcel->getWeight());
                    $atBpost->setCustomsInfo($customsInfo);
                }

                $atBpost->setPugoId(sprintf('%06s', $pickup['id_bpost']));
                $atBpost->setPugoName(BpostTools::cleanStr($pickup['company_name'], 40));
                $atBpost->setPugoAddress($pugoAddress);

                $atBpost->addOption(
                    new Box\Option\Messaging(
                        Box\Option\Messaging::MESSAGING_TYPE_KEEP_ME_INFORMED,
                        $parcel->getLang(),
                        $receiver->getEmailAddress()
                    )
                );
                $options = $parcel->getDeliveryBoxOptions();
                foreach ($options as $option) {
                    $atBpost->addOption($option);
                }

                if (!$parcel->isIntl()) {
                    $box->setNationalBox($atBpost);
                } else {
                    $box->setInternationalBox($atBpost);
                }
            } else {
                // locker @24/7
                $depotAddress = new ParcelsDepotAddress(
                    $pickup['street'],
                    null,
                    $pickup['nr'],
                    'A',
                    $pickup['zipcode'],
                    $pickup['city'],
                    $pickup['iso_country']
                );

                $atUpl = new Box\At247();
                $atUpl->setParcelsDepotId(sprintf('%06s', $pickup['id_bpost']));
                $atUpl->setParcelsDepotName(BpostTools::cleanStr($pickup['company_name'], 40));
                $atUpl->setParcelsDepotAddress($depotAddress);

                $customerContact = $this->customerContactService->getCustomerContactForOrder($order);
                $info = new Box\National\Unregistered();
                $info->setLanguage($parcel->getLang());
                $info->setEmailAddress(
                    \Validate::isEmail($customerContact['email']) ?
                        $customerContact['email'] :
                        $receiver->getEmailAddress()
                );
                $info->setMobilePhone(
                    BpostTools::isBeMobileNumber($customerContact['mobile']) ?
                        $customerContact['mobile'] :
                        $parcel->getMobile()
                );
                $info->setReducedMobilityZone((bool) $customerContact['reduced_mobility_zone']);

                $atUpl->setUnregistered($info);
                $atUpl->setReceiverName(\Tools::substr($receiver->getName(), 0, 40));
                $atUpl->setReceiverCompany(\Tools::substr($receiver->getCompany(), 0, 40));
                $atUpl->setWeight($parcel->getWeight());
                $options = $parcel->getDeliveryBoxOptions();
                foreach ($options as $option) {
                    $atUpl->addOption($option);
                }
                $box->setNationalBox($atUpl);
            }
        }

        $box->setAdditionalCustomerReference(sprintf(
            'PrestaShop v%s / %s module v%s',
            _PS_VERSION_,
            $this->module->name,
            $this->module->version
        ));

        return $box;
    }

    /**
     * Check if we can ship the cart content with bpost
     *
     * @param \Cart $cart the cart
     *
     * @return bool false if the weight of each product of cart doesn't exceed the max weight (30Kg) or true otherwise
     */
    public function canShip(\Cart $cart): bool
    {
        foreach ($cart->getProducts() as $product) {
            $weight = $this->getProductWeight((float) $product['weight'], 1, $cart->id_shop_group, $cart->id_shop);
            if ($weight > ParcelService::MAX_WEIGHT) {
                return false;
            }
        }

        return true;
    }

    /**
     * Retrieves the receiver and sender information for the given order.
     *
     * @param \Order $order the order object
     * @param bool $retour whether the order has a retour or not
     * @param string $carrierMethod The carrier method 'HOME', 'PICKUP' or 'PARCEL LOCKER'
     *
     * @return array An array containing the receiver and sender
     *
     * @throws BpostInvalidLengthException
     */
    public function getReceiverAndSender(\Order $order, bool $retour, string $carrierMethod): array
    {
        $atHome = $carrierMethod == 'HOME';
        $retourAddressId = ($retour && !$atHome) ? $order->id_address_invoice : null;
        $customer = new \Customer($order->id_customer);
        $deliveryAddress = new \Address($retourAddressId ?: $order->id_address_delivery);
        $company = BpostTools::cleanStr($deliveryAddress->company);
        $clientLine1 = BpostTools::cleanStr($deliveryAddress->address1);
        $clientLine2 = BpostTools::cleanStr($deliveryAddress->address2);

        $phoneMobile = $phone = '';
        if (!empty($deliveryAddress->phone_mobile)) {
            $phone = BpostTools::cleanStr($deliveryAddress->phone_mobile);
            $phoneMobile = $phone;
        } elseif (!empty($deliveryAddress->phone)) {
            $phone = BpostTools::cleanStr($deliveryAddress->phone);
        }
        if (!$phoneMobile || !$phone) {
            $invoiceAddress = new \Address($order->id_address_invoice);
            if (!$phoneMobile && !empty($invoiceAddress->phone_mobile)) {
                $phoneMobile = BpostTools::cleanStr($invoiceAddress->phone_mobile);
                if (!$phone) {
                    $phone = $phoneMobile;
                }
            }
            if (!$phone && !empty($invoiceAddress->phone)) {
                $phone = BpostTools::cleanStr($invoiceAddress->phone);
            }
            if (!$phoneMobile) {
                $phoneMobile = $phone;
            }
        }

        $client = [
            'name' => $deliveryAddress->firstname . ' ' . $deliveryAddress->lastname,
            'company' => $company,
            'address1' => $clientLine1,
            'address2' => $clientLine2,
            'city' => $deliveryAddress->city,
            'postcode' => $deliveryAddress->postcode,
            'id_country' => $deliveryAddress->id_country,
            'email' => $customer->email,
            'phone' => $phone,
        ];
        $client = $this->getShipper($client, $atHome);

        $shop = [
            'name' => \Configuration::get(
                'BPOSTSHM_STORE_DETAILS_SENDER',
                null,
                $order->id_shop_group,
                $order->id_shop
            ),
            'company' => '',
            'address1' => \Configuration::get(
                'BPOSTSHM_STORE_DETAILS_ADDRESS',
                null,
                $order->id_shop_group,
                $order->id_shop
            ),
            'address2' => '',
            'city' => \Configuration::get(
                'BPOSTSHM_STORE_DETAILS_CITY',
                null,
                $order->id_shop_group,
                $order->id_shop
            ),
            'postcode' => \Configuration::get(
                'BPOSTSHM_STORE_DETAILS_POSTCODE',
                null,
                $order->id_shop_group,
                $order->id_shop
            ),
            'id_country' => \Country::getByIso(\Configuration::get(
                'BPOSTSHM_STORE_DETAILS_COUNTRY',
                null,
                $order->id_shop_group,
                $order->id_shop
            )),
            'email' => \Configuration::get(
                'BPOSTSHM_STORE_DETAILS_EMAIL',
                null,
                $order->id_shop_group,
                $order->id_shop
            ),
            'phone' => \Configuration::get(
                'BPOSTSHM_STORE_DETAILS_PHONE',
                null,
                $order->id_shop_group,
                $order->id_shop
            ),
        ];
        $shop = $this->getShipper($shop, true);

        $sender = $shop;
        $receiver = $client;
        if ($retour) {
            $sender = $client;
            $receiver = $shop;
        }

        // sender
        $address = new \PrestaShop\Module\BpostShm\Api\Bpost\Order\Address();
        if (!empty($sender['number'])) {
            $address->setNumber(\Tools::substr($sender['number'], 0, 8));
        }
        $address->setStreetName(BpostTools::cleanStr($sender['street'], 40));
        if (!empty($sender['addressLineTwo'])) {
            $address->setAddressLineTwo(BpostTools::cleanStr($sender['addressLineTwo'], 40));
        }
        $address->setPostalCode(BpostTools::cleanStr($sender['postcode'], 32));
        $address->setLocality(BpostTools::cleanStr($sender['locality'], 40));
        $address->setCountryCode($sender['countryCode']);

        $bpostSender = new Sender();
        $bpostSender->setAddress($address);
        $bpostSender->setName(BpostTools::cleanStr($sender['name'], 40));
        if (!empty($sender['company'])) {
            $bpostSender->setCompany(BpostTools::cleanStr($sender['company'], 40));
        }

        $senderPhone = \Tools::substr($sender['phone'], 0, 20);
        if (!(empty($senderPhone))) {
            $bpostSender->setPhoneNumber($senderPhone);
        }

        $senderEmail = \Tools::substr($sender['email'], 0, 50);
        if (!(empty($senderEmail))) {
            $bpostSender->setEmailAddress($senderEmail);
        }

        // receiver
        $address = new \PrestaShop\Module\BpostShm\Api\Bpost\Order\Address();
        if (!empty($receiver['number'])) {
            $address->setNumber(\Tools::substr($receiver['number'], 0, 8));
        }
        $address->setStreetName(BpostTools::cleanStr($receiver['street'], 40));
        if (!empty($receiver['addressLineTwo'])) {
            $address->setAddressLineTwo(BpostTools::cleanStr($receiver['addressLineTwo'], 40));
        }
        $address->setPostalCode(BpostTools::cleanStr($receiver['postcode'], 32));
        $address->setLocality(BpostTools::cleanStr($receiver['locality'], 40));
        $address->setCountryCode($receiver['countryCode']);

        $bpostReceiver = new Receiver();
        $bpostReceiver->setAddress($address);
        $bpostReceiver->setName(BpostTools::cleanStr($receiver['name'], 40));
        if (!empty($receiver['company'])) {
            $bpostReceiver->setCompany(BpostTools::cleanStr($receiver['company'], 40));
        }

        $receiverPhone = \Tools::substr($receiver['phone'], 0, 20);
        if (!(empty($receiverPhone))) {
            $bpostReceiver->setPhoneNumber($receiverPhone);
        }

        $receiverEmail = \Tools::substr($receiver['email'], 0, 50);
        if (!(empty($receiverEmail))) {
            $bpostReceiver->setEmailAddress($receiverEmail);
        }

        $recipient = $receiver['name']
            . ($bpostReceiver->getCompany() ? (' (' . $bpostReceiver->getCompany() . ')') : '') . ', '
            . $address->getStreetName() . ' '
            . ($address->getAddressLineTwo() ? ($address->getAddressLineTwo() . ' ') : '')
            . $address->getPostalCode() . ' ' . $address->getLocality()
            . ' (' . \Tools::strtoupper($address->getCountryCode()) . ')';

        return [
            'receiver' => $bpostReceiver,
            'phone_mobile' => $phoneMobile,
            'sender' => $bpostSender,
            'recipient' => html_entity_decode($recipient),
            'country' => \Tools::strtoupper(\Country::getIsoById($deliveryAddress->id_country)),
        ];
    }

    /**
     * Get parcels with a maximum of MAX_WEIGHT (30kg)
     *
     * @param \Order $order the order
     * @param array $products the concerned products
     * @param string $method the delivery method
     * @param array $options the delivery options
     * @param bool $retour whether the order has a retour or not
     * @param Receiver $receiver
     * @param Sender $sender
     * @param bool $isIntl
     * @param array|null $pickup
     * @param string|null $mobile
     * @param \Language $language
     *
     * @return array An array with parcels
     *
     * @throws HSCodeException
     * @throws OriginCountryException
     * @throws ProductMaxWeightException
     * @throws ProductMinWeightException
     * @throws ProductConstraintException
     */
    private function getParcels(
        \Order $order,
        array $products,
        string $method,
        array $options,
        bool $retour,
        Receiver $receiver,
        Sender $sender,
        bool $isIntl,
        ?array $pickup,
        ?string $mobile,
        \Language $language
    ): array {
        $productWeights = [];
        foreach ($products as $productIndex => $product) {
            for ($q = (int) $product['product_quantity']; $q; --$q) {
                $productWeight = $this->getProductWeight(
                    (float) $product['product_weight'],
                    1,
                    $order->id_shop_group,
                    $order->id_shop
                );
                if ($productWeight > self::MAX_WEIGHT) {
                    throw new ProductMaxWeightException(new ProductId((int) $product['product_id']));
                }
                if ($productWeight <= 0) {
                    throw new ProductMinWeightException(new ProductId((int) $product['product_id']));
                }
                $productWeights[] = [$productWeight, $productIndex];
            }
        }
        arsort($productWeights);

        // Extract individual values from $product_weights array
        $productIndexes = array_column($productWeights, 1);
        $individualWeights = array_column($productWeights, 0);

        $maximumParcels = count($individualWeights);
        $totalWeight = array_sum($individualWeights);
        $minimumBoxes = ceil($totalWeight / self::MAX_WEIGHT);
        $requiredParcels = min($minimumBoxes, $maximumParcels);

        $bags = array_fill(0, $requiredParcels, []);
        $boxes = array_fill(0, $requiredParcels, []);
        // Distribute items into bags while considering parcel weight limit
        foreach ($bags as $i => &$bag) {
            if (empty($bag)) {
                $currentWeight = $individualWeights[$i];
                $productIndex = $productIndexes[$i];
                $bag[] = $currentWeight;
                $boxes[$i][$productIndex] = isset($boxes[$i][$productIndex]) ? $boxes[$i][$productIndex] + 1 : 1;
                unset($individualWeights[$i]);
                unset($productIndexes[$i]);
            }
        }

        // Continue distributing remaining items into bags within weight limit
        foreach ($individualWeights as $i => $itemWeight) {
            foreach ($bags as $j => &$bag) {
                $currentWeightInBag = array_sum($bag);
                if ($currentWeightInBag + $itemWeight <= self::MAX_WEIGHT) {
                    $bag[] = $itemWeight;
                    $productIndex = $productIndexes[$i];
                    $boxes[$j][$productIndex] = isset($boxes[$j][$productIndex]) ? $boxes[$j][$productIndex] + 1 : 1;
                    unset($individualWeights[$i]);
                    unset($productIndexes[$i]);
                    break;
                }
            }
        }

        // Continue distributing remaining items into bags
        foreach ($individualWeights as $i => $itemWeight) {
            $productIndex = $productIndexes[$i];
            $boxes[$i][$productIndex] = 1;
            unset($individualWeights[$i]);
            unset($productIndexes[$i]);
        }

        /** @var DeliveryOptionsService $deliveryOptionsService */
        $deliveryOptionsService = $this->module->get('bpostshm.delivery_options_service');
        $lang = $language->iso_code;
        if (!in_array($lang, $this->pickupService->getAvailableLanguages())) {
            $lang = 'fr';
        }

        // Create final parcel data with product details and aggregated values
        $parcels = [];
        foreach ($boxes as $i => $box) {
            $parcels[$i] = new Parcel(
                $order,
                $method,
                $options,
                $retour,
                $receiver,
                $sender,
                $isIntl,
                $pickup,
                $mobile,
                $lang
            );
            foreach ($box as $productIndex => $qty) {
                $product = $products[$productIndex];
                $productWeight = $this->getProductWeight(
                    (float) $product['product_weight'],
                    $qty,
                    $order->id_shop_group,
                    $order->id_shop
                );
                $productValue = (int) \Tools::ps_round($product['unit_price_tax_incl'] * 100, 2) * $qty;

                $info = $deliveryOptionsService->getHsCodeAndOriginOfCountry(
                    $product['product_id'],
                    $language->id,
                    $order->id_shop_group,
                    $order->id_shop
                );
                $hsCode = $info['hsCode'];
                $originCountry = $info['originCountry'];

                $article = [
                    'id_product' => $product['id_product'],
                    'reference' => $product['product_reference'],
                    'description' => BpostTools::cleanStr($product['product_name'], 30),
                    'quantity' => $qty,
                    'weight' => $productWeight,
                    'value' => $productValue,
                    'hsCode' => $hsCode,
                    'originCountry' => $originCountry,
                ];

                if ($isIntl) {
                    if (!BpostTools::isHsCode($hsCode)) {
                        throw new HSCodeException($this->translator->trans('The product %name% (ID %id%) appears to have an incorrect HS Code: %code%.', ['%name%' => $article['description'], '%id%' => $product['id_product'], '%code%' => $hsCode ?: $this->translator->trans('<empty>', [], 'Modules.Bpostshm.Admin')], 'Modules.Bpostshm.Admin'));
                    }
                    if (!BpostTools::isOriginCountry($originCountry)) {
                        throw new OriginCountryException($this->translator->trans('The product %name% (ID %id%) appears to have an incorrect origin of country: %code%.', ['%name%' => $article['description'], '%id%' => $product['id_product'], '%code%' => $originCountry ?: $this->translator->trans('<empty>', [], 'Modules.Bpostshm.Admin')], 'Modules.Bpostshm.Admin'));
                    }
                }
                $parcels[$i]->addProduct($article);
            }
        }

        return $parcels;
    }

    /**
     * Get the shipper details based on the given person information.
     *
     * @param array $person the person information
     * @param bool $atHome Is the at home method delivery?
     *
     * @return array|false
     */
    private function getShipper(array $person, bool $atHome)
    {
        if (empty($person)) {
            return false;
        }

        $countryCode = isset($person['id_country']) ? \Country::getIsoById($person['id_country']) : 'BE';
        if (BpostTools::isCanaries($countryCode, (int) $person['postcode'])) {
            $countryCode = 'IC';
        }

        $name = $person['name'];
        $company = '';
        $street = $person['address1'];
        if ($atHome) {
            $company = $person['company'];
            if (!empty($person['address2']) && !BpostTools::isIntl($countryCode)) {
                $street .= ' ' . $person['address2'];
            }
        }

        return [
            'name' => $name,
            'company' => $company,
            'number' => '',
            'street' => $street,
            'addressLineTwo' => BpostTools::isIntl($countryCode) ? $person['address2'] : '',
            'postcode' => $person['postcode'],
            'locality' => $person['city'],
            'countryCode' => $countryCode,
            'phone' => $person['phone'],
            'email' => $person['email'],
        ];
    }

    /**
     * Retrieves the EAD categories for the given product IDs.
     *
     * @param array $productIds an array of product IDs
     *
     * @return string the list of EAD categories separated by commas, or false if no product IDs are provided
     */
    private function getEadCategories(array $productIds): string
    {
        if (empty($productIds)) {
            return false;
        }

        $pids = implode(',', $productIds);
        $cats = $this->database->executeS('
			SELECT DISTINCT cl.`name` FROM `' . _DB_PREFIX_ . 'category_product` cp
			LEFT JOIN `' . _DB_PREFIX_ . 'category` c ON (c.id_category = cp.id_category)
			LEFT JOIN `' . _DB_PREFIX_ . 'category_lang` cl
			    ON (cp.`id_category` = cl.`id_category`' . \Shop::addSqlRestrictionOnLang('cl') . ')
			' . \Shop::addSqlAssociation('category', 'c') . '
			WHERE cp.`id_product` IN (' . pSQL($pids) . ')
			AND cl.`id_lang` = ' . (int) $this->context->language->id . '
			ORDER BY cp.`id_category` DESC
			LIMIT 2
		');

        return implode(', ', array_column($cats, 'name'));
    }

    /**
     * Get the delivery box options based on the given bpostshm order.
     *
     * @param array $bpostshmOrder the bpostshm order
     * @param bool $retour (optional) For a retour? Defaults to false
     *
     * @return array the array of delivery box options
     */
    private function getDeliveryBoxOptions(array $bpostshmOrder, bool $retour = false): array
    {
        $options = [];
        if ($bpostshmOrder['opt_saturday']) {
            $options[] = new Box\Option\SaturdayDelivery();
        }
        if ($bpostshmOrder['opt_signature']) {
            $options[] = new Box\Option\Signed();
        }
        if ($bpostshmOrder['opt_insurance']) {
            $options[] = new Box\Option\Insured(
                Box\Option\Insured::INSURANCE_TYPE_BASIC_INSURANCE
            );
        }
        if ($bpostshmOrder['opt_2nd_presentation'] && !$retour) {
            $options[] = new Box\Option\AutomaticSecondPresentation();
        }

        return $options;
    }

    /**
     * Get the product weight in grams.
     *
     * @param float $weight the weight of one product
     * @param int $qty the quantity of products
     * @param int|null $idShopGroup
     * @param int|null $idShop
     *
     * @return float Number of grams
     */
    private function getProductWeight(float $weight, int $qty = 1, ?int $idShopGroup = null, ?int $idShop = null): float
    {
        if ($weight <= 0) {
            $weight = (float) \Configuration::get('BPOSTSHM_LABELS_DEFAULT_PRODUCT_WEIGHT', null, $idShopGroup, $idShop) * $qty;
        } else {
            $weight = BpostTools::inGrams($weight, $idShopGroup, $idShop) * $qty;
        }

        return (float) $weight;
    }
}
