<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Service;

use Address;
use Carrier;
use Cart;
use Country;
use Db;
use Language;
use Order;
use PrestaShop\Module\BpostShm\Utils\BpostTools;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * Class PickupService
 */
class PickupService
{
    /**
     * bpack 24/7
     */
    const PARCEL_LOCKER_TYPE = 4;

    /**
     * (1+2+16, Post Office + Post Point + Kariboo)
     */
    const PICKUP_TYPE = 19;

    private static $mapping = [
        'id' => 'id_bpost',
        'name' => 'company_name',
        'city' => 'city',
        'address' => 'address1',
        'postcode' => 'zipcode',
        'country' => 'iso_country',
    ];

    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * @var \Context
     */
    private $context;

    /**
     * @var \Db
     */
    private $database;

    /**
     * @var TranslatorInterface
     */
    private $translator;

    /**
     * @var ApiService
     */
    private $apiService;

    /**
     * @var CarrierService
     */
    private $carrierService;

    /**
     * Constructor
     *
     * @param \BpostShm $module
     * @param ApiService $apiService
     * @param CarrierService $carrierService
     */
    public function __construct(
        \BpostShm $module,
        ApiService $apiService,
        CarrierService $carrierService
    ) {
        $this->module = $module;
        $this->context = $module->getContext();
        $this->database = $module->getDatabase();
        $this->translator = $module->getTranslator();
        $this->apiService = $apiService;
        $this->carrierService = $carrierService;
    }

    /**
     * Retrieves the list of available languages supported by the application.
     *
     * @return array an array containing the available languages
     */
    public function getAvailableLanguages(): array
    {
        return ['en', 'nl', 'fr', 'de'];
    }

    /**
     * Retrieves the list of available countries supported by the application.
     *
     * @param bool $parcelLocker (optional) Set to true to retrieve only countries that have parcel lockers available
     *
     * @return array An associative array containing the available countries.
     *               The keys represent the country codes, and the values are their corresponding translated names.
     */
    public function getAvailableCountries(bool $parcelLocker = false): array
    {
        $countries = ['BE' => $this->translator->trans('Belgium', [], 'Modules.Bpostshm.Shop')];
        if ($parcelLocker) {
            return $countries;
        }

        return array_merge($countries, [
            'FR' => $this->translator->trans('France', [], 'Modules.Bpostshm.Shop'),
            'NL' => $this->translator->trans('Netherlands', [], 'Modules.Bpostshm.Shop'),
        ]);
    }

    /**
     * Determines if the given carrier supports pickup option.
     *
     * @param mixed $carrier ID Carrier | Carrier
     *
     * @return bool returns true if the carrier supports pickup, false otherwise
     */
    public function isPickup($carrier): bool
    {
        return $this->getPickupType($carrier) > 0;
    }

    /**
     * Checks if the given carrier supports parcel locker service.
     *
     * @param mixed $carrier ID Carrier | Carrier
     *
     * @return bool returns true if the given carrier supports parcel locker service, false otherwise
     */
    public function isParcelLocker($carrier): bool
    {
        return $this->getPickupType($carrier) == PickupService::PARCEL_LOCKER_TYPE;
    }

    /**
     * Get the pickup type
     *  - Parcel Shop -> 4: bpack 24/7
     *  - Pickup -> 11: (1+2+8, Post Office + Post Point + Click & Collect Shop)
     *
     * @param mixed $carrier ID Carrier | Carrier
     *
     * @return int
     */
    public function getPickupType($carrier): int
    {
        if (is_int($carrier) || is_string($carrier)) {
            $carrier = new \Carrier($carrier);
        }

        $types = [
            'PICKUP' => PickupService::PICKUP_TYPE,
            'PARCEL_LOCKER' => PickupService::PARCEL_LOCKER_TYPE,
        ];

        $method = $this->carrierService->getCarrierMethod($carrier);
        if (!$method) {
            foreach ($types as $method => $type) {
                $idCarrier = $this->getCarrierId($method);
                if (\Validate::isLoadedObject($carrier)
                    && $idCarrier
                    && ($carrier->id == $idCarrier || $carrier->id_reference == $idCarrier)) {
                    \Db::getInstance()->insert(
                        'bpostshm_carrier_history',
                        [
                            'id_carrier' => (int) $carrier->id,
                            'shipping_method' => $method,
                        ],
                        false,
                        true,
                        \Db::REPLACE
                    );

                    return $type;
                }
            }
        } elseif (isset($types[$method])) {
            return $types[$method];
        }

        return 0;
    }

    /**
     * Retrieves the carrier ID based on the specified type.
     *
     * @param string $type (Optional) The type of carrier. Default value is 'PICKUP'.
     *
     * @return int the carrier ID associated with the specified type
     */
    public function getCarrierId(string $type = 'PICKUP'): int
    {
        return (int) \Configuration::getGlobalValue('BPOSTSHM_ID_CARRIER_' . $type);
    }

    /**
     * Retrieves the carrier object based on the given type.
     *
     * @param string $type (optional) The type of carrier. Default is 'PICKUP'.
     *
     * @return \Carrier|null the carrier object associated with the given type, or null if not found
     */
    public function getCarrier(string $type = 'PICKUP'): ?\Carrier
    {
        $carrier = new \Carrier($this->getCarrierId($type));
        if (\Validate::isLoadedObject($carrier)) {
            return $carrier;
        }

        return null;
    }

    /**
     * Retrieves the list of pickup points available for a given address.
     *
     * @param string $address the address of the pickup point
     * @param string $postcode the postal code of the pickup point
     * @param string|null $city the city of the pickup point
     * @param string|null $country the country code of the pickup point (defaults to 'BE' if not provided)
     * @param string|null $deliveryDate the delivery date for the pickup point (optional)
     * @param int|null $type the type of pickup points to retrieve (defaults to PickupService::PICKUP_TYPE)
     *
     * @return array the list of available pickup points
     */
    public function getPickupPoints(
        string $address,
        string $postcode,
        ?string $city = null,
        ?string $country = null,
        ?string $deliveryDate = null,
        ?int $type = PickupService::PICKUP_TYPE
    ): array {
        if ($city) {
            $tryZones[2] = "$postcode $city";
        }
        $tryZones[1] = $postcode;

        if (!$country) {
            $country = 'BE';
        }

        $street = $address;
        $nr = '';
        if ('BE' == $country) {
            $info = $this->getAddressStreetNr($address);
            $street = $info['street'];
            $nr = $info['nr'];
        }

        $points = [];
        foreach ($tryZones as $zone) {
            try {
                $points = $this->apiService->getPickupPoints($street, $nr, $zone, $type, $country, $deliveryDate);
            } catch (\Exception $e) {
            }
            if ($points) {
                break;
            }
            if ($street != '' && $nr != '') {
                try {
                    $points = $this->apiService->getPickupPoints('', '', $zone, $type, $country, $deliveryDate);
                } catch (\Exception $e) {
                }
            }
            if ($points) {
                break;
            }
        }

        return $points;
    }

    /**
     * Retrieves the street number and name from a given address string.
     *
     * @param string $address1 the address string to extract the street number and name from
     *
     * @return array an array containing the street number and name extracted from the address
     */
    private function getAddressStreetNr(string $address1 = ''): array
    {
        preg_match('#([0-9]+\w?)?[, ]*([\pL&;\'\. -]+)[, ]*([0-9]+[a-z]*)?[, ]*(.*)?#iu', $address1, $matches);
        $nr = '';
        if (!empty($matches[1])) {
            $nr = $matches[1];
        } elseif (!empty($matches[3])) {
            $nr = $matches[3];
        }

        return [
            'nr' => $nr,
            'street' => !empty($matches[2]) ? $matches[2] : '',
        ];
    }

    /**
     * Retrieves the details of a pickup point using the given ID.
     *
     * @param int $id The ID of the pickup point
     * @param string $isoCode
     * @param string $country The country code of the pickup point (default: 'BE')
     * @param int $type The type of the pickup point (default: 3)
     *
     * @return array an array containing the pickup details if found, or false if the pickup point is not found
     *
     * @throws \Exception
     */
    public function getPickupDetails(
        int $id,
        string $isoCode,
        string $country = 'BE',
        int $type = self::PARCEL_LOCKER_TYPE + self::PICKUP_TYPE
    ): array {
        $pickup = $this->apiService->getPickupPointDetails($id, $isoCode, $country, $type);
        $openingHours = [];
        $note = $pickup->getNote();
        $hours = $pickup->getHours();
        if (count($hours)) {
            foreach ($hours as $day => $hour) {
                $schedule = '';
                $hasScheduleAm = $hour->getAmOpen() && $hour->getAmOpen() != '00:00';
                $hasScheduleAm |= $hour->getAmClose() && $hour->getAmClose() != '00:00';
                $hasSchedulePm = $hour->getPmOpen() && $hour->getPmOpen() != '00:00';
                $hasSchedulePm |= $hour->getPmClose() && $hour->getPmClose() != '00:00';
                if ($hasScheduleAm) {
                    if ($hour->getAmOpen()) {
                        $schedule .= $hour->getAmOpen();
                    }
                    if ($hour->getAmClose()) {
                        $schedule .= '-' . $hour->getAmClose();
                    }
                }
                if ($schedule) {
                    $schedule .= ' ';
                }
                if ($hasSchedulePm) {
                    if ($hour->getPmOpen()) {
                        $schedule .= $hour->getPmOpen();
                    }
                    if ($hour->getPmClose()) {
                        $schedule .= '-' . $hour->getPmClose();
                    }
                }

                if (!$schedule) {
                    $schedule = $this->translator->trans('Closed', [], 'Modules.Bpostshm.Shop');
                }
                $openingHours[] = [
                    'day' => $day,
                    'schedule' => trim($schedule),
                ];
            }
        }

        return [
            'id_bpost' => $pickup->getId(),
            'company_name' => $pickup->getOffice(),
            'nr' => $pickup->getNr(),
            'street' => $pickup->getStreet(),
            'city' => $pickup->getCity(),
            'zipcode' => $pickup->getZip(),
            'iso_country' => $country,
            'type' => $pickup->getType(),
            'note' => $note,
            'openingHours' => $openingHours,
        ];
    }

    /**
     * Sets the pickup point for an order.
     *
     * @param int $idOrder the ID of the order
     * @param array|null $pickupPoint the pickup point data
     *
     * @return array an array containing any error messages encountered during the process
     */
    public function setPickupForOrder(int $idOrder, ?array $pickupPoint = null): array
    {
        $order = new \Order($idOrder);
        if (!\Validate::isLoadedObject($order)) {
            return [
                $this->translator->trans(
                    'Order with ID "%id%" not found!',
                    ['%id%' => $idOrder],
                    'Modules.Bpostshm.Admin'
                ),
            ];
        }

        if ($this->isPickup($order->id_carrier)) {
            if (!is_array($pickupPoint)) {
                $addressDelivery = new \Address($order->id_address_delivery);
                $country = \Country::getIsoById($addressDelivery->id_country) ?: null;
                $pickupPoint = $this->getPickup(
                    (int) $this->getPickupIdByOrderId($order->id),
                    new \Language($order->id_lang),
                    $country
                );
                if (!$pickupPoint) {
                    return [
                        $this->translator->trans(
                            'Pick-up point with ID "%id%" not found!',
                            ['%id%' => $idOrder],
                            'Modules.Bpostshm.Admin'
                        ),
                    ];
                }
            }
            $this->storePickup($order->id_carrier, $order->id_cart, $pickupPoint);

            $deliveryAddress = $this->getSelectedPickupAddress($order->id_lang, $order->id_cart);
            if (!$deliveryAddress || !$order->id_cart) {
                $log = $this->translator->trans(
                    'Validate Order: Unable to find Pick-up point "%pickup%" for order %order% (cart: %cart%).',
                    [
                        '%pickup%' => $this->getSelectedPickupId($order->id_carrier, $order->id_cart),
                        '%order%' => $order->id,
                        '%cart%' => $order->id_cart,
                    ],
                    'Modules.Bpostshm.Admin'
                );

                return [$log];
            } else {
                $cart = new \Cart($order->id_cart);
                if ($cart->id_address_invoice !== $cart->id_address_delivery) {
                    $deliveryAddress->id = $cart->id_address_delivery;
                }

                $oldAddress = new \Address($cart->id_address_delivery);

                $deliveryAddress->firstname = $oldAddress->firstname;
                $deliveryAddress->lastname = $oldAddress->lastname;
                $deliveryAddress->id_customer = $cart->id_customer;
                $deliveryAddress->phone = $oldAddress->phone;
                $deliveryAddress->phone_mobile = $oldAddress->phone_mobile;

                if ($cart->id_address_invoice === $cart->id_address_delivery) {
                    $deliveryAddress->add();
                    $cart->id_address_delivery = $deliveryAddress->id;
                    $cart->update();
                } else {
                    $deliveryAddress->update();
                }
                $order->id_address_delivery = $deliveryAddress->id;
                $order->update();

                $this->database->execute(
                    '
                    UPDATE `' . _DB_PREFIX_ . 'cart_product`
                    SET `id_address_delivery` = ' . (int) $deliveryAddress->id . '
                    WHERE `id_cart` = ' . (int) $cart->id
                );

                $this->database->execute(
                    '
                    UPDATE `' . _DB_PREFIX_ . 'customization`
		            SET `id_address_delivery` = ' . (int) $deliveryAddress->id . '
		            WHERE `id_cart` = ' . (int) $cart->id
                );
            }
        } else {
            return [
                $this->translator->trans(
                    'Order with ID "%d" does not have bpost Carrier!',
                    ['%id%' => $idOrder],
                    'Modules.Bpostshm.Admin'
                ),
            ];
        }

        return [];
    }

    /**
     * Stores the pick-up point information in the database for a given carrier and cart.
     *
     * @param int|null $idCarrier The ID of the carrier. null if we only want to store pickup point in DB
     * @param int|null $idCart The ID of the cart. null if we only want to store pickup point in DB
     * @param array $pickupPoint the pick-up point information
     *
     * @return void
     *
     * @throws \PrestaShopException if any required pick-up point information is missing
     * @throws \PrestaShopException if the carrier ID or the cart ID is missing
     */
    public function storePickup(?int $idCarrier, ?int $idCart, array $pickupPoint): void
    {
        if (null !== $idCarrier || null !== $idCart) {
            if (!$idCarrier) {
                throw new \PrestaShopException($this->translator->trans('Carrier ID is missing!', [], 'Modules.Bpostshm.Admin'));
            }

            if (!$idCart) {
                throw new \PrestaShopException($this->translator->trans('Cart ID is missing!', [], 'Modules.Bpostshm.Admin'));
            }
        }

        foreach (self::$mapping as $keyInfo => $keyTable) {
            if (isset($pickupPoint[$keyInfo])) {
                $pickupPoint[$keyTable] = $pickupPoint[$keyInfo];
            }
        }

        $fields = [
            'id_bpost' => true,
            'id_lang' => true,
            'company_name' => true,
            'nr' => true,
            'street' => true,
            'city' => true,
            'zipcode' => true,
            'iso_country' => true,
            'type' => true,
        ];
        $data = [
            'date_upd' => date('Y-m-d H:i:s'),
        ];
        foreach ($fields as $field => $required) {
            if (!isset($pickupPoint[$field])) {
                if ($required) {
                    throw new \PrestaShopException($this->translator->trans('Pick-up point: %field% info is missing!', ['%field%' => $field], 'Modules.Bpostshm.Admin'));
                }
            } else {
                $data[$field] = pSQL($pickupPoint[$field]);
            }
        }
        $this->database->insert(
            'bpostshm_pickup',
            $data,
            false,
            true,
            \Db::REPLACE
        );
        if (null !== $idCarrier || null !== $idCart) {
            $this->database->insert(
                'bpostshm_cart',
                [
                    'id_cart' => (int) $idCart,
                    'id_carrier' => (int) $idCarrier,
                    'id_bpost' => $data['id_bpost'],
                ],
                false,
                false,
                \Db::ON_DUPLICATE_KEY
            );
        }
    }

    /**
     * Retrieves the selected pickup ID for a given carrier and cart.
     *
     * @param int $idCarrier the ID of the carrier
     * @param int|null $idCart the ID of the cart (optional)
     *
     * @return int|null the selected pickup ID for the given carrier and cart, or null if not found
     */
    public function getSelectedPickupId(int $idCarrier, ?int $idCart = null): ?int
    {
        if (!$idCart) {
            if (\Validate::isLoadedObject($this->context->cart)) {
                $idCart = $this->context->cart->id;
            }
        }

        if ($idCart) {
            $sql = new \DbQuery();
            $sql->select('id_bpost');
            $sql->from('bpostshm_cart');
            $sql->where('id_cart = ' . (int) $idCart . ' AND id_carrier = ' . $idCarrier);

            return $this->database->getValue($sql);
        }

        return null;
    }

    /**
     * Retrieves the selected pickup address based on the provided parameters.
     *
     * @param int $idCarrier the ID of the carrier
     * @param int $idLang the ID of the language
     * @param int|null $idCart The ID of the cart. Defaults to null.
     * @param bool $replaceDeliveryAddress Whether to replace the delivery address. Defaults to false.
     * @param string $firstname The firstname of the address. Defaults to an empty string.
     * @param string $lastname The lastname of the address. Defaults to an empty string.
     *
     * @return \Address|null the selected pickup address as an instance of the Address class,
     *                       or null if there is no selected pickup
     */
    public function getSelectedPickupAddress(
        int $idCarrier,
        int $idLang,
        ?int $idCart = null,
        bool $replaceDeliveryAddress = false,
        string $firstname = '',
        string $lastname = ''
    ): ?\Address {
        $address = $cart = null;

        if (!$idCart) {
            if (\Validate::isLoadedObject($this->context->cart)) {
                $idCart = $this->context->cart->id;
                $cart = $this->context->cart;
            }
        }
        if (!$cart && $idCart) {
            $cart = new \Cart($idCart);
        }
        $country = null;
        if (\Validate::isLoadedObject($cart)) {
            $addressDelivery = new \Address($cart->id_address_delivery);
            $country = \Country::getIsoById($addressDelivery->id_country) ?: null;
        }

        $selectedPickup = $this->getPickup(
            (int) $this->getSelectedPickupId($idCarrier, $idCart),
            new \Language($idLang),
            $country
        );
        if ($selectedPickup) {
            $address = new \Address();
            $address->alias = $selectedPickup['id_bpost'];
            $address->id_country = \Country::getByIso($selectedPickup['iso_country']);
            $address->country = \Country::getNameById($idLang, $address->id_country);
            $address->firstname = $firstname;
            $address->lastname = $lastname;
            $address->company = $selectedPickup['company_name'];
            $address->address1 = \Tools::substr($selectedPickup['address1'], 0, 128);
            $address->postcode = $selectedPickup['zipcode'];
            $address->city = $selectedPickup['city'];
            $address->active = 0;
            $address->deleted = 1;
        }

        if ($replaceDeliveryAddress) {
            if (!$address) {
                $address = new \Address();
                $address->address1 = $this->translator->trans(
                    'Unknown Address, please contact support.',
                    [],
                    'Modules.Bpostshm.Shop'
                );
            }

            if ($this->isParcelLocker($idCarrier)) {
                $address->alias = $this->translator->trans('bpost - Parcel locker', [], 'Modules.Bpostshm.Shop');
            } else {
                $address->alias = $this->translator->trans('bpost - Pick-up point', [], 'Modules.Bpostshm.Shop');
            }
        }

        return $address;
    }

    /**
     * Retrieves the pickup ID associated with the given order ID.
     *
     * @param int $orderId the ID of the order
     *
     * @return int|false the pickup ID if the order is associated with a pickup, otherwise false
     */
    public function getPickupIdByOrderId(int $orderId)
    {
        $order = new \Order($orderId);
        if (!$this->isPickup($order->id_carrier)) {
            return false;
        }

        $idCart = \Cart::getCartIdByOrderId($orderId);

        return $this->getSelectedPickupId($order->id_carrier, $idCart);
    }

    /**
     * Retrieves the pickup details for a given pickup ID.
     * If the pickup address is not found in DB, we try to retrieve it from bpost API.
     *
     * @param int $id the ID of the pickup
     * @param \Language $language the Language of the pickup
     * @param string|null $country The country of the pickup (optional). Default is null.
     *
     * @return array|null An associative array containing the pickup details,
     *                    or null if the $id is not provided.
     *                    [
     *                    `id_bpost`,
     *                    `company_name`,
     *                    `address1`,
     *                    `nr`,
     *                    `street`,
     *                    `city`,
     *                    `zipcode`,
     *                    `iso_country`,
     *                    `type`,
     *                    `date_upd`,
     *                    ]
     */
    public function getPickup(int $id, \Language $language, ?string $country = null): ?array
    {
        if (!$id) {
            return null;
        }

        if (!\Validate::isLoadedObject($language)) {
            $language = $this->context->language;
        }

        if ($country) {
            $country = \Tools::strtoupper($country);
        }

        $sql = new \DbQuery();
        $sql->select('*');
        $sql->from('bpostshm_pickup');
        $sql->where('id_bpost = ' . $id);
        $sql->where('id_lang = ' . $language->id);
        $pickup = $this->database->getRow($sql);

        $pickupAnotherLang = null;
        if (!$pickup) {
            $sqlAnotherLang = new \DbQuery();
            $sqlAnotherLang->select('*');
            $sqlAnotherLang->from('bpostshm_pickup');
            $sqlAnotherLang->where('id_bpost = ' . $id);
            $pickupAnotherLang = $this->database->getRow($sqlAnotherLang);
        }

        if (!$pickup && in_array($country, array_keys($this->getAvailableCountries()))) {
            try {
                $pickupPoint = $this->getPickupDetails($id, $language->iso_code, $country);
                if ($pickupPoint) {
                    $pickupPoint['id_lang'] = $language->id;
                    $this->storePickup(null, null, $pickupPoint);
                    $pickup = $this->database->getRow($sql, false);
                }
            } catch (\Exception $e) {
                $this->module->log($e->getMessage());
            }
        }

        if (!$pickup) {
            $pickup = $pickupAnotherLang;
        }

        if ($pickup && isset($pickup['street']) && isset($pickup['nr'])) {
            $pickup['address1'] = $pickup['street'] . ($pickup['nr'] ? (', ' . $pickup['nr']) : '');
        }

        return $pickup ?: null;
    }

    /**
     * Cleans the address for pickup by removing any unwanted characters or formatting.
     *
     * @param \Address $address the address to be cleaned
     *
     * @return \Address the cleaned address
     */
    public function cleanAddressForPickup(\Address $address): \Address
    {
        $address->address1 = BpostTools::cleanSpecialChars($address->address1);
        $address->address2 = BpostTools::cleanSpecialChars($address->address2);
        $address->postcode = BpostTools::cleanSpecialChars($address->postcode);
        $address->city = BpostTools::cleanSpecialChars($address->city);

        return $address;
    }
}
