<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Utils;

use Carrier;
use Cart;
use Order;

/**
 * Class BpostTools
 */
class BpostTools
{
    /**
     * Checks if a given carrier is a bpost carrier.
     *
     * @param int|string|\Carrier $carrier The carrier to be checked.
     *                                     It can be either the carrier ID or the carrier object.
     *
     * @return bool returns true if the carrier is a bpost carrier, false otherwise
     */
    public static function isBpostCarrier($carrier): bool
    {
        if (!$carrier) {
            return false;
        }

        if (is_int($carrier) || is_string($carrier)) {
            $carrier = new \Carrier($carrier);
        }

        return \Validate::isLoadedObject($carrier) && $carrier->external_module_name == 'bpostshm';
    }

    /**
     * Checks if a given cart is associated with a bpost carrier.
     *
     * @param int|string|\Cart $cart The cart to be checked. It can be either the cart ID or the cart object.
     *
     * @return bool returns true if the cart is associated with a bpost carrier, false otherwise
     */
    public static function isBpostCart($cart): bool
    {
        if (!$cart) {
            return false;
        }

        if (is_int($cart) || is_string($cart)) {
            $cart = new \Cart($cart);
        }

        return self::isBpostCarrier($cart->id_carrier);
    }

    /**
     * Checks if a given order is associated with a bpost carrier.
     *
     * @param int|string|\Order $order The order to be checked. It can be either the order ID or the order object.
     *
     * @return bool returns true if the order is associated with a bpost carrier, false otherwise
     */
    public static function isBpostOrder($order): bool
    {
        if (!$order) {
            return false;
        }

        if (is_int($order) || is_string($order)) {
            $order = new \Order($order);
        }

        return self::isBpostCarrier($order->id_carrier);
    }

    /**
     * Check if the provided string is a valid time.
     *
     * @param string $str the time string to be checked
     *
     * @return bool whether the provided string is a valid time
     */
    public static function isTime(string $str): bool
    {
        return BpostTools::cleanTime($str) !== false;
    }

    /**
     * Checks if the given HS Code is valid.
     *
     * @param string $hsCode The HsCode to check. It should be a numeric string of at least 4 digits.
     *
     * @return bool returns true if the HsCode is valid, false otherwise
     */
    public static function isHsCode(string $hsCode): bool
    {
        return preg_match('/^\d{4,9}$/', $hsCode);
    }

    /**
     * Checks if the given origin country code is valid.
     *
     * @param string $originCountry the origin country code to check
     *
     * @return bool returns true if the origin country code is a two-letter string
     *              containing only letters (case-insensitive)
     */
    public static function isOriginCountry(string $originCountry): bool
    {
        return preg_match('/^[a-zA-Z]{2}$/', $originCountry);
    }

    /**
     * Check the belgium mobile phone
     *
     * @param string $number
     *
     * @return bool returns true if the number is a belgium mobile phone, false otherwise
     */
    public static function isBeMobileNumber(string $number): bool
    {
        $number = BpostTools::cleanPhone($number);

        return (bool) preg_match('/^04[7-9][0-9]{7}$/', $number);
    }

    /**
     * Checks if the country code is considered as international
     *
     * @param string $countryCode the iso country code to check
     *
     * @return bool|null Returns true if the country is considered as international,
     *                   false otherwise or null if the country code is invalid
     */
    public static function isIntl(string $countryCode): ?bool
    {
        if (!preg_match('/^[a-zA-Z]{2}$/', $countryCode)) {
            return null;
        }

        return strtoupper($countryCode) != 'BE';
    }

    /**
     * Checks if the given country code belongs to a country in the European Union (EU).
     *
     * @param string $countryCode the iso country code to check
     *
     * @return bool returns true if the country code belongs to a country in the EU
     */
    public static function isInEU(string $countryCode): bool
    {
        return in_array(\Tools::strtoupper($countryCode), [
            // EU 28
            'AT', 'BE', 'BG', 'HR', 'CY', 'CZ', 'DK', 'EE', 'FI',
            'FR', 'DE', 'GR', 'HU', 'IE', 'IT', 'LV', 'LT', 'LU',
            'MT', 'NL', 'PL', 'PT', 'RO', 'SK', 'SI', 'ES', 'SE', 'XI',
            // OMR + special case
            'GF', 'GP', 'MQ', 'ME', 'YT', 'RE', 'MF', 'GI', 'AX',
            // OCT + microstates
            'PM', 'GL', 'BL', 'SX', 'AW', 'CW', 'WF', 'PF', 'NC',
            'TF', 'AI', 'BM', 'IO', 'VG', 'KY', 'FK', 'MS', 'PN',
            'SH', 'GS', 'TC', 'AD', 'LI', 'MC', 'SM', 'VA', 'JE', 'GG',
        ]);
    }

    /**
     * Checks if the given country code and postcode belong to the Canary Islands in Spain.
     *
     * @param string $countryCode the iso country code to check
     * @param int $postcode the postcode to check
     *
     * @return bool returns true if the country code is 'ES'
     *              and the postcode falls within the range of Canary Islands' postcodes
     */
    public static function isCanaries(string $countryCode, int $postcode): bool
    {
        return \Tools::strtoupper($countryCode) === 'ES'
            && (($postcode >= 35000 && $postcode < 36000) || ($postcode >= 38000 && $postcode < 39000));
    }

    /**
     * Clean String for bpost API.
     *
     * Replaces all characters in the given string that are not letters, numbers,
     * dashes, underscores, periods, whitespace, single quotes, parentheses, or ampersands.
     * It also replaces forward slashes and backslashes with dashes and replaces
     * ampersands with their HTML entity representation &amp; if not already present in the string.
     * Finally, it returns the cleaned string, trimmed and optionally truncated to a maximum length.
     *
     * @param string $str The input string to be cleaned
     * @param int $max Optional. If set to true, the cleaned string will be truncated to the maximum length
     *                 provided in the `$max` parameter. If not set or set to false, the cleaned string will
     *                 not be truncated.
     *
     * @return string The cleaned string
     */
    public static function cleanStr(?string $str, int $max = 0): string
    {
        if ($str === null) {
            return '';
        }
        $pattern = '/[^\pL0-9,-_\.\s\'\(\)\&]/u';
        $rpl = '-';
        $str = preg_replace($pattern, $rpl, trim($str));
        $str = str_replace(['/', '\\'], $rpl, $str);
        if (false === strpos($str, '&amp;')) {
            $str = str_replace('&', '&amp;', $str);
        }

        return mb_substr($str, 0, $max ?: mb_strlen($str));
    }

    /**
     * Cleans the given text by removing any special characters and converting them to their regular counterparts.
     *
     * @param string $text the text to be cleaned
     *
     * @return string the cleaned text
     */
    public static function cleanSpecialChars(string $text): string
    {
        $utf8 = [
            '/[áàâãªä]/u' => 'a',
            '/[ÁÀÂÃÄ]/u' => 'A',
            '/[ÍÌÎÏ]/u' => 'I',
            '/[íìîï]/u' => 'i',
            '/[éèêë]/u' => 'e',
            '/[ÉÈÊË]/u' => 'E',
            '/[óòôõºö]/u' => 'o',
            '/[ÓÒÔÕÖ]/u' => 'O',
            '/[úùûü]/u' => 'u',
            '/[ÚÙÛÜ]/u' => 'U',
            '/ç/' => 'c',
            '/Ç/' => 'C',
            '/ñ/' => 'n',
            '/Ñ/' => 'N',
            '/–/' => '-', // UTF-8 hyphen to "normal" hyphen
            '/[’‘]/u' => '\'', // Literally a single quote
            '/[‹›‚]/u' => ',',
            '/[“”«»„]/u' => '"', // Double quote
            '/ /' => ' ', // nonbreaking space (equiv. to 0x160)
        ];

        return preg_replace(array_keys($utf8), array_values($utf8), $text);
    }

    /**
     * Clean a phone number by removing all non-numeric characters.
     *
     * @param string $str the phone number to be cleaned
     *
     * @return string the cleaned phone number
     */
    public static function cleanPhone(string $str): string
    {
        return preg_replace('/[^0-9]/i', '', $str);
    }

    /**
     * Clean the time string.
     *
     * @param string $str the time string to clean
     *
     * @return string|false the cleaned time string, or false if the time string is invalid
     */
    public static function cleanTime(string $str)
    {
        $H = '';
        $m = '';
        $i = 1;
        foreach (explode(':', $str) as $t) {
            $t = preg_replace('/[^0-9]/i', '', $t);
            if ($t) {
                if ($i === 1 && (int) $t >= 0 && (int) $t <= 23) {
                    $H = (int) $t;
                    if ($H < 10) {
                        $H = '0' . $H;
                    }
                } elseif ($i === 2 && (int) $t >= 0 && (int) $t <= 59) {
                    $m = (int) $t;
                    if ($m < 10) {
                        $m = '0' . $m;
                    }
                }
                ++$i;
            }
        }

        return ($H !== '' && $m !== '') ? ($H . $m) : false;
    }

    /**
     * Convert $weight in grams
     *
     * @param float $weight
     * @param int|null $idShopGroup
     * @param int|null $idShop
     *
     * @return float
     */
    public static function inGrams(float $weight = 0, ?int $idShopGroup = null, ?int $idShop = null): float
    {
        if (!is_numeric($weight) || 0.0 == (float) $weight) {
            return 10;
        }

        switch (\Tools::strtolower(\Configuration::get('PS_WEIGHT_UNIT', null, $idShopGroup, $idShop))) {
            case 'g':
                $ratio = 1;
                break;
            case 'lbs':
            case 'lb':
                $ratio = 453.592;
                break;
            case 'oz':
                $ratio = 28.34952;
                break;
            default: // 'kg'
                $ratio = 1000;
                break;
        }

        return (int) ceil((float) $weight * $ratio);
    }

    /**
     * Check if we need CN23 form
     *
     * @param string $isoFrom
     * @param string $isoTo
     * @param string $postalCode
     *
     * @return bool
     */
    public static function needCN23(string $isoFrom, string $isoTo, string $postalCode): bool
    {
        if (in_array($isoTo, ['ES', 'IC', 'IT', 'DE', 'CH', 'GB', 'GR'])) {
            if (BpostTools::needCN23Exceptions($isoTo, $postalCode)) {
                return true;
            }
        }
        if (in_array($isoFrom, ['FR', 'MC'])) {
            if (BpostTools::isInEU($isoTo) && !in_array($isoTo, ['CH', 'HR', 'IS', 'NO', 'GB'])) {
                return false;
            }
        } elseif (in_array($isoFrom, ['GP', 'MQ', 'GF', 'RE', 'YT', 'PM', 'MF', 'BL'])) {
            if (in_array($isoFrom, ['GP', 'MQ', 'BL', 'MF']) && in_array($isoTo, ['GP', 'MQ', 'BL', 'MF'])) {
                return false;
            }
        }
        // (https://www.bpost.be/en/faq/which-countries-do-not-require-customs-documents-if-i-want-send-parcel)
        $eu27 = ['AT', 'BE', 'BG', 'HR', 'CY', 'CZ', 'DK', 'EE', 'FI', 'FR', 'DE', 'GR', 'HU', 'IE', 'IT', 'LV',
            'LT', 'LU', 'MT', 'NL', 'PL', 'PT', 'RO', 'SK', 'SI', 'ES', 'SE', ];
        $fromEU = in_array($isoFrom, $eu27);
        $toEU = in_array($isoTo, $eu27);

        return !$fromEU || !$toEU;
    }

    /**
     * Check if we need CN23 form
     *
     * @param string $iso
     * @param string $postalCode
     *
     * @return bool
     */
    private static function needCN23Exceptions(string $iso, string $postalCode): bool
    {
        switch ($iso) {
            case 'ES':
                $province = \Tools::substr($postalCode, 0, 2);

                return in_array($province, ['35', '38', '51', '52']);
            case 'IT':
                // Sondrio, Campione D'Italia, Livigno
                return in_array($postalCode, ['23030', '22060', '22061', '23041']);
            case 'DE':
                return in_array($postalCode, ['27498', '78266']);
            case 'CH':
                return $postalCode == '8238';
            case 'GB':
                $postalZone = \Tools::substr($postalCode, 0, 2);

                return in_array($postalZone, ['JE', 'IM', 'GY']);
            case 'GR':
                return in_array($postalCode, [
                    '63075', '60386', '63086', '63087', '630 75', '603 86', '630 86', '630 87',
                ]);
        }

        return false;
    }
}
