<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Utils;

/**
 * Class FileUpgrader
 */
class FileUpgrader
{
    const MODULE_NAME = 'bpostshm';

    /**
     * Moves labels from /pdf/* to /download/bpostshm/*
     *
     * @param string|null $source The source directory
     * @param string|null $destination The destination directory
     * @param bool $isRacine is racine?
     *
     * @return void
     */
    public static function moveOldLabels(?string $source = null, ?string $destination = null, bool $isRacine = true)
    {
        if (!$source) {
            $source = _PS_MODULE_DIR_ . FileUpgrader::MODULE_NAME . '/pdf/';
        }
        if (!$destination) {
            $destination = _PS_DOWNLOAD_DIR_ . FileUpgrader::MODULE_NAME . '/';
        }
        if (!is_dir($destination)) {
            mkdir($destination, 0775, true);
        }

        if (is_dir($source)) {
            $dir = opendir($source);
            if ($dir) {
                while ($element = readdir($dir)) {
                    if (is_dir($source . $element) && $element != '.' && $element != '..') {
                        FileUpgrader::moveOldLabels($source . $element . '/', $destination, false);
                    }
                    if (!$isRacine) {
                        $info = pathinfo($element);
                        if (is_file($source . $element) && isset($info['extension']) && $info['extension'] === 'pdf') {
                            $parent = basename(dirname($source . $element));
                            $newFile = $parent . '.pdf';
                            if (rename($source . $element, $destination . $newFile)) {
                                if (file_exists($source . 'index.php')
                                    && is_writable($source . 'index.php')) {
                                    unlink($source . 'index.php');
                                }
                                @rmdir($source);
                            }
                        }
                    }
                }
                closedir($dir);
            }
        }
    }

    /**
     * Removes files and directories recursively.
     *
     * @param array $paths the file and directory list
     *
     * @return bool
     */
    public static function removeAll(array $paths): bool
    {
        $dir = _PS_MODULE_DIR_ . FileUpgrader::MODULE_NAME;
        $ret = true;
        foreach ($paths as $path) {
            if (is_file($dir . $path)) {
                $ret &= FileUpgrader::removeFiles([$path]);
            }
            if (is_dir($dir . $path)) {
                $ret &= FileUpgrader::removeDir($path);
            }
        }

        return (bool) $ret;
    }

    /**
     * Removes files
     *
     * @param array $files The file list
     *
     * @return bool
     */
    protected static function removeFiles(array $files): bool
    {
        $dir = _PS_MODULE_DIR_ . FileUpgrader::MODULE_NAME;
        $res = true;
        foreach ($files as $file) {
            $file = $dir . $file;
            if (is_writable($file)) {
                $res &= unlink($file);
            }
        }

        return (bool) $res;
    }

    /**
     * Removes directory and its files recursively.
     *
     * @param string $path the path of the directory
     *
     * @return bool the result of deletion
     */
    public static function removeDir(string $path): bool
    {
        $dir = _PS_MODULE_DIR_ . FileUpgrader::MODULE_NAME;
        $res = true;
        if (!empty($path) && $path !== $dir && is_dir($dir . $path)) {
            $res &= FileUpgrader::delTree($dir . $path);
        }

        return (bool) $res;
    }

    /**
     * Renames directory with suffix "_bakYYYYMMDD"
     *
     * @param string $path the path of the directory
     *
     * @return bool true on success (if the directory exists) or false otherwise
     */
    public static function backupDir(string $dir): bool
    {
        return file_exists($dir)
            && rename($dir, $dir . '_bak' . date('YmdHis'));
    }

    /**
     * Removes directory and its files recursively.
     *
     * @param string $dir the path of the directory
     *
     * @return bool the result of deletion
     */
    protected static function delTree(string $dir): bool
    {
        if (empty($dir)) {
            return false;
        }

        $files = array_diff(scandir($dir), ['.', '..']);
        foreach ($files as $file) {
            $branch = $dir . DIRECTORY_SEPARATOR . $file;
            if (is_dir($branch)) {
                FileUpgrader::delTree($branch);
            } else {
                unlink($branch);
            }
        }

        return rmdir($dir);
    }
}
