<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Utils;

use BpostShm;
use Configuration;
use Db;
use PrestaShop\Module\BpostShm\Helper\SettingsFormHelper;
use PrestaShopBundle\Entity\Repository\TabRepository;
use PrestaShopBundle\Translation\TranslatorComponent as Translator;
use Symfony\Component\Console\Style\SymfonyStyle;
use Zone;

/**
 * Class Installer
 *
 * This class is responsible for installing and upgrading the software.
 */
class Installer
{
    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * @var Translator
     */
    private $translator;

    /**
     * Installer Constructor
     *
     * @param \BpostShm $module The BpostShm module object
     *
     * @return void
     */
    public function __construct(\BpostShm $module)
    {
        $this->module = $module;
        $this->translator = $module->getTranslator();
    }

    /**
     * Install method that performs necessary actions for installing the software.
     *
     * This method installs the software by executing the following tasks:
     * - Installs the database using the `installDb` method.
     *     - The `installDb` method is responsible for creating the required database tables.
     * - Sets default values if needed using the `setDefaultValues` method.
     *     - The `setDefaultValues` method is responsible for setting default values in the software.
     * - Installs the shipping methods using the `installShippingMethods` method.
     *     - The `installShippingMethods` method is responsible for installing the available shipping methods.
     * - Installs the `bpostShmTab` using the `installBpostShmTab` method.
     *     - The `installBpostShmTab` method is responsible for installing the `bpostShmTab`.
     *
     * @return void
     *
     * @throws \PrestaShopDatabaseException
     * @throws \PrestaShopException
     */
    public function install(): void
    {
        $this->installDb();
        $this->setDefaultValues(true);
        $this->installShippingMethods();
        $this->installBpostShmTab();
        FileUpgrader::moveOldLabels();
    }

    /**
     * Upgrade method that performs necessary actions for upgrading the software.
     *
     * This method upgrades the software by executing the following tasks:
     * - Upgrades the configuration settings.
     * - Sets default values if needed.
     * - Upgrades the database.
     * - Uninstalls the `bpostShmTab` using the `Uninstaller` class.
     * - Installs the `bpostShmTab`.
     * - Move the old labels.
     *
     * @return void
     *
     * @throws \PrestaShopDatabaseException
     * @throws \PrestaShopException
     */
    public function upgradeV2(): void
    {
        $this->upgradeConfigToV2();
        $this->setDefaultValues();
        $this->upgradeDbToV2();
        (new Uninstaller($this->module))->uninstallBpostShmTab();
        $this->installBpostShmTab();
        FileUpgrader::moveOldLabels();
    }

    /**
     * Adds belgium zone if not exists
     *
     * @return int id zone
     *
     * @throws \PrestaShopDatabaseException
     * @throws \PrestaShopException
     */
    protected function addBelgiumZone(): int
    {
        $idCountry = \Country::getByIso('BE');
        $country = new \Country($idCountry);
        if (!\Validate::isLoadedObject($country)) {
            throw new \PrestaShopException($this->translator->trans('The country "Belgium" does not exist.', [], 'Modules.Bpostshm.Shop'));
        }

        $zone = null;
        $idZone = (int) \Configuration::getGlobalValue('BPOSTSHM_ID_ZONE_BELGIUM');
        if ($idZone) {
            $zone = new \Zone($idZone);
        }
        if (!\Validate::isLoadedObject($zone)) {
            $labels = ['België', 'Belgie', 'Belgique', 'Belgium'];
            foreach ($labels as $label) {
                if ($idZone = \Zone::getIdByName($label)) {
                    break;
                }
            }
        }

        if (!$idZone) {
            $zone = new \Zone();
            $zone->name = $this->translator->trans('Belgium', [], 'Modules.Bpostshm.Shop');
            $zone->active = true;
            $zone->save();
            $idZone = (int) $zone->id;
        }

        if ($country->id_zone != $idZone) {
            $country->id_zone = $idZone;
            $country->save();
        }

        return $idZone;
    }

    /**
     * Installs the bpost shipping methods
     *
     * @return void
     *
     * @throws \PrestaShopDatabaseException
     * @throws \PrestaShopException
     */
    public function installShippingMethods(): void
    {
        $idZone = $this->addBelgiumZone();

        foreach ($this->module->getShippingMethods() as $id => $method) {
            $carrier = null;

            $idCarrier = \Configuration::getGlobalValue('BPOSTSHM_ID_CARRIER_' . $method['key']);
            if ($idCarrier) {
                $carrier = new \Carrier($idCarrier);
            }

            if (!\Validate::isLoadedObject($carrier) || $carrier->deleted) {
                $delays = [];
                foreach (\Language::getLanguages(false) as $language) {
                    $isoCode = substr($language['iso_code'], 0, 2);
                    $delay = $method['delay'][$isoCode] ?? $method['delay']['en'];
                    $delays[$language['id_lang']] = $delay;
                }

                $carrier = new \Carrier();
                $carrier->deleted = false;
                $carrier->active = true;
                $carrier->external_module_name = $this->module->name;
                $carrier->name = $method['name'];
                $carrier->delay = $delays;
                $carrier->need_range = true;
                $carrier->url = $this->module->getCarrierUrl();
                $carrier->shipping_external = true;
                $carrier->shipping_handling = false;
                $carrier->is_module = true;

                if ($carrier->add()) {
                    \Configuration::updateGlobalValue('BPOSTSHM_ID_CARRIER_' . $method['key'], $carrier->id);

                    $carrier->setGroups(\Group::getAllGroupIds());

                    // LOGO
                    \Tools::copy(
                        _PS_MODULE_DIR_ . $this->module->name . '/img/' . strtolower($method['key']) . '.jpg',
                        _PS_SHIP_IMG_DIR_ . (int) $carrier->id . '.jpg'
                    );

                    if (!$carrier->getZone($idZone)) {
                        $carrier->addZone($idZone);
                    }

                    \Db::getInstance()->insert(
                        'bpostshm_carrier_history',
                        [
                            'id_carrier' => (int) $carrier->id,
                            'shipping_method' => $method['key'],
                        ],
                        false,
                        true,
                        \Db::REPLACE
                    );
                } else {
                    throw new \PrestaShopException($this->translator->trans('Unable to add %method% carrier.', ['%method%' => \Tools::ucfirst(str_replace('_', '', $method['key']))], 'Modules.Bpostshm.Shop'));
                }
            }

            \Configuration::deleteByName('BPOST_SHIP_METHOD_' . $id . '_ID_CARRIER');
        }
    }

    /**
     * Install the BpostShm tabs
     *
     * @return void
     *
     * @throws \PrestaShopDatabaseException
     * @throws \PrestaShopException
     */
    public function installBpostShmTab(): void
    {
        /** @var TabRepository $tabRepository */
        $tabRepository = $this->module->get('prestashop.core.admin.tab.repository');
        $idParent = $tabRepository->findOneIdByClassName('AdminParentShipping');

        if ($idParent) {
            $names = $this->module->getTabNames();
            $parentNames = [];
            $openedOrdersNames = [];
            $treatedOrdersNames = [];
            foreach (\Language::getLanguages(false) as $language) {
                $isoCode = substr($language['iso_code'], 0, 2);
                $parentNames[$language['id_lang']] =
                    $names['AdminParentShipping'][$isoCode] ?? $names['AdminParentShipping']['en'];
                $openedOrdersNames[$language['id_lang']] =
                    $names['AdminBpostshmOpenedOrder'][$isoCode] ?? $names['AdminBpostshmOpenedOrder']['en'];
                $treatedOrdersNames[$language['id_lang']] =
                    $names['AdminBpostshmTreatedOrder'][$isoCode] ?? $names['AdminBpostshmTreatedOrder']['en'];
            }

            $enabled = (bool) \Configuration::get('BPOSTSHM_LABELS_USE_PS');
            if (\Shop::isFeatureActive()) {
                foreach (\Shop::getShops(true) as $shop) {
                    $enabled |= (bool) \Configuration::get(
                        'BPOSTSHM_LABELS_USE_PS',
                        null,
                        $shop['id_shop_group'],
                        $shop['id_shop']
                    );
                }
            }

            if ($idTab = $tabRepository->findOneIdByClassName('AdminParentBpostshm')) {
                $tab = new \Tab($idTab);
            } else {
                $tab = new \Tab();
            }
            $tab->class_name = 'AdminParentBpostshm';
            $tab->id_parent = $idParent;
            $tab->module = 'bpostshm';
            $tab->active = $enabled;
            $tab->enabled = $enabled;
            $tab->icon = '';
            $tab->wording = 'bpost Shipping Manager';
            $tab->wording_domain = 'Modules.Bpostshm.Admin';
            $tab->name = $parentNames;
            $tab->save();
            $idParent = $tab->id;

            if ($idTab = $tabRepository->findOneIdByClassName('AdminBpostshmOpenedOrder')) {
                $tab = new \Tab($idTab);
            } else {
                $tab = new \Tab();
            }
            $tab->class_name = 'AdminBpostshmOpenedOrder';
            $tab->id_parent = $idParent;
            $tab->module = 'bpostshm';
            $tab->route_name = 'admin_bpostshm_index';
            $tab->active = true;
            $tab->icon = '';
            $tab->wording = 'bpost opened orders';
            $tab->wording_domain = 'Modules.Bpostshm.Admin';
            $tab->name = $openedOrdersNames;
            $tab->save();

            if ($idTab = $tabRepository->findOneIdByClassName('AdminBpostshmTreatedOrder')) {
                $tab = new \Tab($idTab);
            } else {
                $tab = new \Tab();
            }
            $tab->class_name = 'AdminBpostshmTreatedOrder';
            $tab->id_parent = $idParent;
            $tab->module = 'bpostshm';
            $tab->route_name = 'admin_bpostshm_treated_index';
            $tab->active = true;
            $tab->icon = '';
            $tab->wording = 'bpost treated orders';
            $tab->wording_domain = 'Modules.Bpostshm.Admin';
            $tab->name = $treatedOrdersNames;
            $tab->save();
        }
    }

    /**
     * Install Db
     *
     * @return void
     *
     * @throws \PrestaShopDatabaseException
     */
    public function installDb(): void
    {
        $sql = [];

        $sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'bpostshm_carrier_history` (
            `id_carrier` INT(10) UNSIGNED NOT NULL,
            `shipping_method` ENUM("HOME","PICKUP","PARCEL_LOCKER") NOT NULL,
            PRIMARY KEY (`id_carrier`)
        ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;';

        $sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'bpostshm_cart` (
            `id_cart` INT(10) UNSIGNED NOT NULL,
            `id_carrier` INT(10) UNSIGNED NOT NULL,
            `id_bpost` INT(10) UNSIGNED NULL,
            `email` VARCHAR(255) NULL DEFAULT NULL,
            `mobile` VARCHAR(255) NULL DEFAULT NULL,
            `reduced_mobility_zone` TINYINT(1) UNSIGNED NOT NULL DEFAULT 0,
            `delivery_date` DATE DEFAULT NULL,
            INDEX (`id_bpost`),
            PRIMARY KEY (`id_cart`, `id_carrier`)
        ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;';

        $sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'bpostshm_pickup` (
            `id_bpost` INT(10) UNSIGNED NOT NULL,
            `id_lang` INT(10) UNSIGNED NOT NULL,
            `company_name` VARCHAR(64) NOT NULL,
            `nr` VARCHAR(120) NULL DEFAULT NULL,
            `street` VARCHAR(255) NULL DEFAULT NULL,
            `city` VARCHAR(80) NOT NULL,
            `zipcode` VARCHAR(10) NOT NULL,
            `iso_country` VARCHAR(2) NOT NULL,
            `type` TINYINT(1) NOT NULL,
            `date_upd` DATETIME NOT NULL,
            PRIMARY KEY (`id_bpost`, `id_lang`)
        ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;';

        $sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'bpostshm_order` (
            `id_order` INT(10) UNSIGNED NOT NULL,
            `retour` TINYINT(1) UNSIGNED NOT NULL DEFAULT 0,
            `bpost_reference` VARCHAR(255) NOT NULL,
            `delivery_method` ENUM("' . implode('","', array_keys($this->module->getDeliveryMethods())) . '") NOT NULL,
            `recipient` VARCHAR(255) NOT NULL,
            `drop_date` DATE NULL,
            `opt_saturday` TINYINT(1) UNSIGNED NOT NULL DEFAULT 0,
            `opt_signature` TINYINT(1) UNSIGNED NOT NULL DEFAULT 0,
            `opt_insurance` TINYINT(1) UNSIGNED NOT NULL DEFAULT 0,
            `opt_2nd_presentation` TINYINT(1) UNSIGNED NOT NULL DEFAULT 0,
            `treated` TINYINT(1) UNSIGNED NOT NULL DEFAULT 0,
            `status` ENUM("' . implode('","', array_keys($this->module->getStates())) . '") NULL DEFAULT NULL,
            `date_add` DATETIME NOT NULL,
            `date_upd` DATETIME NOT NULL,
            PRIMARY KEY (`id_order`, `retour`),
            INDEX(`treated`)
        ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;';

        $sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'bpostshm_label` (
            `id_label` INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
            `id_order` INT(10) UNSIGNED NOT NULL,
            `tracking` VARCHAR(255) NOT NULL,
            `tracking_retour` VARCHAR(255) NOT NULL,
            `retour` ENUM("WITHOUT","WITH","ONLY") NOT NULL,
            `date_add` DATETIME NOT NULL,
            `date_upd` DATETIME NOT NULL,
            PRIMARY KEY (`id_label`),
            UNIQUE(`id_order`, `tracking`, `tracking_retour`, `retour`),
            INDEX(`id_order`, `retour`)
        ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;';

        foreach ($sql as $query) {
            if (false === \Db::getInstance()->execute($query)) {
                throw new \PrestaShopDatabaseException($this->translator->trans('An error occurred when executing query %query%.', ['%query%' => $query], 'Modules.Bpostshm.Shop'));
            }
        }
    }

    /**
     * Upgrade Db v1 to v2
     *
     * @return void
     *
     * @throws \PrestaShopDatabaseException
     */
    public function upgradeDbToV2(): void
    {
        $this->installDb();

        $sql = [];
        $sql[] = 'ALTER TABLE `' . _DB_PREFIX_ . 'order_bpost` ADD INDEX(`reference`);';
        $sql[] = 'ALTER TABLE `' . _DB_PREFIX_ . 'cart_bpost` ADD INDEX(`id_cart`);';
        foreach ($sql as $query) {
            if (false === \Db::getInstance()->execute($query)) {
                throw new \PrestaShopDatabaseException($this->translator->trans('An error occurred when executing query %query%.', ['%query%' => $query], 'Modules.Bpostshm.Shop'));
            }
        }

        $maxDays = 30;
        $displayDays = (int) \Configuration::get('BPOSTSHM_LABELS_ORDER_DISPLAY_DAYS');
        if ($displayDays > 0) {
            $maxDays = min($displayDays, $maxDays);
        }
        $this->upgradeDataToV2(date('Y-m-d 00:00:00', strtotime("- $maxDays DAYS")));

        foreach ($this->module->getMethods() as $shippingMethod => $name) {
            $idCarrier = (int) \Configuration::getGlobalValue("BPOSTSHM_ID_CARRIER_$shippingMethod");
            if (\Validate::isLoadedObject(new \Carrier($idCarrier))) {
                \Db::getInstance()->insert(
                    'bpostshm_carrier_history',
                    [
                        'id_carrier' => $idCarrier,
                        'shipping_method' => $shippingMethod,
                    ],
                    false,
                    true,
                    \Db::REPLACE
                );
            }
        }
    }

    /**
     * Upgrade v1 data in DB to v2
     *
     * @param string|null $maxDate The maximum date in the past to consider when selecting orders (format: YYYY-MM-DD HH:MM:SS)
     * @param SymfonyStyle|null $io
     *
     * @return void
     */
    public function upgradeDataToV2(?string $maxDate = null, ?SymfonyStyle $io = null): void
    {
        $sql = [];

        $dateMaxCond = '';
        if ($maxDate) {
            if (strtotime($maxDate) === false) {
                throw new \PrestaShopDatabaseException('Max date is invalid!');
            }
            $dateMaxCond = 'date_add >= "' . $maxDate . '"';
        }

        $sql[] = 'REPLACE INTO `' . _DB_PREFIX_ . 'bpostshm_order` (
                id_order,
                retour,
                bpost_reference,
                delivery_method,
                recipient,
                drop_date,
                opt_saturday,
                opt_signature,
                opt_insurance,
                opt_2nd_presentation,
                treated,
                status,
                date_add,
                date_upd
            )
            SELECT
                SUBSTRING_INDEX(SUBSTRING_INDEX(`reference`, "_", 2), "_", -1),
                0,
                `reference`,
                TRIM(SUBSTRING_INDEX(`delivery_method`, ":", 1)),
                `recipient`,
                CONCAT(LEFT(dt_drop, 4), "-", MID(dt_drop, 5, 2), "-", RIGHT(dt_drop, 2)),
                IF(delivery_method LIKE "%470%" , 1, 0),
                IF(delivery_method LIKE "%300%" , 1, 0),
                IF(delivery_method LIKE "%350%" OR delivery_method LIKE "%540%", 1, 0),
                IF(delivery_method LIKE "%330%" , 1, 0),
                `treated`,
                `status`,
                `date_add`,
                DATE_FORMAT(`date_upd`, "%Y-%m-%d %H:%i:%s")
            FROM `' . _DB_PREFIX_ . 'order_bpost`
            WHERE ' . ($dateMaxCond ?: '1=1') . ' AND TRIM(SUBSTRING_INDEX(`delivery_method`, ":", 1))
                IN ("' . implode('","', array_keys($this->module->getDeliveryMethods())) . '");';

        $sql[] = 'REPLACE INTO `' . _DB_PREFIX_ . "bpostshm_cart` (
                id_cart,
                id_carrier,
                id_bpost,
                email,
                mobile,
                reduced_mobility_zone,
                delivery_date
            )
            SELECT
                cb.`id_cart`,
                o.`id_carrier`,
                cb.`service_point_id`,
                REPLACE(JSON_EXTRACT(cb.upl_info, '$.eml'), '\"', ''),
                REPLACE(JSON_EXTRACT(cb.upl_info, '$.mob'), '\"', ''),
                IF(upl_info LIKE '%\"rmz\":true%', 1, 0),
                bo.`drop_date`
            FROM `" . _DB_PREFIX_ . 'cart_bpost` cb
            INNER JOIN `' . _DB_PREFIX_ . 'orders` o
                ON o.id_cart = cb.id_cart
            LEFT JOIN `' . _DB_PREFIX_ . 'bpostshm_order` bo
                ON bo.id_order = o.id_order
            WHERE ' . ($dateMaxCond ? ('bo.' . $dateMaxCond) : '1=1') . ';';

        $sql[] = 'INSERT INTO `' . _DB_PREFIX_ . 'bpostshm_carrier_history` (
                id_carrier,
                shipping_method
            )
            SELECT
                c.`id_carrier`,
                CASE
                    WHEN ob.shm IN (1, 9, 25) THEN "HOME"
                    WHEN ob.shm IN (2, 10) THEN "PICKUP"
                    WHEN ob.shm = 4 THEN "PARCEL_LOCKER"
                    ELSE ""
                END AS shipping_method
            FROM `' . _DB_PREFIX_ . 'carrier` c
            INNER JOIN `' . _DB_PREFIX_ . 'orders` o
                ON c.id_carrier = o.id_carrier
            INNER JOIN (
                SELECT SUBSTRING_INDEX(SUBSTRING_INDEX(`reference`, "_", 2), "_", -1) as id_order, shm, date_add 
                FROM `' . _DB_PREFIX_ . 'order_bpost`
            ) ob ON ob.id_order = o.id_order
            WHERE ' . ($dateMaxCond ? ('ob.' . $dateMaxCond) : '1=1') . '
            GROUP BY (c.`id_carrier`)
            ON DUPLICATE KEY UPDATE shipping_method = VALUES(shipping_method);';

        $sql[] = 'REPLACE INTO `' . _DB_PREFIX_ . 'bpostshm_label` (
                id_order,
                tracking,
                tracking_retour,
                retour,
                date_add,
                date_upd
            )
            SELECT
                SUBSTRING_INDEX(SUBSTRING_INDEX(ob.reference, "_", 2), "_", -1),
                IF(l.barcode IS NOT NULL AND l.barcode <> "" AND NOT l.is_retour, l.barcode, ""),
                IF(l.barcode_retour IS NOT NULL AND l.barcode_retour <> "" AND l.has_retour, l.barcode_retour,
                    IF(l.barcode IS NOT NULL AND l.barcode <> "" AND l.is_retour, l.barcode, "")
                ),
                IF(l.is_retour, "ONLY", IF(l.has_retour, "WITH", "WITHOUT")),
                l.date_add,
                DATE_FORMAT(l.date_upd, "%Y-%m-%d %H:%i:%s")
            FROM `' . _DB_PREFIX_ . 'order_bpost_label` l
            INNER JOIN `' . _DB_PREFIX_ . 'order_bpost` ob
                ON l.id_order_bpost = ob.id_order_bpost AND ' . ($dateMaxCond ? ('ob.' . $dateMaxCond) : '1=1') . '
            WHERE
               (l.barcode IS NOT NULL AND l.barcode <> "" AND NOT l.is_retour)
                OR (l.barcode_retour IS NOT NULL AND l.barcode_retour <> "" AND l.has_retour);
        ';

        /*
        $sql[] = 'DROP TABLE `' . _DB_PREFIX_ . 'cart_bpost`';
        $sql[] = 'DROP TABLE `' . _DB_PREFIX_ . 'order_bpost`';
        $sql[] = 'DROP TABLE `' . _DB_PREFIX_ . 'order_bpost_label`';
        */

        foreach ($sql as $query) {
            if ($io instanceof SymfonyStyle) {
                $io->note($query . "\n");
            }
            if (false === \Db::getInstance()->execute($query)) {
                throw new \PrestaShopDatabaseException($this->translator->trans('An error occurred when executing query %query%.', ['%query%' => $query], 'Modules.Bpostshm.Shop'));
            }
        }
    }

    /**
     * Set default values on install
     *
     * @param bool $erase erase config in DB?
     *
     * @return void
     */
    public function setDefaultValues(bool $erase = false): void
    {
        if (!\Configuration::getGlobalValue('BPOSTSHM_CRON_SECURE_KEY')) {
            \Configuration::updateGlobalValue('BPOSTSHM_CRON_SECURE_KEY', \Tools::strtoupper(\Tools::passwdGen(16)));
        }

        $defaultValues = $this->module->getSettingsFormHelper()->getConfigFieldsAndDefaultValues();
        foreach ($defaultValues as $configKey => $configData) {
            $value = \Configuration::getGlobalValue($configKey);
            if ($erase || false === $value) {
                $value = $configData['default'];
            }
            \Configuration::updateGlobalValue($configKey, $value);
        }
    }

    /**
     * Upgrade config v1 to v2
     *
     * @return void
     */
    public function upgradeConfigToV2(): void
    {
        foreach ($this->module->getShippingMethods() as $id => $method) {
            $oldConfigKey = 'BPOST_SHIP_METHOD_' . $id . '_ID_CARRIER';
            $configKey = 'BPOSTSHM_ID_CARRIER_' . $method['key'];
            if (\Configuration::getGlobalValue($oldConfigKey)) {
                $carrier = new \Carrier(\Configuration::getGlobalValue($oldConfigKey));
                if (\Validate::isLoadedObject($carrier)) {
                    \Configuration::updateGlobalValue($configKey, $carrier->id);
                    $carrier->name = $method['name'];
                    $carrier->is_module = true;
                    $carrier->external_module_name = $this->module->name;
                    $carrier->url = $this->module->getCarrierUrl();
                    $delays = [];
                    foreach (\Language::getLanguages(false) as $language) {
                        $isoCode = substr($language['iso_code'], 0, 2);
                        $delay = $method['delay'][$isoCode] ?? $method['delay']['en'];
                        $delays[$language['id_lang']] = $delay;
                    }
                    $carrier->delay = $delays;
                    $carrier->update();
                }
            }
            \Configuration::deleteByName($oldConfigKey);
        }

        $idZone = (int) \Configuration::getGlobalValue('BPOST_ID_ZONE_BELGIUM');
        if ($idZone) {
            $zone = new \Zone($idZone);
            if (\Validate::isLoadedObject($zone)) {
                \Configuration::updateGlobalValue('BPOSTSHM_ID_ZONE_BELGIUM', $idZone);
            }
        }

        $this->upgradeStoreDetails();
        $this->upgradeDeliveryOptionList();
        $this->upgradeIntlEad();
        $this->upgradeGeneralConfigs();

        if (\Shop::isFeatureActive()) {
            foreach (\Shop::getShops(true) as $shop) {
                $this->upgradeStoreDetails($shop['id_shop_group'], $shop['id_shop']);
                $this->upgradeDeliveryOptionList($shop['id_shop_group'], $shop['id_shop']);
                $this->upgradeIntlEad($shop['id_shop_group'], $shop['id_shop']);
                $this->upgradeGeneralConfigs($shop['id_shop_group'], $shop['id_shop']);
            }
        }

        // delete unused config (upgrade v1 to v2)
        $defaultValues = $this->module->getSettingsFormHelper()->getConfigFieldsAndDefaultValues();
        foreach ($defaultValues as $configData) {
            if (isset($configData['old'])) {
                \Configuration::deleteByName($configData['old']);
            }
        }
        \Configuration::deleteByName('BPOST_ACCOUNT_API_URL');
        \Configuration::deleteByName('BPOST_DEBUG_LOG_ENABLE');
        \Configuration::deleteByName('BPOST_ID_ZONE_BELGIUM');
        \Configuration::deleteByName('BPOST_HOME_24H_BUSINESS');
        \Configuration::deleteByName('BPOST_INTERNATIONAL_DELIVERY');
        \Configuration::deleteByName('BPOST_DISPLAY_ADMIN_INFO');
        \Configuration::deleteByName('BPOST_STORE_DETAILS');
        \Configuration::deleteByName('BPOST_DELIVERY_OPTIONS_LIST');
        \Configuration::deleteByName('BPOST_INTL_EAD');
        \Configuration::deleteByName('BPOST_INTL_COUNTRIES');
        \Configuration::deleteByName('BPOST_ORDER_STATE_TREATED');
        \Configuration::deleteByName('BPOST_DISPLAY_ORDER_STATES');
        \Configuration::deleteByName('BPOST_CACHED_DATES');
        \Configuration::deleteByName('BPOST_HOME_DELIVERY_ONLY'); // v1.22
    }

    /**
     * Upgrades config values about store details.
     *
     * @param int|null $idShopGroup
     * @param int|null $idShop
     *
     * @return void
     */
    private function upgradeStoreDetails(?int $idShopGroup = null, ?int $idShop = null)
    {
        $store = \Configuration::get('BPOST_STORE_DETAILS', null, $idShopGroup, $idShop);
        if ($store && SettingsFormHelper::isValidJSON($store)) {
            $data = json_decode($store, true);
            $storesDetails = [
                'name' => 'SENDER',
                'address1' => 'ADDRESS',
                'city' => 'CITY',
                'postcode' => 'POSTCODE',
                'email' => 'EMAIL',
                'phone' => 'PHONE',
            ];
            foreach ($storesDetails as $key => $value) {
                if (isset($data[$key])) {
                    \Configuration::updateValue(
                        'BPOSTSHM_STORE_DETAILS_' . $value,
                        $data[$key],
                        false,
                        $idShopGroup,
                        $idShop
                    );
                }
            }
        }
    }

    /**
     * Upgrades config values about delivery options.
     *
     * @param int|null $idShopGroup
     * @param int|null $idShop
     *
     * @return void
     */
    private function upgradeDeliveryOptionList(?int $idShopGroup = null, ?int $idShop = null)
    {
        $deliveryOptionsList = \Configuration::get('BPOST_DELIVERY_OPTIONS_LIST', null, $idShopGroup, $idShop);
        /* For instance:
            {
                "1": {
                    "300": "3.50", // BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE: 'signature' -> BPOSTSHM_DELIVERY_OPTIONS_HOME_SIGNATURE_FROM
                    "330": "4.00", // BPOSTSHM_DELIVERY_OPTIONS_HOME_2ND_PRESENTATION_ENABLED -> BPOSTSHM_DELIVERY_OPTIONS_HOME_2ND_PRESENTATION_FROM
                    "350": "3.00", // BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE: 'insurance' -> BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE_FROM
                    "470": ["1.00", "2.00"] // BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_ENABLED -> BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_FROM / BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_COST
                },
                "2": {
                    "350": "7.00", // BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INSURANCE_ENABLED -> BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INSURANCE_FROM
                    "470": ["5.00", "6.00"] // BPOSTSHM_DELIVERY_OPTIONS_PICKUP_SATURDAY_ENABLED -> BPOSTSHM_DELIVERY_OPTIONS_PICKUP_SATURDAY_FROM / BPOSTSHM_DELIVERY_OPTIONS_PICKUP_SATURDAY_COST
                },
                "4": {
                    "350": "10.00", // BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_INSURANCE_ENABLED -> BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_INSURANCE_FROM
                    "470": ["8.00", "9.00"]  // BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_SATURDAY_ENABLED -> BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_SATURDAY_FROM / BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_SATURDAY_COST
                },
                "9": {
                    "540": "11.00" // BPOSTSHM_DELIVERY_OPTIONS_HOME_INTL_INSURANCE_ENABLED -> BPOSTSHM_DELIVERY_OPTIONS_HOME_INTL_INSURANCE_FROM
                },
                "10": {
                    "540": "12.00" // BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INTL_INSURANCE_ENABLED -> BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INTL_INSURANCE_FROM
                }
            }
        */
        if ($deliveryOptionsList && SettingsFormHelper::isValidJSON($deliveryOptionsList)) {
            $data = json_decode($deliveryOptionsList, true);
            if (isset($data['1']['330'])) {
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_HOME_2ND_PRESENTATION_ENABLED', 1, false, $idShopGroup, $idShop);
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_HOME_2ND_PRESENTATION_FROM', (float) $data['1']['330'], false, $idShopGroup, $idShop);
            }
            if (isset($data['1']['350'])) {
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE', 'insurance', false, $idShopGroup, $idShop);
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE_FROM', (float) $data['1']['350'], false, $idShopGroup, $idShop);
            } elseif (isset($data['1']['300'])) {
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_HOME_INSURANCE', 'signature', false, $idShopGroup, $idShop);
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_HOME_SIGNATURE_FROM', (float) $data['1']['300'], false, $idShopGroup, $idShop);
            }
            if (isset($data['1']['470'])) {
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_ENABLED', 1, false, $idShopGroup, $idShop);
                if (isset($data['1']['470'][0])) {
                    \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_FROM', (float) $data['1']['470'][0], false, $idShopGroup, $idShop);
                }
                if (isset($data['1']['470'][1])) {
                    \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_HOME_SATURDAY_COST', (float) $data['1']['470'][1], false, $idShopGroup, $idShop);
                }
            }
            if (isset($data['2']['350'])) {
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INSURANCE_ENABLED', 1, false, $idShopGroup, $idShop);
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INSURANCE_FROM', (float) $data['2']['350'], false, $idShopGroup, $idShop);
            }
            if (isset($data['2']['470'])) {
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_PICKUP_SATURDAY_ENABLED', 1, false, $idShopGroup, $idShop);
                if (isset($data['2']['470'][0])) {
                    \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_PICKUP_SATURDAY_FROM', (float) $data['2']['470'][0], false, $idShopGroup, $idShop);
                }
                if (isset($data['2']['470'][1])) {
                    \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_PICKUP_SATURDAY_COST', (float) $data['2']['470'][1], false, $idShopGroup, $idShop);
                }
            }
            if (isset($data['4']['350'])) {
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_INSURANCE_ENABLED', 1, false, $idShopGroup, $idShop);
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_INSURANCE_FROM', (float) $data['4']['350'], false, $idShopGroup, $idShop);
            }
            if (isset($data['4']['470'])) {
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_SATURDAY_ENABLED', 1, false, $idShopGroup, $idShop);
                if (isset($data['4']['470'][0])) {
                    \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_SATURDAY_FROM', (float) $data['4']['470'][0], false, $idShopGroup, $idShop);
                }
                if (isset($data['4']['470'][1])) {
                    \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_PARCEL_LOCKER_SATURDAY_COST', (float) $data['4']['470'][1], false, $idShopGroup, $idShop);
                }
            }
            if (isset($data['9']['540'])) {
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_HOME_INTL_INSURANCE_ENABLED', 1, false, $idShopGroup, $idShop);
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_HOME_INTL_INSURANCE_FROM', (float) $data['9']['540'], false, $idShopGroup, $idShop);
            }
            if (isset($data['10']['540'])) {
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INTL_INSURANCE_ENABLED', 1, false, $idShopGroup, $idShop);
                \Configuration::updateValue('BPOSTSHM_DELIVERY_OPTIONS_PICKUP_INTL_INSURANCE_FROM', (float) $data['10']['540'], false, $idShopGroup, $idShop);
            }
        }
    }

    /**
     * Upgrades config values about EAD.
     *
     * @param int|null $idShopGroup
     * @param int|null $idShop
     *
     * @return void
     */
    private function upgradeIntlEad(?int $idShopGroup = null, ?int $idShop = null)
    {
        $intlEad = trim(\Configuration::get('BPOST_INTL_EAD', null, $idShopGroup, $idShop));
        /* for instance
            {
                "hscode": {
                    "opt":1,
                    "index":0,
                    "id":0,
                    "code":"9999"
                },
                "origin": {
                    "opt":1,
                    "index":1,
                    "id":"2",
                    "code":"BE"
                }
            }
        */
        if ($intlEad && SettingsFormHelper::isValidJSON($intlEad)) {
            $data = json_decode($intlEad, true);
            \Configuration::updateValue('BPOSTSHM_INTL_OUTSIDE_EU_ENABLED', 1, false, $idShopGroup, $idShop);
            if (isset($data['hscode']['index'])) {
                if ($data['hscode']['index'] == 0 && isset($data['hscode']['code'])) {
                    \Configuration::updateValue('BPOSTSHM_INTL_HS_CODE_TYPE', 'fixed', false, $idShopGroup, $idShop);
                    \Configuration::updateValue('BPOSTSHM_INTL_HS_CODE_FIXED_VALUE', $data['hscode']['code'], false, $idShopGroup, $idShop);
                } elseif ($data['hscode']['index'] == 1 && isset($data['hscode']['id']) && (int) $data['hscode']['id'] > 0) {
                    \Configuration::updateValue('BPOSTSHM_INTL_HS_CODE_TYPE', 'feature', false, $idShopGroup, $idShop);
                    \Configuration::updateValue('BPOSTSHM_INTL_HS_CODE_FEATURE_VALUE', (int) $data['hscode']['id'], false, $idShopGroup, $idShop);
                }
            }
            if (isset($data['origin']['index'])) {
                if ($data['origin']['index'] == 0 && isset($data['origin']['code'])) {
                    \Configuration::updateValue('BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_TYPE', 'fixed', false, $idShopGroup, $idShop);
                    \Configuration::updateValue('BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_FIXED_VALUE', $data['origin']['code'], false, $idShopGroup, $idShop);
                } elseif ($data['origin']['index'] == 1 && isset($data['origin']['id']) && (int) $data['origin']['id'] > 0) {
                    \Configuration::updateValue('BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_TYPE', 'feature', false, $idShopGroup, $idShop);
                    \Configuration::updateValue('BPOSTSHM_INTL_COUNTRY_OF_ORIGIN_FEATURE_VALUE', (int) $data['origin']['id'], false, $idShopGroup, $idShop);
                }
            }
        } else {
            \Configuration::updateValue('BPOSTSHM_INTL_OUTSIDE_EU_ENABLED', 0, false, $idShopGroup, $idShop);
        }
    }

    /**
     * Upgrades config values about other general configs.
     *
     * @param int|null $idShopGroup
     * @param int|null $idShop
     *
     * @return void
     */
    private function upgradeGeneralConfigs(?int $idShopGroup = null, ?int $idShop = null)
    {
        // business / pro
        foreach ([
            'BPOST_HOME_24H_BUSINESS' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_BUSINESS',
            'BPOST_INTERNATIONAL_DELIVERY' => 'BPOSTSHM_DELIVERY_OPTIONS_HOME_INTL_BUSINESS',
        ] as $oldConfig => $newConfig) {
            \Configuration::updateValue(
                $newConfig,
                \Configuration::get($oldConfig, null, $idShopGroup, $idShop) == '1' ? 'business' : 'pro',
                false,
                $idShopGroup,
                $idShop
            );
        }

        // States
        if (\Configuration::get('BPOST_DISPLAY_ORDER_STATES', null, $idShopGroup, $idShop)) {
            \Configuration::updateValue(
                'BPOSTSHM_LABELS_ORDER_DISPLAY_STATES',
                json_encode(
                    explode(',', \Configuration::get('BPOST_DISPLAY_ORDER_STATES'))
                ),
                false,
                $idShopGroup,
                $idShop
            );
        }

        $defaultValues = $this->module->getSettingsFormHelper()->getConfigFieldsAndDefaultValues($idShopGroup, $idShop);
        foreach ($defaultValues as $configKey => $configData) {
            if (isset($configData['old']) && $configData['old']) {
                $oldValue = \Configuration::get($configData['old'], null, $idShopGroup, $idShop);
                \Configuration::updateValue(
                    $configKey,
                    ($oldValue === false && isset($configData['default'])) ? $configData['default'] : $oldValue,
                    false,
                    $idShopGroup,
                    $idShop
                );
            }
        }

        if (\Configuration::get('BPOSTSHM_DELIVERY_OPTIONS_HOME_BUSINESS', null, $idShopGroup, $idShop) == 'business') {
            $this->module->getSettingsFormHelper()->applyBusinessOptions($idShopGroup, $idShop);
        }
    }
}
