<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

namespace PrestaShop\Module\BpostShm\Utils;

use BpostShm;
use Carrier;
use Configuration;
use PrestaShop\Module\BpostShm\Service\DeliveryDateService;
use PrestaShopBundle\Entity\Repository\TabRepository;
use Tab;

/**
 * Class Uninstaller
 */
class Uninstaller
{
    /**
     * @var \BpostShm
     */
    private $module;

    /**
     * Uninstaller constructor.
     *
     * @param \BpostShm $module
     */
    public function __construct(\BpostShm $module)
    {
        $this->module = $module;
    }

    /**
     * Uninstall the module.
     *
     * This method is responsible for uninstalling the module, which includes
     * removing carriers, tab, database tables, and deleting configuration values.
     *
     * @return void
     */
    public function uninstall(): void
    {
        $this->uninstallCarriers();
        $this->uninstallBpostShmTab();
        $this->uninstallDb();
        $this->deleteConfigValues();
    }

    /**
     * Uninstalls the carriers associated with the module.
     *
     * This method deactivates and deletes the carriers associated with the BpostShm module.
     * It also deletes the relevant configuration values.
     *
     * @return void
     */
    public function uninstallCarriers(): void
    {
        $needsClean = false;
        foreach ($this->module->getShippingMethods() as $method) {
            $idCarrier = (int) \Configuration::getGlobalValue('BPOSTSHM_ID_CARRIER_' . $method['key']);
            $carrier = new \Carrier($idCarrier);
            if (\Validate::isLoadedObject($carrier)) {
                $carrier->active = false;
                $carrier->deleted = true;
                $carrier->save();
                $needsClean = true;
            }
            \Configuration::deleteByName('BPOSTSHM_ID_CARRIER_' . $method['key']);
            \Configuration::deleteByName('BPOST_ID_CARRIER_' . $method['key']);
        }
        if ($needsClean) {
            \Carrier::cleanPositions();
        }
        \Configuration::deleteByName('BPOSTSHM_ID_ZONE_BELGIUM');
    }

    /**
     * Uninstalls the BpostShm tab from the admin panel.
     */
    public function uninstallBpostShmTab(): void
    {
        /** @var TabRepository $tabRepository */
        $tabRepository = $this->module->get('prestashop.core.admin.tab.repository');

        $tabs = [
            'AdminOrdersBpost',
            'AdminParentBpostshm',
            'AdminBpostshmOpenedOrder',
            'AdminBpostshmTreatedOrder',
        ];
        foreach ($tabs as $class) {
            $tabId = $tabRepository->findOneIdByClassName($class);
            if ($tabId) {
                $tab = new \Tab($tabId);
                $tab->delete();
            }
        }
    }

    /**
     * Removes the database tables and data associated with the BpostShm module.
     *
     * @throws \PrestaShopDatabaseException if there is an error executing the SQL query
     */
    public function uninstallDb(): void
    {
        /*
        $sql[] = 'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'bpostshm_carrier_history`';
        $sql[] = 'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'bpostshm_cart`';
        $sql[] = 'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'bpostshm_pickup`';
        $sql[] = 'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'bpostshm_order`';
        $sql[] = 'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'bpostshm_label`';
        */
        $sql = [];

        foreach ($sql as $query) {
            if (false === Db::getInstance()->execute($query)) {
                throw new \PrestaShopDatabaseException($this->translator->trans('An error occurred when executing query %query%.', ['%query%' => $query], 'Modules.Bpostshm.Shop'));
            }
        }
    }

    /**
     * Delete all saved config values
     *
     * @return void
     */
    public function deleteConfigValues(): void
    {
        \Configuration::deleteByName('BPOSTSHM_CRON_SECURE_KEY');
        \Configuration::deleteByName(DeliveryDateService::CACHED_CONFIG_KEY);
        foreach ($this->module->getSettingsFormHelper()->getConfigFieldsAndDefaultValues() as $key => $value) {
            \Configuration::deleteByName($key);
        }
    }
}
